<?php
namespace App\Classes\Finance;

use App\Classes\Finance\DailyJournalHelper;

use App\Models\Finance\ExpenseCode;
use App\Models\Sales\Delegate;
use App\Models\Finance\DelegatePetty;
use App\Models\Finance\Tax;
use App\Models\Finance\DailyJournal;
use App\Models\Finance\Treasury;

use Illuminate\Support\Facades\DB;

class DelegatePettyHelper
{
    function get_next_codes($date , $year , $branch)
    {
        $data['year_ser'] = intval( DB::table('delegate_petty_m')
                ->where('finance_year_id' , $year)
                ->max('year_ser') ) + 1 ;

        $data['month_ser'] = intval( DB::table('delegate_petty_m')
                ->where('finance_year_id' , $year)
                ->where('action_date' , 'like' , date("Y-m" , strtotime($date))."%" )
                ->max('month_ser') ) + 1 ;

        $data['branch_ser'] = intval( DB::table('delegate_petty_m')
                ->where('branch_id' , $branch)
                ->max('branch_ser') ) + 1 ;

        return $data;
    }

    function can_edit($id)
    {
        return true;
    }


    function can_delete($id)
    {

        $master     = DelegatePetty::get_delegate_petty_m($id, 'en');
        $details    = DelegatePetty::get_delegate_petty_d($id, 'en');
        if(!isset($master->m_id) ){
            return [false , 'item not found !'];
        }

        if(collect($details)->where('invoice_id' , 0 )->count() != count($details) ){
            return [false , 'some item has invoices !'];
        }

        return [true];
    }

    function update_balance($id){

        $multi_debit = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            ->where('multi_voucher_active' , 1)
            ->where('voucher_type' , 1)
            ->where('dis_type' , 6)
            ->where('dis_delegate_id' , $id)
            ->sum('amount');

        $multi_credit = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            ->where('multi_voucher_active' , 1)
            ->where('voucher_type' , 2)
            ->where('dis_type' , 6)
            ->where('dis_delegate_id' , $id)
            ->sum('amount');

        $payment = DB::table('payment_voucher_d')
            ->join('payment_voucher_m' , 'payment_voucher_d.payment_voucher_m_id' , '=' , 'payment_voucher_m.payment_voucher_m_id')
            ->where('payment_voucher_active' , 1)
            ->where('delegate_id' , $id)
            ->sum('amount');

        $receipt = DB::table('receipt_voucher_d')
            ->join('receipt_voucher_m' , 'receipt_voucher_d.receipt_voucher_m_id' , '=' , 'receipt_voucher_m.receipt_voucher_m_id')
            ->where('receipt_voucher_active' , 1)
            ->where('delegate_id' , $id)
            ->sum('amount') * -1 ;

        $petty_cash = DB::table('delegate_petty_d')
            ->join('delegate_petty_m' , 'delegate_petty_d.delegate_petty_m_id' , '=' , 'delegate_petty_m.delegate_petty_m_id')
            ->where('delegate_petty_active' , 1)
            ->where('delegate_id' , $id)
            ->sum('amount') * -1 ;


        DB::table('delegates')
            ->where('delegate_id' , $id)
            ->update(['balance' => ( $payment + $receipt + $petty_cash + $multi_debit - $multi_credit ) ]);


    }

    function generate_journal($id)
    {

        $master     = DelegatePetty::get_delegate_petty_m($id, 'en');
        $details    = DelegatePetty::get_delegate_petty_d($id, 'en');

        if($master->delegate_petty_active == 0 && $master->daily_journal_m_id > 0 ){
            DailyJournal::delete_daily_journal_m($master->daily_journal_m_id);
            return array( true ,  '')  ;
        }

        $items          = [];
        $journal_data   = array(
            'finance_year_id'   => $master->finance_year_id  ,
            'branch_id'         => $master->branch_id ,
            'des'               => $master->description ?? 'تصفيه عهده مندوب  '.$master->delegate_name  ,
            'journal_date'      => $master->action_date ,
            'document_id'       => $id ,
            'daily_type_id'     => 20 ,
            'auto_generated'    => 1 ,
            'journal_active'    => 1 ,
            'confirmed'         => 1 ,
            'daily_journal_id'  => $master->daily_journal_m_id
        );

        $credit_account     = Delegate::get_delegate($master->delegate_id, 'en')->account_tree_id ?? 0;

        if( $credit_account == 0  ){
            return array( false ,  'Delegate Account tree id not found ' , '' ,[] );
        }

        foreach ($details as $key => $row) {

            if($row->is_finance){
                $debit_account      = Treasury::get_treasury( $row->treasury_id ,  'en' )->account_tree_id ?? 0 ;
            }else{
                $debit_account      = ExpenseCode::get_expense_code($row->expense_code_id, 'en')->account_tree_id ?? 0;
            }

            if( $debit_account == 0  ){
                return array( false ,  'ExpenseCode or Treasury Account tree id not found ' , '' ,[] );
            }

            $des = " ".(strlen($row->clearance_id) > 0 ? "معامله رقم  " : "" ) .$row->clearance_id." - ".$row->description;

            $items[]    = array(
                'account_tree_id'   => $debit_account ,
                'cost_center_id'    => 0 ,
                'debit'             => $row->amount_without_vat ,
                'credit'            => 0 ,
                'description'       => $des
            );

            if($row->tax_id > 0 ){
                $tax = Tax::get_tax($row->tax_id , 'en');

                if($tax->account_tree_id == null || $tax->account_tree_id == 0 ){
                    return array( false ,  'Account tree id not found in '.$tax->name , '' ,[] );
                }

                $items[]    = array(
                    'account_tree_id'   => $tax->account_tree_id ,
                    'cost_center_id'    => 0 ,
                    'debit'             => $row->total_vat ,
                    'credit'            => 0 ,
                    'description'       => $tax->name.$des
                );
            }

            $items[]    = array(
                'account_tree_id'   => $credit_account ,
                'cost_center_id'    => 0 ,
                'debit'             => 0 ,
                'credit'            => $row->amount ,
                'description'       => $des
            );

        }

        $daily_journal = (new DailyJournalHelper)->generate_journal($journal_data , $items );


        if(!$daily_journal[0]){
            DB::rollback();
            return $daily_journal;
        }
        DelegatePetty::update_delegate_petty_m([ 'daily_journal_m_id' => $daily_journal[2] ] , $id );

        return $daily_journal;


    }
    function check_invoices($clearance_id  = null ,$invoice_id = null, $invoice_type = null ) //check if the detail has invoice to update it
    {

        if($invoice_id > 0){
            if($invoice_type == 'claim'){
                DB::table('clearance_invoices_claim_m')->where('invoice_id' , $invoice_id)->update(['need_update' => 1]);
            }elseif($invoice_type == 'invoice'){
              DB::table('clearance_invoices_m')->where('invoice_id' , $invoice_id)->update(['need_update' => 1]);

            }
        }
        if($clearance_id > 0){
            $last_claim = DB::table('clearance_invoices_claim_m')->where('clearance_id' , $clearance_id)->orderBy('invoice_id', 'desc')->first();
            if(isset($last_claim) && $last_claim->invoice_id > 0){
                DB::table('clearance_invoices_claim_m')->where('invoice_id' , $last_claim->invoice_id)->update(
                    ['need_update' => 1]
                );

            }else{
                $last_invoice = DB::table('clearance_invoices_m')->where('clearance_id' , $clearance_id)->orderBy('invoice_id', 'desc')->first();
                if(isset($last_invoice) && $last_invoice->invoice_id > 0){
                    DB::table('clearance_invoices_m')->where('invoice_id' , $last_invoice->invoice_id)->update(
                        ['need_update' => 1]
                    );
                }
            }

        }

        return true;
    }

}
