<?php
namespace App\Classes\General;
use App\Classes\General\Tafqeet\Tafqeet;

use App\Models\User;
use App\Models\Main\Branch;
use App\Models\Main\FinanceYear;
use App\Models\Main\Setting;
use Carbon\Carbon;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Request;

class General {


    public function numberToText($lang  , $number , $currency = "SAR" , $big = "SAR" , $small = "Halala" ) {

        if($lang == 'ar'){

            return Tafqeet::inArabic(round($number , 2) , strtolower($currency) );

        }elseif($lang == 'en'){

            $digit = new \NumberFormatter('en_US', \NumberFormatter::SPELLOUT);
            $numberParts = explode('.', (string) round($number,2));
            $formatedNumber =  $digit->format($numberParts[0])." ".$big;
            if (isset($numberParts[1] ) ) {
                $formatedNumber .= ' and ' . $digit->format($numberParts[1])." ".$small;
            }
            return ucwords($formatedNumber);

        }

    }


    public function check_date($date){

        $all_years  = $this->get_user_years(auth()->user()->id  , 'en' );
        $year       = array_search( date("Y" , strtotime($date)) , $all_years );
        if($year == ''){
            return array( 'check' => false , 'year_id' => 0 );
        }
        return array( 'check' => true , 'year_id' => $year );
    }

    public function get_user_branchs( $UserId , $lang ){
        return DB::table('branchs')->whereRaw("find_in_set(".$UserId." , users)")->where('branch_active' , 1 )->pluck('name_'.$lang.' as name' , 'branch_id')->toArray();
    }

    public function get_user_years( $UserId , $lang ){
        return DB::table('finance_years')->whereRaw("find_in_set(".auth()->user()->id." , users)")->where('finance_year_active' , 1 )->pluck('name' , 'finance_year_id')->toArray();
    }

    public function get_user_titles($UserId, $lang)
    {
        $user = User::find($UserId);
        $user_titles = explode(',', $user->user_titles);
        return DB::table('title')->whereIn('title_id', $user_titles)->select('title_id', 'title_name_'.$lang.' as name')->get();
    }

    public function get_current_shift($user){
        return DB::table('shifts')
            ->join('branchs' , 'shifts.branch_id' , '=' , 'branchs.branch_id')
            ->where('user_id' , $user)
            ->where('end_date' , null )
            ->first();
    }

    public function general_data($UserId , $lang){

        $user                       = User::find($UserId);
        $data['client_id']          = request()->header('clientID')  ?? $_GET['clientID'] ?? session()->get('clientID') ?? 0;
        $data['shift']              = $this->get_current_shift($UserId);
        $data['branchs']            = $this->get_user_branchs($UserId,$lang);
        $data['years']              = $this->get_user_years($UserId,$lang);
        $data['titles']             = $this->get_user_titles($UserId,$lang);
        $data['current_branch']     = $user->default_branch ?? 0 ;

        $data['current_year']       = $user->default_year ?? 0 ;
        $data['current_year_data']  = FinanceYear::get_finance_year($user->default_year) ?? null ;

        $data['default_port']       = $user->default_port ?? 0 ;
        $data['clearance_status']   = $user->clearance_status ?? 0 ;
        $data['other']              = session()->get('otherDB') ;
        $data['custom_db']          = session()->get('otherDB') ;
        $data['trans_db']           = session()->get('trans_db') ;

        $data['current_title']      = $user->title_id ?? 0;
        $data['branch_data']        = Branch::get_branch( $data['current_branch'] , $lang );

        if( $data['current_branch'] != 0 ){
            $branch                 = Branch::get_branch($data['current_branch']);
            $data['screens']        = [];
            $data['permissions']    = User::get_user_permission( $UserId , $lang , null , null, $data['current_title'] );

            foreach ($data['permissions'] as $key => $value) {
                $data['permissions'][$key]->permission = json_decode( $value->permission );
            }

            $data['mypermissions']  = collect($data['permissions'])->groupBy('screen_code')->toArray();

        }else{
            $data['screens']        = [];
            $data['permissions']    = [];
            $data['mypermissions']  = [];
        }

        return $data;
    }


    public function get_user_profile( $UserId , $lang = null){
        $user                                   = User::get_user( $UserId , $lang);
        $data['container_dest']                 = intval(Setting::get_main_setting_by_code('container_dest')->value ?? 0 );
        $data['display_logs']                   = intval(Setting::get_main_setting_by_code('display_logs')->value ?? 0);
        $data['display_whatsapp']               = intval(Setting::get_main_setting_by_code('display_whatsapp')->value ?? 0);
        $data['display_email']                  = intval(Setting::get_main_setting_by_code('display_email')->value ?? 0);

        $data['api_token']                      = $user->api_token ;
        $data['user_id']                        = $user->id ;

        $data['full_name']                      = $user->user_full_name ;
        $data['email']                          = $user->email ;
        $data['mobile']                         = $user->mobile ;
        $data['session_status']                 = $user->session_status ;
        $data['invoice_issuance_status']        = $user->invoice_issuance_status ;
        $data['player_id']                      = $user->player_id ?? 0 ;
        $data['image']                          = $user->image_url ?? '' ;
        $data['title']                          = $user->title ?? '' ;
        $data['access_reports']                 = $user->access_reports ?? 0;
        $data['edit_trip_price']                = $user->edit_trip_price ?? 0;
        $data['edit_trans_date']                = $user->edit_trans_date ?? 0;
        $data['edit_multi_voucher_attachments'] = $user->edit_multi_voucher_attachments ?? 0;
        $data['out_trans']                      = $user->out_trans ?? 0;
        $data['edit_return_date']               = $user->edit_return_date ?? 0;

        $data['user']                           = $this->general_data( $UserId , $lang );
        $mains                                  = collect($data['user']['permissions'])->groupBy('main_id')->toArray();
        $clients                                = ['buraq_1' , 'buraq_2'];
        $clientID                               = Request::header('clientID');
        $data['client_id_status']               =  in_array($clientID,$clients);
        foreach ($mains as $mains_key => $mains_value) {
            $data['user']['screens'][$mains_key."w"] = array(
                'id'    => $mains_value[0]->main_id ,
                'title' => $mains_value[0]->main_title ,
                'image' => $mains_value[0]->main_image ,
                'cats' => []
            );
            $cats   = collect($mains_value)->groupBy('cat_id')->toArray();
            foreach ($cats as $cats_key => $cats_value) {
                $data['user']['screens'][$mains_key."w"]['cats'][$cats_key."w"] = array(
                    'id'    => $cats_value[0]->cat_id ?? 0 ,
                    'title' => $cats_value[0]->cat_title ?? '' ,
                    'subs' => $cats_value
                );
            }
        }

        unset($data['user']['permissions']);
        return $data;
    }


    function gold_converter($c_18 , $c_21 , $c_22  , $c_24 )
    {

        // $data['c_18'] = round( $c_18 + ( $c_21 * 0.875 / 0.75  ) + ( $c_24 / 0.75 ) , 3 ) ;
        // $data['c_21'] = round( ( $c_18 * 0.75 / 0.875  ) + $c_21  + ( $c_24 / 0.875 ) , 3)  ;
        // $data['c_24'] = round( ( $c_18 * 0.75 ) + ( $c_21 * 0.875 ) +  $c_24 , 3 )  ;


        $data['c_18'] = round( $c_18 + ( $c_21 * 0.875 / 0.75  ) + ( $c_22 * 0.917 / 0.75  ) + ( $c_24 / 0.75 ) , 3 ) ;
        $data['c_21'] = round( ( $c_18 * 0.75 / 0.875  ) + $c_21  + ( $c_22 * 0.917 / 0.875  )  + ( $c_24 / 0.875 ) , 3)  ;
        $data['c_22'] = round( ( $c_18 * 0.75 / 0.917  ) + ( $c_21 * 0.875 /  0.917 )  + $c_22 + ( $c_24 / 0.917 ) , 3)  ;
        $data['c_24'] = round( ( $c_18 * 0.75 ) + ( $c_21 * 0.875 ) + ( $c_22 * 0.917  )  +  $c_24 , 3 )  ;


        return $data;
    }

    function check_not_allowed_times($user_id)
    {
        $currentTime = Carbon::now()->format('H:i:s');
        $dayOfWeek = Carbon::now()->dayOfWeek+1;

        $notAllowedTimesCount  = DB::table('not_allowed_times')
            ->where('user_id', $user_id)
            ->where('day','=',$dayOfWeek)
            ->where('from_time','<=',$currentTime)
            ->where('to_time' ,'>=',$currentTime)
            ->where('not_allowed_time_active','=',1)
            ->count();
        return $notAllowedTimesCount > 0;
    }

    function check_financial_year($action_date){
        $current_year_id  = User::find(auth()->user()->id)->default_year;
        $current_year     = FinanceYear::get_finance_year($current_year_id);
        $fromDate         = Carbon::parse($current_year->from_date);
        $toDate           = Carbon::parse($current_year->to_date);
        $action_date      = Carbon::parse($action_date);
        
        if(!$action_date->between($fromDate, $toDate,$fromDate)){
            return trans('error.this_date_not_in_year_period');
        }elseif(isset($current_year->daily_journal_id) && $current_year->daily_journal_id > 0){
            
            return trans('error.this_finance_year_is_closed');
        }else{
            return "true";
        }

    }

}
?>
