<?php

namespace App\Classes\Sales;

use App\Classes\Finance\DailyJournalHelper;
use App\Models\Sales\NoticeDebtor;
use DB;

class NoticeDebtorHelper
{
    function calculate_master_values($id){
        $details    = NoticeDebtor::get_notice_debtor_d($id, 'en');
        $update = [
            'total_vat'          => collect($details)->sum('vat_amount'),
            'total_without_vat'  => collect($details)->sum('amount_without_vat'),
            'total_with_vat'     => collect($details)->sum('amount_with_vat'),
        ];

        NoticeDebtor::update_notice_debtor_m($update , $id);
    }

    function generate_daily_journal($id){

        $master     = NoticeDebtor::get_notice_debtor_m($id, 'en');
        $details    = NoticeDebtor::get_notice_debtor_d($id, 'en');

        $items      = [];
        $journal_data  = array(
            'finance_year_id'   => $master->finance_year_id  ,
            'branch_id'         => $master->branch_id ,
            'des'               => $master->description ?? 'اشعار مدين رقم '.$id ,
            'journal_date'      => $master->invoice_date ,
            'document_id'       => $id ,
            'daily_type_id'     => 34 ,
            'auto_generated'    => 1 ,
            'journal_active'    => 1 ,
            'confirmed'         => 1
        );

        if($master->daily_journal_id > 0 ){
            $journal_data['daily_journal_id'] = $master->daily_journal_id;
        }
            //check account shared
            $shared = DB::table('account_tree')
                ->where('account_tree_id' , $master->customer_account_tree_id )
                ->where('shared_account' , 1 )
                ->where('shared_type_id'  , 1 ) // customer
                ->first();
            if(isset($shared) ){
                $shared_profile_id = $master->customer_id;
            }
            $items[]   = array(
                'account_tree_id'   => $master->customer_account_tree_id ,
                'shared_profile_id' => $shared_profile_id ?? 0 ,
                'cost_center_id'    => 0 ,
                'debit'             => $master->total_with_vat ,
                'credit'            => 0 ,
                'description'       => $master->description ?? 'اشعار مدين رقم '.$id
            );



        foreach ( $details as $key => $detail ) {

            $items[]    = array(
                'account_tree_id'   => $detail->sales_revenue_id ?? 0 ,
                'cost_center_id'    => 0 ,
                'credit'            => $detail->new_total_without_vat ,
                'debit'             => 0 ,
                'description'       => $master->description ?? 'اشعار مدين رقم '.$id
            );

            $items[]    = array(
                'account_tree_id'   => $detail->tax_account_tree_id ?? 0 ,
                'cost_center_id'    => 0 ,
                'credit'            => $detail->new_total_vat ,
                'debit'             => 0 ,
                'description'       => $master->description ?? 'اشعار مدين رقم '.$id
            );

        }

        return (new DailyJournalHelper)->generate_journal($journal_data , $items);

    }



}
