<?php

namespace App\Http\Controllers\API\V1;

use App\Classes\Excel\InvoicesExport;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Classes\General\General;
use Illuminate\Support\Facades\App;
use Maatwebsite\Excel\Facades\Excel;
use Route;
use Spatie\Browsershot\Browsershot;

class BaseController extends Controller
{

    protected $data = array();

    public function __construct()
    {
        if(request()->lang == 'ar'){App::setlocale('ar');}else{App::setlocale('en');}
        $this->middleware(function ($request, $next) {
            if(auth()->guard('api')->check()){
                $general = new General;
                $this->data = $general->general_data( auth()->user()->id, Route::current()->parameter('lang'));

                if(!isset($this->data['branchs'][$this->data['current_branch']]) ){
                    return $this->sendError('Branch Error', $this->data, 401);
                }

                if( !isset($this->data['years'][$this->data['current_year']]) ){
                    return $this->sendError('Finance Year Error', $this->data, 401);
                }
            }

            return $next($request);
        });
    }
    public function sendResponse($result, $message = '')
    {
    	$response = [
            'check' => true,
            'data'  => $result,
            'msg'   => $message,
        ];

        return response()->json($response, 200);
    }

    public function sendError($error, $errorMessages = [], $code = 200)
    {
    	$response = [
            'check' => false,
            'msg' => $error,
        ];

        if(!empty($errorMessages)){
            $response['data'] = $errorMessages;
        }

        return response()->json($response, $code);
    }

    // public function checkPermission($code, $action)
    // {
    //     $actions = ['view' => 1 ,'add' => 2 ,'edit' => 3 ,'delete' => 4 ,'print' => 5 ];
    //     if(!@in_array($actions[$action] ?? 10 , $this->data['mypermissions'][$code][0]->permission ?? [] ) ){
    //         return $this->sendError( 'permission needed #'.$code , $this->data , 200 )->send();
    //     }
    // }

   public function checkPermission($code, $action)
   {
       $actions = ['view' => 1 ,'add' => 2 ,'edit' => 3 ,'delete' => 4 ,'print' => 5 ];
       if(!@in_array($actions[$action] ?? 10 , $this->data['mypermissions'][$code][0]->permission ?? [] ) ){
           $response = [
               'check' => false,
               'data'  => [] ,
               'msg'   => 'permission needed #'.$code
           ];
           header('Content-Type: application/json; charset=utf-8; ');
           header('Access-Control-Allow-Origin:*');
           header('Access-Control-Allow-Headers:*');
           echo json_encode($response);
           die();
       }
   }
   public function sendPrint($type, $print_temp, $data , $name = 'test')
{
    $data['logo'] = public_path() . ($data['logo'] ?? '');

    switch ($type) {
        case 'excel':
            $export = new InvoicesExport($this->data, $print_temp);
            return Excel::download($export, 'balances.xlsx');

        case 'pdf':
            $pdfPath = storage_path('app/'.$name);
            $html = view($print_temp, ['data' => $data])->render();
            $pdf = Browsershot::html($html)
            ->format('A4')
            ->showBackground()
            ->setOption('args', ['--no-sandbox'])
            ->save($pdfPath);

            return response()->file($pdfPath, [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'inline; filename="document.pdf"',
            ])->deleteFileAfterSend(true);


            // return response($pdf)->header('Content-Type', 'application/pdf');
        case 'image':
            $html = view($print_temp, ['data' => $data])->render();
            return response(
                Browsershot::html($html)
                    ->windowSize(1280, 800)
                    ->showBackground()
                    ->screenshot()
            )->header('Content-Type', 'image/png');

        case 'json':
            return $this->sendResponse($data);

        case 'print':
            return view($print_temp)->with('data', $data);

        default:
            return $this->sendResponse($data);
    }
}
}
