<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Classes\Finance\DelegatePettyHelper;
use App\Models\Finance\DailyJournal;
use App\Classes\General\Upload;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\DelegatePetty;
use App\Models\Finance\ExpenseCode;
use App\Models\Finance\Treasury;
use App\Models\Main\DailyType;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Delegate;
use Illuminate\Http\Request;
use App\Models\Sales\Clearance;
use App\Models\Main\Branch;
use App\Models\Finance\Tax;

use DB;
use PDF;
use App;
use App\Classes\General\General;
use Excel;

class DelegatePettyController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-019','view');


        // $items       = Delegate::all_delegates(['lang'=>'en','active'=>1]);

        // foreach ($items as $key => $value) {
        //     (new DelegatePettyHelper)->update_balance($value->id);
        // }

        $this->filter = array(
            'lang'                  => $lang ,
            'active'                => $request['active'] ?? 1 ,
            // 'status'             => $request['status'] ?? 1 ,
            'rows'                  => $request['rows'] ?? 20 ,
            'page'                  => $request['page'] ?? 1 ,
            'word'                  => $request['word'] ?? null ,
            'is_delegate_petty'     => 1 ,
            'covenant_type_id'      => 1,
            'delegate_ids'          => 1
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-019', $lang)->sub_title ?? '';
        $this->return['delegates']   = Delegate::all_delegates($this->filter);
        $this->return['items']       = DelegatePetty::all_delegate_pettys($this->filter);

        // $all = DelegatePetty::all_delegate_pettys(['lang' => $lang , 'active' => 1]);
        // foreach ($all as $key => $value) {
        //     $journal = (new DelegatePettyHelper)->generate_journal($value->m_id);
        // }


        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('02-019','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-019' , $lang)->sub_title ?? '';
        $this->return['delegate']       = Delegate::get_delegate($request['delegate_id'] , $lang);


        // $this->return['daily_types']   = DailyType::all_daily_types(array('active' => '1', 'lang' => $lang));
        // $this->return['delegates']     = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['expense_codes']  = ExpenseCode::all_expense_codes(array('active' => '1', 'lang' => $lang));
        $this->return['taxes']          = Tax::all_taxes(['lang' => $lang, 'active' => 1 ] );
        $this->return['treasuries']       = Treasury::all_treasury(array('active' => '1', 'lang' => $lang));


        (new DelegatePettyHelper)->update_balance($request['delegate_id']);

        return $this->sendResponse($this->return);

    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-019','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-019' , $lang)->sub_title ?? '';
        $this->return['master']         = DelegatePetty::get_delegate_petty_m($id, $lang);
        $this->return['details']        = DelegatePetty::get_delegate_petty_d($id, $lang);
        $this->return['delegate']       = Delegate::get_delegate($this->return['master']->delegate_id , $lang);

        // $this->return['daily_types']   = DailyType::all_daily_types(array('active' => '1', 'lang' => $lang));
        // $this->return['delegates']     = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['expense_codes']  = ExpenseCode::all_expense_codes(array('active' => '1', 'lang' => $lang));
        $this->return['taxes']          = Tax::all_taxes(['lang' => $lang, 'active' => 1 ] );
        $this->return['treasuries']       = Treasury::all_treasury(array('active' => '1', 'lang' => $lang));




        return $this->sendResponse($this->return);
    }

    public function get_delegate_petty_d($lang, $id)
    {
        $this->checkPermission('02-019','view');


        $this->return['data']        = DelegatePetty::get_delegate_petty_d_by_d_id($id, $lang);
        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('02-019','add');

        $action_date = $request['action_date'] ?? date('Y-m-d');
        $year = array_search(date("Y", strtotime($action_date)), $this->data['years']);
        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($action_date));
        }
        $codes = (new DelegatePettyHelper)->get_next_codes(date('Y-m-d'), $year, $this->data['current_branch']);

        DB::beginTransaction();

            $m_id = DelegatePetty::add_delegate_petty_m( [
                'branch_id'          => $this->data['current_branch'],
                'finance_year_id'    => $year,
                'year_ser'           => $codes['year_ser'],
                'month_ser'          => $codes['month_ser'],
                'branch_ser'         => $codes['branch_ser'],
                'daily_type_id'      => 41 ,
                'action_date'        => $action_date,
                'delegate_id'        => $request['delegate_id'],
                'total_amount'       => collect($request['items'])->sum('amount') ,
                'add_user'           => auth()->user()->id,
            ] );

            foreach ($request['items'] as $item) {

                if($item['tax_id'] > 0 ){
                    $tax                = Tax::get_tax($item['tax_id'] , 'en');
                    $amount             = $item['amount'] ;
                    $amount_without_vat = $item['amount'] / ( 1 + ( $tax->tax_percentage / 100 ) );
                    $total_vat          = $amount - $amount_without_vat;
                }else{
                    $item['tax_id']     = 0;
                    $amount             = $item['amount'] ;
                    $amount_without_vat = $item['amount'];
                    $total_vat          = 0;
                }

                $details_seed = array(
                    'delegate_petty_m_id' => $m_id,

                    'total_vat'           => $total_vat ,
                    'amount'              => $amount ,
                    'amount_without_vat'  => $amount_without_vat ,
                    'tax_id'              => $item['tax_id'] ?? 0 ,

                    'clearance_id'        => $item['clearance_id'] ?? null,
                    'expense_code_id'     => $item['expense_code_id'] ?? null,
                    'receipt_number'      => $item['receipt_number'] ?? "",
                    'description_ar'      => $item['description'] ?? "",
                    'description_en'      => $item['description'] ?? "",
                    'add_user'            => auth()->user()->id,
                    'treasury_id'         => $item['treasury_id'] ?? 0,
                    'taxable'         => $item['taxable'] ?? 0,
                    'is_finance'         => $item['is_finance'] ?? 0
                );
                $validator = validator()->make($details_seed, DelegatePetty::$details_rules);
                if($validator->fails()){
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }

                DelegatePetty::add_delegate_petty_d($details_seed);
                if( $details_seed['clearance_id'] > 0) {
                    (new DelegatePettyHelper)->check_invoices($details_seed['clearance_id']);
                }
            }

            (new DelegatePettyHelper)->update_balance($request['delegate_id']);
            $journal = (new DelegatePettyHelper)->generate_journal($m_id);
           if(!$journal[0])
               return $this->sendError($journal[1], $journal[2] ?? [] );




        DB::commit();
        return $this->sendResponse($m_id, trans('main.add_success'));

    }



    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-019','edit');

        $update = $request->only(['items']);
        $master = DelegatePetty::get_delegate_petty_m($id, $lang);
        if((new General)->check_financial_year($master->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->action_date));
        }
        DB::beginTransaction();

            DelegatePetty::update_delegate_petty_m(['total_amount' => collect($request['items'])->sum('amount') , 'action_date' => $request['action_date'] ] , $id);
            $deleted_items  = DB::table('delegate_petty_d')->where('delegate_petty_m_id', $id)->whereNotIn('delegate_petty_d_id' , collect($update['items'])->pluck('d_id'))->get()->toArray();
            foreach ($deleted_items as $deleted_item) {
                if($deleted_item->invoice_id > 0 ) {
                    (new DelegatePettyHelper)->check_invoices(null ,$deleted_item->invoice_id , $deleted_item->invoice_type );
                }

                DB::table('delegate_petty_d')
                ->where('delegate_petty_d_id', $deleted_item->delegate_petty_d_id)
                ->delete();
            }
            // DB::table('delegate_petty_d')
            //     ->where('delegate_petty_m_id', $id)
            //     ->whereNotIn('delegate_petty_d_id' , collect($update['items'])->pluck('d_id'))
            //     ->where('invoice_id' , 0 )
            //     ->delete();

            if (isset($update['items']) && is_array($update['items'])) {

                foreach ($update['items'] as $item) {

                    if($item['tax_id'] > 0 ){
                        $tax                = Tax::get_tax($item['tax_id'] , 'en');
                        $amount             = $item['amount'] ;
                        $amount_without_vat = $item['amount'] / ( 1 + ( $tax->tax_percentage / 100 ) );
                        $total_vat          = $amount - $amount_without_vat;
                    }else{
                        $item['tax_id']     = 0;
                        $amount             = $item['amount'] ;
                        $amount_without_vat = $item['amount'];
                        $total_vat          = 0;
                    }


                    $details_seed = array(
                        'delegate_petty_m_id' => $id,
                        'total_vat'           => $total_vat ,
                        'amount'              => $amount ,
                        'amount_without_vat'  => $amount_without_vat ,
                        'tax_id'              => $item['tax_id'] ?? 0 ,
                        'clearance_id'        => $item['clearance_id'],
                        'expense_code_id'     => $item['expense_code_id'],
                        'receipt_number'      => $item['receipt_number'] ?? "",
                        'description_ar'      => $item['description'] ?? "",
                        'description_en'      => $item['description'] ?? "",
                        'amount'              => $item['amount'],
                        'treasury_id'         => $item['treasury_id'] ?? 0,
                        'taxable'             => $item['taxable'] ?? 0,
                        'is_finance'          => $item['is_finance'] ?? 0
                    );

                    $validator = validator()->make($details_seed, DelegatePetty::$details_rules);
                    if($validator->fails()){
                        return $this->sendError(trans('error.add_fails'), $validator->errors());
                    }

                    if(isset($item['d_id']) && $item['d_id'] > 0 ){
                        $updated_detail = DB::table('delegate_petty_d')->where('delegate_petty_d_id', $item['d_id'])->first();
                        if($updated_detail->invoice_id > 0 && ($updated_detail->clearance_id != $details_seed['clearance_id'] || $updated_detail->amount != $details_seed['amount'] )){
                            if($updated_detail->clearance_id != $details_seed['clearance_id']){
                                $details_seed['invoice_id'] = 0;
                                (new DelegatePettyHelper)->check_invoices($details_seed['clearance_id']);

                            }
                            (new DelegatePettyHelper)->check_invoices(null ,$updated_detail->invoice_id , $updated_detail->invoice_type );

                        }
                        DB::table('delegate_petty_d')
                            ->where('delegate_petty_d_id', $item['d_id'])
                            // ->where('invoice_id' , 0 )
                            ->update($details_seed);

                    }else{
                        $details_seed['add_user'] =  auth()->user()->id;
                        DelegatePetty::add_delegate_petty_d($details_seed);
                        (new DelegatePettyHelper)->check_invoices($details_seed['clearance_id']);

                    }
                }
            }else{
                return $this->sendError(trans('error.Found 0 Items'));
            }

            (new DelegatePettyHelper)->update_balance($request['delegate_id']);
            $journal = (new DelegatePettyHelper)->generate_journal($id);
            if(!$journal[0])
               return $this->sendError($journal[1], $journal );

        DB::commit();
        return $this->sendResponse($id, trans('main.Delegate Petty Added Successfully'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-019','delete');


        $master = DelegatePetty::get_delegate_petty_m($id, $lang);
        if((new General)->check_financial_year($master->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->action_date));
        }
        $check  = (new DelegatePettyHelper)->can_delete($id);
        if(!$check[0])
            return $this->sendError($check[1]);

        DelegatePetty::delete_delegate_petty_m($id);
        (new DelegatePettyHelper)->generate_journal($id);


        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function export_data($lang , $id , Request $request)
    {

        $this->checkPermission('02-019','print');

        $this->return['lang']           = $lang;

        $this->return['print_title']    = ScreenSub::get_screen_sub_by_screen_code('02-019' , $lang)->sub_title ?? 'Payment Voucher';
        $this->return['master']         = DelegatePetty::get_delegate_petty_m($id, $lang);
        $this->return['details']        = DelegatePetty::get_delegate_petty_d($id, $lang);

        $this->return['branch_data']    = Branch::get_branch($this->return['master']->branch_id);
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = get_logo( $this->return['master']->branch_id );


        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'finance/delegate_petty/print');
                return Excel::download($export, 'delegate_petty.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/delegate_petty/print', array('data' => $this->return ));
                return $pdf->stream();

                break;
            case 'image':
                $this->return['logo']     = public_path().$this->return['logo'];
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Finance/delegate_petty/print', array('data' => $this->return ));
                return $image->stream();

                break;
            case 'print':
                return view('Finance/delegate_petty/print')->with('data' , $this->return);
                break;
        }

    }

    public function toggle_active($lang, $id)
    {
        $item = DelegatePetty::find($id);
        if((new General)->check_financial_year($item->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->action_date));
        }
        if($item->delegate_petty_active){
            $this->checkPermission('02-019','delete');


            $check  = (new DelegatePettyHelper)->can_delete($id);
            if(!$check[0])
                return $this->sendError($check[1]);

            DelegatePetty::delete_delegate_petty_m($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('02-019','edit');


            DelegatePetty::update_delegate_petty_m(array('delegate_petty_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function upload_file($lang,$id,Request $request){
        $this->checkPermission('02-019','edit');

        $request['id'] = $id;
        $validator = validator()->make( $request->all() , DelegatePetty::$upload_rules);

        $request['folder'] = 'DelegatePetty';

        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        if(request()->hasFile('file_path')){

            $upload = new Upload;
            $check  = $upload->uploadFile($request['file_path'] , $request['folder'], $request['attachment_name'] );

            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            else
                $update['file_path'] = $check['url'];

            DelegatePetty::update_delegate_petty_d($id,$update);

            return  $this->sendResponse($check['url'],trans('main.add_success'));
        }
    }



    public function get_delegate_petty_without_attachment_get($lang)
    {
        $this->checkPermission('02-019','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-019' , $lang)->sub_title ?? '';
        $this->return['delegates']      = Delegate::all_delegates(['active' => 1 , 'lang' => $lang]);

        return $this->sendResponse($this->return);
    }


    public function get_delegate_petty_without_attachment_post($lang, $id)
    {
        $this->checkPermission('02-019','view');


        $this->return['items'] = DelegatePetty::get_delegate_petty_d_without_attachment($id, $lang);
        return $this->sendResponse($this->return);
    }


    public function get_delegate_balance($lang,$id){
        $master     = Delegate::get_delegate($id , $lang);
        $this->return['balance']    = DailyJournal::get_account_balance($master->account_tree_id , $lang , array('total' => true , 'before' => date("Y-m-d" , strtotime("+1 days")) ) );
        return $this->sendResponse($this->return);
    }

}
