<?php
namespace App\Http\Controllers\API\V1\Finance;

use App\Classes\Finance\DailyJournalHelper;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\AccountTree;
use App\Models\Finance\DailyJournal;
use Illuminate\Http\Request;


use App\Models\Finance\FinancialPeriod;
use App\Models\Main\FinanceYear;
use App\Models\Main\ScreenSub;
use App\Models\Main\Setting;
use Illuminate\Support\Facades\DB;

class FinancialPeriodController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('02-009','view');


        $this->filter = array(
            'lang'          => $lang ,
            'active'        => $request['active'] ?? 1 ,
            'rows'          => $request['rows'] ?? 10 ,
            'page'          => $request['page'] ?? 1 ,
            'word'          => $request['word'] ?? null ,
            'branch'        => $this->data['current_branch'] ,
            'finance_year'  => $this->data['current_year']
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-009' , $lang)->sub_title ?? '';
        $this->return['items']       = FinancialPeriod::all_financial_periods( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('02-009','add');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-009' , $lang)->sub_title ?? '';
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' ));

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('02-009','add');


        $insert             = $request->only(['finance_year_id', 'months']);
        $insert['months']   = array_unique( ( is_array($insert['months'] ?? []) ) ? $insert['months'] : explode(',', $insert['months'] ?? [] )  ) ;
        $validator = validator()->make( $insert , FinancialPeriod::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $monthsItems        = collect(array_unique($insert['months']))->sort()->values()->all();
        $insert['months']   = (is_array($monthsItems ?? '')) ? implode(',' , $monthsItems) : $monthsItems;
        $insert['add_user'] = auth()->user()->id;

        $this->return['id'] = FinancialPeriod::add_financial_period($insert);
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id)
    {
        $this->checkPermission('02-009','view');


        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('02-009' , $lang)->sub_title ?? '';
        $this->return['year']               = FinanceYear::get_finance_year($id);
        $this->return['account_trees']      = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' ));

        return $this->sendResponse( $this->return );
    }

    public function update($lang , $id , Request $request )
    {
        $this->checkPermission('02-009','edit');

        $update             = $request->only(['finance_year_id', 'months']);
        $update['months']   = array_unique( ( is_array($update['months'] ?? []) ) ? $update['months'] : explode(',', $update['months'] ?? [] )  ) ;
        $validator          = validator()->make( $update , FinancialPeriod::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $monthsItems = collect($update['months'])->sort()->values()->all();
        $update['months'] = (is_array($monthsItems ?? '')) ? implode(',' , $monthsItems) : $monthsItems;

        FinancialPeriod::update_financial_period($update ,  $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('02-009','delete');

        $year = FinanceYear::get_finance_year($id);
        if($year->daily_journal_id > 0){
            DB::beginTransaction();
            DB::table('daily_journal_m')->where('daily_journal_m_id' , $year->daily_journal_id )->delete();
            DB::table('daily_journal_d')->where('daily_journal_m_id' , $year->daily_journal_id)->delete();
            FinanceYear::update_finance_year(['daily_journal_id' => null] , $id);
            DB::commit();
        }

        return $this->sendResponse( [] , trans('main.delete_success') );
    }


    public function get_close_finance_year($lang, $id, Request $request)
    {
        $this->checkPermission('02-009','view');

        $year   = FinanceYear::get_finance_year($id)->name;
        if($year == ''){
            return array( false ,  trans("error.finance_year_not_aval") );
        }
        $closin_year_account_id =  $request['account_tree_id'] ?? intval(Setting::get_main_setting_by_code('closing_year_account')->value);
        $closin_year_account = AccountTree::get_account_tree($closin_year_account_id, $lang )->name;
        $expenses_accounts = DB::table('account_tree')->where([ //debit
            ['active',1],
            ['account_code', 'like', '3%'],
            ['account_type' , 2 ]
            ])->pluck('account_tree_id')->toArray();
        $revenues_accounts = DB::table('account_tree')->where([//credit
            ['active',1],
            ['account_code', 'like', '4%'],
            ['account_type' , 2 ]
            ])->pluck('account_tree_id')->toArray();
            $this->return['accounts'] = [];

            foreach($expenses_accounts as $account_id){
                $journals = DB::table('daily_journal_d')
                    ->join('daily_journal_m','daily_journal_d.daily_journal_m_id','daily_journal_m.daily_journal_m_id')
                    ->where('journal_active' , 1)
                    ->where('confirmed' , 1)
                    ->where('daily_journal_m.finance_year_id' , $id)
                    ->where('daily_journal_d.account_tree_id' , $account_id)
                    ->selectRaw('sum(debit) as debit , sum(credit) as credit , (sum(debit) - sum(credit)) as total')
                    ->first();

                if($journals->total > 0){
                    $credit = $journals->total;
                    $debit = 0;
                }elseif($journals->total < 0){
                    $credit = 0;
                    $debit = abs($journals->total);
                }else{
                    $credit = 0;
                    $debit = 0;
                }
                $account_name = AccountTree::get_account_tree($account_id, $lang )->name;

                $this->return['accounts'][] = array(
                    'account_tree_id'           => $account_id ,
                    'account_name'              => $account_name ,
                    'debit'                     => doubleval($debit) ,
                    'credit'                    => doubleval($credit)  ,
                    'description'               => 'expenses Accounts'
                );

            }
            foreach($revenues_accounts as $account_id){
                $journals = DB::table('daily_journal_d')
                ->join('daily_journal_m','daily_journal_d.daily_journal_m_id','daily_journal_m.daily_journal_m_id')
                ->where('journal_active' , 1)
                ->where('confirmed' , 1)
                ->where('daily_journal_m.finance_year_id' , $id)
                ->where('daily_journal_d.account_tree_id' , $account_id)
                ->selectRaw('sum(debit) as debit , sum(credit) as credit , (sum(debit) - sum(credit)) as total')->first();

                if($journals->total > 0){
                    $credit = $journals->total;
                    $debit = 0;
                }elseif($journals->total < 0){
                    $credit = 0;
                    $debit = abs($journals->total);
                }else{
                    $credit = 0;
                    $debit = 0;
                }
                $account_name = AccountTree::get_account_tree($account_id, $lang )->name;

                $this->return['accounts'][] = array(
                    'account_tree_id'           => $account_id ,
                    'account_name'              => $account_name ,
                    'debit'                     => doubleval($debit) ,
                    'credit'                    => doubleval($credit)  ,
                    'description'               => 'revenues Accounts'
                );


            }
            // check diff between debit and credit
            $sum_depit = collect($this->return['accounts'])->sum('debit');
            $sum_credit = collect($this->return['accounts'])->sum('credit');
            $diff = $sum_credit - $sum_depit ;
            if($diff > 0){
                $credit = 0;
                $debit = $diff;
            }elseif($diff < 0){
                $credit = abs($diff);
                $debit = 0;
            }
            $this->return['accounts'][] = array(
                'account_tree_id'           => intval($closin_year_account_id) ,
                'account_name'              => $closin_year_account ,
                'debit'                     => doubleval($debit) ,
                'credit'                    => doubleval($credit)  ,
                'description'               => 'closing year Accounts'
            );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }
    public function close_finance_year($lang, $id, Request $request)
    {
        $this->checkPermission('02-009','add');

        $year   = FinanceYear::get_finance_year($id);
        if($year->name == ''){
            return array( false ,  trans("error.finance_year_not_aval") );
        }
        if(isset($year->daily_journal_id) && $year->daily_journal_id > 0){
            return $this->sendError(trans('error.this_finance_year_is_closed'));

        }
        $closin_year_account =  $request['account_tree_id'] ?? intval(Setting::get_main_setting_by_code('closing_year_account')->value);

        $expenses_accounts = DB::table('account_tree')->where([ //debit
            ['active',1],
            ['account_code', 'like', '3%'],
            ['account_type' , 2 ]
            ])->pluck('account_tree_id')->toArray();
        $revenues_accounts = DB::table('account_tree')->where([//credit
            ['active',1],
            ['account_code', 'like', '4%'],
            ['account_type' , 2 ]
            ])->pluck('account_tree_id')->toArray();


            $codes = (new DailyJournalHelper)->get_next_codes(date('Y-m-d') , $year->name , $this->data['current_branch'] );

            $master_seed = array(
                'finance_year_id'   => $id,
                'branch_id'         => $this->data['current_branch'] ,
                'description'       => 'Closing a financial year' ,
                'year_ser'          => $codes['year_ser'] ,
                'month_ser'         => $codes['month_ser'] ,
                'branch_ser'        => $codes['branch_ser'] ,
                'journal_date'      => date('Y-m-d') ,
                'daily_type_id'     => 42 ,
                'auto_generated'    => 0 ,
                'document_id'       => 0 ,
                'confirmed'         => 1 ,
                'journal_active'    => 1 ,
                'add_user'          => auth()->user()->id
            );

            $validator = validator()->make( $master_seed , DailyJournal::$master_rules );
            if($validator->fails())
            {
                return $this->sendError(trans('error.add_fails'), $validator->errors());

            }
            DB::beginTransaction();

            $m_id = DailyJournal::add_daily_journal_m($master_seed);
            foreach($expenses_accounts as $account_id){
                $journals = DB::table('daily_journal_d')
                    ->join('daily_journal_m','daily_journal_d.daily_journal_m_id','daily_journal_m.daily_journal_m_id')
                    ->where('journal_active' , 1)
                    ->where('confirmed' , 1)
                    ->where('daily_journal_m.finance_year_id' , $id)
                    ->where('daily_journal_d.account_tree_id' , $account_id)
                    ->selectRaw('sum(debit) as debit , sum(credit) as credit , (sum(debit) - sum(credit)) as total')
                    ->first();

                if($journals->total > 0){
                    $credit = $journals->total;
                    $debit = 0;
                }elseif($journals->total < 0){
                    $credit = 0;
                    $debit = abs($journals->total);
                }else{
                    $credit = 0;
                    $debit = 0;
                }

                $det_seed = array(
                    'daily_journal_m_id'        => $m_id ,
                    'account_tree_id'           => $account_id ,
                    'cost_centers'              =>  "" ,
                    'debit'                     => doubleval($debit) ,
                    'credit'                    => doubleval($credit)  ,
                    'description'               => 'expenses Accounts'
                );

                $validator = validator()->make( $det_seed , DailyJournal::$detaile_rules );
                if($validator->fails())
                {
                    return $this->sendError(trans('error.add_fails'), $validator->errors());

                }

                DailyJournal::add_daily_journal_d($det_seed);
            }
            foreach($revenues_accounts as $account_id){
                $journals = DB::table('daily_journal_d')
                ->join('daily_journal_m','daily_journal_d.daily_journal_m_id','daily_journal_m.daily_journal_m_id')
                ->where('journal_active' , 1)
                ->where('confirmed' , 1)
                ->where('daily_journal_m.finance_year_id' , $id)
                ->where('daily_journal_d.account_tree_id' , $account_id)
                ->selectRaw('sum(debit) as debit , sum(credit) as credit , (sum(debit) - sum(credit)) as total')->first();

                if($journals->total > 0){
                    $credit = $journals->total;
                    $debit = 0;
                }elseif($journals->total < 0){
                    $credit = 0;
                    $debit = abs($journals->total);
                }else{
                    $credit = 0;
                    $debit = 0;
                }
                $det_seed = array(
                    'daily_journal_m_id'        => $m_id ,
                    'account_tree_id'           => $account_id ,
                    'cost_centers'              =>  "" ,
                    'debit'                     => doubleval($debit) ,
                    'credit'                    => doubleval($credit)  ,
                    'description'               => 'revenues Accounts'
                );

                $validator = validator()->make( $det_seed , DailyJournal::$detaile_rules );
                if($validator->fails())
                {
                    return $this->sendError(trans('error.add_fails'), $validator->errors());

                }

                DailyJournal::add_daily_journal_d($det_seed);
            }
            // check diff between debit and credit
            $sum_depit = DB::table('daily_journal_d')->where('daily_journal_m_id',$m_id)->sum('debit');
            $sum_credit = DB::table('daily_journal_d')->where('daily_journal_m_id',$m_id)->sum('credit');
            $diff = $sum_credit - $sum_depit ;
            if($diff > 0){
                $credit = 0;
                $debit = $diff;
            }elseif($diff < 0){
                $credit = abs($diff);
                $debit = 0;
            }
            $det_seed = array(
                'daily_journal_m_id'        => $m_id ,
                'account_tree_id'           => $closin_year_account ,
                'cost_centers'              =>  "" ,
                'debit'                     => doubleval($debit) ,
                'credit'                    => doubleval($credit)  ,
                'description'               => 'closing year Accounts'
            );

            $validator = validator()->make( $det_seed , DailyJournal::$detaile_rules );
            if($validator->fails())
            {
                return $this->sendError(trans('error.add_fails'), $validator->errors());

            }

            DailyJournal::add_daily_journal_d($det_seed);

        FinanceYear::update_finance_year(array('daily_journal_id' => $m_id) , $id );

        DB::commit();
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }

}
