<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Finance\MultiVoucherHelper;
use App\Classes\Finance\DelegatePettyHelper;
use App\Classes\General\General;
use App\Classes\General\Upload;
use App\Classes\LogsHelper;
use App\Models\Finance\AccountTree;
use App\Models\Finance\CostCenter;
use App\Models\Finance\MultiVoucher;
use App\Models\Finance\Treasury;
use App\Models\Main\Bank;
use App\Models\Main\ScreenSub;
use App\Models\Purchases\PurchasesAgent;
use App\Models\Purchases\PurchasesVendor;
use App\Models\Sales\Customer;
use App\Models\Sales\SalesAgent;
use App\Models\Finance\Tax;
use App\Models\Sales\Clearance;
use App\Models\Sales\Delegate;
use App\Models\Finance\ExpenseCode;
use App\Models\Main\Branch;
use App\Models\Finance\DailyJournal;
use Illuminate\Support\Facades\Storage;
use Illuminate\Http\UploadedFile;

use Illuminate\Support\Facades\DB;

class MultiVoucherController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-013','view');
        $this->filter = array(
            'lang'         => $lang ,
            'active'       => $request['active'] ?? 1 ,
            'rows'         => $request['rows'] ?? 20 ,
            'page'         => $request['page'] ?? 1 ,
            'word'         => $request['word'] ?? null ,
            'branch'       => $this->data['current_branch'] ,
            'finance_year' => $this->data['current_year']
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-013', $lang)->sub_title ?? '';
        $this->return['items']       = MultiVoucher::all_multi_vouchers($this->filter);

        // $all = MultiVoucher::all_multi_vouchers(['lang' => $lang , 'active' => 1]);
        // foreach ($all as $key => $value) {
        //     $journal = (new MultiVoucherHelper)->generate_journal($value->id);
        // }

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('02-013','add');

        $this->return['screen_name']       = ScreenSub::get_screen_sub_by_screen_code('02-013', $lang)->sub_title ?? '';
        $this->return['bank']              = Bank::all_banks(array('lang' => $lang, 'active' => 1 ));
        $this->return['purchases_vendors'] = [];
        $this->return['customers']         = Customer::all_customers(array('lang' => $lang, 'active' => 1));
        $this->return['account_trees']     = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => 1));
        $this->return['cost_centers']      = CostCenter::active_cost_centers($lang, array('account_type' => 2, 'active' => 1));
        $this->return['treasurys']         = Treasury::all_treasury(['lang' => $lang, 'active' => 1, 'user' => auth()->user()->id]);
        $this->return['taxes']             = Tax::all_taxes(['lang' => $lang, 'active' => 1 ] );
        $this->return['currencys']         = DB::table('currencys')->selectRaw("currency_id as id , name_".$lang." as name")->get();
        $this->return['types']             = [["id" => 1 , "name" => "مقبوضات"]];
        $this->return['delegates']         = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['expense_codes']     = ExpenseCode::all_expense_codes(array('active' => '1', 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('02-013','add');

        $insert = $request->all();
        $year   = array_search(date("Y", strtotime($insert['action_date'])), $this->data['years']);
        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['action_date']));
        }

        if(count($request['items']) == 0 ){
           return $this->sendError(trans('error.update_fails'), ['details cant be empty']);
        }

        // $coll = collect($request['items']);
        // if($coll->where('voucher_type' , 1)->sum('amount') != $coll->where('voucher_type' , 2)->sum('amount')){
        //    return $this->sendError(trans('error.update_fails'), ['Debit amount not credit amount ']);
        // }

        $codes = (new MultiVoucherHelper)->get_next_codes($insert['action_date'], $year, $this->data['current_branch']);

        $master_seed = [
            'branch_id'          => $this->data['current_branch'],
            'finance_year_id'    => $year,
            'year_ser'           => $codes['year_ser'],
            'month_ser'          => $codes['month_ser'],
            'branch_ser'         => $codes['branch_ser'],
            'daily_journal_m_id' => 0,
            'action_date'        => $insert['action_date'],
            'description'        => $insert['description'] ?? null,
            'conversion_type_id' => $insert['conversion_type_id'] ?? 0 ,
            'currency_id'        => $insert['currency_id'] ?? 1 ,
            'add_user'           => auth()->user()->id,

        ];

        $validator = validator()->make($master_seed, MultiVoucher::$master_rules);
        if($validator->fails()){
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $delegates = [];
        DB::beginTransaction();

            $m_id = MultiVoucher::add_multi_voucher_m($master_seed);
            $details_id = [];
            foreach ($request['items'] as $row )
            {

                if(isset($row['dis_delegate_id']) && $row['dis_delegate_id'] > 0 ){
                    $delegates[] = $row['dis_delegate_id'];
                }

                if(isset($row['tax_id']) && $row['tax_id'] > 0 ){
                    $tax    = Tax::get_tax($row['tax_id'] , 'en');
                    $amount = $row['amount'] ;
                    $amount_without_vat = $row['amount'] / ( 1 + ( $tax->tax_percentage / 100 ) );
                    $total_vat = $amount - $amount_without_vat;
                }else{
                    $amount = $row['amount'] ;
                    $amount_without_vat = $row['amount'];
                    $total_vat = 0;
                }
                $row['cost_centers'] = $row['cost_centers'] ?? [];

                $row['cost_centers'] = (is_array($row['cost_centers'])? implode(',' , $row['cost_centers'] ) : $row['cost_centers'] );
                $details_seed = [
                    'multi_voucher_m_id'  => $m_id,
                    'voucher_type'        => $row['voucher_type'] ,
                    'taxable'             => $row['taxable'] ,
                    'total_vat'           => $total_vat ,
                    'amount'              => $amount ,
                    'amount_without_vat'  => $amount_without_vat ,
                    'cost_centers'        => $row['cost_centers'] ?? '' ,
                    'tax_id'              => $row['tax_id'] ?? 0 ,
                    'description'         => $row['description'] ?? null,
                    'receipt_number'      => $row['receipt_number'] ?? '' ,
                    'dis_type'            => $row['dis_type'] ,
                    'dis_treasury_id'     => $row['dis_treasury_id']  ?? 0,
                    'dis_bank_id'         => $row['dis_bank_id']  ?? 0,
                    'dis_customer_id'     => $row['dis_customer_id'] ?? 0,
                    'dis_supplier_id'     => $row['dis_supplier_id'] ?? 0,
                    'dis_account_tree_id' => $row['dis_account_tree_id'] ?? 0,
                    'dis_delegate_id'     => $row['dis_delegate_id'] ?? 0 ,
                    'dis_expense_code_id' => $row['dis_expense_code_id'] ?? 0,
                    'clearance_id'        => $row['clearance_id'] ?? 0,
                    'add_user'            => auth()->user()->id,
                ];
                $validator = validator()->make($details_seed, MultiVoucher::$details_rules);
                if($validator->fails()){
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }
                if( $details_seed['clearance_id'] > 0) {
                    (new MultiVoucherHelper)->check_invoices($details_seed['clearance_id']);
                }
                $details_id[] = MultiVoucher::add_multi_voucher_d($details_seed);

            }

            foreach ($delegates as $key => $row) {
                (new DelegatePettyHelper)->update_balance($row);
            }

            $daily_journal = (new MultiVoucherHelper)->generate_journal($m_id);
            if(!$daily_journal[0]){
                return response()->json( array('check' => 0 , 'data' => $daily_journal , 'msg' => $daily_journal[1] ) );
            }
        (new LogsHelper)->insert_log(['payload' => $request  , 'screen_code'=>'02-013' ,'function'=> 'store' , 'action_type'=>'multi_voucher' , 'action_id'=>$m_id]);

        DB::commit();
        return $this->sendResponse([ 'id' => $m_id ,'details' => $details_id], trans('main.add_success'));

    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-013','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('02-013' , $lang)->sub_title ?? '';
        $this->return['master']             = MultiVoucher::get_multi_voucher_m($id, $lang);
        $this->return['details']            = MultiVoucher::get_multi_voucher_d($id, $lang);
        $this->return['bank']               = Bank::all_banks(array('lang' => $lang, 'active' => 1 ));
        $this->return['purchases_vendors']  = [];
        $this->return['customers']          = Customer::all_customers(array('lang' => $lang, 'active' => 1));
        $this->return['account_trees']      = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => 1));
        $this->return['cost_centers']       = CostCenter::active_cost_centers($lang, array('account_type' => 2, 'active' => 1));
        $this->return['treasurys']          = Treasury::all_treasury(['lang' => $lang, 'active' => 1, 'user' => auth()->user()->id]);
        $this->return['taxes']              = Tax::all_taxes(['lang' => $lang, 'active' => 1 ] );
        $this->return['currencys']          = DB::table('currencys')->selectRaw("currency_id as id , name_".$lang." as name")->get();
        $this->return['types']              = [["id" => 1 , "name" => "مقبوضات"],["id" => 2 , "name" => "مدفوعات"]];
        $this->return['attachments']        = MultiVoucher::get_attachments($id);
        $this->return['delegates']          = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['expense_codes']      = ExpenseCode::all_expense_codes(array('active' => '1', 'lang' => $lang));

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-013','edit');

        $update = $request->all();
        $year   = array_search(date("Y", strtotime($update['action_date'])), $this->data['years']);
        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($update['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($update['action_date']));
        }
        if(count($request['items']) == 0 ){
           return $this->sendError(trans('error.update_fails'), ['details cant be empty']);
        }
        // $coll = collect($request['items']);
        // if($coll->where('voucher_type' , 1)->sum('amount') != $coll->where('voucher_type' , 2)->sum('amount')){
        //    return $this->sendError(trans('error.update_fails'), ['Debit amount not credit amount ']);
        // }
        $codes = (new MultiVoucherHelper)->get_next_codes($update['action_date'], $year, $this->data['current_branch']);

        $master_seed = [
            'branch_id'          => $this->data['current_branch'],
            'finance_year_id'    => $year,
            'year_ser'           => $codes['year_ser'],
            'month_ser'          => $codes['month_ser'],
            'branch_ser'         => $codes['branch_ser'],
            'action_date'        => $update['action_date'],
            'description'        => $update['description'] ?? null,
            'conversion_type_id' => $insert['conversion_type_id'] ?? 0 ,
            'currency_id'        => $update['currency_id'],
            'update_user'        => auth()->user()->id,
            'update_date'        => date('Y-m-d H:i:s'),

        ];

        $validator = validator()->make($master_seed, MultiVoucher::$master_rules);
        if($validator->fails()){
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        DB::beginTransaction();

            $new_details     = collect($request['items'])->pluck('d_id')->toArray();
            $deleted_details = collect(MultiVoucher::get_multi_voucher_d($id, $lang))->whereNotIn('d_id',$new_details)->pluck('d_id')->toArray();
            foreach($deleted_details as $deleted_item){
                //check if has invoice
                // $invoice = collect(MultiVoucher::get_multi_voucher_d($id, $lang))->where('d_id', $deleted_item)->first()->invoice_id;
                // if($invoice > 0){
                //     return $this->sendError(trans('error.cannot delete item has invoice'));
                // }
                $detail = collect(MultiVoucher::get_multi_voucher_d($id, $lang))->where('d_id', $deleted_item)->first();
                if($detail->invoice_id > 0) {
                    (new MultiVoucherHelper)->check_invoices(null ,$detail->invoice_id , $detail->invoice_type );
                }
                DB::table('multi_voucher_d')->where('multi_voucher_d_id' , $deleted_item)->delete();
            }
            $delegates = [];
            MultiVoucher::update_multi_voucher_m($master_seed,$id);

            $in_active = DB::table('multi_voucher_d')
                ->whereNotIn('multi_voucher_d_id' , collect($request['items'])->pluck('d_id')->toArray() )
                ->where('multi_voucher_m_id' , $id )
                ->where('multi_voucher_d_active' , 1 )
                ->where('invoice_id' , 0 )
                ->update(['multi_voucher_d_active' => 0 , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id ] );

            foreach ($request['items'] as $row )
            {

                if(isset($row['dis_delegate_id']) && $row['dis_delegate_id'] > 0 ){
                    $delegates[] = $row['dis_delegate_id'];
                }

                if(isset($row['tax_id']) && $row['tax_id'] > 0 ){
                    $tax    = Tax::get_tax($row['tax_id'] , 'en');
                    $amount = $row['amount'] ;
                    $amount_without_vat = $row['amount'] / ( 1 + ( $tax->tax_percentage / 100 ) );
                    $total_vat = $amount - $amount_without_vat;
                }else{
                    $amount = $row['amount'] ;
                    $amount_without_vat = $row['amount'];
                    $total_vat = 0;
                }
                $row['cost_centers'] = $row['cost_centers'] ?? [];
                $row['cost_centers'] = (is_array($row['cost_centers'])? implode(',' , $row['cost_centers'] ) : $row['cost_centers'] );
                $details_seed = [
                    'multi_voucher_m_id'  => $id,
                    'voucher_type'        => $row['voucher_type'] ,
                    'total_vat'           => $total_vat ,
                    'amount'              => $amount ,
                    'amount_without_vat'  => $amount_without_vat ,
                    'cost_centers'        => $row['cost_centers'] ?? '',
                    'tax_id'              => $row['tax_id'] ?? 0 ,
                    'description'         => $row['description'] ?? null ,
                    'receipt_number'      => $row['receipt_number'] ?? '' ,
                    'dis_type'            => $row['dis_type'] ,
                    'dis_treasury_id'     => $row['dis_treasury_id']  ?? 0,
                    'dis_bank_id'         => $row['dis_bank_id']  ?? 0,
                    'dis_customer_id'     => $row['dis_customer_id'] ?? 0,
                    'dis_supplier_id'     => $row['dis_supplier_id'] ?? 0,
                    'dis_account_tree_id' => $row['dis_account_tree_id'] ?? 0 ,
                    'dis_delegate_id'     => $row['dis_delegate_id'] ?? 0 ,
                    'dis_expense_code_id' => $row['dis_expense_code_id'] ?? 0 ,
                    'clearance_id'        => $row['clearance_id'] ?? 0
                ];

                $validator = validator()->make($details_seed, MultiVoucher::$details_rules);
                if($validator->fails()){
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }

                if(isset($row['d_id']) && $row['d_id'] > 0){
                    $updated_detail = DB::table('multi_voucher_d')->where('multi_voucher_d_id', $row['d_id'])->first();
                    if($updated_detail->invoice_id > 0 && ($updated_detail->clearance_id != $details_seed['clearance_id'] || $updated_detail->amount != $details_seed['amount'] )){
                        if($updated_detail->clearance_id != $details_seed['clearance_id']){
                            $details_seed['invoice_id'] = 0;
                            (new MultiVoucherHelper)->check_invoices($details_seed['clearance_id']);
                        }
                        (new MultiVoucherHelper)->check_invoices(null ,$updated_detail->invoice_id , $updated_detail->invoice_type );
                    }
                    DB::table('multi_voucher_d')
                        ->where('multi_voucher_d_id', $row['d_id'])
                        // ->where('invoice_id' , 0 )
                        ->update($details_seed);
                }else{
                    $details_seed['add_user'] = auth()->user()->id ;
                    MultiVoucher::add_multi_voucher_d($details_seed);
                    (new MultiVoucherHelper)->check_invoices($details_seed['clearance_id']);

                }

            }

            foreach ($delegates as $key => $row) {
                (new DelegatePettyHelper)->update_balance($row);
            }

            $daily_journal = (new MultiVoucherHelper)->generate_journal($id);
            if(!$daily_journal[0]){
                return response()->json( array('check' => 0 , 'data' => $daily_journal , 'msg' => $daily_journal[1] ) );
            }
            (new LogsHelper)->insert_log(['payload' => $request  , 'screen_code'=>'02-013' ,'function'=> 'update' , 'action_type'=>'multi_voucher' , 'action_id'=>$id]);

        DB::commit();

        return $this->sendResponse([], trans('main.update_success'));

    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-013','delete');

        $master = MultiVoucher::get_multi_voucher_m($id);
        if((new General)->check_financial_year($master->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->action_date));
        }
        $check  = (new MultiVoucherHelper)->can_delete($id);
        if(!$check[0])
            return $this->sendError($check[1]);

        DB::beginTransaction();
            MultiVoucher::delete_multi_voucher_m($id);
            $daily_journal = (new MultiVoucherHelper)->generate_journal($id);
            if(!$daily_journal[0]){
                return response()->json( array('check' => 0 , 'data' => $daily_journal , 'msg' => $daily_journal[1] ) );
            }

            (new LogsHelper)->insert_log(['payload' => [] , 'screen_code'=>'02-013' ,'function'=> 'destroy' , 'action_type'=>'multi_voucher' , 'action_id'=>$id]);

        DB::commit();



        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $master = MultiVoucher::get_multi_voucher_m($id);
        if((new General)->check_financial_year($master->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->action_date));
        }
        if($master->multi_voucher_active){
            $this->checkPermission('02-013','delete');


            $check  = (new MultiVoucherHelper)->can_delete($id);
            if(!$check[0])
                return $this->sendError($check[1]);

            DB::beginTransaction();
                MultiVoucher::delete_multi_voucher_m($id);
                $daily_journal = (new MultiVoucherHelper)->generate_journal($id);
                if(!$daily_journal[0]){
                    return response()->json( array('check' => 0 , 'data' => $daily_journal , 'msg' => $daily_journal[1] ) );
                }

                (new LogsHelper)->insert_log(['payload' => [] , 'screen_code'=>'02-013' ,'function'=> 'toggle_active' , 'action_type'=>'multi_voucher' , 'action_id'=>$id]);

            DB::commit();


            $this->return['active'] = false;

        }else{
            $this->checkPermission('02-013','edit');


            if(!(new MultiVoucherHelper)->can_edit($id)){
                return $this->sendError(trans('error.update_fails'), []);
            }

            DB::beginTransaction();
                MultiVoucher::update_multi_voucher_m(array('multi_voucher_active' => 1), $id);
                $daily_journal = (new MultiVoucherHelper)->generate_journal($id);
                if(!$daily_journal[0]){
                    return response()->json( array('check' => 0 , 'data' => $daily_journal , 'msg' => $daily_journal[1] ) );
                }
                (new LogsHelper)->insert_log(['payload' => [] , 'screen_code'=>'02-013' ,'function'=> 'toggle_active' , 'action_type'=>'multi_voucher' , 'action_id'=>$id]);

            DB::commit();

            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }


    public function export_data($lang , $id , Request $request)
    {

        $this->checkPermission('02-013','print');

        $this->return['lang']           = $lang;
        $this->return['print_title']    = ScreenSub::get_screen_sub_by_screen_code('02-013' , $lang)->sub_title ?? 'Multi Voucher';
        $this->return['master']         = MultiVoucher::get_multi_voucher_m($id, $lang);
        $this->return['details']        = MultiVoucher::get_multi_voucher_d($id, $lang);
        $this->return['branch_data']    = Branch::get_branch($this->return['master']->branch_id);
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = $this->return['branch_data']->branch_image;


        return $this->sendResponse($this->return, trans('main.add_success'));

    }
    public function add_attach($lang , $id , Request $request){

        $check  = (new Upload)->uploadFile($request['file_path'] , 'multi_voucher' );
        if(!$check['check'])
            return $this->sendError( trans('error.add_fails') , $check['msg'] );


        $this->return['id'] = DB::table('multi_voucher_attachments')->insertGetId( [
            'multi_voucher_id'      => $id ,
            'file_name'             => $request['file_name'] ?? date('Y-m-d H:i:s'),
            'file_path'             => $check['url']  ,
            'file_ext'              => $check['extension']  ,
            'add_user'              => auth()->user()->id
        ]);

        return $this->sendResponse( $this->return , '' );
    }
    public function add_details_attach($lang , $id , Request $request){
        $this->checkPermission('02-013','add');
        $item = $request->all();
            $check  = (new Upload)->uploadFile($item['attachment'] , 'multi_voucher' );
            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );

            DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->update(['attachment' => $check['url']]);

        return $this->sendResponse( $this->return );
    }
    public function delete_details_attach($lang , $id , Request $request){
        $this->checkPermission('02-013','delete');

        $file = DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->value('attachment');
        DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->update(['attachment' => null]);

        if(Storage::exists($file)){
            Storage::delete($file);
        }
            return $this->sendResponse( $this->return , trans('main.delete_success') );
        }

    public function download_attach($lang , $id){

        $file = DB::table('multi_voucher_attachments')
            ->where('attachment_id' , $id)
            ->first();

        return response()->file(public_path().$file->file_path);

    }
    public function download_detail_attach($lang , $id){

        $this->checkPermission('02-013','view');

        $file = DB::table('multi_voucher_d')
            ->where('multi_voucher_d_id' , $id)
            ->first();

        return response()->file(public_path().$file->attachment);

    }

    public function delete_attach($lang , $id){

        $file = DB::table('multi_voucher_attachments')
            ->where('attachment_id' , $id)
            ->first();

        DB::table('multi_voucher_attachments')
            ->where('attachment_id' , $id)
            ->delete();

        if(Storage::exists($file->file_path)){
            Storage::delete($file->file_path);
        }

        return $this->sendResponse( $this->return , trans('main.delete_success') );
    }
}
