<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Classes\Finance\DelegatePettyHelper;
use App\Classes\Finance\MaintenancePettyHelper;
use App\Classes\Finance\PaymentVoucherHelper;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\AccountTree;
use App\Models\Finance\CostCenter;
use App\Models\Main\DailyType;
use App\Models\Sales\Delegate;
use App\Models\Finance\PaymentVoucher;
use App\Models\Finance\Treasury;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;
use App\Models\Sales\Clearance;
use App\Models\Main\Branch;

use Illuminate\Support\Facades\DB;
use PDF;
use App;
use App\Classes\General\General;
use Excel;
use Illuminate\Support\Facades\Crypt;
use SimpleSoftwareIO\QrCode\Facades\QrCode;

class PaymentVoucherController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('02-008','view');


        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'status' => $request['status'] ?? 1 ,
            'rows'   => $request['rows'] ?? 20 ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('02-008', $lang)->sub_title ?? '';
        $this->return['items']       = PaymentVoucher::all_payment_vouchers($this->filter);


        // $all = PaymentVoucher::all_payment_vouchers(['lang' => $lang , 'active' => 1]);
        // foreach ($all as $key => $value) {
        //     $journal = (new PaymentVoucherHelper)->generate_journal($value->id);
        // }


        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('02-008','add');


        $this->return['screen_name']   = ScreenSub::get_screen_sub_by_screen_code('02-008', $lang)->sub_title ?? '';
        $this->return['daily_types']   = DailyType::all_daily_types(array('active' => '1', 'lang' => $lang));
        $this->return['account_trees'] = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));
        $this->return['cost_centers']  = CostCenter::active_cost_centers($lang, array('active' => '1'));
        $this->return['delegates']     = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['treasurys']     = Treasury::all_treasury(array('lang' => $lang, 'active' => 1, 'user' => auth()->user()->id));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('02-008','add');

        $insert = $request->only(['action_date', 'daily_type_id', 'arabic_description', 'items' , 'treasury_id']);

        $year   = array_search(date("Y", strtotime($insert['action_date'])), $this->data['years']);

        if($year == ''){
            return $this->sendError(trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['action_date']));
        }
        $codes = (new PaymentVoucherHelper)->get_next_codes($insert['action_date'], $year, $this->data['current_branch']);

        $master_seed = [
            'branch_id'          => $this->data['current_branch'],
            'finance_year_id'    => $year,
            'year_ser'           => $codes['year_ser'],
            'month_ser'          => $codes['month_ser'],
            'branch_ser'         => $codes['branch_ser'],
            'daily_journal_m_id' => 0,
            'action_date'        => $insert['action_date'],
            'daily_type_id'      => $insert['daily_type_id'],
            'arabic_description' => $insert['arabic_description'],
            'treasury_id'        => $insert['treasury_id'] ?? 0,
            'total_amount'       => collect($insert['items'])->sum('amount'),
            'add_user'           => auth()->user()->id
        ];

        $validator = validator()->make($master_seed, PaymentVoucher::$master_rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        DB::beginTransaction();

        $m_id = PaymentVoucher::add_payment_voucher_m($master_seed);

        if (isset($insert['items']) && is_array($insert['items'])){
            foreach ($insert['items'] as $item){
                $details_seed = array(
                    'payment_voucher_m_id' => $m_id,
                    'pay_for'              => $item['pay_for'] ?? 0,
                    'account_tree_id'      => $item['account_tree_id'] ?? 0,
                    'cost_center_id'       => $item['cost_center_id'] ?? 0,
                    'delegate_id'          => $item['delegate_id'] ?? 0,
                    'clearance_id'         => $item['clearance_id'] ?? 0,
                    'container_id'         => $item['container_id'] ?? 0,
                    'arabic_description'   => $item['arabic_description'] ?? 0,
                    'amount'               => $item['amount'] ?? 0,
                    'add_user'             => auth()->user()->id
                );

                $validator = validator()->make($details_seed, PaymentVoucher::$details_rules);

                if($validator->fails())
                {
                    return $this->sendError(trans('error.add_fails'), $validator->errors());
                }

                $d_id = PaymentVoucher::add_payment_voucher_d($details_seed);

                // Creating Delegate Petty Or Maintenance Petty
                switch ($item['pay_for']){
                    case 1:
                        // $year = array_search(date("Y", strtotime(date('Y-m-d'))), $this->data['years']);
                        // if($year == ''){
                        //     return $this->sendError(trans("error.finance_year_not_aval"));
                        // }
                        // $codes = (new DelegatePettyHelper)->get_next_codes(date('Y-m-d'), $year, $this->data['current_branch']);
                        // DB::table('delegate_petty_m')->insert([
                        //     'branch_id'          => $this->data['current_branch'],
                        //     'finance_year_id'    => $year,
                        //     'year_ser'           => $codes['year_ser'],
                        //     'month_ser'          => $codes['month_ser'],
                        //     'branch_ser'         => $codes['branch_ser'],
                        //     'action_date'        => date('Y-m-d'),
                        //     'daily_type_id'      => $insert['daily_type_id'],
                        //     'delegate_id'        => $item['delegate_id'],
                        //     'total_amount'       => $item['amount'],
                        //     'remaining_amount'   => $item['amount'],
                        //     'add_user'           => auth()->user()->id,
                        // ]);

                        (new DelegatePettyHelper)->update_balance($item['delegate_id']);

                        break;
                    case 2:
                        $year = array_search(date("Y", strtotime(date('Y-m-d'))), $this->data['years']);
                        if($year == ''){
                            return $this->sendError(trans("error.finance_year_not_aval"));
                        }
                        $codes = (new MaintenancePettyHelper)->get_next_codes(date('Y-m-d'), $year, $this->data['current_branch']);
                        DB::table('maintenance_petty_m')->insert([
                            'branch_id'          => $this->data['current_branch'],
                            'finance_year_id'    => $year,
                            'year_ser'           => $codes['year_ser'],
                            'month_ser'          => $codes['month_ser'],
                            'branch_ser'         => $codes['branch_ser'],
                            'action_date'        => date('Y-m-d'),
                            'daily_type_id'      => $insert['daily_type_id'],
                            'delegate_id'        => $item['delegate_id'],
                            'total_amount'       => $item['amount'],
                            'remaining_amount'   => $item['amount'],
                            'add_user'           => auth()->user()->id,
                        ]);
                        break;
                }
            }
        }

        $dailyJournal = (new PaymentVoucherHelper)->generate_journal($m_id);

        if(!$dailyJournal[0]){
            DB::rollback();
            return response()->json(array('check' => 0 , 'data' => $dailyJournal[3] ?? [], 'msg' => 'Error In Creating Journal Code '.$dailyJournal[1]));
        }

        DB::table('payment_voucher_m')
            ->where('payment_voucher_m_id' , $m_id)
            ->update(['daily_journal_m_id' => $dailyJournal[2]]);

        DB::commit();
        return $this->sendResponse([], trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('02-008','add');


        $this->return['screen_name']   = ScreenSub::get_screen_sub_by_screen_code('02-008' , $lang)->sub_title ?? '';
        $this->return['master']        = PaymentVoucher::get_payment_voucher_m($id, $lang);
        $this->return['details']       = PaymentVoucher::get_payment_voucher_d($id, $lang);

        $this->return['daily_types']   = DailyType::all_daily_types(array('active' => '1', 'lang' => $lang));
        $this->return['account_trees'] = AccountTree::active_account_trees($lang, array('account_type' => 2, 'active' => '1'));
        $this->return['cost_centers']  = CostCenter::active_cost_centers($lang, array('active' => '1'));
        $this->return['delegates']     = Delegate::all_delegates(array('active' => '1', 'lang' => $lang));
        $this->return['treasurys']     = Treasury::all_treasury(array('lang' => $lang, 'active' => 1, 'user' => auth()->user()->id));

        return $this->sendResponse($this->return);
    }
    public function encryptRouteParameter($data)
    {
        $encryptedData = Crypt::encrypt($data);
        return $encryptedData;
    }
    public function decryptRouteParameter($encryptedData)
    {

        $decryptedData = Crypt::decrypt($encryptedData);
        return $decryptedData;

    }
    public function show_link($encryptedParameter)
    {
        $decryptedData = $this->decryptRouteParameter($encryptedParameter);
        $id = $decryptedData['id'];
        $lang = 'ar';
        $this->return['screen_name']   = ScreenSub::get_screen_sub_by_screen_code('02-008' , $lang)->sub_title ?? '';
        $this->return['master']        = PaymentVoucher::get_payment_voucher_m($id, $lang);
        $this->return['details']       = PaymentVoucher::get_payment_voucher_d($id, $lang);
        $this->return['link_token']    = $this->encryptRouteParameter(array('id' => $id ));
        $this->return['lang']           = $lang;
        $this->return['print_title']    = ScreenSub::get_screen_sub_by_screen_code('02-008' , $lang)->sub_title ?? 'Payment Voucher';
        $this->return['branch_data']    = Branch::get_branch($this->return['master']->branch_id);
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = get_logo( $this->return['master']->branch_id );
        $link = route('payment_voucher_link',$this->return['link_token']).'?clientID='.request()->header('clientID');
        $qr_code  = QrCode::format('png')->size(300)->generate($link);
        $this->return['qr_code'] = 'data:image/png;base64,' . base64_encode($qr_code);

        $this->return['logo']     = public_path().$this->return['logo'];
        $pdf = PDF::loadView('Finance/payment_voucher/print', array('data' => $this->return ));
        return $pdf->stream();
    }
    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('02-008','edit');


        $update = $request->only(['action_date', 'daily_type_id', 'arabic_description' , 'treasury_id']);
        if((new General)->check_financial_year($update['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($update['action_date']));
        }
        $update_seed = [
            'action_date'        => $update['action_date'],
            'daily_type_id'      => $update['daily_type_id'],
            'arabic_description' => $update['arabic_description'],
            'treasury_id'        => $update['treasury_id'] ?? 0,
            'total_amount'       => collect($update['items'])->sum('amount'),
            'add_user'           => auth()->user()->id
        ];
        DB::beginTransaction();

        PaymentVoucher::update_payment_voucher_m($update_seed, $id);


        if (isset($request->items) && is_array($request->items)){
            //1-delete old details
            PaymentVoucher::delete_payment_voucher_d($id);

            // 2- adding new details
            foreach ($request->items as $item){
                $insert = array(
                    'payment_voucher_m_id' => $id,
                    'pay_for'              => $item['pay_for'] ?? 0,
                    'account_tree_id'      => $item['account_tree_id'] ?? 0,
                    'cost_center_id'       => $item['cost_center_id'] ?? 0,
                    'delegate_id'          => $item['delegate_id'] ?? 0,
                    'clearance_id'         => $item['clearance_id'] ?? 0,
                    'container_id'         => $item['container_id'] ?? 0,
                    'arabic_description'   => $item['arabic_description'] ?? 0,
                    'amount'               => $item['amount'] ?? 0,
                    'add_user'             => auth()->user()->id
                );
                $d_id =  PaymentVoucher::add_payment_voucher_d($insert);

                // Creating Delegate Petty Or Maintenance Petty
//                switch ($item['pay_for']){
//                    case 1:
//                        (new DelegatePettyHelper)->update_balance($item['delegate_id']);
//                        break;
//                    case 2:
//                        $year = array_search(date("Y", strtotime(date('Y-m-d'))), $this->data['years']);
//                        if($year == ''){
//                            return $this->sendError(trans("error.finance_year_not_aval"));
//                        }
//                        $codes = (new MaintenancePettyHelper)->get_next_codes(date('Y-m-d'), $year, $this->data['current_branch']);
//                        DB::table('maintenance_petty_m')->insert([
//                            'branch_id'          => $this->data['current_branch'],
//                            'finance_year_id'    => $year,
//                            'year_ser'           => $codes['year_ser'],
//                            'month_ser'          => $codes['month_ser'],
//                            'branch_ser'         => $codes['branch_ser'],
//                            'action_date'        => date('Y-m-d'),
//                            'daily_type_id'      => $update['daily_type_id'],
//                            'delegate_id'        => $item['delegate_id'],
//                            'total_amount'       => $item['amount'],
//                            'remaining_amount'   => $item['amount'],
//                            'add_user'           => auth()->user()->id,
//                        ]);
//                        break;
//                }
            }
        }

        $dailyJournal = (new PaymentVoucherHelper)->generate_journal($id);

        if(!$dailyJournal[0]){
            DB::rollback();
            return response()->json(array('check' => 0 , 'data' => $dailyJournal[3] ?? [], 'msg' => 'Error In Creating Journal Code '.$dailyJournal[1]));
        }
        DB::commit();

        return $this->sendResponse($id, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('02-008','delete');

        $master = PaymentVoucher::get_payment_voucher_m($id , 'en');
        if((new General)->check_financial_year($master->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master->action_date));
        }
        // if(!isset($master->payment_voucher_m_id) || $master->branch_id != $this->data['current_branch'] || !(new PaymentVoucherHelper)->can_edit_or_delete($id)){
        //     return $this->sendError(trans("error.delete_fail"));
        // }

        PaymentVoucher::delete_payment_voucher_m($id);
        (new PaymentVoucherHelper)->generate_journal($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function export_data($lang , $id , Request $request)
    {
        $this->checkPermission('02-008','print');

        $this->return['lang']           = $lang;
        $this->return['print_title']    = ScreenSub::get_screen_sub_by_screen_code('02-008' , $lang)->sub_title ?? 'Payment Voucher';
        $this->return['master']         = PaymentVoucher::get_payment_voucher_m($id, $lang);
        $this->return['details']        = PaymentVoucher::get_payment_voucher_d($id, $lang);

        $this->return['branch_data']    = Branch::get_branch($this->return['master']->branch_id);
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = get_logo( $this->return['master']->branch_id );
        $this->return['link_token']    = $this->encryptRouteParameter(array('id' => $id));
        $link = route('payment_voucher_link',$this->return['link_token']).'?clientID='.request()->header('clientID');
        $qr_code  = QrCode::format('png')->size(300)->generate($link);
        $this->return['qr_code'] = 'data:image/png;base64,' . base64_encode($qr_code);
        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'finance/payment_voucher/print');
                return Excel::download($export, 'payment_voucher.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/payment_voucher/print', array('data' => $this->return ));
                return $pdf->stream();

                break;
            case 'image':
                $this->return['logo']     = public_path().$this->return['logo'];
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Finance/payment_voucher/print', array('data' => $this->return ));
                return $image->stream();

                break;
            case 'print':
                return view('Finance/payment_voucher/print')->with('data' , $this->return);
                break;
        }

    }


    public function toggle_active($lang, $id)
    {
        $item = PaymentVoucher::find($id);
        if((new General)->check_financial_year($item->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->action_date));
        }
        if($item->payment_voucher_active){
            $this->checkPermission('02-008','delete');


            if(!(new PaymentVoucherHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.delete_fails'), []);
            }

            PaymentVoucher::delete_payment_voucher_m($id);
            (new PaymentVoucherHelper)->generate_journal($id);

            $this->return['active'] = false;
        }else{
            $this->checkPermission('02-008','edit');


            if(!(new PaymentVoucherHelper)->can_edit_or_delete($id)){
                return $this->sendError( trans('error.update_fails'), []);
            }

            PaymentVoucher::update_payment_voucher_m(array('payment_voucher_active' => 1), $id);
            (new PaymentVoucherHelper)->generate_journal($id);

            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
