<?php

namespace App\Http\Controllers\API\V1\Finance;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Sales\Delegate;
use Illuminate\Http\Request;

use App\Classes\Finance\AccountTreeHelper;
use App\Classes\Excel\InvoicesExport;
use App\Classes\Finance\CustomerStatmentReportHelper;
use App\Classes\Finance\FinanceReportHelper;

use App\Models\Finance\AccountTree;
use App\Models\Finance\CostCenter;
use App\Models\Finance\DailyJournal;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Customer;


use Illuminate\Support\Facades\DB;
use PDF;
use Excel;
class ReportController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function cost_center_statement_get($lang , Request $request){
        $this->checkPermission('03-038','view');
        $this->return['cost_centers']   = CostCenter::active_cost_centers($lang , array('active' => '1' ));

        return $this->sendResponse( $this->return  );
    }

    public function cost_center_statement_post($lang , Request $request){

        $this->checkPermission('03-038','print');
        $cost_centers = DB::table('cost_center')->where('parent_id', $request['cost_center_id'])->pluck('cost_center_id')->toArray();
        if (empty($cost_centers)) {
            $cost_center_id = $request['cost_center_id'];
        } else {
            $cost_center_id = $cost_centers;
        }
        $this->data['from']             = $request['from'];
        $this->data['to']               = $request['to'];
        $this->data['lang']             = $lang;
        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('02-029' , $lang);
        $this->return['first']            = CostCenter::get_cost_center_balance($cost_center_id , $lang , array('total' => true , 'before' => $this->data['from'] ) );
        $this->return['actions']          = CostCenter::get_cost_center_balance($cost_center_id , $lang , array('from' => $this->data['from']  , 'to' => $this->data['to'] ) );
        $this->return['cost_centers']     = CostCenter::active_cost_centers($lang , array('active' => '1' ));
        // $this->data['logo']             = get_logo(null);

        return $this->sendResponse( $this->return  );
    }


    public function main_account_balances_get($lang , Request $request)
    {
        $this->checkPermission('02-022','view');
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 1 , 'active' => '1' ));

        return $this->sendResponse( $this->return  );
    }

    public function main_account_balances_post($lang , Request $request)
    {

        $this->checkPermission('02-022','view');

        $this->return['from']           = $request['from'];
        $this->return['to']             = $request['to'];
        $this->return['lang']           = $lang;
        $this->return['draft']          = $request['with_draft'] ?? 0;

        $this->return['credit']         = $request['credit'] ?? 1;
        $this->return['debit']          = $request['debit'] ?? 1;
        $this->return['has_balance']    = $request['has_balance'] ?? 1;
        $sort_by                        = $request['sort_by'] ?? null;
        $sort_value                     = $request['sort_value'] ?? 0;

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang);
        $this->return['report']         = [];
        $account_trees                  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' , 'start_with' => $request['account_tree_id'] ));

        foreach ($account_trees as $key => $row) {
            $first      = DailyJournal::get_account_balance($row->id , $lang , array('total' => true , 'before' => $this->return['from'] , 'draft' => $this->return['draft'] ) );
            $actions    = DailyJournal::get_account_balance($row->id , $lang , array('total' => true , 'from' => $this->return['from']  , 'to' => $this->return['to'] , 'draft' => $this->return['draft'] ) );

            $balance = $first->total + $actions->debit - $actions->credit ;
            if($this->return['credit'] == 0 && $balance < 0 ){
                continue;
            }

            if($this->return['debit'] == 0 && $balance > 0 ){
                continue;
            }

            if($this->return['has_balance'] == 1 && $balance  == 0 ){
                continue;
            }

            $this->return['report'][] = [
                'id'        => $row->id ,
                'name'      => $row->name ,
                'code'      => $row->account_code ,
                'first'     => doubleval($first->total ),
                'debit'     => doubleval($actions->debit) ,
                'credit'    => doubleval($actions->credit),
                'balance'   => doubleval($balance)
            ];
        }
        if (!is_null($sort_by)){
            usort($this->return['report'], function ($a, $b) use ($sort_by,$sort_value){
                if ($sort_by == 'name'){
                    $result =  strcmp($a[$sort_by], $b[$sort_by]);
                }elseif (in_array($sort_by,['id','balance'])){
                    $result =  $a[$sort_by] <=> $b[$sort_by];
                }
                return ($sort_value === 'desc') ? -$result : $result;
            });
        }



        $this->return['logo']             = get_logo(1);

        return $this->sendResponse( $this->return  );

    }


    public function account_statment_anal_report_get($lang , Request $request)
    {
        $this->checkPermission('02-004','view');
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' ));
        $this->return['customers']      = Customer::all_customers(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse( $this->return  );
    }

    public function account_statment_anal_report_post($lang , Request $request)
    {

        $this->checkPermission('02-004','view');

        $this->return['from']             = $request['from'];
        $this->return['to']               = $request['to'];
        $this->return['lang']             = $lang;
        $this->return['draft']            = $request['with_draft'] ?? 0;
        $this->return['draft_date']       = $request['invoice_date'] ?? 0;
        $this->return['customer_id']      = $request['customer_id'] ?? null;

        $this->return['without_closing_daily']       = $request['without_closing_daily'] ?? 1;
        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang);

        $this->return['first']            = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('total' => true , 'before' => $this->return['from'] , 'draft' => $this->return['draft'], 'draft_date' => $this->return['draft_date']  ,'without_closing_daily' => $this->return['without_closing_daily'] , 'customer_id' => $this->return['customer_id'] ) );
        $this->return['actions']          = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('from' => $this->return['from']  , 'to' => $this->return['to'] , 'draft' => $this->return['draft'] , 'draft_date' => $this->return['draft_date'] ,'without_closing_daily' => $this->return['without_closing_daily'] , 'customer_id' => $this->return['customer_id']) );
        $this->return['account_tree']     = AccountTree::get_account_tree( $request['account_tree_id'] , $lang );

        if($this->return['first']->total > 0 ){
            $this->return['first']->credit  = 0;
            $this->return['first']->debit   = $this->return['first']->total;
        }else{
            $this->return['first']->debit  = 0;
            $this->return['first']->credit   = $this->return['first']->total * -1;
        }
        $this->return['logo']             = get_logo(1);
        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Finance/report/account_statment_anal');
                return Excel::download($export, 'balances.xlsx');
            break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/report/account_statment_anal', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
            break;
            case 'print':
                return view('Finance/report/account_statment_anal')->with('data' , $this->return);
            break;
            case 'json':
                return $this->sendResponse( $this->return  );
                // return view('finance/report/account_statment_anal_p')->with('data' , $this->return);
            break;
        }
    }
    public function account_statment_yearly_report_get($lang , Request $request)
    {
        $this->checkPermission('03-059','view');
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' ));

        return $this->sendResponse( $this->return );
    }

    public function account_statment_yearly_report_post($lang , Request $request)
    {

        $this->checkPermission('03-059','view');

        $this->return['from']             = $request['from'];
        $this->return['to']               = $request['to'];
        $this->return['lang']             = $lang;
        $this->return['draft']            = $request['with_draft'] ?? 0;
        $this->return['draft_date']       = $request['invoice_date'] ?? 0;
        $this->return['without_closing_daily']       = $request['without_closing_daily'] ?? 1;
        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang);

        $this->return['first']            = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('total' => true , 'before' => $this->return['from'] , 'draft' => $this->return['draft'], 'draft_date' => $this->return['draft_date']  ,'without_closing_daily' => $this->return['without_closing_daily']  ) );
        $this->return['actions']          = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('from' => $this->return['from']  , 'to' => $this->return['to'] , 'draft' => $this->return['draft'] , 'draft_date' => $this->return['draft_date'] ,'without_closing_daily' => $this->return['without_closing_daily']) );
        $this->return['account_tree']     = AccountTree::get_account_tree( $request['account_tree_id'] , $lang );

        if($this->return['first']->total > 0 ){
            $this->return['first']->credit  = 0;
            $this->return['first']->debit   = $this->return['first']->total;
        }else{
            $this->return['first']->debit  = 0;
            $this->return['first']->credit   = $this->return['first']->total * -1;
        }
        $this->return['logo']             = get_logo(1);
        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Finance/report/account_statment_anal');
                return Excel::download($export, 'balances.xlsx');
            break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/report/account_statment_anal', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
            break;
            case 'print':
                return view('Finance/report/account_statment_anal')->with('data' , $this->return);
            break;
            case 'json':
                return $this->sendResponse( $this->return  );
                // return view('finance/report/account_statment_anal_p')->with('data' , $this->return);
            break;
        }
    }
    public function group_account_statment_anal_report_get($lang , Request $request)
    {
        $this->checkPermission('02-021','view');
        $this->return['account_trees']  = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => '1' ));

        return $this->sendResponse( $this->return  );
    }

    public function group_account_statment_anal_report_post($lang , Request $request)
    {

        $this->checkPermission('02-021','view');

        $this->return['from']             = $request['from'];
        $this->return['to']               = $request['to'];
        $this->return['lang']             = $lang;
        $this->return['draft']            = $request['with_draft'] ?? 0;
        $this->return['draft_date']       = $request['invoice_date'] ?? 0;
        $this->return['without_closing_daily']       = $request['without_closing_daily'] ?? 1;

        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang);
        $this->return['first']            = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('total' => true , 'before' => $this->return['from'] , 'draft' => $this->return['draft'], 'draft_date' => $this->return['draft_date'] , 'without_closing_daily' => $this->return['without_closing_daily']) );
        $this->return['actions']          = DailyJournal::get_account_balance($request['account_tree_id'] , $lang , array('from' => $this->return['from']  , 'to' => $this->return['to'], 'draft' => $this->return['draft'] , 'draft_date' => $this->return['draft_date'] ,'without_closing_daily' => $this->return['without_closing_daily']) );
        $this->return['account_tree']     = AccountTree::get_account_tree( $request['account_tree_id'] , $lang );
        $this->return['logo']             = get_logo(1);

        if($this->return['first']->total > 0 ){
            $this->return['first']->credit  = 0;
            $this->return['first']->debit   = $this->return['first']->total;
        }else{
            $this->return['first']->debit  = 0;
            $this->return['first']->credit   = $this->return['first']->total * -1;
        }

        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Finance/report/account_statment_anal');
                return Excel::download($export, 'balances.xlsx');
            break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/report/account_statment_anal', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
            break;
            case 'print':
                return view('Finance/report/account_statment_anal')->with('data' , $this->return);
            break;
            case 'json':
                return $this->sendResponse( $this->return  );
                // return view('finance/report/account_statment_anal_p')->with('data' , $this->return);
            break;
        }
    }

    public function trial_balance_report($lang , Request $request)
    {
        $this->checkPermission('02-005','print');


        $this->data['from']             = $request['from'];
        $this->data['to']               = $request['to'];
        $this->return['lang']           = $lang;
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-005' , $this->return['lang']);
        $this->return['report']         = [];
        $data                           = [];
        // $this->return['first']                  = DB::select('call p_trial_balance("2000-01-01" ,  "'.date("Y-m-d" , strtotime($this->data['from']."- 1 day")).'"  , "'.$this->return['lang'].'" )  ');
        // $this->return['actions']                = DB::select('call p_trial_balance("'.$this->data['from'].'" ,  "'.$this->data['to'].'"  , "'.$this->return['lang'].'" )  ');


        $this->return['first']                  = (new FinanceReportHelper)->trial_balance(['from' => "2000-01-01" , 'to' => date("Y-m-d" , strtotime($this->data['from']."- 1 day")) , 'lang' => $lang]);
        $this->return['actions']                  = (new FinanceReportHelper)->trial_balance(['from' => date("Y-m-d" , strtotime($this->data['from'])) , 'to' => date("Y-m-d" , strtotime($this->data['to'])) , 'lang' => $lang]);






        $this->return['first_debit']            = abs( collect($this->return['first'])->whereIn('account_code', [1,3])->sum('total') );
        $this->return['first_credit']           = abs( collect($this->return['first'])->whereIn('account_code', [2,4])->sum('total') );
        $this->return['actions_debit']          = collect($this->return['actions'])->whereIn('account_code', [1, 2 , 3 , 4])->sum('debit');
        $this->return['actions_credit']         = collect($this->return['actions'])->whereIn('account_code', [1, 2 , 3 , 4])->sum('credit');


        foreach($this->return['first'] as $key => $row){

            if($row->total < 0 ){
                $first_debit    = 0;
                $first_credit   = $row->total * -1;
            }else{
                $first_debit    = $row->total;
                $first_credit   = 0;
            }

            $final_debit    = $this->return['actions'][$key]->debit + $first_debit ;
            $final_credit   = $this->return['actions'][$key]->credit + $first_credit ;

            $final = $final_debit - $final_credit;
            if($final < 0 ){
                $final_debit    = 0;
                $final_credit   = $final * -1;
            }else{
                $final_debit    = $final ;
                $final_credit   = 0;
            }

            $this->return['report'][] = [
                'account_tree_id'   => $row->account_tree_id ?? 0 ,
                'account_code'      => $row->account_code ,
                'account_level'     => $row->account_level ,
                'parent_id'         => $row->parent_id ,
                'name'              => $row->name ,
                'account_type'      => $row->account_type ,
                'first_debit'       => $first_debit ,
                'first_credit'      => $first_credit ,
                'actions_debit'     => $this->return['actions'][$key]->debit ,
                'actions_credit'    => $this->return['actions'][$key]->credit ,
                'final_debit'       => $final_debit ,
                'final_credit'      => $final_credit
            ];
       }

       $this->return['report'][] = [
            'account_tree_id'   => 0 ,
            'account_code'      => 'total' ,
            'account_level'     => '' ,
            'parent_id'         => '' ,
            'name'              => 'total',
            'account_type'      => '',
            'first_debit'       => $this->return['first_debit'] ,
            'first_credit'      => $this->return['first_credit'] ,
            'actions_debit'     => $this->return['actions_debit'] ,
            'actions_credit'    => $this->return['actions_credit'] ,
            'final_debit'       => collect($this->return['report'])->whereIn('account_code', [1, 2 , 3 , 4])->sum('final_debit') ,
            'final_credit'      => collect($this->return['report'])->whereIn('account_code', [1, 2 , 3 , 4])->sum('final_credit')
        ];

        $this->return['logo']     = get_logo(1);

        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Finance/report/trial_balance');
                return Excel::download($export, 'balances.xlsx');
            break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Finance/report/trial_balance', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
            break;
            case 'print':
                return view('Finance/report/trial_balance')->with('data' , $this->return);
            break;
            case 'json':
                return $this->sendResponse( $this->return  );
                // return view('finance/report/account_statment_anal_p')->with('data' , $this->data);
            break;
        }
    }

    public function customer_statement_get($lang, Request $request)
    {
        $this->checkPermission('04-014','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-013' , $lang)->sub_title ?? '';
        $this->return['customers']      = Customer::all_customers(array('active' => 1, 'lang' => $lang));
        return $this->sendResponse( $this->return  );
    }

    public function customer_statement_post($lang, Request $request)
    {

        $this->checkPermission('04-014','view');


        $this->return['from']         = $request['from'] ?? date("Y-m-01");
        $this->return['to']           = $request['to'] ?? date("Y-m-t");
        $this->return['with_draft']   = $request['with_draft'] ?? null ;
        $this->return['customer_id']  = $request['customer_id'] ?? 0 ;
        $this->return['first']        = CustomerStatmentReportHelper::get_account_customer_statment($request['customer_id'] , $lang ,  array('total' => true , 'before' => $this->return['from'] , 'with_draft' => $this->return['with_draft'],'invoice_date' => $request['invoice_date'] ?? 0 ) , $this->data['client_id'] );
        $this->return['action']       = CustomerStatmentReportHelper::get_account_customer_statment($request['customer_id'],$lang , array('from' => $this->return['from']  , 'to' => $this->return['to'] , 'with_draft' => $this->return['with_draft'],'invoice_date' => $request['invoice_date'] ?? 0 )  , $this->data['client_id'] );



        if($this->return['first']->total > 0 ){
            $this->return['first']->credit  = 0;
            $this->return['first']->debit   = $this->return['first']->total;
        }else{
            $this->return['first']->debit  = 0;
            $this->return['first']->credit   = $this->return['first']->total * -1 ;
        }


       return $this->sendResponse( $this->return);
    }

    public function customer_balances_report_get($lang, Request $request)
    {
        $this->checkPermission('04-014','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang)->sub_title ?? '';
        // $this->return['customers']      = Customer::all_customers(array('active' => 1, 'lang' => $lang));
        $this->return['delegates']      = Delegate::all_delegates(array('active' => 1, 'lang' => $lang ));
        return $this->sendResponse( $this->return  );

    }

    public function customer_balances_report_post($lang, Request $request)
    {

        $this->checkPermission('04-014','view');

        $this->return['from']           = $request['from'] ?? date("Y-m-01");
        $this->return['to']             = $request['to'] ?? date("Y-m-t");
        $this->return['before']         = date("Y-m-d" , strtotime($this->return['from']." -1 days"));
        $this->return['credit']         = $request['credit'] ?? 1;
        $this->return['debit']          = $request['debit'] ?? 1;
        $this->return['has_balance']    = $request['has_balance'] ?? 1;
        $this->return['delegate_id']    = $request['delegate_id'] ?? null;
        $this->return['report']         = [];
        $sort_by                        = $request['sort_by'] ?? null;
        $sort_value                     = $request['sort_value'] ?? 0;


        $first      = CustomerStatmentReportHelper::get_account_customer_statment(0,$lang,array('from' => '2000-01-01' , 'to' => $this->return['before']) );
        $actions    = CustomerStatmentReportHelper::get_account_customer_statment(0,$lang,array('from' => $this->return['from'] , 'to' => $this->return['to']) );
        $customers  = Customer::all_customers(['active' => 1 , 'lang' => $lang , 'delegate_id' => $this->return['delegate_id']]);

        foreach ($customers as $key => $value) {

            // $customer_f = collect($first)->where('customer_id' , $value->id)->all();
            // $customer_a = collect($actions)->where('customer_id' , $value->id)->all();

            $f = (object) [
                'debit' => collect($first)->where('customer_id' , $value->id)->where('sumtion' , 1)->sum('amount') ,
                'credit' => collect($first)->where('customer_id' , $value->id)->where('sumtion' , 2)->sum('amount') ,
            ];
            $f->total = $f->debit - $f->credit;

            $a = (object) [
                'debit' => collect($actions)->where('customer_id' , $value->id)->where('sumtion' , 1)->sum('amount') ,
                'credit' => collect($actions)->where('customer_id' , $value->id)->where('sumtion' , 2)->sum('amount') ,
            ];
            $a->total = $a->debit - $a->credit;


            $balance = $f->total + $a->debit - $a->credit ;
            if($this->return['credit'] == 0 && $balance < 0 ){
                continue;
            }

            if($this->return['debit'] == 0 && $balance > 0 ){
                continue;
            }

            if($this->return['has_balance'] == 1 && $balance  == 0 ){
                continue;
            }

            $this->return['report'][] = [
                'id'        => $value->id ,
                'name'      => $value->name ,
                'first'     => $f->total ,
                'debit'     => $a->debit ,
                'credit'    => $a->credit ,
                'balance'   => $balance
            ];
        }
        if (!is_null($sort_by)){
            usort($this->return['report'], function ($a, $b) use ($sort_by,$sort_value){
                if ($sort_by == 'name'){
                    $result =  strcmp($a[$sort_by], $b[$sort_by]);
                }elseif (in_array($sort_by,['id','balance'])){
                    $result =  $a[$sort_by] <=> $b[$sort_by];
                }
                return ($sort_value === 'desc') ? -$result : $result;
            });
        }

       return $this->sendResponse( $this->return);

    }


    public function balanced_by_main_customers_get($lang, Request $request)
    {
        $this->checkPermission('03-026','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang)->sub_title ?? '';
        $this->return['customers']      = Customer::all_customers(['active' => 1 , 'lang' => $lang , 'is_main' => 1 ]);

        return $this->sendResponse( $this->return  );

    }


    public function balanced_by_main_customers_post($lang, Request $request)
    {
        $this->checkPermission('03-026','view');

        $this->filter['date'] = $request['date'] ?? date("Y-m-d");

        $all_main_customers     = Customer::all_customers(['active' => 1 , 'lang' => $lang , 'is_main' => 1 ]);
        $this->return['report'] = [];

        foreach ($all_main_customers as $main_key => $main_row) {

            $seed = [
                'id'            => $main_row->id ,
                'name'          => $main_row->name ,
                'credit'        => 0 ,
                'debit'         => 0 ,
                'total'         => 0 ,
                'total_subs'    => 0 ,
                'customers'     => []
            ];

            $all_sub_customers     = Customer::all_customers(['active' => 1 , 'lang' => $lang , 'customer_main_id' => $main_row->id ]);
            foreach ($all_sub_customers as $sub_key => $sub_row) {
                $x = CustomerStatmentReportHelper::get_account_customer_statment($sub_row->id , $lang ,  array('total' => true , 'before' => $this->filter['date'] ) , 0 );

                 $all_sub_customers[$sub_key]->total_credit = $x->credit;
                 $all_sub_customers[$sub_key]->total_credit = $x->debit;
                 $all_sub_customers[$sub_key]->final_total = $x->total;

            }

            $seed['credit']     = collect($all_sub_customers)->sum('total_credit');
            $seed['debit']      = collect($all_sub_customers)->sum('total_debit');
            $seed['total']      = collect($all_sub_customers)->sum('final_total');
            $seed['total_subs'] = count($all_sub_customers);
            $seed['customers']  = $all_sub_customers;


            $this->return['report'][] = $seed;

        }
        return $this->sendResponse( $this->return);

    }


    public function delegate_statement_get($lang, Request $request)
    {
        $this->checkPermission('03-033','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-013' , $lang)->sub_title ?? '';
        $this->return['delegates']      = Delegate::all_delegates(array('active' => 1, 'lang' => $lang ));
        return $this->sendResponse( $this->return  );
    }

    public function delegate_statement_post($lang, Request $request)
    {

        $this->checkPermission('03-033','view');

        $this->return['from']         = $request['from'] ?? date("Y-m-01");
        $this->return['to']           = $request['to'] ?? date("Y-m-t");
        $this->return['delegate_id']  = $request['delegate_id'] ?? 0 ;
        $this->return['invoice_date'] = $request['invoice_date'] ?? 0 ;
        $this->return['with_draft']   = $request['with_draft'] ?? null ;
        $this->return['first']        = CustomerStatmentReportHelper::get_account_customer_statment(0 , $lang ,  array('total' => true , 'before' => $this->return['from'] , 'invoice_date' => $this->return['invoice_date'] , 'with_draft' => $this->return['with_draft'] ) , $this->data['client_id'],$this->return['delegate_id']);
        $this->return['action']       = CustomerStatmentReportHelper::get_account_customer_statment(0,$lang , array('from' => $this->return['from']  , 'to' => $this->return['to'], 'invoice_date' => $this->return['invoice_date'] , 'with_draft' => $this->return['with_draft']  )  , $this->data['client_id'],$this->return['delegate_id']);


        if($this->return['first']->total > 0 ){
            $this->return['first']->credit  = 0;
            $this->return['first']->debit   = $this->return['first']->total;
        }else{
            $this->return['first']->debit  = 0;
            $this->return['first']->credit   = $this->return['first']->total * -1 ;
        }


        return $this->sendResponse( $this->return);
    }


    public function tax_declaration_post($lang , Request $request){

        $this->checkPermission('03-042','view');
        $this->return['from']           = $request['from'] ?? date("Y-m-01");
        $this->return['to']             = $request['to'] ?? date("Y-m-t");
        $tax_types                      = DB::table('tax_types')->get();
        $report                         = DB::select('call p_tax_declaration("'.$this->return['from'].'" , "'.$this->return['to'].'")');
        $report_collection              = collect($report);
        $this->return['report']         = [];

        foreach ($tax_types as $key => $value) {
            $tax_type_items = $report_collection->where('tax_type_id' , $value->tax_type_id );
            $this->return['report'][] = [
                'id'                => $value->tax_type_id ,
                'name'              => ($lang == 'ar' ? $value->name_ar :  $value->name_en ) ,
                'in_time'           => $tax_type_items->where('in_time' , 1 )->where('action_type' , "1" )->sum('total_without_vat') - $tax_type_items->where('in_time' , 1 )->where('action_type' , "2" )->sum('total_without_vat') ,
                'out_time'          => $tax_type_items->where('in_time' , 0 )->where('action_type' , "1" )->sum('total_without_vat') - $tax_type_items->where('in_time' , 0 )->where('action_type' , "2" )->sum('total_without_vat'),
                'vat'               => $tax_type_items->where('action_type' , "1" )->sum('total_vat') - $tax_type_items->where('action_type' , "2" )->sum('total_vat') ,
                'items_in_time'     => $tax_type_items->where('in_time' , 1 )->values()->all() ,
                'items_out_time'    => $tax_type_items->where('in_time' , 0 )->values()->all()
            ];
        }

        return $this->sendResponse( $this->return );
    }
    public function profit_and_loss_report_get($lang , Request $request)
    {
        $this->checkPermission('03-053','view');

        return $this->sendResponse( $this->return  );
    }

    public function profit_and_loss_report_post($lang , Request $request)
    {

        $this->checkPermission('03-053','view');
        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('04-014' , $lang)->sub_title ?? '';
        $this->return['report']           = [];
        $report_accounts = DB::table('profit_and_loss_accounts')
            ->orderBy('profit_and_loss_accounts.sort_order','asc')
            ->get();


        foreach ($report_accounts as $report_account) {

            $account = DB::table('account_tree')
                ->where('account_tree_id', $report_account->account_tree_id)
                ->first();


            $report = DB::table('daily_journal_d')
                ->join('account_tree' , 'daily_journal_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
                // ->join('profit_and_loss_accounts' , 'daily_journal_d.account_tree_id' , '=' , 'profit_and_loss_accounts.account_tree_id')
                ->join('daily_journal_m' , 'daily_journal_d.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
                ->where('journal_active' , 1)
                ->where('confirmed' , 1)
                ->where(function($where) use($account){
                    $where->where('account_tree.account_tree_id',$account->account_tree_id);
                    $where->where('account_tree.parent_id',$account->account_tree_id);
                    $where->orWhere('account_tree.account_code', 'like',$account->account_code."%");
                })

                // ->groupBy('daily_journal_d.account_tree_id')
                ->selectRaw('
                    daily_journal_d.account_tree_id ,
                    daily_journal_m.journal_date,
                    account_tree.account_code,
                    account_tree.name_'.$lang.' as account_name,
                    debit,
                    credit
                    ')
                ->where('daily_journal_m.journal_date' , '>=' , $request['from_date'] )
                ->where('daily_journal_m.journal_date' , '<=' , $request['to_date'] )
                ->get();

            $this->return['report'][]           = [
                'account_tree_id'   => $account->account_tree_id,
                'journal_date'      => $report[0]->journal_date ?? '' ,
                'account_code'      => $account->account_code,
                'account_type'      => $report_account->account_type,
                'account_name'      => $account->name_ar,
                'debit'             => collect($report)->sum('debit'),
                'credit'            => collect($report)->sum('credit'),
                'total'             => collect($report)->sum('debit') - collect($report)->sum('credit'),
            ];


        }

        $this->return['1'] = collect($this->return['report'])->whereIn('account_code' , ['4101'])->sum('total') ;
        $this->return['2'] = collect($this->return['report'])->whereIn('account_code' , ['3101' , '3102' , '3103'])->sum('total');
        $this->return['final_profit'] = abs(collect($this->return['report'])->whereIn('account_code' , ['4101'])->sum('total')) - abs(collect($this->return['report'])->whereIn('account_code' , ['3101' , '3102' , '3103'])->sum('total'));
        return $this->sendResponse( $this->return);

    }
    public function revenue_analysis($lang , Request $request){

        // $this->checkPermission('03-042','view');
        $this->return['from']           = $request['from'] ?? date("Y-m-01");
        $this->return['to']             = $request['to'] ?? date("Y-m-t");
        $this->return['customer_id']    = $request['customer_id'] ??null;
        $this->return['port_id']        = $request['port_id'] ??null;
        $this->return['revenue_id']     = $request['revenue_id'] ??null;
        $this->return['lang']           = $lang;
        $this->return['report']  = DB::select('call p_revenue_analysis(
            "'.$this->return['from'].'" ,
            "'.$this->return['to'].'" ,
            "'.$this->return['customer_id'].'" ,
            "'.$this->return['port_id'].'" ,
            "'.$this->return['revenue_id'].'" ,
            "'.$this->return['lang'].'"
            )');


        return $this->sendResponse( $this->return );
    }
    public function customer_revenue($lang , Request $request){

        // $this->checkPermission('03-042','view');
        $this->return['from_date']      = $request['from_date'] ?? date("Y-m-01");
        $this->return['to_date']        = $request['to_date'] ?? date("Y-m-t");
        $this->return['customer_id']    = $request['customer_id'] ?? null;
        $this->return['lang']           = $lang;
        $this->return['headers'] = DB::table('revenues')->where('revenues_active',1)
        ->selectRaw('revenue_name_'.$lang.' as revenue_name')
        ->get()->toArray();
        $query = DB::table('clearance_invoices_d')
        ->leftjoin('clearance_invoices_m', 'clearance_invoices_d.invoice_m_id', '=', 'clearance_invoices_m.invoice_id')
        ->leftjoin('customers' , 'clearance_invoices_m.customer_id' , '=' , 'customers.customer_id')
        ->rightJoin('revenues', 'clearance_invoices_d.revenue_id', '=', 'revenues.revenue_id')
        ->groupBy('clearance_invoices_d.revenue_id')
        ->whereBetween('clearance_invoices_m.invoice_date', [$this->return['from_date'], $this->return['to_date']])
        ->selectRaw('
            customers.name_'.$lang.' as customer_name ,
            clearance_invoices_m.customer_id ,
            revenues.revenue_name_'.$lang.' as revenue_name,
            sum(clearance_invoices_d.amount_with_vat) as amount_with_vat ,
            sum(clearance_invoices_d.amount_without_vat) as amount_without_vat ,
            sum(clearance_invoices_d.vat_amount) as vat_amount
        ');
        if(isset($this->return['customer_id'])){
            $query->where('clearance_invoices_m.customer_id', $this->return['customer_id']);
        }

        $data =  $query->get();
        $this->return['report'] = [];
        foreach ($data as $item) {
                $customerId = $item->customer_id;

                if (!isset($this->return['report'][$customerId])) {
                    $this->return['report'][$customerId] = [
                        'customer_id'       => $customerId,
                        'customer_name'     => $item->customer_name,
                        'revenues'          => [],
                    ];
                }

                $this->return['report'][$customerId]['revenues'][] = [
                    'revenue_name'          => $item->revenue_name,
                    'amount_with_vat'       => $item->amount_with_vat,
                    'amount_without_vat'    => $item->amount_without_vat,
                    'vat_amount'            => $item->vat_amount,
                ];

        }
        $this->return['report'] = array_values($this->return['report']);
        return $this->sendResponse( $this->return );
    }
}
