<?php

namespace App\Http\Controllers\API\V1\Main;

use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Models\Main\ScreenSub;
use App\Models\Main\DailyType;
use Validator;


class DailyTypeController extends BaseController
{
    protected $return = array();
    protected $filter = array();
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($lang , Request $request)
    {
        $this->checkPermission('01-005','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null

        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('01-005' , $lang)->sub_title ?? '';
        $this->return['items']       = DailyType::all_daily_types( $this->filter );
        return $this->sendResponse( $this->return );
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $this->checkPermission('01-005','add');


        $insert             = $request->only(['name_en', 'name_ar']);
        $insert['add_user'] = auth()->user()->id;
        $validator          = validator()->make( $insert , DailyType::$rules );

        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $this->return['id'] = DailyType::add_daily_type($insert);
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($lang, $id)
    {
        $this->checkPermission('01-005','view');

        $this->return['screen_name']          = ScreenSub::get_screen_sub_by_screen_code('01-005' , $lang)->sub_title ?? '';
        $this->return['data']                 = DailyType::get_daily_type($id);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update($lang , $id, Request $request)
    {
        $this->checkPermission('01-005','edit');


        $update    = $request->only(['name_en', 'name_ar']);
        $validator = Validator::make( $request->all() , DailyType::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        DailyType::update_daily_type($update ,  $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($lang, $id)
    {
        $this->checkPermission('01-005','delete');

        DailyType::delete_daily_type($id);
        return $this->sendResponse( [] , trans('main.delete_success') );

    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('01-005','edit');


        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            DailyType::update_daily_type(array('daily_type_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = DailyType::find($id);
        if($item->daily_type_active){

            $this->checkPermission('01-005','delete');


            DailyType::delete_daily_type($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('01-005','edit');


            DailyType::update_daily_type(array('daily_type_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }
}
