<?php
namespace App\Http\Controllers\API\V1\Main;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\NotAllowedTime;
use App\Models\Sales\Delegate;
use Illuminate\Http\Request;

use App\Classes\General\Upload;

use App\Models\Main\Branch;
use App\Models\Main\FinanceYear;
use App\Models\Main\ScreenSub;
use App\Models\Main\SalesAgent;
use App\Models\Trans\Port;
use App\Models\User;
use Excel;
use DB;
use Auth;
use Illuminate\Support\Facades\Storage;
use Str;
use Hash;

class UserController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {

        $this->checkPermission('01-008','view');

        $this->filter = array(
            'user_active' => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('01-008', $lang)->sub_title ?? '';
        $this->return['items']       = User::all_users($this->filter);

        return $this->sendResponse($this->return);

    }

    public function create($lang, Request $request)
    {

        $this->checkPermission('01-008','add');


        $this->return['titles']          = DB::table('title')->select('title_id as id', 'title_name_'.$lang.' as name')->get();
        $this->return['all_permissions'] = ScreenSub::all_screen_subs( array( 'lang' => $lang , 'active' => 1 ) );
        $this->return['screen_name']     = ScreenSub::get_screen_sub_by_screen_code('01-008', $lang);
        $this->return['sales_agents']    = SalesAgent::all_sales_agents( array('lang' => $lang , 'active' => 1 ) );
        $this->return['branchs']         = Branch::all_branchs( array('lang' => $lang, 'active' => 1 ));
        $this->return['financial_years'] = FinanceYear::all_finance_years(array('lang' => $lang, 'active' => 1));
        $this->return['ports']           = Port::all_ports(array('lang' => $lang , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang , 'active' => 1 ));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {

        $this->checkPermission('01-008','add');


        $validator = validator()->make($request->all(), User::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $seed = array(
            'user_full_name'            => $request['user_full_name'] ,
            'username'                  => $request['username'] ,
            'email'                     => $request['email'] ,
            'mobile'                    => $request['mobile'] ,
            'session_status'            => $request['session_status'] ,
            'delegate_ids'              => (isset($request['delegate_ids']) && is_array($request['delegate_ids'])? implode(',' , $request['delegate_ids'] ): ""  ) ,
            'password'                  => Hash::make($request['password']) ,
            'title_id'                  => $request['title_id'] ,
            'sales_agent_id'            => $request['sales_agent_id'] ?? 1 ,
            'default_branch'            => $request['default_branch'] ?? 0 ,
            'default_year'              => $request['default_year'] ?? 0 ,
            'default_port'              => $request['default_port'] ?? 0 ,
            'edit_trip_price'           => $request['edit_trip_price'] ?? 0 ,
            'edit_trans_date'           => $request['edit_trans_date'] ?? 0 ,
            'edit_return_date'          => $request['edit_return_date'] ?? 0 ,
            'edit_multi_voucher_attachments' => $request['edit_multi_voucher_attachments'] ?? 0 ,
            'out_trans'                 => $request['out_trans'] ?? 0 ,
            'clearance_status'          => $request['clearance_status'] ?? 0 ,
            'invoice_issuance_status'   => $request['invoice_issuance_status'] ?? 0 ,
            'user_active'               => 1 ,
            'access_reports'            => $request['access_reports'] ?? 0,
            'user_titles'               => (is_array($request['user_titles'] ?? '')) ? implode(',', $request['user_titles']) : $request['user_titles'],
        );
        if($request->hasFile('image_url') ){
            $upload = new Upload;
//            $check  = $upload->uploadBase64Image($request['image_url'], 'profile_images', 300, 300);
            $check  = $upload->uploadImage($request['image_url'], 'profile_images', 300,300);
            if(!$check['check'])
                return $this->sendError(trans('error.add_fails'), $check['msg']);
            else
                $seed['image_url'] = $check['url'];
        }

        DB::beginTransaction();

        $id = User::add_user($seed);

        if (isset($request['user_titles']) && !is_null($request['user_titles']) && is_array($request['user_titles'])){
            $titles = DB::table('title_permissions')->whereIn('title_id', $request['user_titles'])->get();
        }else{
            $titles = DB::table('title_permissions')->where('title_id', $request['user_titles'])->get();
        }

        if (isset($titles)){
            foreach ($titles as $value) {
                DB::table('user_permissions')->insert([
                    'user_id'       => $id ,
                    'title_id'      => $value->title_id ,
                    'screen_sub_id' => $value->screen_sub_id ,
                    'permission'    => $value->permission,
                    'add_user'      => auth()->user()->id ,
                    'add_date'      => date("Y-m-d H:i:s")
                ]);
            }
        }

        $branch = Branch::get_branch($seed['default_branch']);
        if(strlen($branch->users) <= 0){
            $users = $seed['default_branch'];
        }else{
            $users = $branch->users.",".$id;
        }

        Branch::update_branch(['users' => $users], $seed['default_branch']);

        $year = FinanceYear::get_finance_year($seed['default_year']);

        if(strlen($year->users) <= 0){
            $users = $seed['default_year'];
        }else{
            $users = $year->users.",".$id;
        }

        FinanceYear::update_finance_year(['users' => $users], $seed['default_year']);

        DB::commit();
        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id, Request $request)
    {

        $this->checkPermission('01-008','view');


        $this->return['screen_name']          = ScreenSub::get_screen_sub_by_screen_code('01-008', $lang)->sub_title ?? '';
        $this->return['data']                 = User::find($id);
        $this->return['branchs']              = Branch::all_branchs(array('lang' => $lang, 'active' => 1));
        $this->return['delegates']              = Delegate::all_delegates(array('lang' => $lang , 'active' => 1));
        $this->return['financial_years']      = FinanceYear::all_finance_years(array('lang' => $lang, 'active' => 1));
        $this->return['titles']               = DB::table('title')->select('title_id as id', 'title_name_'.$lang.' as name')->get();
        $this->return['all_permissions']      = ScreenSub::all_screen_subs( array('lang' => $lang , 'active' => 1 ) );
        $this->return['ports']                = Port::all_ports(array('lang' => $lang , 'active' => 1 ));
        $user_permissions                     = User::get_user_permission($id,null, true)->groupBy('screen_code')->toArray();
        $this->return['not_allowed_times']    = NotAllowedTime::get_not_allowed_time($id, $lang);


        foreach ($this->return['all_permissions'] as $key => $value) {
            $screens     = array('1' => 'view' , '2' => 'add' , '3' => 'edit' , '4' => 'delete' , '5' => 'print');
            $actions     = explode(',', $value->actions);
            $permissions = json_decode($user_permissions[$value->screen_code][0]->permission ?? "[]");
            $sync        = array();

            foreach($actions as $action_key => $row){
                if(in_array($row, $permissions)){
                    $sync[$screens[$row]] = true;
                }else{
                    $sync[$screens[$row]] = false;
                }
            }

            $this->return['all_permissions'][$key]->permissions = $sync;
        }

        $this->return['sales_agents'] = SalesAgent::all_sales_agents($lang);

        $this->return['all_my_years'] = DB::table('finance_years')
            ->whereRaw("find_in_set(".$id.", users)")
            ->pluck('name', 'finance_year_id')->toArray();

        $this->return['all_my_branchs'] = DB::table('branchs')
            ->whereRaw("find_in_set(".$id.", users)")
            ->pluck('name_'.$lang.' as name', 'branch_id')->toArray();

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function update($lang, $id, Request $request)
    {

        $this->checkPermission('01-008','edit');


        $user   = User::find($id);
        $rules  = User::$rules;
        $rules['username'] = $rules['username'].',id,'.$id;
        $rules['email']    = $rules['email'].',id,'.$id;
        $rules['mobile']   = $rules['mobile'].',id,'.$id;

        $update['user_full_name'] = $request['user_full_name'] ?? $user->user_full_name ;
        $update['email']          = $request['email'] ?? $user->email ;
        $update['mobile']         = $request['mobile'] ?? $user->mobile ;
        $update['session_status'] = $request['session_status'] ?? 0 ;
        $update['delegate_ids']   = (isset($request['delegate_ids']) && is_array($request['delegate_ids'])? implode(',' , $request['delegate_ids'] ): ""  );
        $update['username']       = $request['username'] ?? $user->username ;
        $update['title_id']       = $request['title_id'] ?? $user->title_id ;
        $update['sales_agent_id'] = $request['sales_agent_id'] ?? $user->sales_agent_id ;
        $update['default_branch'] = $request['default_branch'] ?? $user->default_branch ;
        $update['default_year']   = $request['default_year'] ?? $user->default_year ;
        $update['default_port']   = $request['default_port'] ?? $user->default_port ;
        $update['clearance_status']   = $request['clearance_status'] ?? $user->clearance_status ;
        $update['invoice_issuance_status']   = $request['invoice_issuance_status'] ?? $user->invoice_issuance_status ;
        $update['edit_multi_voucher_attachments'] = $request['edit_multi_voucher_attachments'] ?? $user->edit_multi_voucher_attachments ;

        $update['access_reports']   = $request['access_reports'] ?? $user->access_reports;
        $update['edit_trip_price']  = $request['edit_trip_price'] ?? 0;
        $update['edit_trans_date']  = $request['edit_trans_date'] ?? 0;
        $update['out_trans']        = $request['out_trans'] ?? 0;
        $update['edit_return_date'] = $request['edit_return_date'] ?? 0 ;

        $update['user_titles']    = ((is_array($request['user_titles'] ?? '')) ? implode(',', $request['user_titles']) : $request['user_titles']) ?? $user->user_titles ;
        $update['user_active']    = $request['user_active'] ?? $user->user_active ;
        $update['password']       = $user->password;
        // $update['title_id']       = $user->title_id;


        if(isset($request['password']) && strlen($request['password']) > 0){
            $update['password'] = Hash::make($request['password']);
        }

        $validator = validator()->make($update, $rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        if($request->hasFile('image_url')){
            $filePathFromUrl = str_replace('/upload/', '', $user->image_url);
            Storage::exists($filePathFromUrl) ?  Storage::delete($filePathFromUrl) : null ;
            $upload = new Upload;
//            $check  = $upload->uploadBase64Image($request['image_url'], 'profile_images', 300, 300);
            $check  = $upload->uploadImage($request['image_url'], 'profile_images', 300,300);
            if(!$check['check'])
                return $this->sendError(trans('error.add_fails'), $check['msg']);
            else
                $update['image_url'] = $check['url'];
        }



        if (isset($request['user_titles']) && is_array($request['user_titles'])){

            DB::table('user_permissions')
                ->where('user_id', $id)
                ->delete();

            $titles = DB::table('title_permissions')
                ->whereIn('title_id', $request['user_titles'])
                ->get();

            if (isset($titles)){
                foreach ($titles as $value) {
                    DB::table('user_permissions')->insert([
                        'user_id'       => $id ,
                        'title_id'      => $value->title_id ,
                        'screen_sub_id' => $value->screen_sub_id ,
                        'permission'    => $value->permission,
                        'add_user'      => auth()->user()->id ,
                        'add_date'      => date("Y-m-d H:i:s")
                    ]);
                }
            }
        }

        if(count( $update ) > 0){
            User::update_user($update, $id);
        }
        if (isset($request['times']) && is_array($request['times'])){
            $set_not_allowed_time['times'] = $request['times'];
            $set_not_allowed_time['user_id'] = $id;
            $result =  (new NotAllowedTimeController)->update($lang,$id,new Request($set_not_allowed_time))->getData();
            if (!$result->check){
                return $this->sendResponse($result->data,$result->msg);
            }
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {

        $this->checkPermission('01-008','delete');


        User::delete_user($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function change_password($lang, Request $request)
    {
        $user = User::get_user(auth()->user()->id);

        if(!Hash::check($request['old_password'], $user->password)){
            return $this->sendError('Old Password Error', []);
        }

        if($request['new_password'] !== $request['new_password2']){
            return $this->sendError('New Password not match', []);
        }

        User::update_user(array('password' => Hash::make($request['new_password'])), auth()->user()->id);

        return $this->sendResponse([], trans('main.update_success'));
    }

    public function update_permission($lang, $id, Request $request)
    {

        $this->checkPermission('01-008','edit');


        if(!isset($request['data']) || !is_array($request['data']) || count($request['data']) < 1 ){
            return $this->sendError('data array is required', []);
        }

        DB::beginTransaction();

        foreach ($request['data'] as $key => $value) {
            if(!is_array($value['permissions'])){
                $value['permissions'] = explode(',', $value['permissions']);
            }

            DB::table('user_permissions')
                ->where('user_id', $id)
                ->where('screen_sub_id', $value['sub_id'])
                ->delete();

            User::insert_user_permission(
                array(
                    'screen_sub_id' => $value['sub_id'] ,
                    'user_id'       => $id ,
                    'permission'    => json_encode($value['permissions']) ,
                    'add_user'      => auth()->user()->id ,
                    'add_date'      => date("Y-m-d H:i:s")
                )
            );
        }

        DB::commit();
        return $this->sendResponse([], trans('main.update_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = User::find($id);

        if($item->user_active){

            $this->checkPermission('01-008','delete');


            User::delete_user($id);
            $this->return['active'] = false;
        }else{

            $this->checkPermission('01-008','edit');


            User::update_user(array('user_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function change_user_defaults($lang, Request $request)
    {
        if (isset($request['branch_id']) && isset($request['year_id']) && isset($request['title_id'])){
            DB::table('users')
                ->where('id', auth()->user()->id)
                ->update([
                'default_branch' => $request['branch_id'],
                'default_year'   => $request['year_id'],
                'title_id'       => $request['title_id'],
            ]);
        }

        return $this->sendResponse([], trans('User Defaults Data Changed Successfully'));
    }
}
