<?php

namespace App\Http\Controllers\API\V1;

use App\Classes\ReportingHelper;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\ScreenMain;
use Illuminate\Http\Request;
use App\Models\Main\ScreenSub;
use App\Models\Reporting;
use Illuminate\Validation\Rule;

class ReportingController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        // $this->checkPermission('15-001','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('15-001' , $lang)->sub_title ?? '';
        $this->return['items']       = Reporting::all_reports($this->filter);
        return $this->sendResponse($this->return );
    }

    public function create($lang , Request $request)
    {
        // $this->checkPermission('15-001','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('15-001' , $lang)->sub_title ?? '';
        $this->return['screen_mains']   = ScreenMain::all_screen_mains(array('active' =>1 ,'lang'=>$lang));
        $this->return['lookups']        = (new ReportingHelper)->lookups;
        return $this->sendResponse( $this->return);
    }

    public function store(Request $request)
    {
        // $this->checkPermission('15-001','add');

        $input    = $request->all();
        $insert = [
            'name_ar'           => $input['name_ar'] ,
            'name_en'           => $input['name_en'] ,
            'file_en'           => $input['file_en'] ,
            'file_ar'           => $input['file_ar'] ,
            'stored_procedure'  => $input['stored_procedure'] ,
            'screen_main_id'    => $input['screen_main_id'],
            'add_user'          => auth()->user()->id,
        ];

        $validator = validator()->make( $insert , Reporting::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $this->return['id'] = Reporting::add_report_m($insert);
        if(isset($input['filters']) && is_array($input['filters'])){
            foreach($input['filters'] as $item){
                $details_insert = [
                    'report_m_id'   => $this->return['id'],
                    'name_ar'       => $item['name_ar'],
                    'name_en'       => $item['name_en'],
                    'value'         => $item['value'],
                    'lookup'        => $item['lookup'] ?? null,
                    'data_type'     => $item['data_type'],
                    'input_type'    => $item['input_type'],
                    'input_default' => $item['input_default'] ?? null,
                    'required'      => $item['required'] ?? 0,
                    'report_order'  => $item['report_order'],
                ];
                Reporting::add_report_d($details_insert);
            }
        }

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id )
    {
        // $this->checkPermission('15-001','view');

        $this->return['screen_name']      = ScreenSub::get_screen_sub_by_screen_code('15-001' , $lang)->sub_title ?? '';
        $this->return['master']           = Reporting::get_report_m($id,$lang);
        $this->return['details']          = Reporting::get_report_d($id,$lang);
        foreach($this->return['details'] as $item){
            $item->lookup_list = (new ReportingHelper)->get_lookups($item->lookup , $lang);
        }
        $this->return['screen_mains']   = ScreenMain::all_screen_mains(array('active' =>1 ,'lang'=>$lang));
        $this->return['lookups']        = (new ReportingHelper)->lookups;

        return $this->sendResponse( $this->return);
    }

    public function update($lang, $id , Request $request)
    {
        // $this->checkPermission('15-001','edit');

        $input  = $request->all();

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }
    public function destroy($lang,$id)
    {
        // $this->checkPermission('15-001','delete');

        Reporting::delete_report($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }




}
