<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\AccountTree;
use App\Models\Sales\AttachmentName;
use App\Models\Sales\CovenantType;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class AttachmentNameController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
//        $this->checkPermission('03-034','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-034', $lang)->sub_title ?? '';
        $this->return['items']       = AttachmentName::all_attachment_names($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
//        $this->checkPermission('03-034','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-034', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
//        $this->checkPermission('03-034','add');

        $insert             = $request->only(['name_en', 'name_ar']);
        $insert['add_user'] = auth()->user()->id;
        $validator          = validator()->make($insert, AttachmentName::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $insert['id'] = AttachmentName::add_attachment_name($insert);
        $this->return = $insert;

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
//        $this->checkPermission('03-034','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-034', $lang)->sub_title ?? '';
        $this->return['data']           = AttachmentName::get_attachment_name($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
//        $this->checkPermission('03-034','edit');

        $update = $request->only(['name_en', 'name_ar']);

        $validator = validator()->make($update, AttachmentName::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        AttachmentName::update_attachment_name($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
//        $this->checkPermission('03-034','delete');

        AttachmentName::delete_attachment_name($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = AttachmentName::find($id);

        if($item->attachment_name_active){
//            $this->checkPermission('03-034','delete');

            AttachmentName::delete_attachment_name($id);
            $this->return['active'] = false;
        }else{
//            $this->checkPermission('03-034','edit');

            AttachmentName::update_attachment_name(array('attachment_name_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
