<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Classes\Excel\InvoicesExport;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Classes\General\General;
use App\Classes\General\Upload;
use App\Classes\Sales\ClearanceInvoiceHelper;
use App\Models\Finance\AccountTree;
use App\Models\Finance\Treasury;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Clearance;
use App\Models\Sales\ClearanceInvoiceClaim as ClearanceInvoice;
use App\Models\Sales\Customer;
use App\Models\Main\Branch;
use App\Models\Main\Setting;
use App\Models\Finance\DailyJournal;

use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Maatwebsite\Excel\Facades\Excel;
use PDF;

class ClaimController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('03-037','view');
        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
            'from_id' => $request['from_id'] ?? null ,
            'to_id'   => $request['to_id'] ?? null ,
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-037', $lang)->sub_title ?? '';
        $this->return['items']          = ClearanceInvoice::all_invoices($this->filter);
        $this->return['is_journals']    = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );

        return $this->sendResponse($this->return);
    }

    public function show($lang, $id)
    {
        $this->checkPermission('03-037','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-037', $lang)->sub_title ?? '';
        $this->return['master']             = ClearanceInvoice::get_invoice($id, $lang);
        $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
        $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
        $this->return['invoice_journals']   = ClearanceInvoice::get_invoice_journals($id, $lang);
        $this->return['account_tree']       = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['is_journals']        = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );
        $clearance      = Clearance::get_clearance( $this->return['master']->clearance_id , $lang );
        $this->return['customer']          = Customer::get_customer( $this->return['master']->customer_id , $lang );

        $this->return['totals']             = [
            'total_with_vat'    => $this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') ,
            'total_vat'         => $this->return['details']->sum('vat_amount') ,
            'total_without_vat' => $this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_without_vat')
        ];
        $this->return['prices']          = DB::table('customer_transport_prices')
            ->leftjoin('areas as from_areas' , 'customer_transport_prices.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , 'customer_transport_prices.to_area_id' , '=' , 'to_areas.area_id')
            ->where('port_id' , $clearance->port_id)
            ->where('customer_id' , $clearance->customer_id)
            ->select('from_areas.area_name_'.$lang.' as from_area_name','to_areas.area_name_'.$lang.' as to_area_name','from_area_id' , 'to_area_id','price' , 'container_20' , 'container_40','price_special' , 'container_20_special' , 'container_40_special' )
            ->where('customer_transport_price_active' , 1)
            ->get();

            $clearance_id = $this->return['master']->clearance_id;
        // if(!$this->get_invoice_amounts($clearance_id , $lang) ){
        //     return $this->sendError($this->return , [],  200);
        // }

        // if($this->return['deleget_amounts']->sum('amount') == 0 && count($this->return['amounts']) == 0   ){
        //     return $this->sendError( trans('main.no_amount_found'), [], 200);
        // }

        $this->return['next_item']      = ClearanceInvoice::get_next( $id , 0 )->m_id ?? 0;
        $this->return['previous_item']  = ClearanceInvoice::get_previous($id , 0 )->m_id ?? 0;
        $this->return['first_item']     = 1;
        $this->return['last_item']      = ClearanceInvoice::get_last( 0 )->m_id ?? 0;

        return $this->sendResponse($this->return);
    }

    public function print( $id , Request $request)
    {

        // $this->checkPermission('03-037','print');
        $lang = 'ar';
        $this->return['lang']               = $lang;
        $this->return['master']             = ClearanceInvoice::get_invoice($id, $lang);
        $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
        $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
        $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
        $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
        $this->return['containers']         = implode(' , ' ,  Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
        $this->return['qrcode']             = '';
        $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_amount , 2) , 'SAR' );
        $this->return['goods_type']         = DB::table('classifications_m')
            ->join('classifications_d' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
            ->where('clearances_m_id' , $this->return['master']->clearance_id)
            ->first()->item_name ?? $this->return['master']->goods_type;
            $temp_name  = $request['print_temp'] ?? Setting::get_main_setting_by_code('claim_invoice_print')->value ?? 'temp_1' ;
            $print_temp = 'Print.clearance.claim.'.$temp_name;
            $this->return['attachments'] = [];

            if(isset($request['with_attachments']) && $request['with_attachments'] == 1){
                $attachments =  (new ClearanceInvoiceHelper)->attachments_image($id , "claim" , $request['clearance_attachments'] ?? 0);
                $this->return['attachments'] = $attachments['images'];
            }
        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , $print_temp);
                return Excel::download($export, 'balances.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView($print_temp, array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
                break;
            case 'image':
                $image = App::make('snappy.image.wrapper');
                $image->loadView($print_temp, array('data' => $this->data));
                return $image->stream();
                break;
            case 'print':
                return view($print_temp)->with('data' , $this->return);
                break;
            case 'json':
                return view($print_temp)->with('data' , $this->return);
                break;
            case 'save_pdf_file':

                $folder                 = '/temp/'.$request['clientID'].'/invoices_pdf/';
                $name                   = 'invoice_'.$id.'.pdf';
                $file_path              = ($folder.$name);
                $file_real_path         = public_path($folder.$name);
                $this->return['logo']   = public_path().$this->return['logo'];
                $options = [
                    'margin-left' => 0 ,
                    'margin-right' => 0 ,
                    'margin-top' => 0 ,
                    'margin-bottom' => 0 ,
                    // 'footer-html' => view()->make('Print.clearance.invoice.whats_footer')->render()
                ];

                if(!is_dir(public_path($folder))){
                    mkdir(public_path($folder), 0777, true);
                }

                if (file_exists(($file_real_path))) {
                    unlink(($file_real_path));
                }
                // return $file_real_path;


                PDF::loadView($print_temp, array('data' => $this->return ))
                    ->setPaper('A4', 'portrait')
                    ->setOptions($options)
                    ->save($file_real_path);

                return ($file_path);
            break;
        }


        return $this->sendResponse($this->return);
    }
    public function print_invoices($lang ,Request $request){
        // $this->checkPermission('03-023','print');
        $this->filter = array(
            'lang'      => $lang ,
            'active'    => $request['active'] ?? 1 ,
            'rows'      => $request['rows'] ?? null ,
            'page'      => $request['page'] ?? 1 ,
            'word'      => $request['word'] ?? null ,
            'from_id'   => $request['from_id'] ?? null ,
            'to_id'     => $request['to_id'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-037', $lang)->sub_title ?? '';
        $this->return['invoices']    = ClearanceInvoice::all_invoices($this->filter);
        $this->return['logo']        = $this->return['branch_data']->branch_image ?? 0;
        $temp_name  = $request['print_temp'] ?? Setting::get_main_setting_by_code('claim_invoice_print')->value ?? 'temp_1' ;
        $print_temp = 'Print.clearance.claim.'.$temp_name;
        $allInvoicesHtml = '';

        foreach ($this->return['invoices']  as $invoice) {
            $this->return['lang']               = $lang;
            $this->return['master']             = ClearanceInvoice::get_invoice($invoice->id, $lang);
            $this->return['details']            = ClearanceInvoice::get_invoice_d($invoice->id, $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($invoice->id, $lang);
            $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
            $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
            $this->return['containers']         = implode(' , ' ,  Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
            $this->return['qrcode']             = '';
            $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_amount , 2) , 'SAR' );
            $this->return['goods_type']         = DB::table('classifications_m')
                ->join('classifications_d' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
                ->where('clearances_m_id' , $this->return['master']->clearance_id)
                ->first()->item_name ?? $this->return['master']->goods_type;

            $invoice_html = view($print_temp, ['data' => $this->return])->render();
            $invoice_html .= '<div style="page-break-after: always;"></div>';
            $allInvoicesHtml .= $invoice_html;
        }
        $allInvoicesHtml .= '<script type="text/javascript">window.print();</script>';
        return  $allInvoicesHtml;
    }
    public function destroy($lang, $id)
    {
        $this->checkPermission('03-037','delete');

        $master_c =  DB::table('clearance_invoices_claim_m')->where('clearance_invoices_claim_m.invoice_id' , $id)->first();
        if((new General)->check_financial_year($master_c->invoice_date) != "true"){
            return $this->sendError((new General)->check_financial_year($master_c->invoice_date));
        }
        unset($master_c->invoice_id);
        $master_c->invoice_date = $master_c->draft_invoice_date;
        $master_c->draft = 1;
        $daily_journal = $master_c->daily_journal_m_id;
        unset($master_c->daily_journal_m_id);
        unset($master_c->draft_invoice_date);
        unset($master_c->claim);
        unset($master_c->invoice_uuid);

        unset($master_c->container_count);
        unset($master_c->container_small);
        unset($master_c->container_big);
        unset($master_c->containers_numbers);
        unset($master_c->is_first);

        $master_c = (array) $master_c;
        DB::beginTransaction();

        $master_id   = DB::table('clearance_invoices_draft_m')->insertGetId($master_c);

        $details_c = DB::table('clearance_invoices_claim_d')->where('invoice_m_id' , $id)->get()->toArray();
        foreach ($details_c as $detail_c){
            unset($detail_c->clearance_invoices_d_id);
            unset($detail_c->current_vat);
            unset($detail_c->current_with_vat);
            unset($detail_c->current_without_vat);
            $detail_c->invoice_m_id = $master_id;
            $detail_c = (array) $detail_c;

            DB::table('clearance_invoices_draft_d')->insertGetId($detail_c);

        }

        $delegate_details_c = DB::table('clearance_invoices_claim_delegate_d')->where('invoice_m_id' , $id )->get();
        foreach ($delegate_details_c as $delegate_detail_c){
            unset($delegate_detail_c->clearance_invoices_delegate_d_id );
            $delegate_detail_c->invoice_m_id = $master_id;
            $delegate_detail_c = (array) $delegate_detail_c;


            DB::table('delegate_petty_d')
                ->whereIn('delegate_petty_d_id' , explode(',', $delegate_detail_c['petty_ids']) )
                ->update( ['invoice_id' => 0 , 'invoice_type' => null ] );

            DB::table('multi_voucher_d')
                ->whereIn('multi_voucher_d_id' , explode(',', $delegate_detail_c['multi_ids']) )
                ->update( ['invoice_id' => 0 , 'invoice_type' => null ] );


            DB::table('clearance_invoices_draft_delegate_d')->insertGetId($delegate_detail_c);

        }

        $invoice_journals_c = DB::table('clearance_invoices_claim_journal')->where('invoice_m_id' , $id )->get();
        foreach ($invoice_journals_c as $invoice_journal_c){
            unset($invoice_journal_c->clearance_invoices_journal_id);
            $invoice_journal_c->invoice_m_id = $master_id;
            $invoice_journal_c = (array) $invoice_journal_c;

            DB::table('clearance_invoices_draft_journal')->insertGetId($invoice_journal_c);

        }

        DB::table('clearance_invoices_claim_m')->where('invoice_id' , $id)->delete();
        DB::table('clearance_invoices_claim_d')->where('invoice_m_id' , $id)->delete();
        DB::table('clearance_invoices_claim_delegate_d')->where('invoice_m_id' , $id)->delete();
        DB::table('clearance_invoices_claim_journal')->where('invoice_m_id' , $id)->delete();

        DailyJournal::delete_daily_journal_m($daily_journal);

        DB::commit();
        return $this->sendResponse([],trans('main.delete_success'));
    }
    public function send_clearance_invoice($lang, $id, Request $request){

        $this->checkPermission('03-037','view');
        $data['master']      = ClearanceInvoice::get_invoice($id, $lang);
        $data['logo']        = Branch::get_branch($data['master']->branch_id  , $lang)->branch_image;

        $customer_contacts         = $request['customer_contacts'] ?? '';
        if(!is_array($customer_contacts)){
            $customer_contacts = [$customer_contacts];
        }

        $emails = DB::table('cc_mails')
            ->whereRaw('FIND_IN_SET("claim", screens)')
            ->get()
            ->toArray();


        $request->merge([
            'with_attachments'      => intval($request['is_attachment']) ,
            'clearance_attachments'  => intval($request['clearance_attachments']) ?? 0,
            'type'              => 'save_pdf_file',
            'print_temp'        => Setting::get_main_setting_by_code('clearance_invoice_mail_print')->value ?? 'temp_1' ,
            'clientID'          => request()->header('clientID')
            ]);
        $invoice    = $this->print($id,$request);
        $email_temp = 'Print.clearance.invoice.'.(Setting::get_main_setting_by_code('clearance_body_mail_print')->value ?? 'temp_1');

        $subject    = "INV-".$data['master']->id." BL-".$data['master']->shipping_policy_no." BAYAN-".$data['master']->customs_declaration_number." FILE-".$data['master']->clearance_id;



        $response   = Mail::send($email_temp ,[ 'data' => $data], function ($message) use ($subject ,$emails,$customer_contacts,$invoice,$id) {

            foreach ($customer_contacts as $email) {
                if( !filter_var($email, FILTER_VALIDATE_EMAIL) ){
                    continue;
                }
                $message->to($email);
            }

            foreach ($emails as $item) {
                if( !filter_var($item->email, FILTER_VALIDATE_EMAIL) ){
                    continue;
                }
                switch ($item->mail_type) {
                    case 'to':
                        $message->to($item->email);
                    break;
                    case 'cc':
                        $message->cc($item->email);
                    break;
                    case 'bcc':
                        $message->bcc($item->email);
                    break;
                }
            }

            // $message->cc(env("MAIL_CC_ADDRESS"));
            $message->subject($subject);
            $message->attach(public_path($invoice));

        });

        // insert log
        DB::table('invoices_sent_logs')->insert([
            'invoice_id'  => $id,
            'add_user'    => auth()->user()->id,
            'type'        => 'email',
            'data'        => json_encode($request->all()),
            'response'    => isset($response) ? $response : null
        ]);
        return $this->sendResponse( $response ?? [] , trans('main.send mail success') );

    }
    public function upload_attachment($lang , $id , Request $request){
        $this->checkPermission('03-037','edit');
        $item = $request->all();
        $file_name = $item['file_name'] ?? date('Y-m-d H:i:s');

        $check  = (new Upload)->uploadFile($item['file'] , 'invoice_attachments' ,$file_name );
        if(!$check['check'])
            return $this->sendError( trans('error.add_fails') , $check['msg'] );

        $insert = [
            'invoice_id'        => $id ,
            'invoice_type'      =>'claim' ,
            'file_name'         => $file_name ,
            'file_path'         => $check['url'] ,
            'add_user'          => auth()->user()->id ,
        ];
        $rules = [
            'file_name' => 'required|string|unique:invoice_attachments'
        ];
        $validator  = validator()->make( $insert , $rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $this->return['id']  = DB::table('invoice_attachments')->insertGetId($insert);

        return $this->sendResponse( $this->return );
    }

}
