<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Classes\Excel\InvoicesExport;
use App\Classes\General\General;
use App\Exports\ClassificationsExport;
use App\Exports\ClassificationsGeneralExport;
use App\Http\Controllers\API\V1\BaseController;
use App\Http\Resources\Sales\ClassificationCollection;
use App\Imports\ClassificationsGeneralImport;
use App\Imports\UnitImport;
use Illuminate\Http\Request;
use App\Classes\General\Upload;

use App\Models\Finance\AccountTree;
use App\Models\Main\ScreenSub;


use App\Models\Sales\Classification;
use App\Models\Sales\Clearance;
use App\Models\Sales\CustomsItem;
use App\Models\Finance\Currency;
use App\Models\Sales\Unit;
use App\Models\Sales\Country;
use App\Models\Sales\Customer;
use App\Models\Trans\Port;
use App\Models\Main\Delegate;
use App\Models\Trans\Area;
use App\Models\Main\Branch;



use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Response;
use Maatwebsite\Excel\Facades\Excel;

class ClassificationController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('04-007','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-007' , $lang)->sub_title ?? '';
        $this->return['items']          = Classification::all_classifications( $this->filter );
        $this->return['status']         = DB::table('classification_status')->select('id as id' , 'name_'.$lang.' as name')->get();


        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('04-007','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-007' , $lang)->sub_title ?? '';
        $this->return['units']          = Unit::all_units(array('lang' => $lang , 'active' => 1  ));
        $this->return['countrys']       = Country::all_countrys(array('lang' => $lang , 'active' => 1  ));
        $this->return['currencys']      = Currency::all_currencys(array('lang' => $lang , 'active' => 1  ));
        $this->return['status']         = DB::table('classification_status')->select('id as id' , 'name_'.$lang.' as name')->get();
        $this->return['types']          = ['CIF','C&F','C&I','FOB','XFACTORY','Other'];


        return $this->sendResponse( $this->return  );
    }

    public function get_clearance_data($lang  , $id){

        $this->return['master']         = Clearance::get_clearance($id , $lang);
        $this->return['details']        = Clearance::get_clearance_d($id , $lang);
        $this->return['files']          = Clearance::get_files($id , $lang);
        $this->return['comments']       = Clearance::get_clearance_comment($id , $lang);

        $this->return['master']->note  = implode(',', collect($this->return['comments'])->pluck('note')->toArray());
        return $this->sendResponse( $this->return  );

    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('04-007','add');

        $insert = $request->only(['action_date' , 'clearances_m_id', 'notes'  ]);

        $year   = array_search( date("Y" , strtotime($insert['action_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['action_date']));
        }
        $validator = validator()->make( $insert , Classification::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $insert['add_user'] = auth()->user()->id;

        DB::beginTransaction();
            DB::table('classifications_m')->lockForUpdate()->get();
            $clearance          = Clearance::get_clearance($insert['clearances_m_id']);
            if($clearance->classification_id > 0){
                return $this->sendError( trans('error.allready_has_classification'));

            }
            $this->return['id'] = Classification::add_classification($insert);

            $currency           = Currency::get_currency($clearance->currency_id , 'en');
            $details_seed       = [];
            foreach ($request['items'] as $key => $row) {
                $customs_item = DB::table('customs_items')
                    ->where('item_name_ar' , $row['item_name'] )
                    ->orWhere('item_name_en' , $row['item_name'] )
                    ->first();
                if (!$customs_item){
                    $customs_item_id = CustomsItem::add_customs_item([
                        'item_name_ar'  => $row['item_name'],
                        'item_name_en'  => $row['item_name'],
                        'item_type'     => $row['item_type'] ?? null
                    ]);
                    $customs_item = CustomsItem::get_customs_item($customs_item_id);
                }

                $row['containers'] = $row['containers'] ?? [];
                $details_seed[] = array(
                    'classifications_m_id'      =>  $this->return['id'] ,
                    'containers'                => ( is_array($row['containers'])? implode(',', $row['containers']) : $row['containers'] ),
                    'customs_item_id'           => $customs_item->customs_item_id   ,
                    'customs_item_description'  => $row['customs_item_description'] ?? '' ,
                    'qry'                       => $row['qry'] ?? null,
                    'unit_id'                   => $row['unit_id'] ?? null,
                    'weight'                    => floor($row['weight'] ?? 0),
                    'amount'                    => $row['amount'] ?? null,
                    'custom_fees'               => isset($row['amount'] , $currency->equivalent_value ) ? ROUND(( $row['amount'] * $currency->equivalent_value ) * ( $customs_item->item_type / 100 ) , 2): 0 ,
                    'country_id'                => $row['country_id'] ?? optional($clearance)->country_id,
                    'det_type'                  => $row['det_type'] ?? null,
                    'det_category'              => $row['det_category'] ?? null,
                    'item_name'                 => $row['item_name'] ?? null,
                    'customs_item_number'       => $customs_item->name_ar ?? null,
                    'item_type'                 => $row['item_type'] ?? null,
                    'class_slug'                => $row['slug'] ?? substr(base64_encode(random_bytes(10)), 0, 10)
                );
            }

            $diff_weight = round($clearance->weight - collect($details_seed)->sum('weight') , 3);
            $diff_amount = $clearance->invoice_amount - collect($details_seed)->sum('amount');
            $details_seed[count($details_seed)-1]['weight'] =  $details_seed[count($details_seed)-1]['weight'] + $diff_weight;

            Classification::add_classification_d($details_seed);
            Classification::update_classification(['total_custom_fees' => collect($details_seed)->sum('custom_fees')] ,  $this->return['id'] );
            Clearance::update_clearance(['classification_id' => $this->return['id'] ] , $insert['clearances_m_id']);
            //check if isset temp for this clearance and delete it
            $temp_id =  DB::table('classifications_temp')->where('clearances_m_id', $insert['clearances_m_id'])->value('id');
            if(isset($temp_id) && $temp_id > 0){
                DB::table('classifications_temp')->where('clearances_m_id', $insert['clearances_m_id'])->delete();

            }
            Clearance::update_clearance(['under_classification' => 0] , $insert['clearances_m_id']);
        DB::commit();

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('04-007','view');

        $this->return['screen_name']            = ScreenSub::get_screen_sub_by_screen_code('04-007' , $lang)->sub_title ?? '';
        $this->return['master']                 = Classification::get_classification($id , $lang);
        $this->return['details']                = Classification::get_classification_d($id , $lang);
        $this->return['clearances_det']         = Clearance::get_clearance_d($this->return['master']->clearances_m_id , $lang );
        $this->return['units']                  = Unit::all_units(array('lang' => $lang , 'active' => 1  ));
        $this->return['files']                  = Clearance::get_files($this->return['master']->clearances_m_id , $lang);
        $this->return['details_without_group']  = Classification::get_classification_d_without_group($id , $lang );
        $this->return['countrys']       = Country::all_countrys(array('lang' => $lang , 'active' => 1  ));
        $this->return['master']->invoice_id     = implode(',', DB::table('clearance_invoices_m')
            ->where('clearance_id' , $this->return['master']->clearances_m_id)
            ->pluck('invoice_id')->toArray() );

        foreach ($this->return['details'] as $key => $value) {
            if (!empty($this->return['details'][$key]->containers))
                $this->return['details'][$key]->containers = array_map('intval', explode(',', $this->return['details'][$key]->containers));
            else
                $this->return['details'][$key]->containers = [];
        }

        foreach ($this->return['details_without_group'] as $key => $value) {
            if (!empty($this->return['details_without_group'][$key]->containers))
                $this->return['details_without_group'][$key]->containers = array_map('intval', explode(',', $this->return['details_without_group'][$key]->containers));
            else
                $this->return['details_without_group'][$key]->containers = [];
        }


        return $this->sendResponse( $this->return , '' );
    }

    public function update_status($lang , $id , Request $request){
        $this->checkPermission('04-007','edit');
        Classification::update_classification(['classification_status_id' => $request['status_id']] ,  $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }

    public function upload_classification($lang , $id,Request $request){
        $this->checkPermission('04-007','edit');
        Classification::update_classification(['is_upload' => 1,'serial_number' =>$request->serial_number] ,  $id );
        return $this->sendResponse( $this->return , trans('main.update_success') );

    }
    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('04-007','edit');

        $update = $request->only(['action_date' ,  'notes' ]);
        if((new General)->check_financial_year($update['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($update['action_date']));
        }
        $year   = array_search( date("Y" , strtotime($update['action_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        // Classification::$rules['*.items.*.customs_item_number'] = 'required';
        // $validator = validator()->make( $update , Classification::$rules );
        // if($validator->fails())
        // {
        //     return $this->sendError( trans('error.add_fails') , $validator->errors() );
        // }

        DB::beginTransaction();
            $total_custom_fees      = 0;
            $clearance              = Clearance::get_clearance($request['clearances_m_id']);
            $currency               = Currency::get_currency($clearance->currency_id , 'en');
            $update['is_custom']    = 1;
            Classification::update_classification($update ,  $id );

            foreach ($request['items'] as $key => $row) {

                CustomsItem::update_customs_item(
                    [
                        'name_ar'       => $row['customs_item_number'] ,
                        'name_en'       => $row['customs_item_number'] ,
                        'item_type'     => $row['item_type']
                    ] ,
                    $row['customs_item_id']
                );

                DB::table('classifications_d')
                    ->where('classifications_m_id' , $id )
                    ->where('class_slug' , $row['slug'] )
                    ->update(
                         array(
                            'unit_id'               => $row['unit_id'] ,
                            'customs_item_number'   => $row['customs_item_number'] ?? null,
                            'item_type'             => $row['item_type'] ?? null
                        )
                    );

            }

        DB::commit();

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function update_translation_of_classification($lang,$id , Request $request)
    {
        $this->checkPermission('04-007','edit');

        $insert = $request->only(['action_date' , 'clearances_m_id', 'notes'  ]);

        $year   = array_search( date("Y" , strtotime($insert['action_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($insert['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['action_date']));
        }
        $validator = validator()->make( $insert , Classification::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        DB::beginTransaction();
        $clearance          = Clearance::get_clearance($insert['clearances_m_id']);
        $currency           = Currency::get_currency($clearance->currency_id , 'en');
        $details_seed       = $request['items'];


        foreach ($details_seed as $key => $value) {
            $details_seed[$key]['weight'] = floor($details_seed[$key]['weight']);
        }
        $diff_weight        = round($clearance->weight - collect($details_seed)->sum('weight') , 3);


        $diff_amount        = $clearance->invoice_amount - collect($details_seed)->sum('amount');
        $details_seed[count($details_seed)-1]['weight'] =  $details_seed[count($details_seed)-1]['weight'] + $diff_weight;
        // delete details not in array
        $classificationDetailsIds = collect($request->input('items'))->pluck('classifications_d_id')->toArray();


        DB::table('classifications_d')->whereNotIn('classifications_d_id',$classificationDetailsIds)->where('classifications_m_id',$id)->delete();
        foreach ($details_seed as $key => $row) {
            $old = DB::table('classifications_d')->where('classifications_d_id' , $row['classifications_d_id'] ?? 0 )->first();
            if(is_array($row['item_name'])){
                $row['item_name'] = $row['item_name']['item_name'];
                $row['item_type'] = $row['item_name']['item_type'] ?? null;
            }
            $customs_item = DB::table('customs_items')
                ->where('item_name_ar' , $row['item_name'] )
                ->orWhere('item_name_en' , $row['item_name'] )
                ->first();
            if (!$customs_item){
                $customs_item_id = CustomsItem::add_customs_item([
                        'item_name_ar'  => $row['item_name'] ,
                        'item_name_en'  => $row['item_name'] ,
                        'item_type'     => $row['item_type'] ?? null
                    ]);
                $customs_item = CustomsItem::get_customs_item($customs_item_id);
            }
            $row['containers'] = $row['containers'] ?? [];
            $details_seed =   array(
                'classifications_m_id'  =>  $id ,
                'class_slug'            => $row['slug'] ?? substr(base64_encode(random_bytes(10)), 0, 10),
                'item_name'             => $row['item_name'] ?? null,
                'qry'                   => $row['qry'] ?? null,
                'country_id'            => $row['country_id'] ?? optional($clearance)->country_id ,
                'amount'                => $row['amount'] ?? null,
                'containers'            => ( is_array($row['containers'])? implode(',', $row['containers']) : $row['containers'] ),
                'weight'                => $row['weight'] ,
                'customs_item_id'       => $customs_item->customs_item_id  ,
                'item_name'             => $customs_item->item_name_ar  ,
                'customs_item_number'   => (isset($old->customs_item_number) && strlen($old->customs_item_number) < 1 ? $customs_item->name_ar : $old->customs_item_number ?? null ) ,
                'item_type'             => (isset($old->customs_item_number) && strlen($old->item_type) < 1 ? $customs_item->item_type : $old->item_type ?? null ) ,


                'custom_fees'           => isset($row['amount'] ,   $currency->equivalent_value ) ? ROUND(( $row['amount'] * $currency->equivalent_value ) * ( $customs_item->item_type / 100 ) , 2): 0
            );
            if (isset($row['classifications_d_id']) && $row['classifications_d_id'] > 0 ){
                //update_details
                Classification::update_classification_d($details_seed,$row['classifications_d_id']);
            }else{
                //add_new_details
                Classification::add_classification_d($details_seed);
            }

        }

        $total_custom_fees = DB::table('classifications_d')->where('classifications_m_id',$id)->sum('custom_fees');
        Classification::update_classification(['total_custom_fees' => $total_custom_fees] ,  $id );
        DB::commit();

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('04-007','delete');

        DB::beginTransaction();
            $main = Classification::get_classification($id , $lang);
            if((new General)->check_financial_year($main->action_date) != "true"){
                return $this->sendError((new General)->check_financial_year($main->action_date));
            }
            Classification::delete_classification($id);
            Clearance::update_clearance(['classification_id' => 0 ] , $main->clearances_m_id);
        DB::commit();

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    // public function toggle_active($lang , $id){
    //     $item = Classification::find($id);
    //     if($item->classification_active){

    //         if(!@in_array('4', $this->data['mypermissions']['04-007'][0]->permission ?? [] ) ){
    //             return $this->sendError( 'update permission needed #04-007' , [] , 200 );
    //         }

    //         Classification::delete_classification($id);
    //         Clearance::update_clearance(['classification_id' => 0 ] , $id);

    //         $this->return['active'] = false;

    //     }else{

    //         if(!@in_array('3', $this->data['mypermissions']['04-007'][0]->permission ?? [] ) ){
    //             return $this->sendError( 'update permission needed #04-007' , [] , 200 );
    //         }

    //         Classification::update_classification(array('classification_active' => 1 ) , $id);
    //         $this->return['active'] = true;
    //     }
    //     return $this->sendResponse( $this->return , trans('main.update_success') );
    // }


     public function update_customs($lang , $id , Request $request){
         $this->checkPermission('04-007','edit');

         Classification::update_classification(['customs_declaration_date' => $request['customs_declaration_date'] , 'customs_declaration_number' => $request['customs_declaration_number']] ,  $id );

        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function add_comment($lang , $id , Request $request){
        $this->checkPermission('04-007','edit');



        DB::table('classification_comments')
            ->insert(
                array(
                    'classification_id' => $id ,
                    'comment_det'       => $request['comment'] ,
                    'add_date'          => date("Y-m-d H:i:s") ,
                    'add_user'          => auth()->guard('api')->user()->id
                )
            );


        return $this->sendResponse( [] , trans('main.add_success') );

    }


    public function get_comments($lang , $id , Request $request){
        $this->checkPermission('04-007','view');

        $this->return['items'] = Classification::get_comments($id , $lang);

        return $this->sendResponse(  $this->return , trans('main.add_success') );

    }

    public function export_classification($lang , $id , Request $request){

        $this->checkPermission('04-007','print');
        $lang = 'ar';
        $this->return['lang']           = $lang;
        $this->return['master']         = Classification::get_classification($id , $lang);
        $this->return['details']        = Classification::get_classification_d($id , $lang);
        $this->return['branch_data']    = Branch::get_branch(1);



        $this->return['print_title']    = '';
        $this->return['print_right']    = $this->return['branch_data']->print_label_ar ?? '';
        $this->return['print_left']     = $this->return['branch_data']->print_label_en ?? '';
        $this->return['logo']           = get_logo( 1 );


        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $this->return['countries']     = Country::all_countrys();
                $this->return['item_units']     = Unit::all_units(array('lang' => $lang , 'active' => 1 ));
                $this->return['duty_types']     = [(object)['duty_type_code' => 1,'arabic_name' =>'قطعي','english_name' => 'Cash'],(object)['duty_type_code' => 2,'arabic_name' =>'مؤمن','english_name' => 'Insurance'],(object)['duty_type_code' => 3,'arabic_name' =>'مؤمن سندي','english_name' => 'Government Insurance'],(object)['duty_type_code' => 4,'arabic_name' =>'معفي','english_name' => 'Exemption']];
                $this->return['container_sizes']     = [(object)['container_size' => 20],(object)['container_size' => 40],(object)['container_size' => 45]];
                $this->return['date_types']     = [(object)['date_type_id' => 1,'date_type' => 'هجري / Hijri','date_type_format' => '01-01-1440'],(object)['date_type_id' => 2,'date_type' => 'ميلادي /  Gregorian','date_type_format' => '01-01-2021']];


                // return ( string_len('محمود سعيد' , 40,300,'ـ') );
                return Excel::download(new ClassificationsGeneralExport($this->return), ('classification_'.now()->getTimestamp().'.xlsx'));
            break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Print/classification/print', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
            break;
            case 'image':
                $this->return['logo']     = public_path().$this->return['logo'];
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Print/classification/print', array('data' => $this->data));
                return $image->stream();
            break;
            case 'print':
                $this->return['logo']     = url($this->return['logo']);
                return view('Print.classification.print')->with('data' , $this->return);
            break;
            case 'json':

                // $this->return['export_data']        =    ClassificationCollection::make(Classification::get_classification_d($id , $lang));

                $this->return['export_data'] = [];

                foreach ($this->return['details'] as $key => $row) {
                    $this->return['export_data'][] =  [
                        "invoiceSeq"                => 1,                                                          // - 01
                        "countryOriginCode"         => $row->country_id,                                           // - 02
                        "trafficCode"               => string_len($row->customs_item_number , 1 , 16 ,  "_") ,     // - 03
                        "goodsDescription"          => string_len($row->item_name , 50 , 300 , "  البضاعه الوارده  ") ,      // - 04
                        "grossWeight"               => $row->weight,                                               // - 05
                        "netWeight"                 => $row->weight,                                               // - 05
                        "noOfUnit"                  => intval($row->qry),                                                  // - 06
                        "itemValue"                 => $row->amount,                                               // - 07
                        "dutyTypeCode"              => 1,                                                          // - 08
                        "dateTypeCode"              => null ,                                                          // - 09
                        "productionDate"            => null ,                                                         // - 10
                        "expiryDate"                => null ,                                                         // - 11
                        "containerSize"             => $row->container_size ,                                      // - 12
                        "buyingcountry"             => null ,                                                         // - 13
                        "routeDescription"          => null ,                                                         // - 14
                        "salePriceForCustomer"      => null ,                                                         // - 15
                        "batchNumber"               => null ,                                                         // - 16
                        "preNotificationNumber"     => null ,                                                         // - 17
                        "itemQuantity"              => $row->qry,                                                  // - 18
                        "itemUnit"                  => $row->unit_id,                                              // - 19
                        "internationalQuantity"     => $row->qry,                                                  // - 20
                        "trafficQuantity"           => null ,                                                         // - 21
                        "trafficUnit"               => null ,                                                         // - 22
                        "minimumDutyQuantity"       => null ,                                                         // - 23
                        "minimumDutyUnit"           => null ,                                                         // - 24
                        "banderoleQuantity"         => null ,                                                         // - 25
                        "additionalFeeUnit"         => null                                                          // - 26
                    ];
                }

                return $this->sendResponse($this->return);
            break;
        }

        return $this->sendResponse($this->return);
    }

//    private function generateExcelFile($data, $filePath)
//    {
//        // Open a file pointer
//        $file = fopen($filePath, 'w');
//
//        // Write the Excel header
//        fputcsv($file, [
//            'الرقم التسلسلي للفاتورة / Invoice Seq', // - 01
//            'رمز بلد المنشأ / Country of Origin Code',  // - 02
//            'رمز التعرفة / Tariff Code', // - 03
//            'وصف البضائع / Goods Description', // - 04
//            'الوزن الإجمالي / Gross Weight', // - 05
//            'الوزن الصافي / Net Weight', // - 05
//            'عدد الوحدات / No of Unit',// - 06
//            'قيمة الصنف / Item Value',// - 07
//            'رمز نوع الرسوم / Duty Type Code',// - 08
//            'رمز نوع التار يخ / Date Type Code',// - 09
//            'تاريخ الإنتاج / Production Date',// - 10
//            'تاريخ الإنتهاء / Expiry Date',// - 11
//            'حجم الحاوية / Container Size',// - 12
//            'بلد الشراء / Buying Country',// - 13
//            'وصف المسار / Route Description',// - 14
//            'سعر البيع للمستهلك النهائي / Sale Price For Consumer',// - 15
//            'رقم الصنف(في نظام هيئة الغذاء و الدواء) / Batch Number',// - 16
//            'رقم الطلب(في نظام هيئة الغذاء والدواء) / Pre-Notification Number',// - 17
//            'كمية الصنف / Item Quanitity',// - 18
//            'وحدة الصنف / Item Unit',// - 19
//            'الكمية الدولية / International Quantity',// - 20
//            'كمية التعرفة / Tariff Quantity',// - 21
//            'وحدة التعرفة / Tariff Unit',// - 22
//            'كمية الاحد الأدنى للرسوم / Minimum Duty Quantity',// - 23
//            'وحدة الحد الأدنى للرسوم / Minimum Duty Unit',// - 24
//            'كمية البندرول / Banderole Quantity',// - 25
//            'وحدة الرسوم الإضافية / Additional Fee Unit',// - 26
//        ]);
//
//        foreach ($data['details'] as $key => $value) {
//                   $data_row =  [
//            1,//                $value->classifications_d_id,// - 01
//            $value->country_id,// - 02
//            $value->customs_item_name,// - 03
//            $value->customs_item_number,// - 04
//            $value->weight, // ??// - 05
//            $value->weight, // ??// - 05
//            $value->qry,// - 06
//            $value->amount, // ??   // - 07
//            1,//                $value->det_type, // ??// - 08
//            '',//                $value->date_type, // ??// - 09
//            '',//                $value->production_date,  // - 10
//            '',//                $value->expiry_date, // ??  - 11
//            '',//                $value->container_size, // ??// - 12
//            '',//                $value->buying_country, // ??// - 13
//            '',//                $value->route_description, // ?? - 14
//            '',//                $value->sale_price_for_consumer, //  - 15
//            '',//                $value->Batch Number, // ??   - 16
//            '',//                $value->Pre-Notification Number, // ?? - 17
//            $value->qry, // ??  - 18
//            $value->unit_id, //  - 19
//            $value->qry, // ??  - 20
//            '',//                $value->tariff_quantity, // ??  - 21
//            '',//                $value->Tariff Unit, // ?? 22
//            '',//                $value->minimum_duty_quantity, // ?? 23
//            '',//                $value->Minimum Duty Unit, // ?? 24
//            '',//                $value->Banderole Quantity, // ?? - 25
//            ''//                $value->Additional Fee Unit, // ?? - 26
//        ];
//            fputcsv($file, (array)$data_row);
//        }
//
//        // Close the file pointer
//        fclose($file);
//    }
    public function import_classification($lang, $id ,Request $request){
        if ($request->hasFile('excel_file')){
            Excel::import(new ClassificationsGeneralImport, $request->excel_file);

        }

    }


    public function classification_search( $lang , Request $request ){
        // $this->checkPermission('04-024','view');

        // $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('02-024' , $lang)->sub_title ?? '';


        $query = DB::table('classifications_d')
            ->join('classifications_m' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
            ->join('customs_items' , 'classifications_d.customs_item_id' , '=' , 'customs_items.customs_item_id')
            ->selectRaw('
                classifications_m.classification_id as m_id ,
                classifications_m.clearances_m_id as clearance_id ,
                classifications_m.notes ,
                classifications_m.action_date ,
                classifications_d.item_name  ,
                classifications_d.customs_item_number ,
                classifications_d.item_type ,
                classifications_d.weight ,
                classifications_d.amount ,
                classifications_d.qry
            ')
            ->where('classification_active' , 1)
            ->where(function($where) use($request){
                $where->where('classifications_d.customs_item_description' , 'like' , "%{$request['word']}%");
                $where->orWhere('classifications_d.item_name' , 'like' , "%{$request['word']}%");
                $where->orWhere('classifications_d.customs_item_number' , 'like' , "%{$request['word']}%");
                $where->orWhere('classifications_m.clearances_m_id' , 'like' , "%{$request['word']}%");
                $where->orWhere('classifications_m.notes' ,  'like' , "%{$request['word']}%");
            });

        if(isset($request['rows'])){
            $this->return['items'] = $query->paginate($request['rows'], ['*'], 'page', $request['page'] ?? 1);
        }else{
            $this->return['items'] = $query->get();
        }



        return $this->sendResponse( $this->return );

    }
    public function lock_clearance($lang , $id){
        $clearance  = Clearance::get_clearance($id,$lang);
        if($clearance->under_classification == 1){
            Clearance::update_clearance(['under_classification' => 0] , $id);
            $this->return['locked'] = false;
        }elseif($clearance->under_classification == 0){
            Clearance::update_clearance(['under_classification' => 1 , 'locked_at' => now()] , $id);
            $this->return['locked'] = true;
        }
        return $this->sendResponse($this->return);
    }

    public function temp_classifications($lang , Request $request){
        $this->checkPermission('03-056','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-007' , $lang)->sub_title ?? '';
        $this->return['items']          = Classification::get_temp_classification($this->filter);
        return $this->sendResponse($this->return);
    }
    public function add_temp_classification($lang , Request $request){
        $this->checkPermission('03-056','add');

        $insert = [
            'clearances_m_id'   => $request->clearances_m_id ,
            'add_user'          => auth()->user()->id ,
        ];
        DB::beginTransaction();
        //check if is exist
        $old_temp = DB::table('classifications_temp')->where('clearances_m_id', $insert['clearances_m_id'])->value('id');
        if(isset($old_temp) && $old_temp > 0){
            $this->return['locked'] = true;

        }else{
            $this->return['id'] = DB::table('classifications_temp')->insertGetID($insert);
            $clearances_m_id    = $insert['clearances_m_id'];
            $clearance          = Clearance::get_clearance($clearances_m_id,$lang);
            if($clearance->under_classification == 0){
                Clearance::update_clearance(['under_classification' => 1] , $clearances_m_id);
                $this->return['locked'] = true;
            }
        }

        DB::commit();
        return $this->sendResponse($this->return);
    }
    public function delete_temp_classification($lang , $id){
        $this->checkPermission('03-056','delete');
        $item    = DB::table('classifications_temp')->where('id', $id)->first();
        //check if same user
        // if(auth()->user()->id !== $item->add_user){
        //     return $this->sendError(trans('error.must_be_added_user'));
        // }
        $clearance          = Clearance::get_clearance($item->clearances_m_id,$lang);
        // if($clearance->under_classification == 1){
        DB::beginTransaction();
            Clearance::update_clearance(['under_classification' => 0] , $item->clearances_m_id);
            DB::table('classifications_temp')->where('id', $id)->delete();
        DB::commit();
            $this->return['locked'] = false;
        // }
        return $this->sendResponse($this->return , trans('main.delete_success'));
    }

}