<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\Currency;
use App\Models\Sales\Country;
use App\Models\Sales\PolicyType;
use App\Models\Trans\Agent;
use Illuminate\Http\Request;
use App\Classes\General\Upload;
use App\Classes\LogsHelper;
use App\Classes\Sales\ClearanceHelper;
use App\Models\Finance\AccountTree;
use App\Models\Finance\DelegatePetty;
use App\Models\Logs;
use App\Models\Main\ScreenSub;


use App\Models\Sales\Clearance;
use App\Models\Sales\Customer;
use App\Models\Trans\Port;
use App\Models\Main\Delegate;
use App\Models\Sales\ClearanceInvoice;
use App\Models\Trans\Area;


use Illuminate\Support\Facades\DB;
use HanifHefaz\Dcter\Dcter;
use Illuminate\Support\Carbon;
use Illuminate\Support\Facades\File;

class ClearanceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        // $this->checkPermission('04-004','view');

        $this->filter = array(
            'lang'                  => $lang ,
            'active'                => $request['active'] ?? 1 ,
            'rows'                  => $request['rows'] ?? null ,
            'page'                  => $request['page'] ?? 1 ,
            'status'                => $request['status'] ?? null ,
            'customer_id'           => $request['customer_id'] ?? null ,
            'port_id'               => $request['port_id'] ?? null ,
            'from'                  => $request['from_date'] ?? null ,
            'to'                    => $request['to_date'] ?? null ,
            'word'                  => $request['word'] ?? null ,
            'filter_by'             => $request['filter_by'] ?? null ,
            'filter_value'          => $request['filter_value'] ?? null ,
            'sort_by'               => $request['sort_by'] ?? null ,
            'sort_value'            => $request['sort_value'] ?? null ,
            'container_number'      => $request['container_number'] ?? null ,
            'port_withdraw'         => $request['port_withdraw'] ?? null ,
            'withdrawal_note'       => $request['withdrawal_note'] ?? null ,
            'has_withdrawal_note'   => $request['has_withdrawal_note'] ?? null ,

            'clearance_id'          => $request['clearance_id'] ?? null ,
            'shipping_policy_no'    => $request['shipping_policy_no'] ?? null ,
            'has_problem'           => $request['has_problem'] ?? null ,
            'is_classification'     => $request['is_classification'] ?? null,
            'customer_main_id'      => $request['customer_main_id'] ?? null,
            'agent_id'              => $request['agent_id'] ?? null,
            'delegate_id'           => $request['delegate_id'] ?? null,
            'has_invoice'           => $request['has_invoice'] ?? null,

        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-004' , $lang)->sub_title ?? '';
        $this->return['items']          = Clearance::all_clearances( $this->filter );
        $this->return['status']         = DB::table('clearance_status')
            ->leftjoin('clearances_m' , 'clearances_m.clearance_status_id' , '=' , 'clearance_status.clearance_status_id')
            ->selectRaw('clearance_status.clearance_status_id as id , clearance_status.name_'.$lang.' as name , count(clearances_m.clearance_id) as counter')
            ->groupBy('clearance_status.clearance_status_id')
            ->get();
        $this->return['file_names']     = DB::table('clearance_file_names')->where('clearance_file_names.attachment_name_active',1)->select('id as id' , 'name_'.$lang.' as name')->get();
        $this->return['ports']          = Port::all_ports(array('lang' => $lang , 'active' => 1  ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang , 'active' => 1 ));


        // $all = DB::table('clearances_m')->get();
        // foreach ($all as $key => $value) {
        //     $count = DB::table('clearance_invoices_m')
        //         ->where('clearance_id' , $value->clearance_id)
        //         ->count();
        //     DB::table('clearances_m')
        //         ->where('clearance_id' , $value->clearance_id)
        //         ->update(['invoices_count' => $count]);
        // }

        return $this->sendResponse( $this->return );
    }

    public function get_clearance_attachment($lang,$id){
        $this->return = DB::table('clearance_file_names')
            ->leftJoin('clearance_attachments', function ($join) use ($id) {
                $join->on('clearance_file_names.id', '=', 'clearance_attachments.clearance_file_name_id')
                    ->where('clearance_attachments.clearance_id', $id);
            })
            ->where('clearance_file_names.attachment_name_active',1)
            ->selectRaw('clearance_file_names.id, clearance_file_names.name_'.$lang.' as name, clearance_attachments.clearance_attachment_id, clearance_attachments.file_path')
            ->get();
        return $this->sendResponse( $this->return );
    }
    public function full_index($lang , Request $request)
    {
        $this->checkPermission('04-004','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'status'      => 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-004' , $lang)->sub_title ?? '';
        $this->return['items']          = Clearance::all_clearances( $this->filter );
        $this->return['status']         = DB::table('clearance_status')
            ->leftjoin('clearances_m' , 'clearances_m.clearance_status_id' , '=' , 'clearance_status.clearance_status_id')
            ->selectRaw('clearance_status.clearance_status_id as id , clearance_status.name_'.$lang.' as name , count(clearances_m.clearance_id) as counter')
            ->groupBy('clearance_status.clearance_status_id')
            ->get();

        return $this->sendResponse( $this->return );
    }

    public function check_clearance_container($lang , Request $request){

        $this->return = DB::table('clearances_d')
            ->leftjoin('clearances_m' , 'clearances_d.clearances_m_id' , '=' , 'clearances_m.clearance_id')
            ->where('clearances_d.container_number' , $request['code']  )
            ->orderBy('clearances_m.clearance_id' , 'desc')
            ->selectRaw('clearances_m.action_date , clearances_m.clearance_id as id')
            ->get();

        return $this->sendResponse( $this->return ?? [] );
    }

    public function check_clearance_shipping_policy_no($lang , Request $request){

        $this->return = DB::table('clearances_m')
            ->where('clearances_m.shipping_policy_no' , $request['code']  )
            ->orderBy('clearances_m.clearance_id' , 'desc')
            ->selectRaw('clearances_m.action_date , clearances_m.clearance_id as id')
            ->get();

        return $this->sendResponse( $this->return ?? [] );
    }



    public function check_clearance_saber_no($lang , Request $request){

        $this->return = DB::table('clearances_m')
            ->where('clearances_m.saber_no' , $request['code']  )
            ->orderBy('clearances_m.clearance_id' , 'desc')
            ->selectRaw('clearances_m.action_date , clearances_m.clearance_id as id')
            ->get();

        return $this->sendResponse( $this->return ?? [] );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('04-004','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-004' , $lang)->sub_title ?? '';
        $this->return['ports']          = Port::all_ports(array('lang' => $lang , 'active' => 1 ));
        $this->return['delegates']      = Delegate::all_delegates(array('lang' => $lang , 'active' => 1 ));
        $this->return['currencys']      = Currency::all_currencys(array('lang' => $lang , 'active' => 1  ));
        $this->return['types']          = ['CIF','C&F','C&I','FOB','XFACTORY','Other','XFACTORY'];
        $this->return['status']         = DB::table('clearance_status')->select('clearance_status_id as id' , 'name_'.$lang.' as name')->get();
        $this->return['file_names']     = DB::table('clearance_file_names')->select('id as id' , 'name_'.$lang.' as name')->get();
        $this->return['countrys']       = Country::all_countrys(array('lang' => $lang , 'active' => 1  ));
        $this->return['policies_types'] = PolicyType::all_policies_types(array('lang' => $lang   ));
        $this->return['agents']         = Agent::all_agents( ['active' => 1 , 'lang' => $lang] );
        $this->return['citys']          = DB::table('areas')->where('area_active' , 1 )->selectRaw('area_id as city_id , area_name_'.$lang.' as name')->get();




        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('04-004','add');

        $insert = $request->only(['customer_id', 'sub_customer_id', 'has_problem' ,'delegate_id', 'port_id', 'classification_id', 'action_date',
                                  'clearance_type', 'container_count', 'is_lcl', 'lcl_count' , 'old_sys_id'  , 'shipping_policy_no' ,'file_no' ,
                'goods_type','invoice_type' , 'supplier_invoice_code' , 'invoice_amount' , 'currency_id' , 'shipping_amount' , 'shipping_currency_id',
            'arrival_date','agent_number'  , 'saber_no' ,'note', 'exporting_company' , 'weight' , 'invoice_date' , 'country_id' , 'invoice_2' ,
             'invoice_3' , 'invoice_4' , 'policy_type_id' ,'majira_policy', 'pledge_number','pledge_date' ,'unloading_ship_date' , 'unloading_ship_name' , 'tracking_number']);

        $insert['container_count']  = count($request['items'] ?? []);
        $insert['first_container']  = $request['items'][0]['container_number'] ?? null;

        $year   = array_search( date("Y" , strtotime($insert['action_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if($insert['container_count'] < 1 && $insert['is_lcl'] < 1)
        {
            return $this->sendError( trans('error.container_number_cannot_be_zero') );
        }
        $validator = validator()->make( $insert , Clearance::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $insert['weight_factor']    = (isset($request['weight'],$request['invoice_amount']) && $request['weight'] != 0  && $request['invoice_amount'] != 0) ? ($request['weight']/$request['invoice_amount']) :  0;
        $insert['add_user']         = auth()->user()->id;
        $insert['finance_year_id']  = $year;
        $insert['branch_id']        = $this->data['current_branch'];
        $insert['container_small']  = collect( $request['items'] ?? [] )->where('container_size' , 20)->count();
        $insert['container_big']    = collect( $request['items'] ?? [] )->where('container_size' , 40)->count();
        $insert['unloading_ship_date']     = $insert['unloading_ship_date'] ?? null;

        DB::beginTransaction();

            DB::table('clearances_m')->lockForUpdate()->get();
            $insert['clearance_id'] = intval( DB::table('clearances_m')->max("clearance_id") ) + 1;
            $this->return['id']     = Clearance::add_clearance($insert);

            if($insert['is_lcl'] == 0 ){
                $details_seed = [];
                foreach ($request['items'] as $key => $row) {
                    $details_seed[] = array(
                        'clearances_m_id'   =>  $this->return['id'] ,
                        'container_size'    => $row['container_size'] ,
                        'container_number'  => $row['container_number'] ,
                        'seal_number'       => $row['seal_number'] ?? null ,
                        'dest_id'           => $row['dest_id'] ?? null,

                        'recommended_return_date'  => $row['recommended_return_date'] ,
                        'from_area'         => 0 ,
                        'to_area'           => 0 ,
                        'add_user'          => $insert['add_user']
                    );
                }

                Clearance::add_clearance_d($details_seed);
            }
        //(new LogsHelper)->insert_log(['payload' => $insert  , 'screen_code'=>'04-004' ,'function'=> 'store' , 'action_type'=>'clearance' , 'action_id'=>$this->return['id']]);
        DB::commit();

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('04-004','view');

        $this->return['screen_name']            = ScreenSub::get_screen_sub_by_screen_code('04-004' , $lang)->sub_title ?? '';
        $this->return['master']                 = Clearance::get_clearance($id , $lang);
        $this->return['details']                = Clearance::get_clearance_d($id , $lang);
        $this->return['comments']               = Clearance::get_clearance_comment($id , $lang);
        $this->return['ports']                  = Port::all_ports(array('lang' => $lang , 'active' => 1 ));
        $this->return['delegates']              = Delegate::all_delegates(array('lang' => $lang , 'active' => 1 ));
        $this->return['currencys']              = Currency::all_currencys(array('lang' => $lang , 'active' => 1  ));
        $this->return['types']                  = ['CIF','C&F','C&I','FOB','XFACTORY','Other','XFACTORY'];
        $this->return['countrys']               = Country::all_countrys(array('lang' => $lang , 'active' => 1  ));
        $this->return['policies_types']         = PolicyType::all_policies_types(array('lang' => $lang ));
        $this->return['agents']                 = Agent::all_agents( ['active' => 1 , 'lang' => $lang] );
        $this->return['status']                 = DB::table('clearance_status')->select('clearance_status_id as id' , 'name_'.$lang.' as name')->get();
        $this->return['file_names']             = DB::table('clearance_file_names')->where('attachment_name_active',1)->select('id as id' , 'name_'.$lang.' as name')->get();


        $this->return['delegate_petty_attachment']  = Clearance::delegate_petty_attachment($lang , $id);
        $this->return['multi_voucher_attachment']   = Clearance::multi_voucher_attachment($lang , $id);
        $this->return['files']                      = Clearance::get_files($id , $lang);


        // $this->return['files'] = $this->return['clearance_files']->merge($this->return['delegate_petty_attachment'])->merge($this->return['multi_voucher_attachment']);

        $this->return['citys']                  = DB::table('areas')->where('area_active' , 1 )->selectRaw('area_id as city_id , area_name_'.$lang.' as name')->get();

        $delegate_petty    = DB::table('delegate_petty_d')
            ->leftjoin('expense_codes' , 'delegate_petty_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('users' , 'delegate_petty_d.add_user' , '=' , 'users.id')
            ->leftjoin('users as attached' , 'delegate_petty_d.attached_user' , '=' , 'attached.id')

            ->where('delegate_petty_d.clearance_id' , $id )
            ->selectRaw('
                delegate_petty_m_id as m_id ,
                "delegate_petty" as from_type ,
                delegate_petty_d_id as d_id ,
                clearance_id ,
                users.user_full_name as added_by,
                attached.user_full_name as attached_by,
                delegate_petty_d.file_path as attachment,
                expense_codes.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                receipt_number ,
                ifnull( sum(amount) , 0 ) as amount ,
                description_'.$lang.' as description ,
                delegate_petty_d.add_date ,
                invoice_id
            ')->groupBy('delegate_petty_d.delegate_petty_d_id')->get();

            $multi_voucher = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            ->leftjoin('expense_codes' , 'multi_voucher_d.dis_expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->leftjoin('users' , 'multi_voucher_d.add_user' , '=' , 'users.id')
            ->leftjoin('users as attached' , 'multi_voucher_d.attached_user' , '=' , 'attached.id')
            // ->where('multi_voucher_d.invoice_id' , 0 )
            ->where('multi_voucher_m.multi_voucher_active' , 1 )
            ->where('multi_voucher_d.multi_voucher_d_active' , 1 )
            ->where('multi_voucher_d.clearance_id' , $id )
            ->where('voucher_type' , 1 )
            ->selectRaw('
                "multi_voucher" as from_type ,
                multi_voucher_d.multi_voucher_m_id as m_id ,
                multi_voucher_d.multi_voucher_d_id as d_id ,
                multi_voucher_d.clearance_id ,
                multi_voucher_d.attachment,
                users.user_full_name as added_by,
                attached.user_full_name as attached_by,
                multi_voucher_d.dis_expense_code_id as expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                multi_voucher_d.receipt_number ,
                multi_voucher_d.amount_without_vat as amount ,
                multi_voucher_d.description as description ,
                multi_voucher_d.add_date ,
                multi_voucher_d.invoice_id
            ')->get();
        $this->return['reimburses_expenses'] = $delegate_petty->merge($multi_voucher);
        $check_invoice  = DB::table('clearance_invoices_m')->where('clearance_id',$id)->count();
        $check_draft    = DB::table('clearance_invoices_draft_m')->where('clearance_id',$id)->count();
        $check_claim    = DB::table('clearance_invoices_claim_m')->where('clearance_id',$id)->count();

        if($check_invoice > 0 || $check_draft > 0 || $check_claim > 0){
            $this->return['clearance_has_invoice'] = true;
        }else{
            $this->return['clearance_has_invoice'] = false;
        }


        return $this->sendResponse( $this->return , '' );
    }

    public function clearance_get_comment($lang , $id ){

        $this->return['items']       = Clearance::get_clearance_comment($id , $lang);
        return $this->sendResponse( $this->return , '' );

    }

    public function update_status($lang , $id , Request $request){
//        $this->checkPermission('04-004','edit');
        Clearance::update_clearance(['clearance_status_id' => $request['status_id']] ,  $id );
        //(new LogsHelper)->insert_log(['payload' => $request  , 'screen_code'=>'04-004' ,'function'=> 'update_status' , 'action_type'=>'clearance' , 'action_id'=>$id]);

        return $this->sendResponse( $this->return , trans('main.update_success') );

    }
    public function update($lang , $id , Request $request)
    {
//        $this->checkPermission('04-004','edit');

        $update = $request->only(['customer_id', 'sub_customer_id','has_problem' ,'delegate_id', 'port_id', 'classification_id', 'action_date',
                                  'clearance_type', 'container_count', 'is_lcl', 'lcl_count' , 'old_sys_id'  , 'shipping_policy_no' ,'file_no' ,'goods_type'
            ,'invoice_type' , 'supplier_invoice_code' , 'invoice_amount' , 'currency_id' , 'shipping_amount' , 'shipping_currency_id' ,"agent_number" ,
            "arrival_date" , 'saber_no' , 'note','exporting_company' , 'weight' , 'invoice_date' , 'country_id' , 'invoice_2' , 'invoice_3' ,
             'invoice_4' , 'policy_type_id' ,'majira_policy','pledge_number','pledge_date' , 'unloading_ship_date' , 'unloading_ship_name' , 'tracking_number']);

        $update['container_count']  = count($request['items'] ?? []);


        $check_invoice  = DB::table('clearance_invoices_m')->where('clearance_id',$id)->count();
        $check_draft    = DB::table('clearance_invoices_draft_m')->where('clearance_id',$id)->count();
        $check_claim    = DB::table('clearance_invoices_claim_m')->where('clearance_id',$id)->count();

        if($check_invoice > 0 || $check_draft > 0 || $check_claim > 0){
            $clearance = Clearance::get_clearance($id , $lang);
            $update['customer_id'] = $clearance->customer_id;
        }

        $year   = array_search( date("Y" , strtotime($update['action_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if($update['container_count'] < 1 && $update['is_lcl'] < 1)
        {
            return $this->sendError( trans('error.container_number_cannot_be_zero') );
        }
        $validator = validator()->make( $update , Clearance::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.update_fails') , $validator->errors() );
        }


        $update['weight_factor']    = (isset($request['weight'],$request['invoice_amount']) && $request['weight'] != 0  && $request['invoice_amount'] != 0) ? ($request['weight']/$request['invoice_amount']) :  0;

        DB::beginTransaction();

            $update['container_small']  = collect( $request['items'] ?? [] )->where('container_size' , 20)->count();
            $update['container_big']    = collect( $request['items'] ?? [] )->where('container_size' , 40)->count();
            $update['first_container']  = $request['items'][0]['container_number'] ?? null;
            $update['unloading_ship_date']     = $update['unloading_ship_date'] ?? null;

            Clearance::update_clearance($update ,  $id );

            if($update['is_lcl'] == 0 ){
                $deleted_details = DB::table('clearances_d')
                    ->where('clearances_m_id' , $id )
                    ->whereNotIn( 'clearances_d_id' , collect( $request['items'] )->pluck('clearances_d_id') )
                    ->pluck('clearances_d_id');
                $check_trips = DB::table('trips')
                    ->whereIn('clearance_d_id' , $deleted_details )
                    ->where('trip_active' , 1 )
                    ->count();
                $check_warehouse_transaction = DB::table('wearhouse_transactions')
                    ->whereIn('clearance_d_id' , $deleted_details )
                    ->where('wearhouse_transaction_active' , 1 )
                    ->count();
                if($check_trips > 0 || $check_warehouse_transaction > 0){
                    return $this->sendError( trans('error.this_container_has_trip_or_transaction') );
                }

                DB::table('clearances_d')
                    ->where('clearances_m_id' , $id )
                    ->whereNotIn( 'clearances_d_id' , collect( $request['items'] )->pluck('clearances_d_id') )
                    ->delete();
                foreach ($request['items'] as $key => $row) {

                    if(!isset($row['clearances_d_id'] ) || $row['clearances_d_id'] == 0 ){
                        DB::table('clearances_d')->insert(
                            array(
                                'clearances_m_id'   => $id ,
                                'container_size'    => $row['container_size'] ,
                                'container_number'  => $row['container_number'] ,
                                'seal_number'       => $row['seal_number'] ?? null ,
                                'dest_id'           => $row['dest_id'] ?? null,
                                'recommended_return_date'    => $row['recommended_return_date'] ,
                                'add_user'          => auth()->user()->id
                            )
                         );
                    }else{
                        $check_trips = DB::table('trips')
                        ->where('clearance_d_id' , $row['clearances_d_id'] )
                        ->where('trip_active' , 1 )
                        ->count();
                        $check_warehouse_transaction = DB::table('wearhouse_transactions')
                        ->where('clearance_d_id' , $row['clearances_d_id'] )
                        ->where('wearhouse_transaction_active' , 1 )
                        ->count();
                        if($check_trips > 0|| $check_warehouse_transaction > 0){
                            continue;

                            // return $this->sendError( trans('error.this_container_has_trip_or_transaction') );

                            return $this->sendError( trans('error.this_container_has_trip_or_transaction') );

                        }
                        DB::table('clearances_d')
                            ->where('clearances_m_id' , $id )
                            ->where('clearances_d_id' , $row['clearances_d_id'] )
                            ->update(['container_size'    => $row['container_size'] ,'dest_id' => $row['dest_id'], 'container_number'    => $row['container_number'],'seal_number'=> $row['seal_number'] ?? null ,'recommended_return_date' => $row['recommended_return_date']]);
                    }

                }

            }else{
                $old_details = DB::table('clearances_d')
                    ->where('clearances_m_id' , $id )
                    ->pluck('clearances_d_id');
                    $check_trips = DB::table('trips')
                    ->whereIn('clearance_d_id' , $old_details)
                    ->where('trip_active' , 1 )
                    ->count();
                    $check_warehouse_transaction = DB::table('wearhouse_transactions')
                    ->whereIn('clearance_d_id' , $old_details)
                    ->where('wearhouse_transaction_active' , 1 )
                    ->count();
                if($check_trips > 0|| $check_warehouse_transaction > 0){
                    return $this->sendError( trans('error.this_container_has_trip_or_transaction') );
                }
                 DB::table('clearances_d')
                ->where('clearances_m_id' , $id )
                ->delete();
            }
        //(new LogsHelper)->insert_log(['payload' => $update  , 'screen_code'=>'04-004' ,'function'=> 'update' , 'action_type'=>'clearance' , 'action_id'=>$id]);
        DB::commit();

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function update_clearance_item($lang , $id , Request $request){
        $update = $request->only(['customer_id','customs_declaration_file', 'has_problem' ,'delegate_id', 'port_id', 'action_date','clearance_type', 'is_lcl', 'lcl_count' , 'old_sys_id'  , 'shipping_policy_no' ,'file_no' ,'goods_type'  , 'arrival_date' , 'mainfest_date' , 'mainfest_date_hijri' , 'mainfest_no'  , 'customs_declaration_date' , 'customs_declaration_number' , 'customs_floor_date','unloading_date' ,'unloading_date_hijri', 'port_withdraw' , 'inspection_number' , 'withdrawal_note' , 'clearance_status_id']);
        if(isset($request['customs_declaration_file']) && strlen($request['customs_declaration_file']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['customs_declaration_file'] , 'clearance_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
            $update['customs_declaration_file'] = $check['url'] ;
        }
        DB::table('clearances_m')
            ->where('clearance_id' , $id )
            ->update($update);
        //(new LogsHelper)->insert_log(['payload' => $update  , 'screen_code'=>'04-004' ,'function'=> 'update_clearance_item' , 'action_type'=>'clearance' , 'action_id'=>$id]);
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('04-004','delete');
        $check_trips = DB::table('trips')
            ->where('clearance_m_id' , $id)
            ->where('trip_active' , 1 )
            ->count();
        if($check_trips > 0){
            return $this->sendError( trans('error.this_clearance_has_trip_or_transaction') );
        }
        Clearance::delete_clearance($id);
        //(new LogsHelper)->insert_log(['payload' => null  , 'screen_code'=>'04-004' ,'function'=> 'destroy' , 'action_type'=>'clearance' , 'action_id'=>$id]);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('04-004','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Clearance::update_clearance(array('clearance_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    // update to status 2
    public function update_clearance_arrival($lang , $id , Request $request){
        $this->checkPermission('04-009','edit');

        Clearance::update_clearance([
            'clearance_status_id' => 2 ,
            'mainfest_date' => $request['mainfest_date'] ?? null ,
            'mainfest_date_hijri' => $request['mainfest_date_hijri'] ?? null ,
            'mainfest_no' => $request['mainfest_no'] ?? ''  ,
        ],  $id);

        $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')
            ->insert([
                'clearances_m_id'   => $id ,
                'hijri_date'        => $hijri_date,
                'status_id'         => 2 ,
                'note'              => $request['note'] ?? '' ,
                'add_user'          => auth()->guard('api')->user()->id
            ]);
        //(new LogsHelper)->insert_log(['payload' => $request  , 'screen_code'=>'04-004' ,'function'=> 'update_clearance_arrival' , 'action_type'=>'clearance' , 'action_id'=>$id]);

        return $this->sendResponse( [] , trans('main.update_success') );
    }

    // update to status 3
    public function update_clearance_customs_declaration($lang , $id , Request $request){
        $this->checkPermission('04-010','edit');

        $update = [
            'clearance_status_id'               => 3 ,
            'customs_declaration_date'          => $request['customs_declaration_date'],
            'customs_declaration_date_hijri'    => $request['customs_declaration_date_hijri'],
            'customs_declaration_number'        => $request['customs_declaration_number'],
            'unloading_date'                    => $request['unloading_date'],
            'unloading_date_hijri'              => $request['unloading_date_hijri']
        ];

        $validator = validator()->make( $update , Clearance::$customs_declaration_rules );

        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }


        if(isset($request['customs_declaration_file']) && strlen($request['customs_declaration_file']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['customs_declaration_file'] , 'clearance_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
            $update['customs_declaration_file'] = $check['url'] ;
        }

        Clearance::update_clearance( $update,  $id);

        $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')
            ->insert([
                'clearances_m_id'   => $id ,
                'hijri_date'        => $hijri_date,
                'status_id'         => 3 ,
                'note'              => $request['note'] ?? '' ,
                'add_user'          => auth()->guard('api')->user()->id
            ]);
        //(new LogsHelper)->insert_log(['payload' => $update  , 'screen_code'=>'04-004' ,'function'=> 'update_clearance_customs_declaration' , 'action_type'=>'clearance' , 'action_id'=>$id]);
        return $this->sendResponse( [] , trans('main.update_success') );
    }

    // update to status 4
    public function update_preview_done($lang , $id , Request $request){
        $this->checkPermission('04-011','edit');

        Clearance::update_clearance(array('clearance_status_id' => 4 ),  $id);

        $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')
            ->insert([
                'clearances_m_id'   => $id ,
                'hijri_date'        => $hijri_date,
                'status_id'         => 4 ,
                'note'              => $request['note'] ?? '' ,
                'add_user'          => auth()->guard('api')->user()->id
            ]);
        //(new LogsHelper)->insert_log(['payload' => $request  , 'screen_code'=>'04-004' ,'function'=> 'update_preview_done' , 'action_type'=>'clearance' , 'action_id'=>$id]);
        return $this->sendResponse( [] , trans('main.update_success') );
    }

    // update to status 5
    public function update_clearance_paymant($lang , $id , Request $request){
        $this->checkPermission('04-020','edit');

        DB::beginTransaction();

            if(isset($request['final_note_file']) && strlen($request['final_note_file']) > 10 ){

                $upload = new Upload;
                $check  = $upload->uploadBase64File($request['final_note_file'] , 'clearance_files' );

                if(!$check['check']){
                    return $this->sendError( trans('error.add_fails') , $check['msg'] );
                }
            }
            $update_seed    = array(
                'clearance_status_id'   => 5 ,
                'final_note_file'       => $check['url'] ?? '' ,
                'customs_floor_date'    => $request['customs_floor_date'] ,
                'paid_by'               => $request['paid_by'] ?? 1
            );
        if (isset($request['clearance_finished']) && $request['clearance_finished']){
            $update_seed['clearance_status_id'] = 6;
            $update_seed['expiry_date'] = $request['expiry_date'] ;
        }

            Clearance::update_clearance($update_seed , $id );
            //check if clearance has invoice
            $draft_invoice_id       = DB::table('clearance_invoices_draft_m')->where('clearance_id' , $id)->value('invoice_id');
            $clearance_invoice_id   = DB::table('clearance_invoices_m')->where('clearance_id' , $id)->value('invoice_id');

            if($draft_invoice_id > 0 && $update_seed['paid_by'] == 1){
                DB::table('clearance_invoices_draft_delegate_d')->where([['expense_code_id',1],['invoice_m_id',$draft_invoice_id]])->update([
                    'description_ar' => DB::raw("
                    CONCAT(
                        COALESCE(description_ar, ''),
                        CASE
                            WHEN description_ar IS NOT NULL AND description_ar != '' THEN ', '
                            ELSE ''
                        END,
                        '".("سداد عميل")."'
                    )"),
                    'description_en' => DB::raw("
                    CONCAT(
                        COALESCE(description_en, ''),
                        CASE
                            WHEN description_en IS NOT NULL AND description_en != '' THEN ', '
                            ELSE ''
                        END,
                        '".("سداد عميل")."'
                    )")
                ]);
            }elseif($clearance_invoice_id > 0 && $update_seed['paid_by'] == 1){
                DB::table('clearance_invoices_delegate_d')->where([['expense_code_id',1],['invoice_m_id',$clearance_invoice_id]])->update([
                    'description_ar' => DB::raw("
                    CONCAT(
                        COALESCE(description_ar, ''),
                        CASE
                            WHEN description_ar IS NOT NULL AND description_ar != '' THEN ', '
                            ELSE ''
                        END,
                        '".("سداد عميل")."'
                    )"),
                    'description_en' => DB::raw("
                    CONCAT(
                        COALESCE(description_en, ''),
                        CASE
                            WHEN description_en IS NOT NULL AND description_en != '' THEN ', '
                            ELSE ''
                        END,
                        '".("سداد عميل")."'
                    )")
                ]);
            }

            $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
            DB::table('clearances_comments')
                ->insert([
                    'clearances_m_id'   => $id ,
                    'hijri_date'        => $hijri_date,
                    'status_id'         => 5 ,
                    'note'              => $request['note'] ?? '' ,
                    'add_user'          => auth()->guard('api')->user()->id
                ]);
        //(new LogsHelper)->insert_log(['payload' => $update_seed  , 'screen_code'=>'04-004' ,'function'=> 'update_clearance_paymant' , 'action_type'=>'clearance' , 'action_id'=>$id]);
        DB::commit();


        return $this->sendResponse( [] , trans('main.update_success') );
    }

    // update to status 6
    public function update_completed($lang , $id , Request $request){
        $this->checkPermission('04-018','edit');
        Clearance::update_clearance(array('clearance_status_id' => 6 ,'expiry_date' => date("Y-m-d") ),  $id);

        $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')
            ->insert([
                'clearances_m_id'   => $id ,
                'hijri_date'        => $hijri_date,
                'status_id'         => 6 ,
                'note'              => $request['note'] ?? '' ,
                'add_user'          => auth()->guard('api')->user()->id
            ]);
        //(new LogsHelper)->insert_log(['payload' => null  , 'screen_code'=>'04-004' ,'function'=> 'update_completed' , 'action_type'=>'clearance' , 'action_id'=>$id]);

        return $this->sendResponse( [] , trans('main.update_success') );
    }

    public function clearance_transport_fees($lang , Request $request){
        $this->checkPermission('04-010','edit');

        foreach ($request['items'] as $key => $row) {
            DB::table('clearances_d')
                ->where('clearances_m_id' , $row['clearances_m_id'])
                ->where('clearances_d_id' , $row['clearances_d_id'])
                ->update( ['from_area' => $row['from_area'] , 'to_area' => $row['to_area'] , ] );
        }

        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function update_clearance_withdrawal_note($lang , $id , Request $request){
        // $this->checkPermission('04-004','edit');

        DB::beginTransaction();

            $update_seed    = array('withdrawal_note' => $request['withdrawal_note'] );
            Clearance::update_clearance( $update_seed , $id);
            $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
            DB::table('clearances_comments')
                ->insert([
                    'clearances_m_id'   => $id ,
                    'hijri_date'        => $hijri_date,
                    'status_id'         => 0 ,
                    'note'              => 'تم اضافه رقم السحب --- '.$request['note'] ?? '' ,
                    'add_user'          => auth()->guard('api')->user()->id
                ]);

        DB::commit();

        return $this->sendResponse( [] , trans('main.update_success') );
    }

    public function update_clearance_inspection_number($lang , $id , Request $request){
        // $this->checkPermission('04-004','edit');

        DB::beginTransaction();

            $update_seed    = array('inspection_number' => $request['inspection_number'] );

            Clearance::update_clearance( $update_seed , $id);
            $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
            DB::table('clearances_comments')
                ->insert([
                    'clearances_m_id'   => $id ,
                    'hijri_date'        => $hijri_date,
                    'status_id'         => 0 ,
                    'note'              => 'تم اضافه رقم التفتيش --- '.$request['note'] ?? '' ,
                    'add_user'          => auth()->guard('api')->user()->id
                ]);

        DB::commit();

        return $this->sendResponse( [] , trans('main.update_success') );
    }

    public function update_clearance_withdrawal_port($lang , $id , Request $request){
        // $this->checkPermission('04-004','edit');


        DB::beginTransaction();

            $update_seed    = array('port_withdraw' => $request['port_withdraw'] );

            Clearance::update_clearance( $update_seed , $id);
            $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
            DB::table('clearances_comments')
                ->insert([
                    'clearances_m_id'   => $id ,
                    'hijri_date'        => $hijri_date,
                    'status_id'         => 0 ,
                    'note'              => 'تم اضافه رقم المةاني --- '.$request['note'] ?? '' ,
                    'add_user'          => auth()->guard('api')->user()->id
                ]);

        DB::commit();

        return $this->sendResponse( [] , trans('main.update_success') );
    }

    public function get_clearance_customs_declaration($lang , $id , Request $request){
        $this->return['item'] = DB::table('clearances_m')
            ->where('clearance_id' , $id)
            ->selectRaw('clearance_id as id , customs_declaration_date , customs_declaration_number , customs_declaration_file')
            ->first();


        $rest = substr( $this->return['item']->customs_declaration_file , -6);
        $this->return['item']->ext = explode('.', $rest)[1] ?? "";

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function download_clearance_customs_declaration($lang , $id , Request $request){

        $this->return['item'] = DB::table('clearances_m')
            ->where('clearance_id' , $id)
            ->selectRaw('customs_declaration_file')
            ->first();

        return response()->download( public_path($this->return['item']->customs_declaration_file) );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }
    public function download_final_note_file($lang , $id , Request $request){

        $this->return['item'] = DB::table('clearances_m')
            ->where('clearance_id' , $id)
            ->selectRaw('final_note_file')
            ->first();

        return response()->download( public_path($this->return['item']->final_note_file) );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function add_comment($lang , $id , Request $request){
        $this->checkPermission('04-011','edit');

        $hijri_date = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')
            ->insert([
                'clearances_m_id'   => $id ,
                'hijri_date'        => $hijri_date,
                'note'              => $request['comment']  ,
                'add_user'          => auth()->guard('api')->user()->id
            ]);

        return $this->sendResponse( [] , trans('main.add_success') );
    }

    public function clearance_toggle_has_problem($lang , $id , Request $request){
        $this->checkPermission('04-011','edit');


        $master         = Clearance::get_clearance($id , $lang);
        $update_seed    = [
            'clearances_m_id'   => $id ,
            'note'              => $request['comment']  ,
            'add_user'          => auth()->guard('api')->user()->id ,
            'has_problem'       => 0
        ];

        if( $master->has_problem == 1 ){
            Clearance::update_clearance( ['has_problem' => 0 ] ,  $id);
        }else{
            $update_seed['has_problem'] = 1 ;
            Clearance::update_clearance( ['has_problem' => 1 ] ,  $id);
        }
        $update_seed['hijri_date'] = Dcter::GregorianToHijri(date('Y-m-d'));
        DB::table('clearances_comments')->insert($update_seed);

        return $this->sendResponse( [ $update_seed['has_problem'] ] , trans('main.update_success') );
    }
    public function holding_container_get($lang , Request $request){
        $this->checkPermission('03-041','view');
        $this->filter = array(
            'lang'        => $lang ,
            'active'      => 1 ,
            'rows'        => $request['rows'] ?? 15 ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );
        $this->return['containers'] = (new ClearanceHelper)->all_clearance_d($this->filter);
        return $this->sendResponse( $this->return);
    }

    public function update_holding_container($lang,$id ,Request $request){
        $this->checkPermission('03-041','edit');
        $container = DB::table('clearances_d')->where('clearances_d.clearances_d_id' ,$id)->first();
        if($container->container_status_id > 1){
            return $this->sendError([],trans('error.cannot_hold_this_container'));
        }
        if($container->is_hold == 0){
            DB::table('clearances_d')->where('clearances_d.clearances_d_id' ,$id)->update(array('is_hold' => 1));

        }elseif($container->is_hold == 1) {
            DB::table('clearances_d')->where('clearances_d.clearances_d_id' ,$id)->update(array('is_hold' => 0));
        }

        return $this->sendResponse( [] , trans('main.update_success') );
    }
    public function get_problem_comment($lang , $id , Request $request){

        $this->return['item']   = DB::table('clearances_comments')
            ->where('clearances_m_id' , $id)
            ->where('has_problem' , 1 )
            ->orderBy('clearances_comment_id' , 'desc')
            ->first();

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function toggle_active($lang , $id){
        $item = Clearance::find($id);
        if($item->clearance_active){

            $this->checkPermission('04-004','delete');
            $check_trips = DB::table('trips')
                ->where('clearance_m_id' , $id)
                ->where('trip_active' , 1 )
                ->count();
            if($check_trips > 0){
                return $this->sendError( trans('error.this_clearance_has_trip_or_transaction') );
            }
            Clearance::delete_clearance($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('04-004','edit');

            Clearance::update_clearance(array('clearance_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function add_file($lang , $id , Request $request){
        $this->checkPermission('04-004','edit');

        $upload = new Upload;
        $check  = $upload->uploadBase64File($request['file'] , 'clearance_files' );

        if(!$check['check']){
            return $this->sendError( trans('error.add_fails') , $check['msg'] );
        }
        DB::beginTransaction();

            DB::table('clearance_attachments')
                ->where('clearance_id' , $id )
                ->where('clearance_file_name_id' , $request['clearance_file_name_id'])
                ->delete();

           $this->return['id']  = DB::table('clearance_attachments')
                ->insertGetId(
                    array(
                        'clearance_id'  => $id ,
                        'clearance_file_name_id'    => $request['clearance_file_name_id'] ,
                        'file_type'                 => $check['ext'] ,
                        'file_path'                 => $check['url'] ,
                        'add_user'                  => auth()->guard('api')->user()->id
                    )
                );

        DB::commit();

        return $this->sendResponse( $this->return, trans('main.update_success') );
    }


    public function download_file($lang , $id , Request $request){

        $this->checkPermission('04-004','edit');

        $file = DB::table('clearance_attachments')
            ->where('clearance_attachment_id' , $id )
            ->first();

        if(file_exists(public_path($file->file_path ?? '' ))){
            return response()->download(public_path($file->file_path ?? '' ));
        }

        return $this->sendError( '404 File not found ' , '' );

    }
    public function download_all_files($lang , $id , Request $request){

        $this->checkPermission('04-004','view');

        $clearance  = Clearance::get_clearance($id , $lang);
        $clearance_attachments      = DB::table('clearance_attachments')
            ->where('clearance_id', $id)
            ->pluck('file_path')->toArray();

            $delegate_petty_attachment  = ClearanceInvoice::delegate_petty_attachment($lang,$id);
            $multi_voucher_attachment   = ClearanceInvoice::multi_voucher_attachment($lang,$id);
            $delegate_petty_attachment_files    = collect($delegate_petty_attachment)->pluck('file_path')->toArray();
            $multi_voucher_files                = collect($multi_voucher_attachment)->pluck('attachment')->toArray();
            $files = array_merge($clearance_attachments, $delegate_petty_attachment_files, $multi_voucher_files);

        if(strlen($clearance->customs_declaration_file) > 10){
            $files[] = $clearance->customs_declaration_file;
        }

        if(strlen($clearance->final_note_file) > 10){
           $files[] = $clearance->final_note_file;
        }

        if (empty($files)) {
            return $this->sendError('No files found for this clearance.', '' , 404);
        }

        $zip = new \ZipArchive();
        $zipFileName = 'attachments-' . $id . '.zip';
        $zipFilePath = public_path($zipFileName);

        if ($zip->open($zipFilePath, \ZipArchive::CREATE) === TRUE) {
            foreach ($files as $file) {
                $filePath = public_path($file);
                if (file_exists($filePath)) {
                    // Add files to zip
                    $zip->addFile($filePath, basename($file));
                } else {
                    // Optionally log missing files
                    \Log::warning("File not found: $filePath");
                }
            }
            $zip->close();
        } else {
            return $this->sendError('Could not create a zip file.', '');
        }

        // Check if the zip file has been created and has files
        if (file_exists($zipFilePath) && filesize($zipFilePath) > 0) {
            return response()->download($zipFilePath)->deleteFileAfterSend(true);
        } else {
            return $this->sendError('Failed to download the zip file or zip file is empty.', '');
        }

    }

    public function delete_file($lang,$id){
        $this->checkPermission('04-004','delete');

        Clearance::delete_clearance_file($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }
    public function upload_delegate_petty_file($lang,$id,Request $request){


        if(request()->hasFile('file_path')){
            //check if its new attach
            $old_attach = DB::table('delegate_petty_d')->where('delegate_petty_d_id' , $id)->value('file_path');
            if(isset($old_attach) || strlen($old_attach) > 0){
                $this->checkPermission('04-004','edit');
            }else{
                $this->checkPermission('04-004','view');
            }
            $update['attached_user'] = auth()->user()->id;
            $upload = new Upload;
            $check  = $upload->uploadFile($request['file_path'] , 'DelegatePetty');
            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }else{
                $update['file_path'] = $check['url'];
            }
            DelegatePetty::update_delegate_petty_d($id,$update);

            return  $this->sendResponse($check['url'],trans('main.add_success'));
        }
    }
    public function add_multi_voucher_attach($lang , $id , Request $request){


        $item = $request->all();
        //check if its new attach
        $old_attach = DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->value('attachment');
        if(isset($old_attach) || strlen($old_attach) > 0){
            $this->checkPermission('04-004','edit');
        }else{
            $this->checkPermission('04-004','view');
        }
            $check  = (new Upload)->uploadFile($item['file_path'] , 'multi_voucher' );
            if(!$check['check'])
                return $this->sendError( trans('error.add_fails') , $check['msg'] );

            DB::table('multi_voucher_d')->where('multi_voucher_d_id',$id)->update(['attachment' => $check['url'] , 'attached_user' => auth()->user()->id]);

        return $this->sendResponse( $this->return );
    }
    public function upload_reimburses_expenses_attach($lang , $id , Request $request){
        // $this->checkPermission('04-004','edit');

        $item = $request->all();
        if($item['from_type'] == 'delegate_petty'){
            $this->upload_delegate_petty_file($lang , $id , $request);
        }elseif($item['from_type'] == 'multi_voucher'){
            $this->add_multi_voucher_attach($lang , $id , $request);
        }

        return $this->sendResponse( $this->return );
    }
    public function update_container_return_file($lang , $id , Request $request){
        $this->checkPermission('04-004','edit');
        $container  = DB::table('clearances_d')->where('clearances_d_id' , $id)->first();
        if(isset($container->return_file) || strlen($container->return_file) > 0){
            DB::beginTransaction();
            //delete old file
            $path = public_path($container->return_file ?? '' );
            if (File::exists($path)) {
                File::delete($path);
            }

            $upload = new Upload;
            $check  = $upload->uploadFile(request()->file('file_path') , 'clearances_d_return_file' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
            $return_file = $check['url'] ;
            DB::table('clearances_d')
            ->where('clearances_d_id' , $id )
            ->update(['return_file' => $return_file]);
            DB::commit();
        }


        return $this->sendResponse( $this->return, trans('main.update_success') );
    }
}
