<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\AccountTree;
use App\Models\Sales\CovenantType;
use App\Models\Sales\CustomsItem;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class CustomsItemController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('04-005','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('04-005', $lang)->sub_title ?? '';
        $this->return['customs_items']      = CustomsItem::all_customs_items($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('04-005','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-005', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('04-005','add');

        $insert             = $request->only(['name_en', 'name_ar' , 'item_name_ar' , 'item_name_en' , 'item_type','slug']);


        $validator          = validator()->make($insert, CustomsItem::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $insert['name_en'] = ( strlen($insert['name_en'] ?? '') > 0 )? $insert['name_en'] : $insert['name_ar'];
        $insert['item_name_en'] = ( strlen($insert['item_name_en'] ?? '') > 0 )? $insert['item_name_en'] : $insert['item_name_ar'];

        $insert['add_user'] = auth()->user()->id;
        $insert['id']       = CustomsItem::add_customs_item($insert);
        $this->return       = $insert;
        $insert['name']     = $insert['name_ar'];
        $insert['item_name']= $insert['item_name_ar'];

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('04-005','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-005', $lang)->sub_title ?? '';
        $this->return['data']           = CustomsItem::get_customs_item($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('04-005','edit');

        $update = $request->only(['name_en', 'name_ar' , 'item_name_ar' , 'item_name_en' , 'item_type']);

        $validator = validator()->make($update, CustomsItem::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        CustomsItem::update_customs_item($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('04-005','delete');

        CustomsItem::delete_customs_item($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $customs_item = CustomsItem::find($id);

        if($customs_item->customs_item_active){
            $this->checkPermission('04-005','delete');

            CustomsItem::delete_customs_item($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('04-005','edit');

            CustomsItem::update_customs_item(array('customs_item_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
