<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Classes\General\General;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Sales\ClearanceInvoiceHelper;
use App\Classes\Sales\ClearanceInvoiceClaimHelper;

use App\Models\Finance\AccountTree;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Clearance;
use App\Models\Sales\ClearanceInvoiceDraft as ClearanceInvoice;
use App\Models\Sales\Revenue;
use App\Models\Sales\Customer;
use App\Models\Trans\Area;
use App\Models\Main\Branch;
use App\Models\Main\Setting;
use Illuminate\Support\Facades\DB;
use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;
use PDF;
use App\Classes\General\ZatcaHelper;

use Str;
class DraftController extends BaseController
{
    protected $return = array();
    protected $filter = array();
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($lang, Request $request)
    {
        $this->checkPermission('03-023','view');
        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
            'from_id' => $request['from_id'] ?? null ,
            'to_id'   => $request['to_id'] ?? null ,
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
        $this->return['items']          = ClearanceInvoice::all_invoices($this->filter);
        $this->return['is_journals']    = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );

//        $invoices = DB::table('clearance_invoices_draft_m')->get();
//        foreach ($invoices as $key => $value) {
//            $this->update_delegate_amounts($value->invoice_id , $value->clearance_id);
//        }


        return $this->sendResponse($this->return);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create($lang , Request $request)
    {
        $this->checkPermission('03-023','add');

        $clearance_id                   = $request['clearance_id'] ?? 0;
        if($request['clearance_id'] == 0 ){
            $this->return['clearances']     = Clearance::all_clearances( ['lang' => $lang , 'active' => 1  ] );
        }else{

            $check = DB::table('clearance_invoices_draft_m')
                ->where('clearance_id' , $request['clearance_id'])
                ->count();
            if($check > 0  ){
                return $this->sendError( trans('error.A draft has already been created for the current transaction'), [], 200);
            }

            $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
            $this->return['areas']          = Area::all_areas( ['lang' => $lang , 'active' => 1] );
            $this->return['clearance']      = Clearance::get_clearance( $clearance_id , $lang );
            $this->return['customer']       = Customer::get_customer( $this->return['clearance']->customer_id , $lang );
            $this->return['account_tree']   = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
            $this->return['count_invoices'] = DB::table('clearance_invoices_draft_m')->where('clearance_id' , $request['clearance_id'] )->where('invoice_active' , 1 )->count() ;
            $this->return['is_journals']    = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );
            $this->return['citys']          = DB::table('citys')->selectRaw('city_id , name_'.$lang.' as name')->get();

            $this->return['prices']         = DB::table('customer_transport_prices')
                ->leftjoin('areas as to_areas' , 'customer_transport_prices.to_area_id' , '=' , 'to_areas.area_id')
                ->leftjoin('areas as from_areas' , 'customer_transport_prices.from_area_id' , '=' , 'from_areas.area_id')
                ->where('port_id' , $this->return['clearance']->port_id)
                ->where('customer_id' , $this->return['clearance']->customer_id)
                ->select('from_areas.area_name_'.$lang.' as from_area_name','to_areas.area_name_'.$lang.' as to_area_name','from_area_id' , 'to_area_id','price' , 'container_20' , 'container_40','price_special' , 'container_20_special' , 'container_40_special' )
                ->where('customer_transport_price_active' , 1)
                ->get();


            if(!$this->get_invoice_amounts($clearance_id , $lang) ){
                return $this->sendError($this->return , [],  200);
            }

            if($this->return['deleget_amounts']->sum('amount') == 0 && count($this->return['amounts']) == 0   ){
                return $this->sendError( trans('main.no_amount_found'), [], 200);
            }
        }

        return $this->sendResponse($this->return);
    }


    private function get_invoice_amounts($clearance_id ,$lang){

        $this->return['clearance_m']    = Clearance::get_clearance($clearance_id , $lang);

        //check if clearance is have classification
        $classification = DB::table('classifications_d')
        ->leftjoin('classifications_m','classifications_m.classification_id','classifications_d.classifications_m_id')
        ->where('classifications_m.clearances_m_id',$clearance_id)
        ->first();
        if(isset($classification)){
            $this->return['clearance_m']->goods_type = $classification->item_name;

        }else{
            $this->return['clearance_m']->goods_type = null;

        }
        if(!$this->return['clearance_m']){
            $this->return = 'Clearance Not found';
            return false;
        }
        $this->return['clearance_d']    = Clearance::get_clearance_d($clearance_id , $lang);

        if($this->return['clearance_d']->where('from_area' , 0)->count() > 0 ){
            $this->return['transport_fees'] = 1;
        }else{
            $this->return['transport_fees'] = 0;
        }

        $this->return['amounts']            = array();


        $petty = DB::table('delegate_petty_d')
            ->join('delegate_petty_m' , 'delegate_petty_d.delegate_petty_m_id' , '=' , 'delegate_petty_m.delegate_petty_m_id')
            ->where('invoice_id' , 0 )
            ->where('delegate_petty_m.delegate_petty_active' , 1 )
            ->where('delegate_petty_d.clearance_id' , '>',  0 )
            ->where('delegate_petty_d.clearance_id' , $clearance_id )
            ->selectRaw('
                "delegate_petty" as from_type ,
                delegate_petty_d.delegate_petty_m_id as m_id ,
                delegate_petty_d.delegate_petty_d_id as d_id ,
                delegate_petty_d.clearance_id ,
                delegate_petty_d.expense_code_id ,
                delegate_petty_d.receipt_number ,
                delegate_petty_d.amount_without_vat as amount ,
                delegate_petty_d.description_'.$lang.' as description ,
                delegate_petty_d.add_date
            ');

        $all = DB::table('multi_voucher_d')
            ->join('multi_voucher_m' , 'multi_voucher_d.multi_voucher_m_id' , '=' , 'multi_voucher_m.multi_voucher_m_id')
            ->where('multi_voucher_d.invoice_id' , 0 )
            ->where('multi_voucher_m.multi_voucher_active' , 1 )
            ->where('multi_voucher_d.multi_voucher_d_active' , 1 )
            ->where('multi_voucher_d.clearance_id' , '>',  0 )
            ->where('multi_voucher_d.clearance_id' , $clearance_id )
            ->where('voucher_type' , 1 )
            ->selectRaw('
                "multi_voucher" as from_type ,
                multi_voucher_d.multi_voucher_m_id as m_id ,
                multi_voucher_d.multi_voucher_d_id as d_id ,
                multi_voucher_d.clearance_id ,
                multi_voucher_d.dis_expense_code_id as expense_code_id ,
                multi_voucher_d.receipt_number ,
                multi_voucher_d.amount_without_vat as amount ,
                multi_voucher_d.description as description ,
                "" as add_date
            ')
            ->union($petty);


        $this->return['deleget_amounts']    = DB::table('expense_codes')
            ->leftjoinSub($all, 'all', function ($join) {
                $join->on('all.expense_code_id' , '=' , 'expense_codes.expense_code_id');
            })
            ->where('expense_codes.expense_code_id' , '<>' , 8)
            ->groupBy('expense_codes.expense_code_id')
            ->selectRaw('
                m_id ,
                d_id ,
                group_concat( case when from_type = "delegate_petty" then d_id end) as petty_ids ,
                group_concat( case when from_type = "multi_voucher" then d_id end) as multi_ids ,
                clearance_id ,
                expense_codes.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                group_concat(ifnull( receipt_number , "-" )) as receipt_number ,
                ifnull( sum(amount) , 0 ) as amount ,
                description ,
                all.add_date
            ')->get();
            //check if invoice paid by customer
            if($this->return['clearance_m']->paid_by == 1 && in_array($this->return['clearance_m']->clearance_status_id , [5,6]) ){
                foreach($this->return['deleget_amounts'] as $deleget_amount){
                    if($deleget_amount->expense_code_id == 1){
                        $deleget_amount->description = $deleget_amount->description . "سداد عميل";
                    }
                }
            }


        $count = $this->return['clearance_m']->container_count ?? 0;

        $revenues = collect( Revenue::all_revenues( ['lang' => $lang , 'active' => 1 ] ) )->groupBy('id')->all();



        if(DB::table('clearance_invoices_draft_m')->where('clearance_id' , $clearance_id )->where('invoice_active' , 1 )->count() == 0  ){
            $prices = DB::table('customer_prices')
                ->where('customer_id' , $this->return['clearance_m']->customer_id)
                ->where('port_id' , $this->return['clearance_m']->port_id )
                ->first();

            if(!$prices){

                $prices = (object) array();
                $prices->cont_s_first           = $prices->cont_s_first ?? 0;
                $prices->cont_s_sec             = $prices->cont_s_sec ?? 0;
                $prices->cont_s_other           = $prices->cont_s_other ?? 0;
                $prices->cont_b_first           = $prices->cont_b_first ?? 0;
                $prices->cont_b_sec             = $prices->cont_b_sec ?? 0;
                $prices->cont_b_other           = $prices->cont_b_other ?? 0;
                $prices->loading_type           = $prices->loading_type ?? 0;
                $prices->loading_price          = $prices->loading_price ?? 0;
                $prices->lcl_loading_price      = $prices->lcl_loading_price ?? 0;
                $prices->translate_type         = $prices->translate_type ?? 0;
                $prices->translate_price        = $prices->translate_price ?? 0;
                $prices->lcl_translate_price    = $prices->lcl_translate_price ?? 0;
                $prices->other_type             = $prices->other_type ?? 0;
                $prices->other_price            = $prices->other_price ?? 0;
                $prices->lcl_other_price        = $prices->lcl_other_price ?? 0;


                if(in_array(request()->header('clientID') , ['trans' , 'amwag'] )){
                    $this->return = trans('main.no_prices_found');
                    return false;
                }
            }


            if( isset($revenues[1]) ){

                if($this->return['clearance_m']->is_lcl){
                    $t = $prices->cont_s_first;
                }else{
                    $count_s = $this->return['clearance_m']->container_small;
                    $count_b = $this->return['clearance_m']->container_big;
                    $amount_s = ( ($count_s > 0? $prices->cont_s_first : 0 ) ) + ( ( ($count_s - 1) > 0? $prices->cont_s_sec : 0)  ) + ( ( ($count_s - 2) > 0? (($count_s - 2) * $prices->cont_s_other) : 0) );
                    $amount_b = ( ($count_b > 0? $prices->cont_b_first : 0 ) ) + ( ( ($count_b - 1) > 0? $prices->cont_b_sec : 0)  ) + ( ( ($count_b - 2) > 0? (($count_b - 2) * $prices->cont_b_other) : 0) );
                    $t        = $amount_s+$amount_b;
                }



                $this->return['amounts'][] = [
                    'revenue_id'        => 1,
                    'name_ar'           => $revenues[1][0]->revenue_name_ar ,
                    'name_en'           => $revenues[1][0]->revenue_name_en ,
                    'amount_without_vat'=> $t ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => $t * 0.15  ,
                    'amount_with_vat'=> $t + ($t * 0.15 )
                ];

            }

            if( isset($revenues[2]) ){
                if($this->return['clearance_m']->is_lcl){
                    $loading_amount = $prices->lcl_loading_price;
                }else{
                    $loading_amount = ( $prices->loading_type == 1? $prices->loading_price:($count*$prices->loading_price) );
                }
                $this->return['amounts'][] = [
                    'revenue_id'        => 2,
                    'name_ar'           => $revenues[2][0]->revenue_name_ar ,
                    'name_en'           => $revenues[2][0]->revenue_name_en ,
                    'amount_without_vat'=> $loading_amount ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => $loading_amount * 0.15  ,
                    'amount_with_vat'=> $loading_amount + ($loading_amount * 0.15 )
                ];

            }

            if( isset($revenues[3]) ){
                if($this->return['clearance_m']->is_lcl){
                    $trans_amount = $prices->lcl_translate_price;
                }else{
                    $trans_amount = ( $prices->translate_type == 1? $prices->translate_price:($count*$prices->translate_price) );
                }
                $this->return['amounts'][] = [
                    'revenue_id'        => 3,
                    'name_ar'           => $revenues[3][0]->revenue_name_ar ,
                    'name_en'           => $revenues[3][0]->revenue_name_en ,
                    'amount_without_vat'=> $trans_amount ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => $trans_amount * 0.15  ,
                    'amount_with_vat'=> $trans_amount + ($trans_amount * 0.15 )
                ];

            }

            if( isset($revenues[4]) ){
                if($this->return['clearance_m']->is_lcl){
                    $other_amount = $prices->lcl_other_price;
                }else{
                    $other_amount = ( $prices->other_type == 1? $prices->other_price:($count*$prices->other_price) );
                }

                $this->return['amounts'][] = [
                    'revenue_id'        => 4,
                    'name_ar'           => $revenues[4][0]->revenue_name_ar ,
                    'name_en'           => $revenues[4][0]->revenue_name_en ,
                    'amount_without_vat'   => $other_amount ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => $other_amount * 0.15  ,
                    'amount_with_vat'=> $other_amount + ($other_amount * 0.15 )
                ];
            }

            if( isset($revenues[5]) ){

                $trans_price = 0;

                if($this->return['clearance_m']->is_lcl == 1 ){

                    $trans_price = 0;

                }else{

                    foreach ($this->return['clearance_d'] as $key => $row) {
                        $price = DB::table('customer_transport_prices')
                            ->where('port_id' , $this->return['clearance_m']->port_id)
                            ->where('from_area_id' , $row->from_area)
                            ->where('to_area_id' , $row->to_area)
                            ->select('price' , 'container_20' , 'container_40')
                            ->first();

                        if($row->container_size == 40 ){
                            $size_price = $price->container_40 ?? 0 ;
                        }else{
                            $size_price = $price->container_20 ?? 0 ;
                        }
                        $trans_price = $trans_price + ( $size_price ) ;

                    }
                }

                $this->return['amounts'][] = [
                    'revenue_id'        => 5,
                    'name_ar'           => $revenues[5][0]->revenue_name_ar ,
                    'name_en'           => $revenues[5][0]->revenue_name_en ,
                    'amount_without_vat'=> $trans_price ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => $trans_price * 0.15 ,
                    'amount_with_vat'=> $trans_price + ($trans_price * 0.15)
                ];

            }

            foreach ($revenues as $key => $revenue) {

                if($revenue[0]->id > 5 ){
                    $this->return['amounts'][] = [
                        'revenue_id'        => $revenue[0]->id ,
                        'name_ar'           => $revenue[0]->revenue_name_ar ,
                        'name_en'           => $revenue[0]->revenue_name_en ,
                        'amount_with_vat'   => 0 ,
                        'vat_per'           => 15 ,
                        'vat_amount'        => 0 ,
                        'amount_without_vat'=> 0
                    ];
                }
            }

        }else{

            foreach ($revenues as $key => $revenue) {

                $this->return['amounts'][] = [
                    'revenue_id'        => $revenue[0]->id ,
                    'name_ar'           => $revenue[0]->revenue_name_ar ,
                    'name_en'           => $revenue[0]->revenue_name_en ,
                    'amount_with_vat'   => 0 ,
                    'vat_per'           => 15 ,
                    'vat_amount'        => 0 ,
                    'amount_without_vat'=> 0
                ];

            }

        }

        return true;

    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store($lang, Request $request)
    {

        $this->checkPermission('03-023','add');
        $clearance_id = $request['clearance_id'] ?? 0;
        if($request['clearance_id'] == 0 ){
            return $this->sendError( 'clearance ID is required', [], 200);
        }

        $check = DB::table('clearance_invoices_draft_m')
            ->where('clearance_id' , $request['clearance_id'])
            ->count();
        if($check > 0  ){
            return $this->sendError( trans('error.A draft has already been created for the current transaction'), [], 200);
        }

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
        if(!$this->get_invoice_amounts($clearance_id , $lang) ){
            return $this->sendError($this->return , [],  200);
        }

        if($this->return['deleget_amounts']->sum('amount') == 0 && count($this->return['amounts']) == 0   ){
            return $this->sendError('No Amounts found', [], 200);
        }

        $year   = array_search( date("Y" , strtotime($request['invoice_date'])) , $this->data['years'] );
        if($year == ''){
            return $this->sendError( trans("error.finance_year_not_aval"));
        }
        if((new General)->check_financial_year($request['invoice_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($request['invoice_date']));
        }
        DB::beginTransaction();

        if(isset($request['clearance_transport_fees']) && is_array(isset($request['clearance_transport_fees'])) && count($request['clearance_transport_fees']) > 0 ){
            foreach ($request['clearance_transport_fees'] as $key => $row) {
                DB::table('clearances_d')
                    ->where('clearances_m_id' , $row['clearances_m_id'])
                    ->where('clearances_d_id' , $row['clearances_d_id'])
                    ->update( ['from_area' => $row['from_area'] , 'to_area' => $row['to_area'] , ] );
            }
        }
        $counter = DB::table('clearance_invoices_draft_m')->where('clearance_id' , $request['clearance_id'] )->where('invoice_active' , 1 )->count();
        if($counter >= 0  ){
            $this->return['amounts']    = $request['extra_amounts'];
            $this->return['journals']   = $request['invoice_journals'] ?? [];
        }

        $id = ClearanceInvoice::add_invoice([
            'clearance_id'      => $clearance_id  ,
            'goods_type'        => $request['goods_type'] ,
            'handler'           => $request['handler'] ?? '',
            // 'branch_id'      => $this->return['clearance_m']->branch_id ,
            'finance_year_id'   => $year ,
            'port_id'           => $this->return['clearance_m']->port_id ,
            'customer_id'       => $this->return['clearance_m']->customer_id ,
            'invoice_date'      => $request['invoice_date'] ?? date("Y-m-d") ,
            'total_delegate'    => collect($this->return['deleget_amounts'])->sum('amount') ,
            'total_cost'        => collect($this->return['amounts'])->sum('amount_with_vat') ,
            'total_cost_without_vat'    => collect($this->return['amounts'])->sum('amount_without_vat') ,
            'total_journals'    => collect($this->return['journals'])->sum('amount') ,
            'total_amount'      => collect($this->return['amounts'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,



            // 'total_without_vat'=> collect($this->return['amounts'])->sum('amount_without_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,
            // 'total_vat'     => collect($this->return['amounts'])->sum('vat_amount')
            // 'total_with_vat'=> collect($this->return['amounts'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount')


            'vat_per'           => 15 ,
            'add_user'          => auth()->guard('api')->user()->id,
            'draft'             =>  1
        ]);

        foreach ($this->return['amounts'] as $key => $row) {

            unset($this->return['amounts'][$key]['revenue_name_ar'] ,$this->return['amounts'][$key]['revenue_name_en'] ,$this->return['amounts'][$key]['account_tree_id'],$this->return['amounts'][$key]['account_tree_id'] ,$this->return['amounts'][$key]['add_date'],$this->return['amounts'][$key]['add_user']  );

            $this->return['amounts'][$key]['invoice_m_id'] = $id;
            $this->return['amounts'][$key]['description'] = $this->return['amounts'][$key]['description'] ?? '';
            // $this->return['amounts'][$key]['revenue_id'] = $key+1;
        }
        if(isset($request['pledge_number']) || isset($request['pledge_date'])){
            $clearance = DB::table('clearances_m')->where('clearance_id',$clearance_id)->first();
            DB::table('clearances_m')->where('clearance_id',$clearance_id)->update([
                'pledge_number' => $request['pledge_number'] ?? $clearance->pledge_number,
                'pledge_date'   => $request['pledge_date'] ?? $clearance->pledge_date
            ]);
        }
        ClearanceInvoice::add_invoice_d( $this->return['amounts'] );
        $this->return['deleget_amounts'] = $request['delegate_amounts'];

        foreach ($this->return['deleget_amounts'] as $key => $row) {
            DB::table('clearance_invoices_draft_delegate_d')->insert([
                'invoice_m_id'          => $id ,
                'delegate_petty_d_id'   => $row['d_id'] ?? 0 ,
                'multi_ids'             => $row['multi_ids'] ,
                'petty_ids'             => $row['petty_ids'] ,
                'expense_code_id'       => $row['expense_code_id'] ,
                'receipt_number'        => $row['receipt_number'] ?? 0 ,
                'amount'                => $row['amount'] ,
                'description_ar'        => $row['description'] ?? '' ,
                'description_en'        => $row['description'] ?? ''
            ]);
        }

        foreach ($this->return['journals'] as $key => $row) {
            DB::table('clearance_invoices_draft_journal')->insert([
                'invoice_m_id'          => $id ,
                'account_tree_id'       => $row['account_tree_id']  ,
                'amount'                => $row['amount'] ,
            ]);
        }

        DB::commit();

        return $this->sendResponse($id, trans('main.add_success'));
    }

    public function convert_draft_to_invoice($lang,$invoice_id){


        $this->checkPermission('04-012','add');


        $is_journals    = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );
        $journals       = ClearanceInvoice::get_invoice_journals($invoice_id, $lang);
        $details        = ClearanceInvoice::get_invoice_d($invoice_id, $lang);
        if( $is_journals && collect($journals)->sum('amount') != collect($details)->sum('amount_without_vat') ){
            return $this->sendError( 'Amounts not equal '.collect($details)->sum('amount_without_vat')." - ".collect($journals)->sum('amount') , [], 200);
        }

        $invoice_m          = "clearance_invoices_m";
        $invoice_d          = "clearance_invoices_d";
        $invoice_delefate_d = "clearance_invoices_delegate_d";
        $invoice_journals_t   = "clearance_invoices_journal";
        $invoice_type       = "invoice";
        $helper             = (new ClearanceInvoiceHelper);
        if(collect($details)->sum('amount_with_vat') == 0 ){
            $invoice_m          = "clearance_invoices_claim_m";
            $invoice_d          = "clearance_invoices_claim_d";
            $invoice_delefate_d = "clearance_invoices_claim_delegate_d";
            $invoice_journals_t   = "clearance_invoices_claim_journal";
            $invoice_type       = "claim";
            $helper             = (new ClearanceInvoiceclaimHelper);
        }

       // try {

            DB::beginTransaction();
                DB::table($invoice_m)->lockForUpdate()->get();
                $id                 = intval( DB::table($invoice_m)->max("invoice_id") ) + 1;
                $master             = DB::table('clearance_invoices_draft_m')->where('invoice_id' , $invoice_id)->first();
                $details            = DB::table('clearance_invoices_draft_d')->where('invoice_m_id' , $invoice_id)->get();
                //update clearance details
                $trans_amounts = collect($details)->where('revenue_id',5)->first();
                if(isset($trans_amounts)){
                    $clearances_details = Clearance::get_clearance_d($master->clearance_id,'ar');
                    $clearances_details_count = collect($clearances_details)->count();
                    if($clearances_details_count > 0){
                        foreach($clearances_details as $clearance_detail){
                            DB::table('clearances_d')->where('clearances_d_id',$clearance_detail->clearances_d_id)->update([
                                'trans_vat_amount'          => $trans_amounts->vat_amount / $clearances_details_count,
                                'trans_amount_without_vat'  => $trans_amounts->amount_without_vat /$clearances_details_count,
                                'trans_amount_with_vat'     => $trans_amounts->amount_with_vat / $clearances_details_count,
                            ]);
                        }
                    }

                }

                $delegate_details   = DB::table('clearance_invoices_draft_delegate_d')->where('invoice_m_id' , $invoice_id)->get();
                $invoice_journals   = DB::table('clearance_invoices_draft_journal')->where('invoice_m_id' , $invoice_id)->get();
                // $delegate_ids       = collect($delegate_details)->where('from_type' , 'delegate_petty')->pluck('d_ids');
                // $multi_voucher_ids  = collect($delegate_details)->where('from_type' , 'multi_voucher')->pluck('d_ids');
                //get all containers numbers from the clearances details
                $clearance_m = Clearance::get_clearance($master->clearance_id , $lang);
                $container_numbers = Clearance::get_clearance_d($master->clearance_id,'ar')->pluck('container_number')->toArray();
                $is_first = $clearance_m->invoices_count < 1 ? 1 : 0;

                $master->invoice_id = $id;
                $master->invoice_uuid = Str::uuid();
                $master->draft      = 0;
                $master->draft_invoice_date  = $master->invoice_date;
                $master->invoice_date = date("Y-m-d");
                $master->add_date   = date("Y-m-d H:i:s");
                $master->add_user   = auth()->guard('api')->user()->id;
                $master->is_first           = $is_first ;
                $master->container_count    = $clearance_m->container_count ;
                $master->container_small    = $clearance_m->container_small ;
                $master->container_big      = $clearance_m->container_big ;
                $master->containers_numbers = implode(',',$container_numbers);
                if((new General)->check_financial_year($master->invoice_date) != "true"){
                    return $this->sendError((new General)->check_financial_year($master->invoice_date));
                }
                $master             = (array) $master;
                DB::table($invoice_m)->insert($master);

                foreach ($details as $row){
                    $row = (array) $row;
                    $row['invoice_m_id']        = $id;
                    $row['current_vat']         = $row['vat_amount'] ;
                    $row['current_with_vat']    = $row['amount_with_vat'] ;
                    $row['current_without_vat'] = $row['amount_without_vat'] ;
                    unset($row['clearance_invoices_d_id']);
                    DB::table($invoice_d)->insert($row);
                }
                foreach ($delegate_details as $row){
                    $row = (array) $row;
                    $row['invoice_m_id']    = $id;
                    $row['add_date']        = date("Y-m-d H:i:s");
                    unset($row['clearance_invoices_delegate_d_id']);
                    DB::table($invoice_delefate_d)->insert($row);

                    DB::table('delegate_petty_d')
                        ->whereIn('delegate_petty_d_id' , explode(',', $row['petty_ids']) )
                        ->update( ['invoice_id' => $id , 'invoice_type' => $invoice_type ] );

                    DB::table('multi_voucher_d')
                        ->whereIn('multi_voucher_d_id' , explode(',', $row['multi_ids']) )
                        ->update( ['invoice_id' => $id , 'invoice_type' => $invoice_type ] );

                }
                foreach ($invoice_journals as $row){
                    $row = (array) $row;
                    $row['invoice_m_id'] = $id;
                    unset($row['clearance_invoices_journal_id']);
                    DB::table($invoice_journals_t)->insert($row);
                }

                $journal = $helper->generate_journal($id);
                if(!$journal[0]){
                    DB::rollback();
                    return $this->sendError($journal[1], $journal );
                }

                if( $invoice_type == "invoice" && in_array( request()->header('clientID') ,[ 'buraq_1'])){

                    $ztca   = new ZatcaHelper;
                    $ztca->prepare_clearnace_invoice($id);
                    $res    = $ztca->send_invoice();

                    if(!$res[0]){

                        DB::rollback();
                        DB::table('zatca_log')
                            ->insert([
                                'invoice_id'    => $id  ,
                                'source'        => 'ClearanceInvoice' ,
                                'respones'      => $res[2] ,
                                'add_user'      => auth()->user()->id ,
                                'is_done'       => 0
                            ]);

                        return $this->sendError($res[1], $res ?? [] );
                    }
                }elseif( $invoice_type == "invoice" && in_array( request()->header('clientID') ,[ 'amwaj']) ){
                    // (new ZatcaHelper)->send_clearance_zatca($id);
                }


                DB::table('clearance_invoices_draft_m')->where('invoice_id' , $invoice_id)->delete();
                DB::table('clearance_invoices_draft_d')->where('invoice_m_id' , $invoice_id)->delete();
                DB::table('clearance_invoices_draft_delegate_d')->where('invoice_m_id' , $invoice_id)->delete();
                DB::table('clearance_invoices_draft_journal')->where('invoice_m_id' , $invoice_id)->delete();

                if($invoice_type == "invoice"){
                    DB::table('clearances_m')
                        ->where('clearance_id' ,  $master['clearance_id'])
                        ->update(['clearance_status_id' => 6]);
                }


            // update clearance invoices_count
            DB::table('clearances_m')->where('clearance_id', $master['clearance_id'])->increment('invoices_count');

            DB::commit();

       // }catch (\Exception $e){
       //     DB::rollback();
       //     return $this->sendResponse($this->return, trans('main.update_failed'));

       // }

        return $this->sendResponse($this->return, trans('main.update_success'));

    }



    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($lang, $id)
    {
        $this->checkPermission('03-023','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
        $this->return['master']             = ClearanceInvoice::get_invoice($id, $lang);
        if(!isset($this->return['master']->id)){
            return $this->sendError(trans('error.transaction_not_found' , ['i' => trans('main.draft')]) , [],  200);
        }
        $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
        $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
        $this->return['invoice_journals']   = ClearanceInvoice::get_invoice_journals($id, $lang);
        $this->return['account_tree']       = AccountTree::active_account_trees($lang , array('account_type' => 2 , 'active' => 1 ));
        $this->return['is_journals']        = boolval( Setting::get_main_setting_by_code('invoice_journals')->value ?? 0 );
        $clearance      = Clearance::get_clearance( $this->return['master']->clearance_id , $lang );
        $this->return['customer']          = Customer::get_customer( $this->return['master']->customer_id , $lang );
        $this->return['citys']          = DB::table('citys')->selectRaw('city_id , name_'.$lang.' as name')->get();

        $this->return['totals']             = [
            'total_with_vat'    => $this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') ,
            'total_vat'         => $this->return['details']->sum('vat_amount') ,
            'total_without_vat' => $this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_without_vat')
        ];
        $this->return['prices']          = DB::table('customer_transport_prices')
            ->leftjoin('areas as from_areas' , 'customer_transport_prices.from_area_id' , '=' , 'from_areas.area_id')
            ->leftjoin('areas as to_areas' , 'customer_transport_prices.to_area_id' , '=' , 'to_areas.area_id')
            ->where('port_id' , $clearance->port_id)
            ->where('customer_id' , $clearance->customer_id)
            ->select('from_areas.area_name_'.$lang.' as from_area_name','to_areas.area_name_'.$lang.' as to_area_name','from_area_id' , 'to_area_id','price' , 'container_20' , 'container_40','price_special' , 'container_20_special' , 'container_40_special' )
            ->where('customer_transport_price_active' , 1)
            ->get();

            $clearance_id = $this->return['master']->clearance_id;
        if(!$this->get_invoice_amounts($clearance_id , $lang) ){
            return $this->sendError($this->return , [],  200);
        }

        if($this->return['deleget_amounts']->sum('amount') == 0 && count($this->return['amounts']) == 0   ){
            return $this->sendError( trans('main.no_amount_found'), [], 200);
        }


        $this->return['next_item']      = ClearanceInvoice::get_next( $id , 0 )->m_id ?? 0;
        $this->return['previous_item']  = ClearanceInvoice::get_previous($id , 0 )->m_id ?? 0;
        $this->return['first_item']     = 1;
        $this->return['last_item']      = ClearanceInvoice::get_last( 0 )->m_id ?? 0;




        return $this->sendResponse($this->return);
    }

    protected function update_delegate_amounts($draft_id , $clearance_id)
    {
        $lang = 'ar';
        $this->get_invoice_amounts($clearance_id , $lang);

        DB::table('clearance_invoices_draft_delegate_d')->where('invoice_m_id',$draft_id)->delete();
        foreach ($this->return['deleget_amounts'] as $key => $row) {
            DB::table('clearance_invoices_draft_delegate_d')->insert([
                'invoice_m_id'          => $draft_id ,
                'delegate_petty_d_id'   => $row->d_id ?? 0 ,
                'petty_ids'             => $row->petty_ids ,
                'multi_ids'             => $row->multi_ids ,
                'expense_code_id'       => $row->expense_code_id ,
                'receipt_number'        => $row->receipt_number ?? 0 ,
                'amount'                => $row->amount ,
                'description_ar'        => $row->description ?? '' ,
                'description_en'        => $row->description ?? ''
            ]);
        }

        $this->return['master']             = ClearanceInvoice::get_invoice($draft_id, $lang);
        $this->return['details']            = ClearanceInvoice::get_invoice_d($draft_id, $lang);
        $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($draft_id, $lang);
        $this->return['invoice_journals']   = ClearanceInvoice::get_invoice_journals($draft_id, $lang);
        ClearanceInvoice::update_invoice([
            'total_delegate'        => collect($this->return['deleget_amounts'])->sum('amount') ,
            'total_cost'            => collect($this->return['details'])->sum('amount_with_vat') ,
            'total_amount'          => collect($this->return['details'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,
            'total_cost_without_vat'=> collect($this->return['details'])->sum('amount_without_vat') ,

            // 'total_without_vat'=> collect($this->return['details'])->sum('amount_without_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,
            // 'total_vat'     => collect($this->return['details'])->sum('vat_amount')
            // 'total_with_vat'=> collect($this->return['details'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount')



        ],$draft_id);

    }

    public function update($lang,$draft_id,Request $request)
    {
        $this->checkPermission('03-023','edit');

        $get_draft = ClearanceInvoice::get_invoice($draft_id,$lang);
        if((new General)->check_financial_year($get_draft->invoice_date) != "true"){
            return $this->sendError((new General)->check_financial_year($get_draft->invoice_date));
        }
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
        if(!$this->get_invoice_amounts($get_draft->clearance_id , $lang) ){
            return $this->sendError($this->return , [],  200);
        }

        if($this->return['deleget_amounts']->sum('amount') == 0 && count($this->return['amounts']) == 0   ){
            return $this->sendError('No Amounts found', [], 200);
        }

        DB::beginTransaction();

        if(isset($request['clearance_transport_fees']) && is_array(isset($request['clearance_transport_fees'])) && count($request['clearance_transport_fees']) > 0 ){
            foreach ($request['clearance_transport_fees'] as $key => $row) {
                DB::table('clearances_d')
                    ->where('clearances_m_id' , $row['clearances_m_id'])
                    ->where('clearances_d_id' , $row['clearances_d_id'])
                    ->update( ['from_area' => $row['from_area'] , 'to_area' => $row['to_area'] , ] );
            }
        }

        $counter = DB::table('clearance_invoices_draft_m')->where('clearance_id' , $request['clearance_id'] )->where('invoice_active' , 1 )->count();
        if($counter >= 0  ){
            $this->return['amounts']    = $request['extra_amounts'];
            $this->return['journals']   = $request['invoice_journals'];
        }

        ClearanceInvoice::update_invoice([
            'goods_type'         => $request['goods_type'] ,
            'total_delegate'=> collect($this->return['deleget_amounts'])->sum('amount') ,
            'total_cost'    => collect($this->return['amounts'])->sum('amount_with_vat') ,
            'total_amount'  => collect($this->return['amounts'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,
            'total_journals'    => collect($this->return['journals'])->sum('amount') ,
            'total_cost_without_vat'    => collect($this->return['amounts'])->sum('amount_without_vat') ,


            // 'total_without_vat'=> collect($this->return['amounts'])->sum('amount_without_vat') + collect($this->return['deleget_amounts'])->sum('amount') ,
            // 'total_vat'     => collect($this->return['amounts'])->sum('vat_amount')
            // 'total_with_vat'=> collect($this->return['amounts'])->sum('amount_with_vat') + collect($this->return['deleget_amounts'])->sum('amount')

        ],$draft_id);

        foreach ($this->return['amounts'] as $key => $row) {
            unset($this->return['amounts'][$key]['revenue_name_ar'] ,$this->return['amounts'][$key]['revenue_name_en'] ,$this->return['amounts'][$key]['account_tree_id'],$this->return['amounts'][$key]['account_tree_id'],$this->return['amounts'][$key]['add_date'],$this->return['amounts'][$key]['add_user'],$this->return['amounts'][$key]['revenues_active'],$this->return['amounts'][$key]['revenues_order']   );
            $this->return['amounts'][$key]['invoice_m_id'] = $draft_id;
            $this->return['amounts'][$key]['description'] = $this->return['amounts'][$key]['description'] ?? '';
        }
        ClearanceInvoice::delete_invoice_d($draft_id);
        ClearanceInvoice::add_invoice_d( $this->return['amounts'] );

        if(isset($request['pledge_number']) || isset($request['pledge_date'])){
            $clearance = DB::table('clearances_m')->where('clearance_id',$get_draft->clearance_id)->first();
            DB::table('clearances_m')->where('clearance_id',$get_draft->clearance_id)->update([
                'pledge_number' => $request['pledge_number'] ?? $clearance->pledge_number,
                'pledge_date'   => $request['pledge_date'] ?? $clearance->pledge_date
            ]);
        }

        DB::table('clearance_invoices_draft_delegate_d')->where('invoice_m_id',$draft_id)->delete();
        foreach ($this->return['deleget_amounts'] as $key => $row) {
            DB::table('clearance_invoices_draft_delegate_d')->insert([
                'invoice_m_id'          => $draft_id ,
                'delegate_petty_d_id'   => $row->d_id ?? 0 ,
                'petty_ids'             => $row->petty_ids ,
                'multi_ids'             => $row->multi_ids ,
                'expense_code_id'       => $row->expense_code_id ,
                'receipt_number'        => $row->receipt_number ?? 0 ,
                'amount'                => $row->amount ,
                'description_ar'        => $row->description ?? '' ,
                'description_en'        => $row->description ?? ''
            ]);
        }

        DB::table('clearance_invoices_draft_journal')->where('invoice_m_id',$draft_id)->delete();
        foreach ($this->return['journals'] as $key => $row) {
            DB::table('clearance_invoices_draft_journal')->insert([
                'invoice_m_id'          => $draft_id ,
                'account_tree_id'       => $row['account_tree_id']  ,
                'amount'                => $row['amount']
            ]);
        }


        DB::commit();

        return $this->sendResponse([], trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('03-023','delete');
        $get_draft = ClearanceInvoice::get_invoice($id,$lang);

        if((new General)->check_financial_year($get_draft->invoice_date) != "true"){
            return $this->sendError((new General)->check_financial_year($get_draft->invoice_date));
        }
        DB::beginTransaction();

        ClearanceInvoice::hard_delete_invoice($id);

        DB::commit();

        return $this->sendResponse([], trans('main.delete_success'));
    }


    public function print( $id , Request $request)
    {

        // $this->checkPermission('03-023','print');
        $lang = 'ar';
        $this->return['lang']               = $lang;
        $this->return['master']             = ClearanceInvoice::get_invoice($id, $lang);
        $this->return['details']            = ClearanceInvoice::get_invoice_d($id, $lang);
        $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($id, $lang);
        $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
        $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
        $this->return['containers']         = implode(' , ' , Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
        $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_amount , 2) , 'SAR' );

        $this->return['qrcode']             = GenerateQrCode::fromArray([
            new Seller($this->return['branch_data']->vat_register_name),
            new TaxNumber($this->return['branch_data']->vat_number),
            new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
            new InvoiceTotalAmount(($this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') )), // Invoice Total Amount
            new InvoiceTaxAmount(round($this->return['details']->sum('vat_amount') , 2)) // Invoice Tax Amount
        ])->render();
        $temp_name  = $request['print_temp'] ?? Setting::get_main_setting_by_code('draft_invoice_print')->value ?? 'temp_1' ;
        $print_temp = 'Print.clearance.draft.'.$temp_name;

        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , $print_temp);
                return Excel::download($export, 'balances.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView($print_temp, array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
                break;
            case 'image':
                $image = App::make('snappy.image.wrapper');
                $image->loadView($print_temp, array('data' => $this->data));
                return $image->stream();
                break;
            case 'print':
                return view($print_temp)->with('data' , $this->return);
                break;
            case 'json':
                return view($print_temp)->with('data' , $this->return);
                break;
        }


        return $this->sendResponse($this->return);
    }
    public function print_invoices($lang ,Request $request){
        $this->checkPermission('03-023','print');

        $this->filter = array(
            'lang'      => $lang ,
            'active'    => $request['active'] ?? 1 ,
            'rows'      => $request['rows'] ?? null ,
            'page'      => $request['page'] ?? 1 ,
            'word'      => $request['word'] ?? null ,
            'from_id'   => $request['from_id'] ?? null ,
            'to_id'     => $request['to_id'] ?? null ,
        );

        $this->return['screen_name']  = ScreenSub::get_screen_sub_by_screen_code('03-023', $lang)->sub_title ?? '';
        $this->return['invoices']     = ClearanceInvoice::all_invoices($this->filter);
        $this->return['logo']         = $this->return['branch_data']->branch_image ?? 0;
        $temp_name  = $request['print_temp'] ?? Setting::get_main_setting_by_code('draft_invoice_print')->value ?? 'temp_1' ;

        $allInvoicesHtml = '';

        foreach ($this->return['invoices']  as $invoice) {
            $this->return['logo']               = public_path().$this->return['logo'];
            $this->return['lang']               = $lang;
            $this->return['master']             = ClearanceInvoice::get_invoice($invoice->id, $lang);
            $this->return['details']            = ClearanceInvoice::get_invoice_d($invoice->id, $lang);
            $this->return['delegate_details']   = ClearanceInvoice::get_invoice_delegate_d($invoice->id, $lang);
            $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id  , $lang);
            $this->return['containers']         = implode(' , ' , Clearance::get_clearance_d($this->return['master']->clearance_id)->pluck('container_number')->toArray() );
            $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_amount , 2) , 'SAR' );

            $this->return['qrcode']             = GenerateQrCode::fromArray([
                new Seller($this->return['branch_data']->vat_register_name),
                new TaxNumber($this->return['branch_data']->vat_number),
                new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
                new InvoiceTotalAmount(($this->return['delegate_details']->sum('amount') + $this->return['details']->sum('amount_with_vat') )), // Invoice Total Amount
                new InvoiceTaxAmount(round($this->return['details']->sum('vat_amount') , 2)) // Invoice Tax Amount
            ])->render();
            $invoice_html = view('Print/clearance/'.$temp_name, ['data' => $this->return])->render();
            $invoice_html .= '<div style="page-break-after: always;"></div>';
            $allInvoicesHtml .= $invoice_html;
        }
        $allInvoicesHtml .= '<script type="text/javascript">window.print();</script>';
        return  $allInvoicesHtml;
    }

}
