<?php
namespace App\Http\Controllers\API\V1\Sales;

use App\Classes\General\General;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Sales\InsuranceHelper;
use App\Classes\General\Upload;



use App\Models\Finance\AccountTree;
use App\Models\Sales\CovenantType;
use App\Models\Sales\Insurance;
use App\Models\Sales\Clearance;
use App\Models\Main\ScreenSub;
use App\Models\Trans\Agent;
use App\Models\Trans\Port;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Storage;

class InsuranceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('03-027','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-027', $lang)->sub_title ?? '';
        $this->return['items']       = Insurance::all_insurances($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('03-027','add');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-027', $lang)->sub_title ?? '';
        $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('03-027','add');

        $insert = $request->only(['action_date', 'clearance_id', 'amount', 'agent_id' , 'return_date']);

        if((new General)->check_financial_year($insert['action_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['action_date']));
        }
        $insert['add_user'] = auth()->user()->id;

        $validator = validator()->make($insert, Insurance::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $check = Insurance::all_insurances(['active' => 1 , 'lang' => 'en' , 'clearance_id' => $insert['clearance_id']]);

        if(count($check) > 0 ){
            return $this->sendError( 'Already have insurance transaction # '.$check[0]->id , [], 200 );
        }

        $this->return['id'] = Insurance::add_insurance($insert);
        (new InsuranceHelper)->update_action($this->return['id']);


        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('03-027','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-027', $lang)->sub_title ?? '';
        $this->return['data']           = Insurance::get_insurance($id, $lang);
        $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('03-027','edit');

        $update = $request->only(['action_date', 'clearance_id', 'amount', 'agent_id' , 'return_amount' , 'return_date' ]);

        $item = Insurance::find($id);
        if((new General)->check_financial_year($item->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->action_date));
        }
        if(request()->hasFile('file_upload')){

            $upload = new Upload;
            $check  = $upload->uploadFile(request()->file('file_upload') , 'insurance_return_file' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
            $update['return_file'] = $check['url'] ;
        }

        $validator = validator()->make($update, Insurance::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        Insurance::update_insurance($update, $id);
        (new InsuranceHelper)->update_action($id);

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('03-027','delete');
        $item = Insurance::find($id);
        if((new General)->check_financial_year($item->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->action_date));
        }
        Insurance::delete_insurance($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = Insurance::find($id);
        if((new General)->check_financial_year($item->action_date) != "true"){
            return $this->sendError((new General)->check_financial_year($item->action_date));
        }
        if($item->insurance_active){
            $this->checkPermission('03-027','delete');

            Insurance::delete_insurance($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('03-027','edit');

            Insurance::update_insurance(array('insurance_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function clearance_d_insurances_get($lang,Request $request)
    {
        // $this->checkPermission('03-027','view');

        $this->filter = array(
            'lang'          => $lang ,
            'active'        => $request['active'] ?? 1 ,
            'rows'          => $request['rows'] ?? 15 ,
            'page'          => $request['page'] ?? 1 ,
            'word'          => $request['word'] ?? null ,
            'agent_id'      => $request['agent_id'] ?? null ,
            'archive_id'    => $request['archive_id'] ?? null ,
            'clearance_id'  => $request['clearance_id'] ?? null ,
            'from_date'     => $request['from_date'] ?? null ,
            'to_date'       => $request['to_date'] ?? null
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-027', $lang)->sub_title ?? '';
        $this->return['items']          = Clearance::get_clearance_d_insurance($this->filter);
        $this->return['agents']         = Agent::all_agents(array('active' => 1, 'lang' => $lang));
        $this->return['ports']          = Port::all_ports(array('active' => 1, 'lang' => $lang));



        return $this->sendResponse($this->return);
    }


    public function clearance_d_insurances_post($lang  , $id , Request $request)
    {
        // $this->checkPermission('03-027','view');

        $update = [
            'return_date'   => $request['return_date'] ,
            'archive_id'    => ( intval(DB::table('clearances_d')->max('archive_id')) + 1 )
        ];

        $upload = new Upload;
        $check  = $upload->uploadFile(request()->file('return_file') , 'clearances_d_return_file' );

        if(!$check['check']){
            return $this->sendError( trans('error.add_fails') , $check['msg'] );
        }
        $update['return_file'] = $check['url'] ;

        DB::connection('other_mysql')->beginTransaction();
            DB::connection('other_mysql')->table('clearances_d')
                ->where('clearances_d_id' , $id )
                ->update( $update );

            $insurance_id = DB::table('insurances')
                ->where('clearance_id' , DB::table('clearances_d')->where('clearances_d_id' , $id )->first()->clearances_m_id ?? 0 )
                ->first()->insurance_id ?? 0 ;

            if($insurance_id > 0 ){
                (new InsuranceHelper)->update_action($insurance_id );
            }


        DB::connection('other_mysql')->commit();

        return $this->sendResponse([], trans('main.update_success'));
    }
    public function update_clearance_d_insurances($lang  , $id , Request $request)
    {
        // $this->checkPermission('03-027','edit');

        $upload = new Upload;
        $check  = $upload->uploadFile(request()->file('return_file') , 'clearances_d_return_file' );

        if(!$check['check']){
            return $this->sendError( trans('error.add_fails') , $check['msg'] );
        }
        $return_file = $check['url'] ;

        DB::connection('other_mysql')->beginTransaction();
            DB::connection('other_mysql')->table('clearances_d')
                ->where('clearances_d_id' , $id )
                ->update( ['return_file' => $return_file] );

            $old_return_file = DB::connection('other_mysql')->table('clearances_d')
            ->where('clearances_d_id' , $id )
            ->value('return_file');
            if(Storage::exists($old_return_file)){
                Storage::delete($old_return_file);
            }
        DB::connection('other_mysql')->commit();

        return $this->sendResponse([], trans('main.update_success'));
    }
    public function update_all_completed_insurances($lang , Request $request)
    {
        // $this->checkPermission('03-027','edit');

        //Validations
        $validator = validator()->make($request->only('insurance_ids'), ['insurance_ids' => 'required|array','insurance_ids.*' => 'numeric']);
        if($validator->fails())
        {
            return $this->sendError(trans('error.update_fails'), $validator->errors());
        }

        try {
            $insurance_ids = $request['insurance_ids'];
            DB::table('insurances')
                ->whereIn('insurance_id',$insurance_ids)
                ->update([
                'status'        => 2 ,
                'return_amount' => DB::raw('amount'),
                'return_date'   => now()
            ]);
        }catch (\Exception $e){
            return $this->sendError($e->getMessage(), [], 500);
        }


        return $this->sendResponse([], trans('main.update_success'));

    }
    public function send_return_file($lang, $id, Request $request){

        $this->checkPermission('04-012', 'view');

        // Fetch file path from DB
        $file = DB::connection('other_mysql')
            ->table('clearances_d')
            ->where('clearances_d.clearances_d_id', $id)
            ->value('clearances_d.return_file');

        if (!$file || !file_exists(public_path($file))) {
            return $this->sendError('File not found.');
        }

        // Get customer contacts
        $customer_contacts = $request->input('customer_contacts', []);
        if (!is_array($customer_contacts)) {
            $customer_contacts = [$customer_contacts];
        }

        // Filter valid emails only
        $valid_emails = array_filter($customer_contacts, function ($email) {
            return filter_var($email, FILTER_VALIDATE_EMAIL);
        });

        if (empty($valid_emails)) {
            return $this->sendError('No valid customer contacts found.');
        }

        $file_path =public_path($file);
        // Send email to each valid contact
        foreach ($valid_emails as $email) {
            Mail::send([], [], function ($message) use ($email, $file_path) {

                $message->to($email)
                    ->subject('Return File')
                    ->setBody('Please find the attached return file.', 'text/html');
                    // attach it here
                $message->attach($file_path);
            });
        }

        // insert log
        DB::table('invoices_sent_logs')->insert([
            'invoice_id'  => $id,
            'add_user'    => auth()->user()->id,
            'type'        => 'email',
            'data'        => json_encode($request->all()),
            'response'    => isset($response) ? json_encode($response) : null
        ]);
        return $this->sendResponse( $response ?? [] , trans('main.send mail success') );

    }

}
