<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Finance\AccountTree;
use App\Models\Sales\CovenantType;
use App\Models\Sales\Item;
use App\Models\Main\ScreenSub;
use Illuminate\Http\Request;

class ItemController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('04-006','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-006', $lang)->sub_title ?? '';
        $this->return['items']       = Item::all_items($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('04-006','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-006', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('04-006','add');

        $insert             = $request->only(['name_en', 'name_ar']);
        $insert['add_user'] = auth()->user()->id;
        $validator          = validator()->make($insert, Item::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $insert['id'] = Item::add_item($insert);
        $this->return = $insert;

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('04-006','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-006', $lang)->sub_title ?? '';
        $this->return['data']           = Item::get_item($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('04-006','edit');

        $update = $request->only(['name_en', 'name_ar']);

        $validator = validator()->make($update, Item::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        Item::update_item($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('04-006','delete');

        Item::delete_item($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = Item::find($id);

        if($item->item_active){
            $this->checkPermission('04-006','delete');

            Item::delete_item($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('04-006','edit');

            Item::update_item(array('item_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
