<?php
namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Models\Main\ScreenSub;
use App\Models\Sales\Quotation;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\Rule;

class QuotationController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('03-057','view');

        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-057', $lang)->sub_title ?? '';
        $this->return['items']       = Quotation::get_all($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('03-057','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-057', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('03-057','add');
        $inputs = $request->all();
        $insert = [
            'customer_name'         => $inputs['customer_name'] ?? null,
            'responsible_person'    => $inputs['responsible_person'] ?? null,
            'customer_code'         => $inputs['customer_code'] ?? null,
            'add_user'              => auth()->user()->id,

        ];
        $validator = validator()->make($insert, Quotation::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }
        DB::beginTransaction();
        $this->return['id'] = Quotation::add_master($insert);
        if(isset($inputs['details']) && is_array($inputs['details']) && count($inputs['details']) > 0){
            foreach($inputs['details'] as $item){
                $detail = ['m_id' =>$this->return['id'] , 'statement' => $item['statement'] , 'amount' => $item['amount']];
                $d_id = Quotation::add_detail($detail);

            }
        }
        DB::commit();
        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id )
    {
        $this->checkPermission('03-057','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-057', $lang)->sub_title ?? '';
        $this->return['item']           = Quotation::get_one($id, $lang);
        $this->return['details']        = Quotation::get_details($id, $lang);

        return $this->sendResponse($this->return);
    }
    public function get_by_customer($lang, Request $request )
    {
        $this->checkPermission('03-057','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-057', $lang)->sub_title ?? '';
        if(isset($request->customer_name)){
            $this->return['item']           = Quotation::get_by_customer($request->customer_name, $lang);
            if(!isset($this->return['item'])){
                return $this->sendResponse(null);
            }
            $this->return['details']        = Quotation::get_details($this->return['item']->id, $lang);
        }
        return $this->sendResponse($this->return);
    }
    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('03-057','edit');

        $inputs = $request->all();
        $update = [
            'customer_name'         => $inputs['customer_name'] ?? null,
            'responsible_person'    => $inputs['responsible_person'] ?? null,
            'customer_code'         => $inputs['customer_code'] ?? null,
        ];
        $update_rules = [
            'customer_name' => [
                'required',
                'string',
                Rule::unique('quotations')->ignore($id,'id'),
            ]

        ];
        $validator = validator()->make($update, $update_rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }
        DB::beginTransaction();
        Quotation::update_one($update, $id);
        if(isset($inputs['details']) && is_array($inputs['details']) && count($inputs['details']) > 0){
            DB::table('quotation_details')->where('m_id', $id)->delete();
            foreach($inputs['details'] as $item){
                $detail = ['m_id' => $id , 'statement' => $item['statement'] , 'amount' => $item['amount']];
                 Quotation::add_detail($detail);

            }
        }
        DB::commit();
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('03-057','delete');

        Quotation::delete_one($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $item = Quotation::find($id);

        if($item->active){
            $this->checkPermission('03-057','delete');

            Quotation::delete_one($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('03-057','edit');

            Quotation::update_one(array('active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }

}
