<?php

namespace App\Http\Controllers\API\V1\Sales;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Sales\CovenantType;
use App\Models\Sales\Unit;
use App\Models\Main\ScreenSub;

class UnitController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang, Request $request)
    {
        $this->checkPermission('04-008','view');


        $this->filter = array(
            'lang'   => $lang ,
            'active' => $request['active'] ?? 1 ,
            'rows'   => $request['rows'] ?? null ,
            'page'   => $request['page'] ?? 1 ,
            'word'   => $request['word'] ?? null ,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('04-008', $lang)->sub_title ?? '';
        $this->return['units']       = Unit::all_units($this->filter);

        return $this->sendResponse($this->return);
    }

    public function create($lang, Request $request)
    {
        $this->checkPermission('04-008','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-008', $lang)->sub_title ?? '';

        return $this->sendResponse($this->return);
    }

    public function store($lang, Request $request)
    {
        $this->checkPermission('04-008','add');

        $insert = $request->only(['name_en', 'name_ar']);
        $insert['add_user'] = auth()->user()->id;

        $validator = validator()->make($insert, Unit::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        $insert['id'] = Unit::add_unit($insert);
        $this->return = $insert;

        return $this->sendResponse($this->return, trans('main.add_success'));
    }

    public function show($lang, $id)
    {
        $this->checkPermission('04-008','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('04-008', $lang)->sub_title ?? '';
        $this->return['data']           = Unit::get_unit($id, $lang);

        return $this->sendResponse($this->return);
    }

    public function update($lang, $id, Request $request)
    {
        $this->checkPermission('04-008','edit');

        $update = $request->only(['name_en', 'name_ar']);

        $validator = validator()->make($update, Unit::$rules);

        if($validator->fails())
        {
            return $this->sendError(trans('error.add_fails'), $validator->errors());
        }

        Unit::update_unit($update, $id);
        return $this->sendResponse($this->return, trans('main.update_success'));
    }

    public function destroy($lang, $id)
    {
        $this->checkPermission('04-008','delete');

        Unit::delete_unit($id);
        return $this->sendResponse([], trans('main.delete_success'));
    }

    public function toggle_active($lang, $id)
    {
        $unit = Unit::find($id);

        if($unit->unit_active){
            $this->checkPermission('04-008','delete');

            Unit::delete_unit($id);
            $this->return['active'] = false;
        }else{
            $this->checkPermission('04-008','edit');

            Unit::update_unit(array('unit_active' => 1), $id);
            $this->return['active'] = true;
        }

        return $this->sendResponse($this->return, trans('main.update_success'));
    }
}
