<?php

namespace App\Http\Controllers\API\V1\Trans;

use App\Http\Controllers\API\V1\BaseController;
use App\Http\Controllers\Controller;
use App\Models\Finance\AccountTree;
use App\Models\Main\ScreenSub;
use App\Models\Trans\Agent;
use App\Models\Trans\Area;
use Illuminate\Http\Request;

class AgentController extends BaseController
{
    protected $return = array();
    protected $filter = array();
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index($lang , Request $request)
    {
        $this->checkPermission('03-022','view');
        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-022' , $lang)->sub_title ?? '';
        $this->return['items']       = Agent::all_agents( $this->filter );

        return $this->sendResponse( $this->return );
    }


    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\JsonResponse
     */
    public function create($lang , Request $request)
    {
        $this->checkPermission('03-022','add');
        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-022' , $lang)->sub_title ?? '';
        $this->return['areas']      = Area::all_areas(array('lang' => $lang  , 'active' => 1 ));
        return $this->sendResponse( $this->return  );
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store($lang , Request $request)
    {
        $this->checkPermission('03-022','add');
        $insert = $request->only(['agent_name_ar', 'agent_name_en','area_id','mobile']);


        $validator = validator()->make( $insert , Agent::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Agent::add_agent($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }


    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-022','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-022' , $lang)->sub_title ?? '';
        $this->return['agent']           = Agent::get_agent($id,$lang);
        $this->return['areas']          = Area::all_areas(['lang' => $lang, 'active' => 1 ]);

        return $this->sendResponse( $this->return , '' );
    }


    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-022','edit');

        $update = $request->only(['agent_name_ar', 'agent_name_en','area_id','mobile']);

        $rules            = Agent::$rules;
        $rules['agent_name_ar'] = $rules['agent_name_ar'].',agent_name_ar,'.$id.',agent_id';
        $rules['agent_name_en'] = $rules['agent_name_en'].',agent_name_en,'.$id.',agent_id';

        $validator  = validator()->make( $update , $rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        Agent::update_agent($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($lang , $id)
    {
        $this->checkPermission('03-022','delete');

        Agent::delete_agent($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }


    public function sort_post($lang , Request $request){
        $this->checkPermission('03-022','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Customer::update_customer(array('customer_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Agent::find($id);
        if($item->agent_active){
            $this->checkPermission('03-022','delete');

            Agent::delete_agent($id);
            $this->return['active'] = false;

        }else{
            $this->checkPermission('03-022','edit');
            Agent::update_agent(array('agent_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
