<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Trans\Area;
use App\Models\Main\ScreenSub;


use Config;
use DB;

class AreaController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-003','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-003' , $lang)->sub_title ?? '';
        $this->return['items']       = Area::all_areas( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-003','add');

        if(request()->header('clientID') != 'trans' )
            return $this->sendError( trans('error.add_fails') , [] );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-003' , $lang)->sub_title ?? '';

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-003','add');


        $insert = $request->only(['area_name_en', 'area_name_ar']);
        $validator = validator()->make( $insert , Area::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Area::add_area($insert);


        DB::connection('other_mysql')->table('areas')->insert( $insert );


        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-003','view');


        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-003' , $lang)->sub_title ?? '';
        $this->return['data']           = Area::find($id);

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-003','edit');

        // $check = $this->can_update($id);
        // if($check[0] ==  false){
        //     return $this->sendError( $check[1] , []);
        // }

        $update = $request->only(['area_name_en', 'area_name_ar']);

        $rules            = Area::$rules;
        $rules['area_name_ar'] = $rules['area_name_ar'].',area_name_ar,'.$id.',area_id';
        $rules['area_name_en'] = $rules['area_name_en'].',area_name_en,'.$id.',area_id';


        $validator  = validator()->make( $update , $rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['area_id'] , $update['area_active'] , $update['_method'] );

        Area::update_area($update ,  $id );
        DB::connection('other_mysql')->table('areas')->where('area_id' , $id)->update( $update );


        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-003','delete');

        // $check = $this->can_delete($id);
        // if($check[0] ==  false){
        //     return $this->sendError( $check[1] , []);
        // }



        Area::delete_area($id);


        DB::connection('other_mysql')->table('areas')->where('area_id' , $id)->update( ['area_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id] );


        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-003','edit');


        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Area::update_area(array('area_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){



        $item = Area::find($id);
        if($item->area_active){

            $this->checkPermission('03-003','delete');




            Area::delete_area($id);


            DB::connection('other_mysql')->table('areas')->where('area_id' , $id)->update( ['area_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id] );

            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-003','edit');





            Area::update_area(array('area_active' => 1 ) , $id);
            DB::connection('other_mysql')->table('areas')->where('area_id' , $id)->update( ['area_active' => 1] );


            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function can_update($id){

        if(request()->header('clientID') != 'trans' )
            return [ false , 'لا يمكن التعديل الان' ];

        return [true];

    }

    public function can_delete($id){

        if(request()->header('clientID') != 'trans' )
            return [false , 'لا يمكن المسح الان' ];


        $trans = DB::table('customer_transport_prices')
            ->where('customer_transport_price_active' , 1)
            ->where(function($where) use($id){
                $where->where('from_area_id' , $id)
                ->orWhere('to_area_id' , $id);
            })
            ->count();
        if($trans > 0 )
            return [false , 'لا يمكن المسح لوجود اسعر ردود' ];

        $trans2 = DB::table('area_prices')
            ->where('area_price_active' , 1)
            ->where(function($where) use($id){
                $where->where('from_area_id' , $id)
                ->orWhere('to_area_id' , $id);
            })
            ->count();
        if($trans2 > 0 )
            return [false , 'لا يمكن المسح لوجود اسعر نقل' ];

        $amwaj = DB::connection('other_mysql')->table('customer_transport_prices')
            ->where('customer_transport_price_active' , 1)
            ->where(function($where) use($id){
                $where->where('from_area_id' , $id)
                ->orWhere('to_area_id' , $id);
            })
            ->count();
        if($amwaj > 0 )
            return [false , 'لا يمكن المسح لوجود اسعر ردود' ];

        $amwaj2 = DB::connection('other_mysql')->table('area_prices')
            ->where('area_price_active' , 1)
            ->where(function($where) use($id){
                $where->where('from_area_id' , $id)
                ->orWhere('to_area_id' , $id);
            })
            ->count();

        if($amwaj2 > 0 )
            return [false , 'لا يمكن المسح لوجود اسعر نقل' ];

        return [true];

    }

}
