<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Trans\Area;
use App\Models\Trans\AreaPrice;
use App\Models\Main\ScreenSub;


class AreaPriceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-006','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'from_area'   => $request['from_area'] ?? null ,
            'to_area'     => $request['to_area'] ?? null ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-006' , $lang)->sub_title ?? '';
        $this->return['items']       = AreaPrice::all_area_prices( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-006','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-006' , $lang)->sub_title ?? '';
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-006','add');
        $insert = $request->only(['from_area_id', 'to_area_id' , 'driver_price_in' , 'driver_price_out' , 'trip_price_in' , 'trip_price_out']);
        $validator = validator()->make( $insert , AreaPrice::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = AreaPrice::add_area_price($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-006','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-006' , $lang)->sub_title ?? '';
        $this->return['data']           = AreaPrice::find($id);
        $this->return['areas']          = Area::all_areas( ['active' => 1 , 'lang' => $lang] );

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-006','edit');

        $update = $request->only(['from_area_id', 'to_area_id' , 'driver_price_in' , 'driver_price_out' , 'trip_price_in' , 'trip_price_out']);
        $validator  = validator()->make( $update , AreaPrice::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['area_price_id'] , $update['area_price_active'] , $update['_method'] );
        AreaPrice::update_area_price($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-006','delete');

        AreaPrice::delete_area_price($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-006','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            AreaPrice::update_area_price(array('area_price_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = AreaPrice::find($id);
        if($item->area_price_active){

            $this->checkPermission('03-006','delete');

            AreaPrice::delete_area_price($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-006','edit');

            AreaPrice::update_area_price(array('area_price_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
