<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Classes\General\Upload;


use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Trans\Car;
use App\Models\Main\ScreenSub;
use App\Models\Trans\Driver;


class CarController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-008','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-008' , $lang)->sub_title ?? '';
        $this->return['items']       = Car::all_cars( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-008','add');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-008' , $lang)->sub_title ?? '';
        $this->return['driver']         = Driver::all_drivers( ['lang' => $lang , 'acrive' => 1 ] );

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-008','add');
        $insert = $request->only(['car_name_en', 'car_name_ar' , 'default_driver' , 'weight' ,'car_code' ,'surface_number' , 'axes_number' , 'car_type' , 'form_expiry_date' , 'form_expiry_image' , 'card_expiry_date' , 'card_expiry_image']);
        $validator = validator()->make( $insert , Car::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        if(isset($request['form_expiry_image']) && strlen($request['form_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['form_expiry_image'] , 'car_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }


        if(isset($request['card_expiry_image']) && strlen($request['card_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['card_expiry_image'] , 'car_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }


        $insert['add_user'] = auth()->user()->id;
        $this->return['id'] = Car::add_car($insert);

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-008','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-008' , $lang)->sub_title ?? '';
        $this->return['data']           = Car::find($id);
        $this->return['driver']         = Driver::all_drivers( ['lang' => $lang , 'acrive' => 1 ] );

        return $this->sendResponse( $this->return , '' );
    }

    public function update($lang , $id , Request $request)
    {
        $this->checkPermission('03-008','edit');

        $update = $request->only(['car_name_en', 'car_name_ar', 'default_driver' ,'weight' , 'car_code' ,'surface_number' , 'axes_number' , 'car_type' , 'form_expiry_date' , 'form_expiry_image' , 'card_expiry_date' , 'card_expiry_image']);
        $validator  = validator()->make( $update , Car::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }
        unset($update['car_id'] , $update['car_active'] , $update['_method'] );


        if(isset($request['form_expiry_image']) && strlen($request['form_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['form_expiry_image'] , 'car_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }


        if(isset($request['card_expiry_image']) && strlen($request['card_expiry_image']) > 10 ){

            $upload = new Upload;
            $check  = $upload->uploadBase64File($request['card_expiry_image'] , 'car_files' );

            if(!$check['check']){
                return $this->sendError( trans('error.add_fails') , $check['msg'] );
            }
        }


        Car::update_car($update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-008','delete');

        Car::delete_car($id);

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function sort_post($lang , Request $request){
        $this->checkPermission('03-008','edit');

        $sort = $request->all();
        foreach ($sort['items'] as $key => $value) {
            Car::update_car(array('car_order' => $key ),  $value['id']);
        }
        return $this->sendResponse( [] , trans('main.update_success') );

    }

    public function toggle_active($lang , $id){
        $item = Car::find($id);
        if($item->car_active){

            $this->checkPermission('03-008','delete');

            Car::delete_car($id);
            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-008','edit');

            Car::update_car(array('car_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
