<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use App\Models\Main\Branch;
use Illuminate\Http\Request;


use App\Models\Finance\AccountTree;
use App\Models\Trans\Area;
use App\Models\Trans\DriverCard;
use App\Models\Trans\Driver;
use App\Models\Trans\Trip;
use App\Models\Main\ScreenSub;

use DB;
class DriverCardController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-036','view');

        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null,
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-036' , $lang)->sub_title ?? '';
        $this->return['items']       = DriverCard::all_driver_cards( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-036','add');

        $this->filter = array(
            'lang'          => $lang ,
            'active'        => $request['active'] ?? 1 ,
            'rows'          => $request['rows'] ?? null ,
            'page'          => $request['page'] ?? 1 ,
            'word'          => $request['word'] ?? null,
            'driver_id'     => $request['driver_id'] ?? null,
            'is_returned'   => $request['is_returned'] ?? null,
            'from'          => $request['from_date'] ??  date("Y-m-01") ,
            'to'            => $request['to_date'] ?? date("Y-m-t") ,
            'driver_paid'   => 0 ,
        );

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-036' , $lang)->sub_title ?? '';
        $this->return['drivers']        = Driver::all_drivers( ['active' => 1 , 'lang' => $lang] );
        if($this->filter['driver_id'] > 0 ){
            $this->return['items']          = Trip::all_trips( $this->filter );

            foreach ($this->return['items']  as $key => $value) {
                $this->return['items'][$key]->total_price = $this->return['items'][$key]->driver_price;
                $this->return['items'][$key]->trip_price = $this->return['items'][$key]->driver_price;
            }
        }

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-036','add');
        $insert = $request->only([ 'driver_id' , 'action_date' , 'from_date' , 'to_date' ]);

        $insert['add_user']             = auth()->user()->id;
        $insert['amount']               = collect($request['items'])->sum('driver_price');
        $insert['driver_card_items']    = json_encode($request['items']);

        $validator = validator()->make( $insert , DriverCard::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        DB::beginTransaction();

            DB::table('driver_cards')->lockForUpdate()->get();
            $insert['driver_card_id'] = intval( DB::table('driver_cards')->max("driver_card_id") ) + 1;
            $this->return['id'] = DriverCard::add_driver_card($insert);
            DB::table('trips')
                ->whereIn('trip_id' , collect($request['items'])->pluck('id')->toArray() )
                ->update( ['driver_paid' => $this->return['id'] ] );

        DB::commit();
        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-036','view');

        $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-036' , $lang)->sub_title ?? '';
        $this->return['data']           = DriverCard::get_driver_card($id,$lang);
        $this->return['items']          = json_decode($this->return['data']->driver_card_items ?? '');

        return $this->sendResponse( $this->return , '' );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-036','delete');
        DB::beginTransaction();

            $item       = DriverCard::find($id);
            $trip_ids   = collect( json_decode($item->driver_card_items) )->pluck('id')->toArray();
            DriverCard::delete_driver_card($id);
            DB::table('trips')
                ->whereIn('trip_id' , $trip_ids )
                ->update( ['driver_paid' => 0 ] );

        DB::commit();

        return $this->sendResponse( [] , trans('main.delete_success') );
    }


    public function toggle_active($lang , $id){
        $item = DriverCard::find($id);
        if($item->driver_card_active){

            $this->checkPermission('03-036','delete');

            DB::beginTransaction();


                $trip_ids   = collect( json_decode($item->driver_card_items) )->pluck('id')->toArray();
                DriverCard::delete_driver_card($id);
                DB::table('trips')
                    ->whereIn('trip_id' , $trip_ids )
                    ->update( ['driver_paid' => 0 ] );

            DB::commit();

            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-036','edit');

            DriverCard::update_driver_card(array('driver_card_active' => 1 ) , $id);

            DB::beginTransaction();


                $item = DriverCard::find($id);
                DriverCard::update_driver_card(array('driver_card_active' => 1 ) , $id);
                DB::table('trips')
                    ->whereIn('trip_id' , collect( json_decode($item->driver_card_items) )->pluck('id')->toArray()  )
                    ->update( ['driver_paid' => $id ] );

            DB::commit();


            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

}
