<?php
namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;

use App\Classes\Trans\WearhouseTransactionHelper;
use App\Classes\Trans\TripsHelper;
use App\Classes\General\ZatcaHelper;
use App\Classes\General\General;

use App\Models\Finance\AccountTree;
use App\Models\Main\Delegate;
use App\Models\Trans\Invoice;
use App\Models\Trans\Trip;
use App\Models\Trans\WearhouseTransaction;
use App\Models\Main\ScreenSub;
use App\Models\Main\Branch;
use App\Models\Sales\Customer;
use App\Models\Trans\ContainerDamage;

use Salla\ZATCA\GenerateQrCode;
use Salla\ZATCA\Tags\InvoiceDate;
use Salla\ZATCA\Tags\InvoiceTaxAmount;
use Salla\ZATCA\Tags\InvoiceTotalAmount;
use Salla\ZATCA\Tags\Seller;
use Salla\ZATCA\Tags\TaxNumber;
use DB;
use PDF;
use Excel;
use App;
use App\Classes\Trans\InvoiceTransHelper;
use Str;
use DNS2D;
use Image;
class InvoiceController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-011','view');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'customer'    => $request['customer'] ?? null ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'type'        => 1,
            'word'        => $request['word'] ?? null
        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-011' , $lang)->sub_title ?? '';
        $this->return['items']       = Invoice::all_invoices( $this->filter );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-011','add');
        $this->filter = [
            'customer'      => $request['customer_id'] ?? null  ,
            'from'          => $request['from_date'] ?? null  ,
            'to'            => $request['to_date'] ?? null  ,
            'invoice_id'    => 0  ,
            'lang'          => $lang  ,
            'active'        => 1 ,
            'customer_pay'  => 1 ,
            'status'        => 4
        ];

        if( $this->filter['customer']  == null ){
            $this->return['screen_name']    = ScreenSub::get_screen_sub_by_screen_code('03-011' , $lang)->sub_title ?? '';
            $this->return['customers']      = Customer::all_customers(  $this->filter );
        }else{
            $this->return['trips']          = Trip::all_trips(  $this->filter );
            $this->return['transactions']   = WearhouseTransaction::all_wearhouse_transactions(  $this->filter );
            $this->return['containers']     = ContainerDamage::all_containers( $this->filter);

            foreach ($this->return['trips'] as $row){
                ( new tripsHelper )->calc_price($row->id);
            }
            foreach ($this->return['transactions'] as $row){
                ( new WearhouseTransactionHelper )->calc_price($row->id);
            }

            $this->return['trips']          = Trip::all_trips(  $this->filter );
            $this->return['transactions']   = WearhouseTransaction::all_wearhouse_transactions(  $this->filter );

        }

        return $this->sendResponse( $this->return  );
    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-011','add');
        $insert                 = $request->only(['customer_id', 'trips' , 'transactions' , 'containers' , 'invoice_date', 'description', 'from_date', 'to_date']);
        $insert['trips']        = (is_array($insert['trips'] ?? [])? ($insert['trips'] ?? [] ) : explode( ',' , $insert['trips'] ?? [] ));
        $insert['transactions'] = (is_array($insert['transactions'] ?? [])? ($insert['transactions'] ?? []) : explode(',',$insert['transactions'] ?? [] ));
        $trips                  = Trip::all_trips(['active' => 1 , 'lang' => $lang , 'ids' => $insert['trips']]);
        $transactions           = WearhouseTransaction::all_wearhouse_transactions(['active' => 1 , 'lang' => $lang , 'ids' => $insert['transactions']]);
        $total                  = $trips->sum('trip_price') + $transactions->sum('total_cost');
        if($total < 1){
            return $this->sendError( trans("error.cannot_create_invoice_without_value"));
        }
        $year   = array_search( date("Y" , strtotime($request['invoice_date'])) , $this->data['years'] );
        // if($year == ''){
        //     return $this->sendError( trans("error.finance_year_not_aval"));
        // }
        if((new General)->check_financial_year($insert['invoice_date']) != "true"){
            return $this->sendError((new General)->check_financial_year($insert['invoice_date']));
        }
        foreach ($trips as $row){
            if($row->invoice_id > 0 ){
                return $this->sendError("تم اصدار فاتوره للرحله رقم ".$row->id  ,  [] );
            }
        }
        foreach ($transactions as $row){
            if($row->invoice_id > 0 ){
                return $this->sendError("تم اصدار فاتوره للتخزين رقم ".$row->id  ,  [] );
            }
        }
        $seed = [

            'customer_id'           => $insert['customer_id'] ,
            'from_date'             => $insert['from_date'] ,
            'to_date'               => $insert['to_date'] ,
            'description'           => $insert['description'] ,
            'invoice_date'          => $insert['invoice_date'] ,
            'branch_id'             => $this->data['current_branch'] ,
            'finance_year_id'       => $year ,
            'trips'                 => implode(',', $insert['trips']) ,
            'wearhouse_transactions'=> implode(',', $insert['transactions']) ,
            'trips_total'           => $trips->sum('trip_price')  ,
            'transactions_total'    => $transactions->sum('total_cost')  ,
            'total'                 => $total ,
            'discount'              => 0 ,
            'total_without_vat'     => $total ,
            'vat_per'               => 15.00 ,
            'total_vat'             => $total * 0.15 ,
            'total_with_vat'        => $total * 1.15 ,
            'add_user'              => auth()->user()->id
        ];

        // return $this->sendError('', $seed ?? [] );

        DB::beginTransaction();

        DB::table('invoices')->lockForUpdate()->get();
        $seed['invoice_id'] = intval( DB::table('invoices')->max("invoice_id") ) + 1;
        $this->return['id'] = Invoice::add_invoice($seed);

        DB::table('trips')
            ->whereIn('trip_id' , $insert['trips'] )
            ->update(['invoice_id' => $this->return['id']]);

        DB::table('wearhouse_transactions')
            ->whereIn('wearhouse_transaction_id' , $insert['transactions'] )
            ->update(['invoice_id' => $this->return['id']]);

        $journal = (new InvoiceTransHelper)->generate_journal($this->return['id']);
        if(!$journal[0])
            return $this->sendError($journal[1], $journal ?? [] );

        DB::commit();

        return $this->sendResponse( $this->return , trans('main.add_success') );
    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-011','view');

        $this->return['screen_name']        = ScreenSub::get_screen_sub_by_screen_code('03-011' , $lang)->sub_title ?? '';
        $this->return['data']               = Invoice::get_invoice($id , $lang);
        $this->return['trips']              = Trip::all_trips( ['active' => 1 , 'lang' => $lang ,'invoice_id' => $id ,'customer' => $request['customer'] ] );
        $this->return['transactions']       = WearhouseTransaction::all_wearhouse_transactions( ['active' => 1 , 'lang' => $lang , 'invoice_id' => $id  , 'customer' => $request['customer'] ]);


        return $this->sendResponse( $this->return , '' );
    }



    public function print( $id , Request $request)
    {

        // $this->checkPermission('04-012','print');
        $lang                               = 'ar';
        $this->return['lang']               = $lang;
        $this->return['master']             = Invoice::get_invoice($id , $lang);
        $this->return['trips']              = Trip::all_trips( ['active' => 1 , 'lang' => $lang ,'invoice_id' => $id ,'customer' => $request['customer'] ] );
        $this->return['transactions']       = WearhouseTransaction::all_wearhouse_transactions( ['active' => 1 , 'lang' => $lang , 'invoice_id' => $id  , 'customer' => $request['customer'] ]);
        $this->return['branch_data']        = Branch::get_branch($this->return['master']->branch_id ?? 0  , $lang);
        $this->return['logo']               = $this->return['branch_data']->branch_image ?? 0;
        $this->return['tafqeet']            =  ( new General )->numberToText($this->return['lang'], round($this->return['master']->total_with_vat , 2) , 'SAR' );


        $this->return['qrcode']             = GenerateQrCode::fromArray([
            new Seller($this->return['branch_data']->vat_register_name),
            new TaxNumber($this->return['branch_data']->vat_number),
            new InvoiceDate(date('Y-m-d\TH:i:s', strtotime($this->return['master']->add_date))),
            new InvoiceTotalAmount(round($this->return['master']->total_with_vat , 2)), // Invoice Total Amount
            new InvoiceTaxAmount(round($this->return['master']->total_vat , 2)) // Invoice Tax Amount
        ])->render();
        // $this->return['qrcode']             =

        switch (@$request['type']) {
            case 'excel':
                $this->return['logo']     = public_path().$this->return['logo'];
                $export = new InvoicesExport($this->return , 'Print/clearance/invoice');
                return Excel::download($export, 'balances.xlsx');
                break;
            case 'pdf':
                $this->return['logo']     = public_path().$this->return['logo'];
                $pdf = PDF::loadView('Print/trans/invoice', array('data' => $this->return ))->setOption("footer-center", "[page] / [topage]");
                return $pdf->stream();
                break;
            case 'image':
                $image = App::make('snappy.image.wrapper');
                $image->loadView('Print/trans/invoice', array('data' => $this->data));
                return $image->stream();
                break;
            case 'print':
                return view('Print.trans.invoice')->with('data' , $this->return);
                break;
            case 'json':
                return view('Print.trans.invoice')->with('data' , $this->return);
                break;
        }


        return $this->sendResponse($this->return);
    }



}
