<?php

namespace App\Http\Controllers\API\V1\Trans;
use App\Http\Controllers\API\V1\BaseController;
use Illuminate\Http\Request;
use App\Models\Finance\AccountTree;
use App\Models\Trans\WearhouseTransaction;
use App\Models\Sales\Customer;
use App\Models\Trans\Wearhouse;
use App\Models\Main\ScreenSub;
use App\Models\Trans\Area;
use App\Models\Trans\Trip;
use App\Models\Trans\Driver;
use App\Models\Trans\Car;
use App\Models\Trans\Port;
use Illuminate\Support\Facades\DB;


class WearhouseTransactionController extends BaseController
{
    protected $return = array();
    protected $filter = array();

    public function index($lang , Request $request)
    {
        $this->checkPermission('03-009','add');


        $this->filter = array(
            'lang'        => $lang ,
            'active'      => $request['active'] ?? 1 ,
            'status'      => $request['status'] ?? null ,
            'customer'    => $request['customer'] ?? null ,
            'wearhouse'   => $request['wearhouse'] ?? null ,
            'port_id'     => $request['port_id'] ?? null ,
            'rows'        => $request['rows'] ?? null ,
            'page'        => $request['page'] ?? 1 ,
            'word'        => $request['word'] ?? null ,
            'branch_id'   => $request['branch_id'] ?? $this->data['current_branch']

        );

        $this->return['screen_name'] = ScreenSub::get_screen_sub_by_screen_code('03-009' , $lang)->sub_title ?? '';
        $this->return['items']       = WearhouseTransaction::all_wearhouse_transactions( $this->filter );
        $this->return['areas']       = Area::all_areas( ['active' => 1 , 'lang' => $lang] );
        $this->return['ports']       = Port::all_ports(['lang'=>$lang , 'active'=>1]);
        $this->return['cars']        = Car::all_cars( ['active' => 1 , 'lang' => $lang] );
        $this->return['drivers']     = Driver::all_drivers( ['active' => 1 , 'lang' => $lang] );
        $this->return['wearhouses']     = Wearhouse::all_wearhouses( ['active' => 1 , 'lang' => $lang] );

        return $this->sendResponse( $this->return );
    }

    public function create($lang , Request $request)
    {
        $this->checkPermission('03-009','add');


        $this->return['wearhouses']     = Wearhouse::all_wearhouses( ['lang' => $lang , 'active' => 1 ] );
        $this->return['customers']   = Customer::all_customers( ['lang' => $lang , 'active' => 1 ] );
        return $this->sendResponse( $this->return );

    }

    public function store($lang , Request $request)
    {
        $this->checkPermission('03-009','add');


        $insert = $request->only(['status' , 'notes'  , 'customer_id' , 'wearhouse_id' , 'in_date' , 'is_box' , 'is_empty' ,'container_number' ,'container_size', 'in_driver_residency_number' , 'in_driver_name' , 'in_driver_mobile' , 'in_car_number', 'out_driver_residency_number' , 'out_driver_name' , 'out_driver_mobile'  , 'store_place' , 'cost_per_day','company_bears_expenses']);
        $validator = validator()->make( $insert , WearhouseTransaction::$rules );
        if($validator->fails())
        {
            return $this->sendError( trans('error.add_fails') , $validator->errors() );
        }

        $insert['transaction_type'] = 2;
        $insert['status']       = 2;
        $insert['in_date']      = $insert['in_date'] ?? date("Y-m-d H:i:s") ;
        $insert['in_user']      = auth()->user()->id;
        $insert['code_number']  = "0-".( $insert['transaction_number'] ?? "t_n" )."-".( $insert['container_number'] ?? "c_n" );
        $insert['branch_id']    = $this->data['current_branch'];
        $this->return['id']     = WearhouseTransaction::add_wearhouse_transaction($insert);

        return $this->sendResponse( $this->return );

    }

    public function show($lang , $id , Request $request)
    {
        $this->checkPermission('03-009','view');


        $this->return['wearhouses']     = Wearhouse::all_wearhouses( ['lang' => $lang , 'active' => 1 ] );
        $this->return['customers']      = Customer::all_customers( ['lang' => $lang , 'active' => 1 ] );
        $this->return['item']           = WearhouseTransaction::get_wearhouse_transaction($id , $lang);

        return $this->sendResponse( $this->return );

    }

    public function update($lang , $id , Request $request)
    {
        // $this->checkPermission('03-009','edit');

        $update = $request->only(['status' , 'notes'  , 'customer_id' ,'in_date' , 'wearhouse_id' , 'is_box' , 'is_empty' , 'container_number' ,'container_size' , 'in_driver_residency_number' , 'in_driver_name' , 'in_driver_mobile' , 'in_car_number', 'out_driver_residency_number' , 'out_driver_name' , 'out_driver_mobile' , 'out_car_number', 'store_place' , 'cost_per_day','company_bears_expenses']);

        $item = WearhouseTransaction::get_wearhouse_transaction($id , $lang);
        if($item->invoice_id > 0 ){
            return $this->sendError( trans('error.transaction_has_invoice'));
        }
        //check container holding
        if($item->is_hold == 1){
            return $this->sendError( trans('error.container_is_holding') );
        }elseif($item->transaction_type == 1 ){
            // return $this->sendError( trans('error.has_trip') );
        }

        if($item->status >= $update['status'] ){

            unset($update['status'] , $update['notes'] );
            if(count($update) > 0 )
                WearhouseTransaction::update_wearhouse_transaction( $update ,  $id );
            return $this->sendResponse( $this->return , trans('main.update_success') );

        }

        if($update['status'] == 2){
            if($item->in_trip_id > 0 ){
                $in_date = Trip::get_trip($item->in_trip_id,$lang)->related_trans_date;
            }
            $update['in_notes']     = $update['notes'];
            $update['in_date']      = $in_date ?? date("Y-m-d H:i:s");
            $update['in_user']      = auth()->user()->id;
            $update['requested_by'] = $request['requested_by'] ??  1 ; //1=>customer , 2=>>company
            unset( $update['notes'] );

        }elseif($update['status'] == 4){
            if($item->out_trip_id > 0 ){
                $out_date = Trip::get_trip($item->out_trip_id,$lang)->related_trans_date;
            }
            $price                  = Wearhouse::get_wearhouse($item->wearhouse_id , $lang);
            $now                    = \Carbon\Carbon::now();
            $lengthOfAd             = $now->diffInDays($item->in_date);

            if($item->transaction_type == 2 ){
                $price = $item->cost_per_day;
            }else{
                $price = $price->price;
            }

            $update['total_days']   = $lengthOfAd + 1 ;
            $update['total_cost']   = $price * $update['total_days'];

            if($item->status != 3 ){

                $update['out_driver_residency_number']  = $update['out_driver_residency_number'] ?? '';
                $update['out_car_number']               = $update['out_car_number'] ?? '';
                $update['out_driver_mobile']            = $update['out_driver_mobile'] ?? '';
                $update['out_driver_name']              = $update['out_driver_name'] ?? '';

            }

            $update['out_notes']    = $update['notes'];
            $update['out_date']     = $out_date ?? date("Y-m-d H:i:s");
            $update['out_user']     = auth()->user()->id;
            unset( $update['notes'] );

        }

        unset($update['notes']);
        WearhouseTransaction::update_wearhouse_transaction( $update ,  $id );

        return $this->sendResponse( $this->return , trans('main.update_success') );
    }

    public function destroy($lang , $id)
    {
        $this->checkPermission('03-009','delete');
        $item = WearhouseTransaction::find($id);

        if($item->invoice_id > 0 ){
            return $this->sendError( trans('error.transaction_has_invoice'));
        }

        if($item->in_trip_id > 0 || $item->out_trip_id > 0){
            $update = [];
            if($item->status == 2){

                $update['in_notes']     = null;
                $update['status']       = 1;
                $update['in_date']      = null;
                $update['in_user']      = null;
                $update['requested_by'] = null; //1=>customer , 2=>>company

            }elseif($item->status == 4){


                $update['total_days']   = 0;
                $update['status']       = ($item->out_trip_id > 0? 3:2 );
                $update['total_cost']   = 0;
                $update['out_notes']    = '';
                $update['out_date']     = null;
                $update['out_user']     = null;
            }elseif($item->status == 10){
                $update['status']       = 1;
            }else{
                return $this->sendError( trans('error.cannot_delete_related_with_trip') , [] );
            }
            WearhouseTransaction::update_wearhouse_transaction( $update ,  $id );
        // }elseif($item->out_trip_id == 0  || !isset($item->out_trip_id) ){

        }else{

            $update = [];
            if($item->status == 4){

                $update['total_days']   = 0;
                $update['status']       = 2;
                $update['total_cost']   = 0;
                $update['out_notes']    = '';
                $update['out_date']     = null;
                $update['out_user']     = null;
                WearhouseTransaction::update_wearhouse_transaction( $update , $id );

            }else{
                WearhouseTransaction::delete_wearhouse_transaction($id);
            }

        }

        return $this->sendResponse( [] , trans('main.delete_success') );
    }

    public function trip_change($lang , Request $request){

        $item = WearhouseTransaction::get_wearhouse_transaction($request['warehouse_transaction_id'] , 'en');
        $trip = Trip::find($item->in_trip_id)->toArray();
        if($item->is_hold == 1){
            return $this->sendError( trans('error.container_is_holding') );
        }
        unset(
            $trip['add_date'],
            $trip['add_user'],
            $trip['trip_id'],
            $trip['disable_date'],
            $trip['disable_user'],
            $trip['invoice_id']
        );

        $trip['parent_id']          = $item->in_trip_id;
        $trip['to_port_id']         = 0;
        $trip['to_wearhouse_id']    = 0;
        $trip['to_area_id']         = $request['area_id'] ?? 0;;
        $trip['car_id']             = $request['car_id'] ?? 0;
        $trip['driver_id']          = $request['driver_id'] ?? 0;
        $trip['trip_price']         = 30;
        $trip['driver_price']       = 0;
        $trip['total_price']        = 30;

        DB::beginTransaction();
            DB::table('trips')->lockForUpdate()->get();
            $trip['trip_id'] = (Trip::max('trip_id')+1);
            $id = Trip::add_trip( $trip );
            Trip::update_trip(  ['has_child' => 1 ] , $item->in_trip_id );
            WearhouseTransaction::update_wearhouse_transaction( ['status' => 10 , 'in_date'=>date('Y-m-d H:i:s')] ,  $request['warehouse_transaction_id'] );
        DB::commit();

        return $this->sendResponse($id  , trans('main.add_success') );
    }


    public function toggle_active($lang , $id){
        $item = WearhouseTransaction::find($id);


        if($item->wearhouse_transaction_active){
            $this->checkPermission('03-009','delete');
            if($item->invoice_id > 0 ){
                return $this->sendError( trans('error.transaction_has_invoice'));
            }
            if($item->in_trip_id > 0 || $item->out_trip_id > 0){
                $update = [];
                if($item->status == 2){

                    $update['in_notes']     = null;
                    $update['status']       = 1;
                    $update['in_date']      = null;
                    $update['in_user']      = null;
                    $update['requested_by'] = null ; //1=>customer , 2=>>company

                }elseif($item->status == 4){

                    $update['total_days']   = 0;
                    $update['status']       = ($item->out_trip_id > 0? 3:2 );
                    $update['total_cost']   = 0;
                    $update['out_notes']    = '';
                    $update['out_date']     = null;
                    $update['out_user']     = null;
                }elseif($item->status == 10){
                    $update['status']       = 1;
                }

                WearhouseTransaction::update_wearhouse_transaction( $update ,  $id );
            }else{
                WearhouseTransaction::delete_wearhouse_transaction($id);

            }

            $this->return['active'] = false;

        }else{

            $this->checkPermission('03-009','edit');

            WearhouseTransaction::update_wearhouse_transaction(array('wearhouse_transaction_active' => 1 ) , $id);
            $this->return['active'] = true;
        }
        return $this->sendResponse( $this->return , trans('main.update_success') );
    }


}
