<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class BankTransfer extends Model
{
    protected $primaryKey     = 'bank_transfer_m_id';
    public static $main_table = 'bank_transfer_m';
    public $table             = 'bank_transfer_m';

    public static $master_rules = array(
        'action_date'   => 'required|date',
        'daily_type_id' => 'required|numeric',
    );

    public static $details_rules = array(
        'bank_transfer_m_id' => 'required|numeric',
        'transfer_from'      => 'required|numeric',
        'transfer_to'        => 'required|numeric',
        'amount'             => 'required|numeric',
    );

    public static function all_bank_transfers($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->orderBy(self::$main_table.'.bank_transfer_m_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                bank_transfer_m_id as m_id ,'.
                self::$main_table.'.branch_id ,
                branchs.name_'.$params['lang'].' as branch_name ,'.
                self::$main_table.'.finance_year_id ,
                finance_years.name as finance_year_name ,'.
                self::$main_table.'.daily_journal_m_id ,'.
                self::$main_table.'.daily_type_id ,
                daily_types.name_'.$params['lang'].' as daily_type_name ,'.
                self::$main_table.'.action_date ,'.
                self::$main_table.'.total_amount ,'.
                self::$main_table.'.arabic_description ,'.
                self::$main_table.'.bank_transfer_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('bank_transfer_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.arabic_description' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('daily_types.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('daily_types.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('bank_transfer_m_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_bank_transfer_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->where('bank_transfer_m_id' , $id);

        $query->selectRaw('
            bank_transfer_m_id ,'.
            self::$main_table.'.branch_id ,'.
            self::$main_table.'.finance_year_id ,'.
            self::$main_table.'.daily_journal_m_id ,'.
            self::$main_table.'.daily_type_id ,
            daily_types.name_'.$lang.' as daily_type_name ,'.
            self::$main_table.'.action_date ,'.
            self::$main_table.'.total_amount ,'.
            self::$main_table.'.arabic_description ,'.
            self::$main_table.'.bank_transfer_active,'.
            self::$main_table.'.add_date,'.
            self::$main_table.'.add_user
        ');

        $data = $query->first();
        return $data;
    }

    public static function get_bank_transfer_d($id, $lang = null)
    {
        $query = DB::table('bank_transfer_d')
            ->where('bank_transfer_m_id', $id);

        if($lang != null){
            $query->selectRaw('
                bank_transfer_m_id as m_id ,
                bank_transfer_d_id as d_id ,
                transfer_from ,
                from_bank_id ,
                from_customer_id ,
                transfer_to ,
                to_bank_id ,
                to_customer_id ,
                clearance_id ,
                arabic_description ,
                amount ,
                add_date ,
                add_user
            ');
        }

        $data = $query->get();
        return $data;
    }

    public static function add_bank_transfer_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_bank_transfer_d($seed)
    {
        return DB::table('bank_transfer_d')->insertGetId($seed);
    }

    public static function update_bank_transfer_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('bank_transfer_m_id', $id)->update($seed);
    }

    public static function delete_bank_transfer_m($id)
    {
        return DB::table(self::$main_table)->where('bank_transfer_m_id', $id)->update(array('bank_transfer_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public static function delete_bank_transfer_d($id)
    {
        return DB::table('bank_transfer_d')->where('bank_transfer_m_id', $id)->delete();
    }
}
