<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class DailyJournal extends Model
{

    protected $primaryKey           = 'daily_journal_m_id';
    public static $master_table     = 'daily_journal_m';
    public $table     = 'daily_journal_m';

    public static $master_rules = array(
        'description'       => 'string|nullable',
        'year_ser'          => 'required|numeric',
        'month_ser'         => 'required|numeric',
        'branch_ser'        => 'required|numeric',
        'journal_date'      => 'required|date'
    );

    public static $detaile_rules = array(
        'description'           => 'string|nullable',
        'account_tree_id'       => 'required|numeric|min:1' ,
        'daily_journal_m_id'    => 'required|numeric',
        'debit'                 => 'required|numeric|min:0',
        'credit'                => 'required|numeric|min:0'
    );

    public static function all_daily_journals($params = null)
    {
        $query =  DB::table(self::$master_table)
            ->leftjoin('daily_types' , self::$master_table.'.daily_type_id' , '=' , 'daily_types.daily_type_id')
            ->join('branchs' , self::$master_table.'.branch_id' , '=' , 'branchs.branch_id')
            ->join('finance_years' , self::$master_table.'.finance_year_id' , '=' , 'finance_years.finance_year_id')
            ->orderBy(self::$master_table.'.year_ser' , 'desc');

        if(isset($params['lang'])){
            $query->selectRaw('
                daily_journal_m_id as m_id  ,
                year_ser as journal_code ,
                description ,
                branchs.branch_id  ,
                branchs.name_'.$params['lang'].' as branch_name ,
                finance_years.finance_year_id ,
                finance_years.name as finance_year_name ,
                journal_date ,
                year_ser ,
                month_ser ,
                branch_ser ,
                daily_types.daily_type_id ,
                daily_types.name_'.$params['lang'].' as daily_type ,
                document_id ,
                auto_generated ,
                journal_active ,'.
                self::$master_table.'.add_date
            ');
        }

        if(isset($params['active'])){
            $query->where('journal_active' , $params['active']);
        }

        if(isset($params['branch'])){
            $query->where(self::$master_table.'.branch_id' , $params['branch']);
        }

        if(isset($params['finance_year'])){
            $query->where(self::$master_table.'.finance_year_id' , $params['finance_year']);
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_daily_journal_m($id , $lang = null)
    {
        $query =  DB::table(self::$master_table)
            ->leftjoin('daily_types' , self::$master_table.'.daily_type_id' , '=' , 'daily_types.daily_type_id')
            ->leftjoin('branchs' , self::$master_table.'.branch_id' , '=' , 'branchs.branch_id')
            ->leftjoin('clearances_m', self::$master_table.'.daily_journal_m_id', '=', 'clearances_m.daily_journal_id')

            ->where('daily_journal_m_id' , $id);

        if($lang != null){
            $query->selectRaw('
            daily_journal_m_id as m_id ,
            year_ser as journal_code ,
            description ,
            daily_journal_m.finance_year_id ,
            branchs.branch_id  ,
            branchs.name_'.$lang.' as branch_name ,
            journal_date ,
            daily_journal_m.year_ser ,
            daily_journal_m.month_ser ,
            daily_journal_m.branch_ser ,
            daily_types.daily_type_id ,
            daily_types.name_'.$lang.' as daily_type ,
            document_id ,
            auto_generated ,
            journal_active ,'.
            self::$master_table.'.add_date,
            clearances_m.clearance_id
            ');
        }

        $data = $query->first();
        return $data;

    }


    public static function get_daily_journal_d($id , $lang = null)
    {
        $query =  DB::table('daily_journal_d')
            ->leftjoin('account_tree' , 'daily_journal_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->where('daily_journal_m_id' , $id);

        if($lang != null){
            $query->selectRaw('
            daily_journal_m_id as m_id  ,
            daily_journal_d_id as d_id ,
            description ,
            if( debit > 0 , 2 , 1) as d_type ,
            debit ,
            credit ,
            account_tree.account_tree_id ,
            account_code ,
            account_tree.name_'.$lang.' as account_tree ,
            cost_centers  ,
            attach_ext
            ');
        }

        $data = $query->get();
        return $data;

    }

    public static function get_next($id , $branch){
        return DB::table(self::$master_table)
            ->where('daily_journal_m_id' , '>' , $id)
            ->where('journal_active' , 1)
            ->where('confirmed' , 1)
            ->where('branch_id' , $branch)
            ->selectRaw('daily_journal_m_id as m_id')
            ->first();
    }

    public static function get_previous($id, $branch){
        return DB::table(self::$master_table)
            ->where('daily_journal_m_id' , '<' , $id)
            ->where('journal_active' , 1)
            ->where('confirmed' , 1)
            ->where('branch_id' , $branch)
            ->selectRaw('daily_journal_m_id as m_id')
            ->orderBy('daily_journal_m_id' , 'desc')
            ->first();
    }

    public static function get_last($branch){
        return DB::table(self::$master_table)
            ->where('journal_active' , 1)
            ->where('confirmed' , 1)
            ->where('branch_id' , $branch)
            ->selectRaw('daily_journal_m_id as m_id')
            ->orderBy('daily_journal_m_id' , 'desc')
            ->first();
    }

    public static function get_files($id){
        return DB::table('daily_journal_files')
            ->where('daily_journal_id' , $id)
            ->get();
    }

    public static function get_account_balance($id , $lang , $params = null)
    {

        if(isset($params['draft']) && $params['draft'] == 1 ){
            $draft = DB::table('clearance_invoices_draft_journal')
                ->join('account_tree' , 'clearance_invoices_draft_journal.account_tree_id' , '=' , 'account_tree.account_tree_id' )
                ->join('clearance_invoices_draft_m' , 'clearance_invoices_draft_journal.invoice_m_id' , '=' , 'clearance_invoices_draft_m.invoice_id' )
                ->join('clearances_m' , 'clearances_m.clearance_id' , '=' , 'clearance_invoices_draft_m.clearance_id' )
                ->join('customers' , 'clearance_invoices_draft_m.customer_id' , '=' , 'customers.customer_id' )
                ->selectRaw('
                    "0" as m_id ,
                    "0" as journal_code ,
                    "0" as branch_ser ,
                    "0" as month_ser ,
                    clearance_invoices_draft_m.invoice_date as journal_date ,
                    0 as debit ,
                    amount as credit ,
                    concat("مسوده رقم  " , clearance_invoices_draft_journal.invoice_m_id ) as daily_type ,
                    concat(customers.name_'.$lang.' , " - ن/ " , ifnull(clearances_m.customs_declaration_number," - ") , " -م/ " , clearance_invoices_draft_m.clearance_id , "- ب/ " , ifnull(clearances_m.shipping_policy_no," - ") ) as description ,
                    clearance_invoices_draft_journal.invoice_m_id as document_id
                ');

            if( is_array($id) ){
                $draft->whereIn('clearance_invoices_draft_journal.account_tree_id' , $id);
            }else{
                $draft->where('clearance_invoices_draft_journal.account_tree_id' , $id);
            }
            if(isset($params['customer_id']) && $params['customer_id'] > 0){
                $draft->where('clearance_invoices_draft_m.customer_id' , $params['customer_id']);
            }
            if(isset($params['before'])){
                $draft->where('clearance_invoices_draft_m.invoice_date' , '<' , $params['before'] );
            }elseif(isset($params['from']) && isset($params['to']) ){
                $draft->where('clearance_invoices_draft_m.invoice_date' , '>=' , $params['from'] )
                    ->where('clearance_invoices_draft_m.invoice_date' , '<=' , $params['to'] );
            }

            $draft_data = $draft->get();

        }else{
            $draft_data = [];
        }



        $query = DB::table('daily_journal_d')
            ->join('account_tree' , 'daily_journal_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->join('daily_journal_m' , 'daily_journal_d.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->join('branchs' , 'daily_journal_m.branch_id' , '=' , 'branchs.branch_id')
            ->join('finance_years' , 'daily_journal_m.finance_year_id' , '=' , 'finance_years.finance_year_id')
            ->join('daily_types' , 'daily_journal_m.daily_type_id' , '=' , 'daily_types.daily_type_id')
            ->leftjoin('clearance_invoices_m' , function($join){
                $join->on('daily_journal_m.document_id' , '=' , 'clearance_invoices_m.invoice_id');
                $join->where('daily_journal_m.daily_type_id' , 13);
            })
            ->orderBy('journal_date' ,'asc')
            ->orderBy('document_id','desc')
            ->where('journal_active' , 1)
            ->where('confirmed' , 1)
            ->selectRaw('
                daily_journal_d.daily_journal_m_id as m_id ,
                year_ser as journal_code ,
                branch_ser ,
                month_ser  ,
                if("'.($params['draft_date'] ?? 0).'" = "1" and clearance_invoices_m.invoice_id > 0 ,  clearance_invoices_m.draft_invoice_date , journal_date ) as journal_date ,
                debit ,
                credit ,
                daily_types.name_'.$lang.' as daily_type ,
                daily_journal_d.description as description ,
                clearance_invoices_m.invoice_id ,
                document_id');


        if( is_array($id) ){
            $query->whereIn('daily_journal_d.account_tree_id' , $id);
        }else{
            $query->where('daily_journal_d.account_tree_id' , $id);
        }
        if(isset($params['without_closing_daily']) && $params['without_closing_daily'] == 1){
            $query->where('daily_journal_m.daily_type_id' ,'!=', 42);
        }
        if(isset($params['customer_id']) && $params['customer_id'] > 0){
            $query->where('daily_journal_d.shared_profile_id' , $params['customer_id']);
        }
        if(isset($params['before'])){
            $query->having('journal_date' , '<' , $params['before'] );
        }elseif(isset($params['from']) && isset($params['to']) ){
            $query->having('journal_date' , '>=' , $params['from'] )
                ->having('journal_date' , '<=' , $params['to'] );
        }

        if(isset($params['total'])){

            $data = $query->get();
            $data = (object) [
                'debit' => $data->sum('debit') ,
                'credit' => $data->sum('credit') ,
                'total' => $data->sum('debit') - $data->sum('credit') ,
            ];

            if(isset($params['draft']) && $params['draft'] == 1){
                $data->credit = $data->credit + collect($draft_data)->sum('credit');
                $data->total = $data->total - collect($draft_data)->sum('credit');
            }


        }else{

            $data = $query->get();

            if(isset($params['draft']) && $params['draft'] == 1 ){
                $data = array_merge($data->toArray(), $draft_data->toArray());
                $data = collect($data)->sortBy([
                    ['journal_date', 'asc'],
                    ['document_id', 'asc'],
                ])->values()->all();
            }

        }

        return $data;

    }



    public static function add_daily_journal_m($seed)
    {
        return DB::table(self::$master_table)->insertGetId($seed);
    }

    public static function add_daily_journal_d($seed)
    {
        return DB::table('daily_journal_d')->insertGetId($seed);
    }

    public static function update_daily_journal_m($seed ,  $id)
    {
        return DB::table(self::$master_table)->where('daily_journal_m_id' , $id )->update($seed);
    }

    public static function delete_daily_journal_m($id)
    {
        return DB::table(self::$master_table)->where('daily_journal_m_id' , $id )->update( array('journal_active' => 0 , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id )  );
    }

    public static function delete_daily_journal_d($id)
    {
        return DB::table('daily_journal_d')->where('daily_journal_m_id' , $id)->delete();
    }

}
