<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class MaintenancePetty extends Model
{
    protected $primaryKey     = 'maintenance_petty_m_id';
    public static $main_table = 'maintenance_petty_m';
    public $table             = 'maintenance_petty_m';

    public static $details_rules = array(
        // 'clearance_id'    => 'required|numeric',
        // 'expense_code_id' => 'required|numeric',
        'amount'          => 'required|numeric',
    );

    public static function all_maintenance_pettys($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->leftjoin('delegates', self::$main_table.'.delegate_id', '=', 'delegates.delegate_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            ->orderBy(self::$main_table.'.maintenance_petty_m_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                maintenance_petty_m_id as m_id ,'.
                self::$main_table.'.branch_id ,
                branchs.name_'.$params['lang'].' as branch_name ,'.
                self::$main_table.'.finance_year_id ,
                finance_years.name as finance_year_name ,'.
                self::$main_table.'.delegate_id ,
                delegates.name_'.$params['lang'].' as delegate_name ,'.
                self::$main_table.'.source ,'.
                self::$main_table.'.action_date ,'.
                self::$main_table.'.total_amount ,'.
                self::$main_table.'.remaining_amount ,'.
                self::$main_table.'.maintenance_petty_status,'.
                self::$main_table.'.maintenance_petty_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user,
                users.user_full_name as added_by
            ');

        if(isset($params['active']))
            $query->where('maintenance_petty_active', $params['active']);

        if(isset($params['status']))
            $query->where('maintenance_petty_status', $params['status']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.arabic_description' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('delegates.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('delegates.name_ar' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_maintenance_petty_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->leftjoin('delegates', self::$main_table.'.delegate_id', '=', 'delegates.delegate_id')
            ->where('maintenance_petty_m_id' , $id);

        $query->selectRaw('
            maintenance_petty_m_id as m_id ,
            maintenance_petty_m_id  ,'.
            self::$main_table.'.branch_id ,'.
            self::$main_table.'.finance_year_id ,'.
            self::$main_table.'.daily_type_id ,
            daily_types.name_'.$lang.' as daily_type_name ,'.
            self::$main_table.'.delegate_id ,
            delegates.name_'.$lang.' as delegate_name ,'.
            self::$main_table.'.source ,'.
            self::$main_table.'.action_date ,'.
            self::$main_table.'.total_amount ,'.
            self::$main_table.'.remaining_amount ,'.
            self::$main_table.'.maintenance_petty_status,'.
            self::$main_table.'.maintenance_petty_active,'.
            self::$main_table.'.add_date,'.
            self::$main_table.'.add_user
        ');

        $data = $query->first();
        return $data;
    }

    public static function get_maintenance_petty_d($id, $lang = null)
    {
        $query = DB::table('maintenance_petty_d')
            ->leftjoin('account_tree' , 'maintenance_petty_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->leftjoin('treasurys' , 'maintenance_petty_d.treasury_id' , '=' , 'treasurys.treasury_id')
            ->leftjoin('clearances_m' , 'clearances_m.clearance_id' , '=' , 'maintenance_petty_d.clearance_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->where('maintenance_petty_m_id', $id);

        if($lang != null){
            $query->selectRaw('
                maintenance_petty_m_id as m_id ,
                maintenance_petty_d_id as d_id ,
                maintenance_petty_d.clearance_id ,
                maintenance_petty_d.account_tree_id ,
                account_tree.account_code ,
                account_tree.name_'.$lang.' as account_tree_name ,
                maintenance_petty_d.treasury_id ,
                treasurys.treasury_name_'.$lang.' as treasury_name ,
                receipt_number ,
                amount ,
                description_'.$lang.' as description ,
                description_en ,
                description_ar ,
                maintenance_petty_d.add_date ,
                concat(clearances_m.clearance_id  ," - " , customers.name_'.$lang.' , " - " ,clearances_m.shipping_policy_no) as label,
                maintenance_petty_d.add_user
            ');
        }

        $data = $query->get();
        return $data;
    }

    public static function add_maintenance_petty_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_maintenance_petty_d($seed)
    {
        return DB::table('maintenance_petty_d')->insertGetId($seed);
    }

    public static function update_maintenance_petty_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('maintenance_petty_m_id', $id)->update($seed);
    }

    public static function delete_maintenance_petty_m($id)
    {
        return DB::table(self::$main_table)->where('maintenance_petty_m_id', $id)->update(array('maintenance_petty_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public static function delete_maintenance_petty_d($id)
    {
        return DB::table('maintenance_petty_d')->where('maintenance_petty_m_id', $id)->delete();
    }
}
