<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class MultiVoucher extends Model
{
        protected $primaryKey     = 'multi_voucher_m_id';
        public static $main_table = 'multi_voucher_m';
        public $table             = 'multi_voucher_m';

        public static $master_rules = array(
            'branch_id'       => 'required|numeric' ,
            'finance_year_id' => 'required|numeric' ,
            'action_date'     => 'required|date',
            'currency_id'     =>'required|Integer',
        );

        public static $details_rules = array(
            'voucher_type'=>'required|Integer',
            'amount'=>'required',
            'dis_type'=>'required|Integer'
        );


    public static function all_multi_vouchers($params = null)
    {

        $query = DB::table(self::$main_table)
            ->leftJoin('multi_voucher_d',self::$main_table.'.multi_voucher_m_id','multi_voucher_d.multi_voucher_m_id')
            ->leftJoin('daily_journal_m',self::$main_table.'.daily_journal_m_id','daily_journal_m.daily_journal_m_id')
            ->leftJoin('currencys',self::$main_table.'.currency_id','currencys.currency_id')
            ->leftJoin('users',self::$main_table.'.add_user','users.id')
            ->leftJoin('users as update_user',self::$main_table.'.update_user','update_user.id')
            ->groupBy(self::$main_table.'.multi_voucher_m_id')
            ->orderBy(self::$main_table.'.multi_voucher_m_id', 'desc');

            if($params['lang'] != null){
                $query->selectRaw('
                    multi_voucher_m.multi_voucher_m_id as id ,
                    multi_voucher_m.daily_journal_m_id ,
                    multi_voucher_m.daily_journal_m_id as daily_journal_id ,
                    daily_journal_m.year_ser as journal_code ,
                    conversion_type_id,
                    currencys.name_'.$params['lang'].',
                    action_date ,
                    multi_voucher_m.description ,
                    multi_voucher_m.update_user ,
                    update_user.user_full_name as updated_by ,
                    multi_voucher_m.update_date ,
                    multi_voucher_active ,'.
                    self::$main_table.'.add_date ,'.
                    self::$main_table.'.add_user ,
                    users.user_full_name as added_by
                ');
            }

        if(isset($params['active']))
            $query->where('multi_voucher_active', $params['active']);

        if(isset($params['branch'])){
            $query->where(self::$main_table.'.branch_id' , $params['branch']);
        }

        if(isset($params['finance_year'])){
            $query->where(self::$main_table.'.finance_year_id' , $params['finance_year']);
        }

        if(isset($params['word']) && strlen($params['word']) >  0){
            $query->where(function ($stmt) use($params) {
                $stmt->where('multi_voucher_m.description', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_m.multi_voucher_m_id', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_m.action_date', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_d.description', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_d.amount_without_vat', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_d.total_vat', 'LIKE', "%{$params['word']}%")
                    ->orWhere('multi_voucher_d.amount', 'LIKE', "%{$params['word']}%");
            });
        }



        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_multi_voucher_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftJoin('users',self::$main_table.'.add_user','users.id')
            ->leftJoin('users as update_user',self::$main_table.'.update_user','update_user.id')
            ->where('multi_voucher_m_id', $id);

        if($lang != null){
            $query->selectRaw('
                multi_voucher_m_id as id ,
                branch_id ,
                year_ser ,
                month_ser ,
                branch_ser ,
                action_date ,
                conversion_type_id ,
                multi_voucher_active ,
                currency_id ,
                finance_year_id ,
                daily_journal_m_id ,
                multi_voucher_m.update_user ,
                update_user.user_full_name as updated_by ,
                multi_voucher_m.update_date ,
                users.user_full_name as added_by ,
                description ,'.
                self::$main_table.'.add_date ,'.
                self::$main_table.'.add_user
            ');
        }

        $data = $query->first();
        return $data;
    }

    public static function get_multi_voucher_d($id, $lang = null , $params = null)
    {
        $query = DB::table('multi_voucher_d')
            ->where('multi_voucher_m_id', $id)
            ->orderBy('multi_voucher_d_id')
            ->where('multi_voucher_d_active' , 1 );

        if($lang != null){
            $query
                ->leftjoin('account_tree', 'multi_voucher_d.dis_account_tree_id', '=', 'account_tree.account_tree_id')
                ->leftjoin('treasurys', 'multi_voucher_d.dis_treasury_id', '=', 'treasurys.treasury_id')
                ->leftjoin('banks', 'multi_voucher_d.dis_bank_id', '=', 'banks.bank_id')
                ->leftjoin('customers', 'multi_voucher_d.dis_customer_id', '=', 'customers.customer_id')
                ->leftjoin('clearances_m', 'multi_voucher_d.clearance_id', '=', 'clearances_m.clearance_id')
                ->leftjoin('agents' , 'clearances_m.agent_number' , '=' , 'agents.agent_id')

                ->leftjoin('delegates', 'multi_voucher_d.dis_delegate_id', '=', 'delegates.delegate_id')
                ->leftjoin('expense_codes', 'multi_voucher_d.dis_expense_code_id', '=', 'expense_codes.expense_code_id')
                ->selectRaw('
                multi_voucher_m_id as m_id ,
                multi_voucher_d_id as d_id ,
                multi_voucher_d.clearance_id ,
                multi_voucher_d.voucher_type ,
                cost_centers ,
                amount_without_vat ,
                multi_voucher_d.attachment ,

                total_vat ,
                amount ,
                tax_id ,
                receipt_number ,
                dis_type ,
                dis_account_tree_id ,
                dis_treasury_id ,
                dis_bank_id ,
                dis_customer_id ,
                dis_supplier_id ,
                dis_delegate_id ,
                dis_expense_code_id ,

                multi_voucher_d.invoice_id ,
                account_tree.name_'.$lang.' as account_tree_name ,
                treasurys.treasury_name_'.$lang.' as treasury_name ,
                banks.name_'.$lang.' as bank_name ,
                customers.name_'.$lang.' as customer_name ,
                delegates.name_'.$lang.' as delegate_name ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                agents.agent_name_'.$lang.' agent_name ,
                clearances_m.invoices_count ,
                multi_voucher_d.description ,
                multi_voucher_d.add_date ,
                multi_voucher_d.add_user
            ');
        }
        if(isset($params['dis_type'])){
            $query->where('dis_type', $params['dis_type']);
        }

        $data = $query->get();
        return $data;
    }
    public static function get_attachments($id)
    {
        return DB::table('multi_voucher_attachments')
            ->where('multi_voucher_id' , $id)
            ->selectRaw('
                attachment_id as id ,
                multi_voucher_id ,
                file_name ,
                file_path ,
                file_ext ,
                add_date ,
                add_user
        ')->get();
    }

    public static function add_multi_voucher_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_multi_voucher_d($seed)
    {
        return DB::table('multi_voucher_d')->insertGetId($seed);
    }

    public static function update_multi_voucher_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('multi_voucher_m_id', $id)->update($seed);
    }

    public static function update_multi_voucher_d($seed, $id)
    {
        return DB::table('multi_voucher_d')->where('multi_voucher_d_id', $id)->update($seed);
    }

    public static function delete_multi_voucher_m($id)
    {
        return DB::table(self::$main_table)->where('multi_voucher_m_id', $id)->update(array('multi_voucher_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public static function delete_multi_voucher_d($id)
    {
        return DB::table('multi_voucher_d')->where('multi_voucher_m_id', $id)->delete();
    }
}
