<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class PaymentVoucher extends Model
{
    protected $primaryKey     = 'payment_voucher_m_id';
    public static $main_table = 'payment_voucher_m';
    public $table             = 'payment_voucher_m';

    public static $master_rules = array(
        'action_date'   => 'required|date',
        'daily_type_id' => 'required|numeric',
    );

    public static $details_rules = array(
        'payment_voucher_m_id' => 'required|numeric',
        'pay_for'              => 'required|numeric',
        'amount'               => 'required|numeric',
    );

    public static function all_payment_vouchers($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->leftjoin('branchs', self::$main_table.'.branch_id', '=', 'branchs.branch_id')
            ->leftjoin('finance_years', self::$main_table.'.finance_year_id', '=', 'finance_years.finance_year_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            ->orderBy(self::$main_table.'.payment_voucher_m_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                payment_voucher_m_id as id ,
                payment_voucher_m_id as m_id ,'.
                self::$main_table.'.branch_id ,
                branchs.name_'.$params['lang'].' as branch_name ,'.
                self::$main_table.'.finance_year_id ,
                finance_years.name as finance_year_name ,'.
                self::$main_table.'.daily_journal_m_id ,'.
                self::$main_table.'.daily_type_id ,
                daily_types.name_'.$params['lang'].' as daily_type_name ,'.
                self::$main_table.'.action_date ,'.
                self::$main_table.'.total_amount ,'.
                self::$main_table.'.arabic_description ,'.
                self::$main_table.'.payment_voucher_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user,
                users.user_full_name
            ');

        if(isset($params['active']))
            $query->where('payment_voucher_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.arabic_description' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('daily_types.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('daily_types.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('daily_types.name_ar' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_payment_voucher_m($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('treasurys', self::$main_table.'.treasury_id', '=', 'treasurys.treasury_id')
            ->leftjoin('daily_types', self::$main_table.'.daily_type_id', '=', 'daily_types.daily_type_id')
            ->leftjoin('users', self::$main_table.'.add_user', '=', 'users.id')
            ->where('payment_voucher_m_id' , $id);

        if($lang != null){
            $query->selectRaw('
                payment_voucher_m_id as m_id ,'.
                self::$main_table.'.branch_id ,'.
                self::$main_table.'.finance_year_id ,'.
                self::$main_table.'.daily_journal_m_id ,'.
                self::$main_table.'.daily_type_id ,
                daily_types.name_'.$lang.' as daily_type_name ,'.
                self::$main_table.'.treasury_id ,
                treasurys.account_tree_id as treasury_account_tree_id , 
                treasurys.treasury_name_'.$lang.' as treasury_name ,'.
                self::$main_table.'.action_date ,'.
                self::$main_table.'.total_amount ,'.
                self::$main_table.'.arabic_description ,'.
                self::$main_table.'.payment_voucher_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user,
                users.user_full_name
            ');
        }

        $data = $query->first();
        return $data;
    }

    public static function get_payment_voucher_d($id, $lang = null)
    {
        $query = DB::table('payment_voucher_d')
            ->leftjoin('account_tree', 'payment_voucher_d.account_tree_id', '=', 'account_tree.account_tree_id')
            ->leftjoin('cost_center', 'payment_voucher_d.cost_center_id', '=', 'cost_center.cost_center_id')
            ->leftjoin('delegates', 'payment_voucher_d.delegate_id', '=', 'delegates.delegate_id')
            ->leftjoin('clearances_d' , 'payment_voucher_d.container_id' , '=' , 'clearances_d.clearances_d_id')

            ->where('payment_voucher_m_id', $id);

        if($lang != null){
            $query->selectRaw('
                payment_voucher_m_id as m_id ,
                payment_voucher_d_id as d_id ,
                pay_for ,
                payment_voucher_d.account_tree_id ,
                account_tree.name_'.$lang.' as account_tree_name,
                payment_voucher_d.cost_center_id ,
                cost_center.name_'.$lang.' as cost_center_name,
                payment_voucher_d.delegate_id ,
                delegates.name_'.$lang.' as delegate_name,
                container_id ,
                clearances_d.container_number,
                clearance_id ,
                arabic_description ,
                amount ,
                payment_voucher_d.add_date ,
                payment_voucher_d.add_user
            ');
        }

        $data = $query->get();
        return $data;
    }

    public static function add_payment_voucher_m($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_payment_voucher_d($seed)
    {
        return DB::table('payment_voucher_d')->insertGetId($seed);
    }

    public static function update_payment_voucher_d($seed, $id)
    {
        return DB::table(self::$main_table)->where('payment_voucher_m_id', $id)->update($seed);
    }

    public static function update_payment_voucher_m($seed, $id)
    {
        return DB::table(self::$main_table)->where('payment_voucher_m_id', $id)->update($seed);
    }

    public static function delete_payment_voucher_m($id)
    {
        return DB::table(self::$main_table)->where('payment_voucher_m_id', $id)->update(array('payment_voucher_active' => '0', 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public static function delete_payment_voucher_d($id)
    {
        return DB::table('payment_voucher_d')->where('payment_voucher_m_id', $id)->delete();
    }
}
