<?php

namespace App\Models\Finance;

use Illuminate\Database\Eloquent\Model;
use DB;

class RelatedPartie extends Model
{
    protected $primaryKey     = 'related_partie_id';
    public static $main_table = 'related_parties';
    public $table             = 'related_parties';

    public static $rules = array(
        'name_en'         => 'required|string',
        'name_ar'         => 'required|string',
        'account_tree_id' => 'required|numeric',
    );

    public static function all_related_parties($params = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('account_tree' , self::$main_table.'.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->orderBy(self::$main_table.'.related_partie_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                related_partie_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.account_tree_id ,
                account_tree.name_'.$params['lang'].' as account_tree_name ,'.
                self::$main_table.'.related_partie_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('related_partie_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('related_partie_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_related_partie($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('account_tree' , self::$main_table.'.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->where('related_partie_id', $id);

        if($lang != null)
            $query->selectRaw('
                related_partie_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.account_tree_id ,
                account_tree.name_'.$lang.' as account_tree_name ,'.
                self::$main_table.'.related_partie_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_related_partie($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_related_partie($seed, $id)
    {
        return DB::table(self::$main_table)->where('related_partie_id', $id)->update($seed);
    }

    public static function delete_related_partie($id)
    {
        return DB::table(self::$main_table)->where('related_partie_id', $id)->update(array('related_partie_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
