<?php

namespace App\Models\Sales;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;


class AttachmentName extends Model
{
    protected $primaryKey     = 'id';
    public static $main_table = 'clearance_file_names';
    public $table             = 'clearance_file_names';

    public static $rules = array(
        'name_en'          => 'required|string',
        'name_ar'          => 'required|string',
    );

    public static function all_attachment_names($params = null)
    {
        $query = DB::table(self::$main_table)
            ->where(self::$main_table.'.attachment_name_active',1)
            ->orderBy(self::$main_table.'.id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                id  ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.attachment_name_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('attachment_name_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_attachment_name($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('clearance_file_names.attachment_name_active',1)
            ->where('id', $id);

        if($lang != null)
            $query->selectRaw('
                 id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.attachment_name_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_attachment_name($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_attachment_name($seed, $id)
    {
        return DB::table(self::$main_table)->where('id', $id)->update($seed);
    }

    public static function delete_attachment_name($id)
    {
        return DB::table(self::$main_table)->where('id', $id)->update(array('attachment_name_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
