<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class Classification extends Model
{

    protected $primaryKey = 'classification_id';

    public static $main_table = 'classifications_m';
    public static $detail_table = 'classifications_d';
    public $table             = 'classifications_m';

    public static $rules = array(
        'clearances_m_id'              => 'required|numeric|min:1' ,
        'action_date'               => 'required|date' ,
        '*.items.*.item_name' => 'required'
    );

    public static function get_comments($id , $lang){

        return DB::table('classification_comments')
            ->leftjoin('users' , 'classification_comments.add_user' , '=' , 'users.id')
            ->where('classification_id' , $id)
            ->orderBy('classification_comments.add_date' , 'desc')
            ->selectRaw('
                classification_comments.id as id ,
                comment_det ,
                classification_comments.add_date ,
                user_full_name
            ')
            ->get();

    }

    public static function all_classifications($params = null)
    {
        $query =  DB::table('classifications_m')
            ->leftjoin('clearances_m' , 'classifications_m.clearances_m_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('delegates' , 'clearances_m.delegate_id' , '=' , 'delegates.delegate_id')
            ->leftjoin('ports' , 'clearances_m.port_id' , '=' , 'ports.port_id')
            ->leftjoin('classification_comments' , 'classifications_m.classification_id' , '=' , 'classification_comments.classification_id')
            ->leftjoin('classifications_d' , 'classifications_m.classification_id' , '=' , 'classifications_d.classifications_m_id')
            ->leftjoin('classification_status' , 'classifications_m.classification_status_id' , '=' , 'classification_status.id')
            ->leftjoin('currencys as cur1' , 'clearances_m.currency_id' , '=' , 'cur1.currency_id')
            ->leftjoin('currencys as cur2' , 'clearances_m.shipping_currency_id' , '=' , 'cur2.currency_id')->orderBy('classifications_m.classification_id','desc')->groupBy('classifications_m.classification_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                classifications_m.classification_id as id ,
                classifications_m.daily_journal_id ,
                classifications_m.is_custom ,
                classifications_m.is_upload ,
                classifications_m.serial_number ,
                classifications_m.finance_year_id ,
                classifications_m.action_date ,
                supplier_invoice_code ,
                invoice_type ,
                invoice_amount ,
                clearances_m.currency_id ,
                cur1.name_'.$params['lang'].' as currency_name ,
                shipping_amount ,
                clearances_m.shipping_currency_id ,
                cur2.name_'.$params['lang'].' as shipping_currency_name ,
                classifications_m.classification_status_id ,
                classification_status.name_'.$params['lang'].' status_name ,
                total_custom_fees ,
                classifications_m.notes ,
                classifications_m.clearances_m_id ,
                clearances_m.container_count ,
                clearances_m.container_small ,
                clearances_m.container_big ,
                clearances_m.is_lcl ,
                clearances_m.old_sys_id ,
                clearances_m.customer_id ,
                customers.name_'.$params['lang'].' customer_name ,
                clearances_m.delegate_id ,
                delegates.name_'.$params['lang'].' delegate_name ,
                clearances_m.port_id ,
                ports.port_name_'.$params['lang'].' port_name


            ');

        if(isset($params['active']))
            $query->where('classification_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.clearance_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.invoice_type' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.supplier_invoice_code' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('classifications_d.item_name' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('classifications_d.customs_item_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('classification_comments.comment_det' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('classifications_m.action_date' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('classifications_m.classification_id' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_classification($id , $lang = null)
    {
        $query =  DB::table(self::$main_table)
            ->where('classifications_m.classification_id' , $id);

        if( isset($lang) )
            $query
                ->leftjoin('clearances_m' , 'classifications_m.clearances_m_id' , '=' , 'clearances_m.clearance_id')
                ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
                ->leftjoin('delegates' , 'clearances_m.delegate_id' , '=' , 'delegates.delegate_id')
                ->leftjoin('delegates as cd' , 'clearances_m.delegate_id' , '=' , 'cd.delegate_id')
                ->leftjoin('countrys' , 'clearances_m.country_id' , '=' , 'countrys.country_id')
                ->leftjoin('ports' , 'clearances_m.port_id' , '=' , 'ports.port_id')
                ->leftjoin('classification_status' , 'classifications_m.classification_status_id' , '=' , 'classification_status.id')
                ->leftjoin('currencys as cur1' , 'clearances_m.currency_id' , '=' , 'cur1.currency_id')
                ->leftjoin('currencys as cur2' , 'clearances_m.shipping_currency_id' , '=' , 'cur2.currency_id')
                ->leftjoin('branchs' , 'branchs.branch_id' , '=' , 'clearances_m.branch_id')
                ->leftjoin('agents' , 'clearances_m.agent_number' , '=' , 'agents.agent_id')
                ->selectRaw('
                    classifications_m.classification_id as id ,
                    classifications_m.daily_journal_id ,
                    classifications_m.is_custom ,
                    classifications_m.is_upload ,
                    classifications_m.serial_number ,
                    classifications_m.finance_year_id ,
                    classifications_m.action_date ,
                    supplier_invoice_code ,
                    invoice_type ,
                    invoice_amount ,
                    weight_factor ,
                    clearances_m.currency_id ,
                    cur1.name_'.$lang.' as currency_name ,
                    cur1.equivalent_value ,
                    shipping_amount ,
                    clearances_m.shipping_currency_id ,
                    cur2.name_'.$lang.' as shipping_currency_name ,
                    classifications_m.classification_status_id ,
                    classification_status.name_'.$lang.' status_name ,
                    total_custom_fees ,
                    classifications_m.notes ,
                    clearances_m.container_small ,
                    clearances_m.container_big ,
                    classifications_m.clearances_m_id ,
                    clearances_m.container_count ,
                    clearances_m.first_container ,
                    clearances_m.exporting_company ,
                    clearances_m.note ,
                    clearances_m.is_lcl ,
                    clearances_m.lcl_count ,
                    clearances_m.old_sys_id ,
                    clearances_m.customer_id ,
                    customers.name_'.$lang.' customer_name ,
                    customers.alireda_code ,
                    clearances_m.delegate_id ,
                    delegates.name_'.$lang.' delegate_name ,
                    cd.name_'.$lang.' customer_delegate_name ,
                    clearances_m.port_id ,
                    clearances_m.shipping_policy_no ,
                    ports.port_name_'.$lang.' port_name,
                    countrys.country_id ,
                    countrys.name_'.$lang.' country_name,
                    clearances_m.weight,
                    clearances_m.saber_no,
                    branchs.print_label_'.$lang.' as print_label,
                    clearances_m.invoice_2,
                    clearances_m.invoice_3,
                    clearances_m.invoice_4,
                    clearances_m.invoice_date,
                    clearances_m.action_date  as clearance_date,
                    clearances_m.arrival_date,
                    agents.agent_name_'.$lang.' as agent_name,
                    agents.mobile as agent_number,
                    DATE_FORMAT(classifications_m.add_date, "%Y-%m-%d") as classification_add_date
                ');

        $data = $query->first();

        return $data;
    }

    public static function get_classification_d_without_group($id , $lang = null)
    {
        $data =  DB::table('classifications_d')
            ->leftjoin('customs_items' , 'classifications_d.customs_item_id' , '=' , 'customs_items.customs_item_id')
            ->leftjoin('countrys' , 'classifications_d.country_id' , '=' , 'countrys.country_id')
            ->leftjoin('units' , 'classifications_d.unit_id' , '=' , 'units.unit_id')
            ->leftjoin('clearances_d' , 'classifications_d.containers' , '=' , 'clearances_d.clearances_d_id')
            ->where('classifications_d.classifications_m_id' , $id )
            ->selectRaw('* ,
                SUM( classifications_d.amount) as amount ,
                SUM( classifications_d.qry) as qry ,
                SUM(classifications_d.weight) as weight ,
                SUM(classifications_d.custom_fees) as custom_fees ,
                classifications_d.item_type , countrys.name_'.$lang.' as country_name   ,
                units.name_'.$lang.' as unit_name ,
                customs_items.name_'.$lang.' as customs_item_name ,
                classifications_d.item_name as item_name,
                classifications_d.customs_item_number as customs_item_number ,
                classifications_d.class_slug as slug')
            ->groupBy('classifications_d.classifications_d_id')
            ->orderBy('classifications_d.classifications_d_id')
            ->get();
        return $data;
    }

    public static function get_classification_d($id , $lang = null)
    {
        $data =  DB::table('classifications_d')
            ->leftjoin('customs_items' , 'classifications_d.customs_item_id' , '=' , 'customs_items.customs_item_id')
            ->leftjoin('countrys' , 'classifications_d.country_id' , '=' , 'countrys.country_id')
            ->leftjoin('units' , 'classifications_d.unit_id' , '=' , 'units.unit_id')
            ->leftjoin('clearances_d' , 'classifications_d.containers' , '=' , 'clearances_d.clearances_d_id')
            ->where('classifications_d.classifications_m_id' , $id )
            ->selectRaw('* ,
                SUM( classifications_d.amount) as amount ,
                SUM( classifications_d.qry) as qry ,
                SUM(classifications_d.weight) as weight ,
                SUM(classifications_d.custom_fees) as custom_fees ,
                classifications_d.item_type ,
                countrys.name_'.$lang.' as country_name   ,
                units.name_'.$lang.' as unit_name ,
                customs_items.name_'.$lang.' as customs_item_name ,
                classifications_d.item_name as item_name,
                classifications_d.customs_item_number as customs_item_number ,
                classifications_d.class_slug as slug')
            ->groupBy('classifications_d.class_slug')
            ->orderBy('classifications_d.classifications_d_id')
            ->get();

        return $data;
    }
    public static function get_first_classification_d($id , $lang = null)
    {
        $data =  DB::table('classifications_d')
            ->leftjoin('customs_items' , 'classifications_d.customs_item_id' , '=' , 'customs_items.customs_item_id')
            ->leftjoin('countrys' , 'classifications_d.country_id' , '=' , 'countrys.country_id')
            ->leftjoin('units' , 'classifications_d.unit_id' , '=' , 'units.unit_id')
            ->leftjoin('clearances_d' , 'classifications_d.containers' , '=' , 'clearances_d.clearances_d_id')
            ->where('classifications_d.classifications_m_id' , $id )
            ->selectRaw('* , SUM( classifications_d.amount) as amount , SUM( classifications_d.qry) as qry , SUM(classifications_d.weight) as weight , SUM(classifications_d.custom_fees) as custom_fees ,classifications_d.item_type , countrys.name_'.$lang.' as country_name   , units.name_'.$lang.' as unit_name , customs_items.name_'.$lang.' as customs_item_name , customs_items.item_name_'.$lang.' as item_name,customs_items.name_'.$lang. ' as customs_item_number , classifications_d.class_slug as slug')
            ->orderBy('classifications_d.classifications_d_id')
            ->get();

        return $data;
    }
    public static function get_temp_classification($params = null)
    {
        $query =  DB::table('classifications_temp')
        ->leftjoin('users','users.id' ,'classifications_temp.add_user')
        ->selectRaw('
            classifications_temp.id,
            classifications_temp.clearances_m_id,
            classifications_temp.add_user,
            classifications_temp.add_date,
            users.user_full_name as added_by');

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('classifications_temp.clearances_m_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }
        return $data;
    }



    public static function add_classification($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_classification_d($seed)
    {
        return DB::table('classifications_d')->insert($seed);
    }

    public static function update_classification($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('classification_id' , $id )->update($seed);
    }

    public static function update_classification_d($seed ,  $id)
    {
        return DB::table(self::$detail_table)->where('classifications_d_id' , $id )->update($seed);
    }

    public static function delete_classification($id)
    {
        return DB::table(self::$main_table)->where('classification_id' , $id )->update( array('classification_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
