<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Request;

class Clearance extends Model
{

    protected $primaryKey = 'clearance_id';

    public static $main_table = 'clearances_m';
    public $table             = 'clearances_m';

    public static $maps    = [
        'action_date'       => 'clearances_m.action_date' ,
        'id'                => 'clearances_m.clearance_id' ,
        'customer_name'     => 'customers.name_ar' ,
        'container_number'     => 'clearances_d.container_number' ,
        'unloading_ship_date'     => 'clearances_m.unloading_ship_date' ,
        'expiry_date'       => 'clearances_m.expiry_date' ,
        'delegate_name'     => 'delegates.name_ar' ,
        'container_count'   => 'clearances_m.container_count' ,
        'shipping_policy_no'=> 'clearances_m.shipping_policy_no' ,
        'port_withdraw'     => 'clearances_m.port_withdraw' ,
        'withdrawal_note'   => 'clearances_m.withdrawal_note',
        'inspection_number' => 'clearances_m.inspection_number' ,
        'expiry_date'       => 'clearances_m.expiry_date',
        'customs_declaration_number'=>'clearances_m.customs_declaration_number',
        'customs_floor_date'        => 'clearances_m.customs_floor_date',
        'draft_id'                  =>'clearance_invoices_draft_m.invoice_id',
        'classification_id'         => 'clearances_m.classification_id'



    ];
    public static $rules = array(
        'customer_id'               => 'required|numeric|min:1' ,
        'delegate_id'               => 'required|numeric|min:1' ,
        'currency_id'               => 'required|exists:currencys,currency_id',
        'port_id'                   => 'required|numeric|min:1' ,
        'action_date'               => 'required|numeric|min:1' ,
        // 'container_count'           => 'required|numeric' ,
        // 'is_lcl'                    => 'required|boolean' ,
        'action_date'               => 'required|date' ,
        'policy_type'               => 'in:1,2,3'

        // 'invoice_type'           => 'required' ,
        // 'invoice_amount'           => 'required' ,
        // 'currency_id'           => 'required' ,
        // 'shipping_amount'           => 'required' ,
        // 'shipping_currency_id'           => 'required' ,
        // 'supplier_invoice_code'           => 'required' ,

    );

    public static $customs_declaration_rules = array(
        'customs_declaration_number'   => 'required',
        'customs_declaration_date'   => 'required|Date',
        'customs_declaration_date_hijri'   => 'required',
        'unloading_date'   => 'nullable|Date',
        'unloading_date_hijri'   => 'nullable',
    );
    public static function get_files($id , $lang){

        return DB::table('clearance_attachments')
            ->leftjoin('clearance_file_names' , 'clearance_attachments.clearance_file_name_id' , '=' , 'clearance_file_names.id')
            ->where('clearance_id' , $id)
            ->where('clearance_file_names.attachment_name_active',1)
            ->selectRaw('
                clearance_attachment_id as id ,
                name_'.$lang.' as file_name ,
                file_type ,
                file_path
            ')
            ->get();

    }
    public static function delegate_petty_attachment($lang,$clearance_id){
        return  DB::table('delegate_petty_d')
            ->where('delegate_petty_d.clearance_id',$clearance_id)
            ->whereNotNull('file_path')
            ->selectRaw('
            "delegate_attachment" as file_name ,
             "" as file_type ,
             delegate_petty_d.file_path,
             delegate_petty_d_id as id')->get();
    }
    public static function multi_voucher_attachment($lang,$clearance_id){
        return  DB::table('multi_voucher_d')
            ->where('multi_voucher_d.clearance_id',$clearance_id)
            ->whereNotNull('attachment')
            ->selectRaw('
            "multi_voucher_attachments" as file_name ,
             "" as file_type ,
             "multi_voucher_attachments" as type_attachment ,
             attachment as file_path ,
             multi_voucher_d_id as id')->get();
    }
    public static function all_clearances($params = null)
    {
        // $comments = DB::table('clearances_comments')
        //     ->limit(1)
        //     ->orderBy('clearances_comment_id',  'desc');


        $query =  DB::table(self::$main_table)
            // ->leftjoin('daily_journal_m', self::$main_table.'.daily_journal_id', '=', 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('delegates as customer_delgates' , 'customers.delegate_id' , '=' , 'customer_delgates.delegate_id')
            ->leftjoin('customers as sub_customers' , self::$main_table.'.sub_customer_id' , '=' , 'sub_customers.customer_id')

            // ->leftjoin('customers as main_customers' , 'customers.customer_main_id' , '=' , 'main_customers.customer_id')
            ->leftjoin('delegates' , self::$main_table.'.delegate_id' , '=' , 'delegates.delegate_id')
            ->leftjoin('ports' , self::$main_table.'.port_id' , '=' , 'ports.port_id')
            ->leftjoin('clearance_status' , self::$main_table.'.clearance_status_id' , '=' , 'clearance_status.clearance_status_id')
            ->leftjoin('clearances_comments' , self::$main_table.'.clearance_id' , '=' , 'clearances_comments.clearances_m_id')
             ->leftjoin('clearance_invoices_draft_m' , 'clearance_invoices_draft_m.clearance_id' , '=' , 'clearances_m.clearance_id')
             ->leftjoin('clearance_invoices_m' , 'clearance_invoices_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('clearances_d' , self::$main_table.'.clearance_id' , '=' , 'clearances_d.clearances_m_id')
            // ->leftjoin('policies_types' , self::$main_table.'.policy_type_id' , '=' , 'policies_types.policy_type_id')
            // ->leftjoin('agents' , self::$main_table.'.agent_number' , '=' , 'agents.agent_id')

            ->groupBy('clearances_m.clearance_id');
            // ->orderBy('clearances_m.customs_floor_date' , 'desc')
            // ->orderBy('clearances_m.clearance_id' , 'desc');

        if( isset($params['lang']) )


            // policies_types.policy_type_name_'.$params['lang'].' ,
//             group_concat(distinct clearance_invoices_m.invoice_id ) as invoices
            // agents.agent_name_'.$params['lang'].' agent_name ,
                // daily_journal_m.year_ser as journal_code ,



            $query->selectRaw('
                clearances_m.clearance_id as id ,
                saber_no ,
                '.self::$main_table.'.note ,
                if(is_lcl = 1 , "LCL" , clearances_m.first_container ) as first_container_number ,
                clearances_m.finance_year_id ,
                clearances_m.customer_id ,
                customers.name_'.$params['lang'].' customer_name ,

                clearances_m.sub_customer_id ,
                sub_customers.name_'.$params['lang'].' sub_customer_name ,

                customers.delegate_id as customer_delgate,
                customer_delgates.name_'.$params['lang'].' customer_delegate_name ,

                clearances_m.delegate_id ,
                delegates.name_'.$params['lang'].' delegate_name ,
                clearances_m.port_id ,
                clearances_m.exporting_company ,
                ports.port_name_'.$params['lang'].' port_name ,
                clearances_m.clearance_status_id ,
                clearance_status.name_'.$params['lang'].' status_name ,
                clearances_m.tracking_number ,
                clearances_m.daily_journal_id ,
                clearances_m.pledge_number ,
                clearances_m.pledge_date ,
                clearances_m.unloading_ship_date ,
                clearances_m.unloading_ship_name ,
                file_no ,
                clearances_m.goods_type ,
                customs_declaration_date ,
                customs_declaration_date_hijri ,
                customs_declaration_number ,
                customs_declaration_file ,
                mainfest_date ,
                mainfest_date_hijri ,
                mainfest_no ,
                customs_floor_date ,
                arrival_date ,
                expiry_date ,
                agent_number ,
                inspection_number ,
                unloading_date ,
                unloading_date_hijri ,
                withdrawal_note ,
                clearances_m.classification_id ,
                clearances_m.action_date ,
                clearances_m.shipping_policy_no ,
                clearances_m.clearance_type ,
                clearances_m.container_count ,
                clearances_m.final_note_file ,
                clearances_m.container_small ,
                clearances_m.container_big ,
                clearances_m.is_lcl ,
                clearances_m.weight ,
                clearances_m.lcl_count ,
                clearances_m.port_withdraw ,
                clearances_m.has_problem ,
                clearances_m.old_sys_id,
                clearances_m.invoice_type,
                clearances_m.supplier_invoice_code,
                clearances_m.invoice_amount,
                clearances_m.currency_id,
                clearances_m.shipping_amount,
                clearances_m.shipping_currency_id,
                clearances_m.majira_policy,
                clearance_invoices_draft_m.invoice_id as draft_id,
                customers.alireda_code ,
                group_concat(distinct clearance_invoices_m.invoice_id ) as invoices,
                COUNT(clearances_comments.clearances_comment_id) as is_comments_exist
            ');
//        IF(EXISTS (SELECT 1 FROM clearances_comments WHERE clearances_comments.clearances_m_id = clearances_m.clearance_id), true, false) as is_comments_exist

        if(isset($params['classification_added']))
            $query->where('classification_id' , '>' , 0  );

        if(isset($params['classification_free']))
            $query->where('classification_id' ,  0  );

        if(isset($params['is_classification'])  &&  $params['is_classification'] == 0 ){
            $query->where('clearances_m.classification_id' , 0 )
                    ->where('clearances_m.under_classification' , 0);
        }


        if (isset($params['has_invoice'])) {
            if ($params['has_invoice'] == 1) {
                $query->havingRaw('COUNT(clearance_invoices_m.invoice_id) > 0');
            } elseif ($params['has_invoice'] == 0) {
                $query->havingRaw('COUNT(clearance_invoices_m.invoice_id) < 1');
            }
        }

        if(isset($params['container_number']))
            $query->where('clearances_d.container_number' , 'LIKE' , "%{$params['container_number']}%"  );

        if(isset($params['has_problem']))
            $query->where('clearances_m.has_problem' ,  $params['has_problem']  );

        if(isset($params['port_id']))
            $query->where('clearances_m.port_id' , $params['port_id'] );

        if(isset($params['agent_id']))
            $query->where('clearances_m.agent_number' , $params['agent_id'] );


        if(isset($params['delegate_id'])){
            if($params['delegate_type'] == 2){
                $query->where('customers.delegate_id' , $params['delegate_id']);

            }else{
                $query->where('clearances_m.delegate_id' , $params['delegate_id'] );
            }
        }

        if(isset($params['customer_id']))
            $query->where('clearances_m.customer_id' , $params['customer_id'] );

        if(isset($params['full_customer_id'])){
            $query->where(function($where) use($params){
                $where->where('clearances_m.customer_id' , $params['full_customer_id'] )
                      ->orWhere('customers.customer_main_id' , $params['full_customer_id'] );
            });
        }

        if(isset($params['clearance_id']))
            $query->where('clearances_m.clearance_id' , $params['clearance_id'] );

        if(isset($params['customer_main_id']))
            $query->where('customers.customer_main_id' , $params['customer_main_id'] );

        if(isset($params['shipping_policy_no']))
            $query->where('clearances_m.shipping_policy_no' , $params['shipping_policy_no'] );

        if(isset($params['from']))
            $query->where('clearances_m.action_date' , '>=' , $params['from'] );

        if(isset($params['to']))
            $query->where('clearances_m.action_date' , '<=' , $params['to'] );

        if(isset($params['withdrawal_note']))
            $query->whereNotNull('clearances_m.withdrawal_note' );

        if(isset($params['has_withdrawal_note'])){
            if($params['has_withdrawal_note'] == 1){
                $query->whereNotNull('clearances_m.withdrawal_note');
            }else{
                $query->whereNull('clearances_m.withdrawal_note');
            }
        }


        if(isset($params['port_withdraw']))
            $query->where('clearances_m.port_withdraw' ,  1  );



        if(isset($params['status']) && is_array($params['status'])){
            $query->whereIn('clearances_m.clearance_status_id' ,  $params['status'] );
        }elseif(isset($params['status'])){
            $query->where('clearances_m.clearance_status_id' ,  $params['status'] );
        }

        if(isset($params['active']))
            $query->where('clearance_active' , $params['active'] );

        if(isset($params['filter_by']) && isset($params['filter_value']) ){
            $query->where(self::$maps[$params['filter_by']] ?? 'clearances_m.clearance_id' , $params['filter_value'] ?? 0 );
        }

        if(isset($params['sort_by']) && isset($params['sort_value']) ){
            $query->orderBy(self::$maps[$params['sort_by']] ?? 'clearances_m.clearance_id' , $params['sort_value'] ?? 0 );
        }else{
            $query->orderBy('clearances_m.clearance_id' , 'desc');
        }

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->leftjoin('customers as main_customers' , 'customers.customer_main_id' , '=' , 'main_customers.customer_id')
                  // ->leftjoin('clearance_invoices_m' , 'clearance_invoices_m.clearance_id' , '=' , 'clearances_m.clearance_id')
                  // ->leftjoin('clearances_d' , self::$main_table.'.clearance_id' , '=' , 'clearances_d.clearances_m_id')
                  ->where(function ($stmt) use($params) {
                    $stmt->where('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('main_customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('main_customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.shipping_policy_no' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('ports.port_name_ar' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('ports.port_name_en' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.file_no' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.customs_declaration_number' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.first_container' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('delegates.name_ar' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('delegates.name_en' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.majira_policy' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('clearances_m.supplier_invoice_code' , 'LIKE' , "%{$params['word']}%")
                        ->orWhere('customers.name_ar' , $params['word'])
                        ->orWhere('clearances_m.shipping_policy_no' , $params['word'])
                        ->orWhere('clearances_m.withdrawal_note' , $params['word'])
                        ->orWhere('ports.port_name_ar' , $params['word'])
                        ->orWhere('ports.port_name_en' , $params['word'])
                        ->orWhere('clearances_m.file_no' , $params['word'])
                        ->orWhere('clearances_m.clearance_id' , $params['word'] );

            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }elseif(isset($params['count'])){
            $data = $query->take($params['count'])->get();
        }else{
            $data = $query->get();
        }

        return $data;
    }


    public static function all_clearances_short($params = null)
    {


        $query =  DB::table(self::$main_table)
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('customers as main_cust' , 'customers.customer_main_id' , '=' , 'main_cust.customer_id')
            ->leftjoin('clearances_d' , self::$main_table.'.clearance_id' , '=' , 'clearances_d.clearances_m_id')
            ->leftJoin('customer_claims_m', 'customer_claims_m.clearance_id', '=', 'clearances_m.clearance_id')
            ->leftjoin('agents' , self::$main_table.'.agent_number' , '=' , 'agents.agent_id')
            ->groupBy('clearances_m.clearance_id');
//         $classification = Classification::all_classifications(['active' => 1])->pluck('clearance_id')->toArray();

        if( isset($params['lang']) )
            $query->selectRaw('
                clearances_m.clearance_id  ,
                clearances_m.customer_id ,
                customers.customer_main_id ,
                customers.name_'.$params['lang'].' customer_name,
                main_cust.name_'.$params['lang'].' main_customer_name,
                agents.agent_name_'.$params['lang'].' agent_name ,
                clearances_m.shipping_policy_no,
                clearances_m.invoices_count ,
                clearances_m.withdrawal_note,
                clearances_m.is_lcl ,
                customers.issuing_invoice as has_price ,
                main_cust.issuing_invoice as main_has_price ,
                concat(clearances_m.clearance_id  ," - " , customers.name_'.$params['lang'].' , " - " ,clearances_m.shipping_policy_no) as label,
                CASE
                    WHEN COUNT(customer_claims_m.clearance_id) > 0 THEN 1 ELSE 0
                END as is_customer_claim

            ');

        if(isset($params['is_classification']) &&  $params['is_classification'] == 0)
            $query->leftjoin('classifications_temp','classifications_temp.clearances_m_id','clearances_m.clearance_id')
            ->where('clearances_m.classification_id' , 0 )
            ->where('clearances_m.under_classification' , 0)
            ->orWhere('classifications_temp.add_user', auth()->user()->id);



        if(isset($params['is_withdrawal']) &&  $params['is_withdrawal'] == 1)
            $query->whereNotNull('clearances_m.withdrawal_note');

        if(isset($params['status']) && is_array($params['status'])){
            $query->whereIn('clearances_m.clearance_status_id' ,  $params['status'] );
        }elseif(isset($params['status'])){
            $query->where('clearances_m.clearance_status_id' ,  $params['status'] );
        }


        if(isset($params['active']))
            $query->where('clearance_active' , $params['active'] );

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.clearance_id' , 'LIKE' , "%{$params['word']}%" )
                    ->orWhere('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%" )
                    ->orWhere('clearances_m.shipping_policy_no' , 'LIKE' , "%{$params['word']}%" );

            });

        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }



    public static function get_clearance($id , $lang = null)
    {
        $query =  DB::table(self::$main_table)
            ->where('clearances_m.clearance_id' , $id);

        if( isset($lang) )
            $query
                ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id')
                ->leftjoin('customers as sub_customers' , self::$main_table.'.sub_customer_id' , '=' , 'sub_customers.customer_id')
                ->leftjoin('customers as main_cust' , 'customers.customer_main_id' , '=' , 'main_cust.customer_id')
                ->leftjoin('countrys' , self::$main_table.'.country_id' , '=' , 'countrys.country_id')
                ->leftjoin('currencys as cur1' , self::$main_table.'.currency_id' , '=' , 'cur1.currency_id')
                ->leftjoin('currencys as cur2' , self::$main_table.'.shipping_currency_id' , '=' , 'cur2.currency_id')
                ->leftjoin('delegates' , self::$main_table.'.delegate_id' , '=' , 'delegates.delegate_id')
                ->leftjoin('ports' , self::$main_table.'.port_id' , '=' , 'ports.port_id')
                ->leftjoin('clearance_status' , self::$main_table.'.clearance_status_id' , '=' , 'clearance_status.clearance_status_id')
                ->leftjoin('policies_types' , self::$main_table.'.policy_type_id' , '=' , 'policies_types.policy_type_id')
                ->leftjoin('agents' , self::$main_table.'.agent_number' , '=' , 'agents.agent_id')
                ->selectRaw('
                    clearance_id as id ,
                    saber_no ,
                    note ,
                    clearances_m.daily_journal_id ,
                    finance_year_id ,
                    clearances_m.customer_id ,
                    clearances_m.sub_customer_id ,
                    sub_customers.name_'.$lang.' sub_customer_name ,
                    clearances_m.branch_id ,
                    customers.alireda_code ,
                    main_cust.customer_id  as main_customer_id,
                    customers.name_'.$lang.' customer_name ,
                    agents.agent_name_'.$lang.' agent_name ,
                    customers.vat_register_number ,
                    customers.address ,
                    clearances_m.delegate_id ,
                    delegates.name_'.$lang.' delegate_name ,
                    clearances_m.port_id ,
                    ports.port_name_'.$lang.' port_name ,
                    clearances_m.clearance_status_id ,
                    clearances_m.paid_by ,
                    clearance_status.name_'.$lang.' status_name ,
                    file_no ,
                    clearances_m.tracking_number ,
                    clearances_m.pledge_number ,
                    clearances_m.pledge_date ,
                    clearances_m.unloading_ship_date ,
                    clearances_m.unloading_ship_name ,
                    clearances_m.goods_type ,
                    customs_declaration_date ,
                    customs_declaration_date_hijri ,
                    customs_declaration_number ,
                    customs_declaration_file ,
                    withdrawal_note ,
                    mainfest_date ,
                    mainfest_date_hijri,
                    mainfest_no ,
                    arrival_date ,
                    agent_number,
                    inspection_number ,
                    unloading_date ,
                    unloading_date_hijri ,
                    clearances_m.classification_id ,
                    clearances_m.under_classification ,
                    clearances_m.action_date ,
                    clearances_m.majira_policy ,
                    clearances_m.shipping_policy_no ,
                    clearances_m.clearance_type ,
                    clearances_m.container_count ,
                    clearances_m.container_small ,
                    clearances_m.container_big ,
                    clearances_m.is_lcl ,
                    clearances_m.port_withdraw ,
                    clearances_m.lcl_count ,
                    clearances_m.has_problem ,
                    clearances_m.old_sys_id ,
                    clearances_m.customs_floor_date ,
                    clearances_m.final_note_file ,
                    clearances_m.add_date,
                     clearances_m.invoice_type,
                    clearances_m.supplier_invoice_code,
                    clearances_m.invoice_amount,
                    clearances_m.weight ,
                    clearances_m.weight_factor,
                    clearances_m.currency_id,
                    cur1.name_'.$lang.' as currency_name,
                    clearances_m.shipping_amount,
                    clearances_m.shipping_currency_id,
                    cur2.name_'.$lang.' as shipping_currency_name,
                    clearances_m.exporting_company,
                    clearances_m.invoice_date,
                    clearances_m.country_id,
                    countrys.name_'.$lang.' as country_name,
                    clearances_m.invoice_2,
                    clearances_m.invoice_3,
                    clearances_m.invoice_4 ,
                    clearances_m.invoices_count ,
                    if(clearances_m.weight > 0 and clearances_m.invoice_amount > 0 , clearances_m.weight / clearances_m.invoice_amount , 0 ) as weight_factor ,
                    policies_types.policy_type_name_'.$lang.' as policy_type_name ,
                    policies_types.policy_type_id
                ');

        $data = $query->first();
        return $data;
    }


    public static function get_clearance_d($id , $lang = 'ar')
    {
        $data =  DB::table('clearances_d')
            ->leftjoin('trips','trips.clearance_d_id','clearances_d.clearances_d_id')
            ->leftjoin('areas','areas.area_id','clearances_d.dest_id')
            ->where('clearances_d.clearances_m_id' , $id)
            ->groupBy("clearances_d.clearances_d_id")
            ->selectRaw('
                clearances_d.clearances_d_id ,
                clearances_d.clearances_m_id,
                clearances_d.recommended_return_date,
                clearances_d.container_size,
                clearances_d.container_number,
                clearances_d.seal_number,
                clearances_d.from_area,
                clearances_d.dest_id,
                areas.area_name_'.$lang.' as dest_name ,
                count(trips.trip_id) as count_trips ,
                clearances_d.to_area,
                clearances_d.container_status_id,
                clearances_d.container_trip_id ,
                clearances_d.add_date,
                clearances_d.add_user,
                clearances_d.archive_id,
                clearances_d.return_date,
                clearances_d.return_file,
                clearances_d.is_hold
                ')
            ->get();


        return $data;
    }


    public static function get_clearance_d_insurance($params)
    {
        $query =  DB::connection('other_mysql')->table('clearances_d')
            ->leftjoin('clearances_m' , 'clearances_d.clearances_m_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('insurances' , 'clearances_m.clearance_id' , '=' , 'insurances.clearance_id')
            ->where('clearances_m.clearance_active' , 1 )
            // ->where('insurances.status' , 1 )
            ->selectRaw('
                clearances_d.clearances_d_id as d_id ,
                container_size ,
                container_number ,
                clearances_m.clearance_id ,
                clearances_d.archive_id ,
                insurances.insurance_id,
                clearances_d.recommended_return_date,
                clearances_d.return_date,
                clearances_d.return_file
            ');


        if(isset($params['agent_id']) && $params['agent_id'] > 0 )
            $query->where('insurances.agent_id' ,$params['agent_id'] );

        if(isset($params['clearance_id']) && $params['clearance_id'] > 0 )
            $query->where('clearances_d.clearances_m_id' ,$params['clearance_id'] );

        if(isset($params['archive_id']))
            $query->where('clearances_d.archive_id' ,$params['archive_id'] );

        if(isset($params['from_date']) && $params['from_date'] > "2000-01-01" )
            $query->where('insurances.action_date' , '>=' , $params['from_date'] );

        if(isset($params['to_date']) && $params['from_date'] > "2000-01-01" )
            $query->where('insurances.action_date' , '<=' , $params['to_date'] );

        if(isset($params['port_id']) && $params['port_id'] > 0 )
            $query->where('clearances_m.port_id' ,$params['port_id'] );

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }


        return $data;
    }


    public static function get_clearance_comment($id , $lang = 'ar')
    {
        $data =  DB::table('clearances_comments')
            ->leftjoin('clearance_status' , 'clearances_comments.status_id' , '=' , 'clearance_status.clearance_status_id')
            ->leftjoin('users' , 'clearances_comments.add_user' , '=' , 'users.id')
            ->where('clearances_m_id' , $id)
            ->selectRaw('
                clearances_comment_id as id ,
                has_problem ,
                note ,
                add_date ,
                user_full_name as add_user  ,
                clearances_comments.status_id ,
                clearance_status.name_'.$lang.' status_name
            ')
            ->get();

        return $data;
    }

    public static function get_available_container($params = null)
    {
        $other_db = session()->get('otherDB');
        $query =  DB::connection('other_mysql')->table('clearances_d')
            ->leftjoin('clearances_m' , 'clearances_m.clearance_id' , '=' , 'clearances_d.clearances_m_id')
            ->leftjoin('areas','areas.area_id','clearances_d.dest_id')
            ->leftjoin('ports' , 'clearances_m.port_id' , '=' , 'ports.port_id')
            ->leftjoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('drivers' , 'clearances_d.driver_id' , '=' , 'drivers.driver_id')
            ->whereNull('clearances_d.container_trip_id')
            ->whereNotNull('clearances_m.withdrawal_note')
            ->whereNotNull('clearances_m.port_withdraw')
            ->where('clearances_m.clearance_status_id' , 5 )
            ->selectRaw('
                clearances_d.clearances_d_id ,
                clearances_d.clearances_m_id,
                clearances_d.recommended_return_date,
                clearances_d.container_size,
                clearances_d.container_number,
                clearances_d.seal_number,
                clearances_d.from_area,
                clearances_d.dest_id,
                areas.area_name_'.$params['lang'].' as dest_name ,
                ports.branch_id,
                ports.port_name_'.$params['lang'].' port_name ,
                customers.name_'.$params['lang'].' customer_name ,
                clearances_m.port_id ,
                clearances_m.mainfest_date,
                clearances_m.unloading_ship_date ,
                clearances_m.unloading_ship_name ,
                clearances_m.shipping_policy_no ,
                clearances_m.customs_declaration_number ,
                clearances_m.receipt_date ,
                clearances_d.port_gate ,
                clearances_d.location_date ,
                clearances_d.container_weight ,
                clearances_m.final_note_file ,
                clearances_d.customs_date ,
                clearances_d.hazardous ,
                clearances_d.shipment_date ,
                clearances_d.driver_id ,
                clearances_d.notes ,
                clearances_d.status ,
                drivers.driver_name_'.$params['lang'].' as driver_name ,
                clearances_d.to_area,
                clearances_d.container_status_id,
                clearances_d.add_date,
                clearances_d.add_user,
                clearances_d.archive_id,
                clearances_d.return_date,
                clearances_d.return_file,
                clearances_d.is_hold
                ');
            if(isset($params['from']) && $params['from'] != 'null')
                $query->where('clearances_m.add_date' , '>=' , $params['from']." 00:00:00" );

            if(isset($params['to']) && $params['to'] != 'null')
                $query->where('clearances_m.add_date' , '<=' , $params['to']." 23:59:59" );


            if(isset($params['clearance_id']) && $params['clearance_id'] > 0)
                $query->where('clearances_m.clearance_id' , $params['clearance_id'] );

            if(isset($params['container_number']))
                $query->where('clearances_d.container_number', $params['container_number'] );

            if(isset($params['branch_id']) && $params['branch_id'] > 0)
                $query->where('ports.branch_id' , '=' , $params['branch_id'] );

            if(isset($params['port_id']) && $params['port_id'] > 0)
                $query->where('clearances_m.port_id' , '=' , $params['port_id'] );

            if(isset($params['ship_name']) && strlen($params['ship_name']) > 0)
                $query->where('clearances_m.unloading_ship_name' , '=' , $params['ship_name'] );

            if(isset($params['port_ids']) && is_array($params['port_ids']) && count( $params['port_ids'] ) > 0)
                $query->whereIn('clearances_m.port_id' , $params['port_ids'] );

            if(isset($params['filter_by']) && isset($params['filter_value']) ){
                $query->where(self::$maps[$params['filter_by']] ?? 'clearances_m.clearance_id' , $params['filter_value'] ?? 0 );
            }


            if(isset($params['sort_by']) && isset($params['sort_value']) ){
            $query->orderBy(self::$maps[$params['sort_by']] ?? 'clearances_m.clearance_id' , $params['sort_value'] ?? 0 );
        }

            // if(isset($params['word']) && strlen($params['word']) >  0){
            //     $query->where(function ($stmt) use($params) {
            //             $stmt->where('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%")
            //                 ->orWhere('clearances_d.clearances_d_id' , $params['word'] )
            //                 ->orWhere('clearances_d.clearances_m_id' , $params['word'] )
            //                 ->orWhere('clearances_d.clearances_d_id' , $params['word'] )
            //                 ->orWhere('clearances_d.clearances_m_id' , $params['word'] );
            //     });
            // }
            $searchableColumns = [
                'clearances_d.clearances_d_id',
                'clearances_d.clearances_m_id',
                'clearances_d.recommended_return_date',
                'clearances_d.container_size',
                'clearances_d.container_number',
                'clearances_d.seal_number',
                'clearances_d.from_area',
                'areas.area_name_' . $params['lang'],
                'ports.port_name_' . $params['lang'],
                'customers.name_' . $params['lang'],
                'clearances_m.shipping_policy_no',
                'clearances_m.customs_declaration_number',
                'drivers.driver_name_' . $params['lang'],
                'clearances_d.notes'
            ];

            $query->where(function ($stmt) use ($params, $searchableColumns) {
                foreach ($searchableColumns as $column) {
                    $stmt->orWhere($column, 'LIKE', "%{$params['word']}%");
                }
            });
            if(isset($params['rows'])){
                $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);

            }else{
                $data = $query->get();
            }

        return $data;
    }

    public static function add_clearance($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_clearance_d($seed)
    {
        return DB::table('clearances_d')->insert($seed);
    }

    public static function update_clearance($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('clearance_id' , $id )->update($seed);
    }

    public static function delete_clearance($id)
    {
        return DB::table(self::$main_table)->where('clearance_id' , $id )->update( array('clearance_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }

    public  static function delete_clearance_file($id){
        $file = DB::table('clearance_attachments')
            ->where('clearance_attachment_id' , $id )
            ->first();


        $path = public_path($file->file_path ?? '' );

        if (File::exists($path)) {
            File::delete($path);
            DB::table('clearance_attachments')
                ->where('clearance_attachment_id' , $id )
                ->delete();
        }

        if ($file)
            DB::table('clearance_attachments')
                ->where('clearance_attachment_id' , $id )
                ->delete();

        return true;

    }
}
