<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class CustomerClaim extends Model
{

    protected $primaryKey = 'customer_claim_id';

    public static $main_table = 'customer_claims_m';
    public $table             = 'customer_claims_m';

    public static $rules = array(
        'name_en'               => 'required|string' ,
        'name_ar'               => 'required|string' ,
        'mobile'                => 'required|string' ,
        'vat_register_number'   => 'required|string' ,
    );

    public static function all_customer_claims($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('daily_journal_m' , 'customer_claims_m.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('customers' , 'customer_claims_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('customers_info' , 'customers_info.customer_id' , '=' , 'customer_claims_m.customer_id')
            ->leftjoin('clearances_m' , 'customer_claims_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('ports' , 'customer_claims_m.port_id' , '=' , 'ports.port_id')
            ->leftjoin('users' , 'users.id' , '=' , self::$main_table.'.add_user')
            ->orderBy('customer_claims_m.customer_claim_id','DESC')
            ->groupBy('customer_claims_m.clearance_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                customer_claims_m.customer_claim_id as id ,
                customer_claims_m.clearance_id ,
                customers.name_'.$params['lang'].' as customer_name  ,
                MAX(customers_info.customers_info_email) as customer_email,
                ports.port_name_'.$params['lang'].' as port_name  ,
                customer_claims_m.customer_claim_date ,
                customer_claims_m.daily_journal_m_id ,
                customer_claims_m.daily_journal_m_id as daily_journal_id ,
                daily_journal_m.year_ser as journal_code ,
                total_delegate,
                total_cost ,
                total_amount ,
                total_journals ,
                total_cost_without_vat ,
                doc_id  ,
                doc_code  ,
                draft  ,
                vat_per ,
                customer_claims_m.add_date,
                users.user_full_name as added_by
            ');

        if(isset($params['active']))
            $query->where('customer_claim_active' , $params['active'] );


       if(isset($params['draft']))
            $query->where('draft' , $params['draft'] );

       if(isset($params['clearance_id']))
            $query->where(self::$main_table.'.clearance_id' , $params['clearance_id'] );


       if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customer_claims_m.customer_claim_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_en' , 'LIKE' , "%{$params['word']}%");
            });

        }

       if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
       }else{
            $data = $query->get();
       }

        return $data;
    }

    public static function get_customer_claim($clearance_id , $lang = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('customers' , 'customer_claims_m.customer_id' , '=' , 'customers.customer_id')
            ->leftjoin('customers_info' , 'customers_info.customer_id' , '=' , 'customer_claims_m.customer_id')
            ->leftjoin('clearances_m' , 'customer_claims_m.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftjoin('currencys as cur1' , 'clearances_m.currency_id' , '=' , 'cur1.currency_id')
            ->leftjoin('ports' , 'customer_claims_m.port_id' , '=' , 'ports.port_id')
            ->where('customer_claims_m.clearance_id' , $clearance_id)
            ->where('customer_claims_m.customer_claim_active' , 1);

        if( $lang != null ){
            $query->selectRaw('
                clearances_m.branch_id ,
                clearances_m.lcl_count ,
                clearances_m.invoice_type ,
                clearances_m.goods_type ,
                clearances_m.exporting_company ,
                clearances_m.invoice_amount ,
                cur1.name_'.$lang.' as currency_name ,
                customer_claims_m.customer_claim_active ,
                customer_claims_m.customer_id ,
                MAX(customers_info.customers_info_email) as customer_email,
                customer_claims_m.customer_claim_id   ,
                customer_claims_m.finance_year_id,
                customer_claims_m.clearance_id ,
                customers.name_'.$lang.' as customer_name  ,
                ports.port_name_'.$lang.' as port_name  ,
                customer_claims_m.customer_balance ,

                customers.vat_register_number ,
                customers.address ,
                customers.account_tree_id as customer_account_tree_id ,
                customer_claims_m.customer_claim_date ,
                customer_claims_m.daily_journal_m_id ,

                clearances_m.file_no ,
                clearances_m.goods_type ,
                clearances_m.customs_declaration_date ,
                clearances_m.customs_declaration_number ,
                clearances_m.customs_declaration_file ,
                clearances_m.withdrawal_note ,
                clearances_m.classification_id ,
                clearances_m.action_date ,
                clearances_m.shipping_policy_no ,
                clearances_m.clearance_type ,
                clearances_m.container_count ,
                clearances_m.container_small ,
                clearances_m.container_big ,
                clearances_m.is_lcl ,


                total_delegate,
                total_cost ,
                total_amount ,
                total_journals ,
                total_cost_without_vat ,
                vat_per ,
                customer_claims_m.add_date,
                doc_id  ,
                doc_code  ,
                draft
            ');
        }

        $data = $query->first();

        return $data;
    }

    public static function get_customer_claim_delegate_d($id , $lang)
    {
        return DB::table('customer_claims_delegate_d')
            ->leftjoin('expense_codes' , 'customer_claims_delegate_d.expense_code_id' , '=' , 'expense_codes.expense_code_id')
            ->where('customer_claim_m_id' , $id )
            ->where('expense_code_active',1)
            ->selectRaw('
                customer_claims_delegate_d_id as d_id ,
                customer_claims_delegate_d.expense_code_id ,
                expense_codes.name_'.$lang.' as expense_code_name ,
                receipt_number ,
                expense_codes.account_tree_id as expense_code_account_tree_id ,
                amount ,
                description_'.$lang.' as description ,
                customer_claims_delegate_d.add_date
            ')
            ->whereNotNull('expense_codes_order')
            ->orderBy('expense_codes_order', 'asc')
            ->get();
    }


//    public static function get_invoice_journals($id , $lang)
//    {
//        return DB::table('clearance_invoices_journal')
//            // ->leftjoin('account_tree' , 'customer_claims_delegate_d.account_tree_id' , '=' , 'account_tree.account_tree_id')
//            ->where('invoice_m_id' , $id )
//            ->selectRaw('
//                clearance_invoices_journal_id as d_id ,
//                invoice_m_id as m_id ,
//                clearance_invoices_journal.account_tree_id ,
//                amount
//            ')
//            ->get();
//    }




    public static function get_customer_claim_d($id , $lang = null)
    {
        return DB::table('customer_claims_d')
            ->leftjoin('revenues' , 'customer_claims_d.revenue_id' , '=' , 'revenues.revenue_id')
            ->selectRaw('*,revenues.revenue_name_'.$lang.' as revenue_name, customer_claims_d.amount_without_vat')
            ->where('customer_claim_m_id' , $id)
            ->where('revenues_active',1)
            ->whereNotNull('revenues_order')
            ->orderBy('revenues_order' , 'asc')
            ->get();

    }
    public static function add_customer_claim($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function add_customer_claim_d($seed)
    {
        return DB::table('customer_claims_d')->insert($seed);
    }

    public static function update_customer_claim($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('customer_claim_id' , $id )->update($seed);
    }

//    public static function delete_customer_claim($id)
//    {
//        return DB::table(self::$main_table)->where('customer_claim_id' , $id )->update( array('customer_claim_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
//    }
    public static function hard_delete_customer_claim_m($customer_claim_id)
    {
        return DB::table(self::$main_table)->where('customer_claim_id' , $customer_claim_id )->delete();
    }

    public static function hard_delete_customer_claim_d($customer_claim_m_id)
    {
        return DB::table('customer_claims_d')->where('customer_claim_m_id',$customer_claim_m_id)->delete();
    }

    public static function hard_delete_customer_claim_delegate_d($customer_claim_m_id)
    {
        return DB::table('customer_claims_delegate_d')->where('customer_claim_m_id',$customer_claim_m_id)->delete();
    }

    public static function all_attachment($lang,$clearance_id){
        return  DB::table('clearance_attachments')
            ->leftJoin('delegate_petty_d','clearance_attachments.clearance_id','=','delegate_petty_d.clearance_id')
            ->leftjoin('clearance_file_names' , 'clearance_attachments.clearance_file_name_id' , '=' , 'clearance_file_names.id')
            ->where('clearance_attachments.clearance_id',$clearance_id)
            ->orWhere('delegate_petty_d.clearance_id',$clearance_id)
            ->selectRaw('clearance_file_names.name_'.$lang.' as file_name ,clearance_attachments.file_path,clearance_attachments.file_type,clearance_attachment_id as id,clearance_attachments.clearance_id')
            ->get();
    }

    public static function clearance_attachment($lang,$clearance_id){
        return  DB::table('clearance_attachments')
            ->leftjoin('clearance_file_names' , 'clearance_attachments.clearance_file_name_id' , '=' , 'clearance_file_names.id')
            ->where('clearance_attachments.clearance_id',$clearance_id)
            ->selectRaw('"clearance_attachment" as type_attachment  , clearance_file_names.name_'.$lang.' as file_name ,clearance_attachments.file_path,clearance_attachment_id as id')
            ->get();
    }
    public static function delegate_petty_attachment($lang,$clearance_id){
        return  DB::table('delegate_petty_d')
            ->where('delegate_petty_d.clearance_id',$clearance_id)
            ->selectRaw('"delegate_attachment" as type_attachment , "" as file_name,delegate_petty_d.file_path,delegate_petty_d_id as id')
            ->get();
    }
}
