<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class Delegate extends Model
{
    protected $primaryKey     = 'delegate_id';
    public static $main_table = 'delegates';
    public $table             = 'delegates';

    public static $rules = array(
        'name_en'          => 'required|string',
        'name_ar'          => 'required|string',
        'account_tree_id'  => 'required|numeric',
        'covenant_type_id' => 'required|numeric',
        'is_delegate_petty' => 'required|numeric',
    );

    public static function all_delegates($params = null)
    {
        $userDelegateIds = auth()->user()->delegate_ids;
        $query = DB::table(self::$main_table)
            ->leftjoin('users' , self::$main_table.'.add_user' , '=' , 'users.id')
            ->leftjoin('account_tree' , self::$main_table.'.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->leftjoin('covenant_types' , self::$main_table.'.covenant_type_id' , '=' , 'covenant_types.covenant_type_id')
            ->orderBy(self::$main_table.'.delegate_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                delegate_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.account_tree_id ,
                account_tree.name_'.$params['lang'].' as account_tree_name ,'.
                self::$main_table.'.covenant_type_id ,
                balance ,
                is_delegate_petty ,
                users.user_full_name as added_by,
                covenant_types.name_'.$params['lang'].' as covenant_type_name ,'.
                self::$main_table.'.delegate_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('delegate_active', $params['active']);

        if(isset($params['delegate_ids']) && !is_null($userDelegateIds))
            $query->whereIn('delegate_id',explode(',',auth()->user()->delegate_ids));

        if(isset($params['covenant_type_id']))
            $query->where(self::$main_table.'.covenant_type_id', $params['covenant_type_id']);

        if(isset($params['is_delegate_petty']))
            $query->where(self::$main_table.'.is_delegate_petty', $params['is_delegate_petty']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('covenant_types.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('covenant_types.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.delegate_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }
        return $data;
    }

    public static function get_delegate($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('account_tree' , self::$main_table.'.account_tree_id' , '=' , 'account_tree.account_tree_id')
            ->leftjoin('covenant_types' , self::$main_table.'.covenant_type_id' , '=' , 'covenant_types.covenant_type_id')
            ->where('delegate_id', $id);

        if($lang != null)
            $query->selectRaw('
                delegate_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.account_tree_id ,
                account_tree.name_'.$lang.' as account_tree_name ,'.
                self::$main_table.'.covenant_type_id ,
                balance ,
                is_delegate_petty ,
                covenant_types.name_'.$lang.' as covenant_type_name ,'.
                self::$main_table.'.delegate_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_delegate($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_delegate($seed, $id)
    {
        return DB::table(self::$main_table)->where('delegate_id', $id)->update($seed);
    }

    public static function delete_delegate($id)
    {
        return DB::table(self::$main_table)->where('delegate_id', $id)->update(array('delegate_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
