<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Insurance extends Model
{

    protected $primaryKey = 'insurance_id';

    public static $main_table = 'insurances';
    public $table             = 'insurances';

    public static $rules = array(
        'clearance_id'      => 'required|numeric' ,
        'agent_id'          => 'required|numeric' ,
        'action_date'       => 'required|date' ,
    );



    public static function all_insurances($params = null)
    {
        $query =  DB::connection('other_mysql')->table(self::$main_table)
            ->leftJoin('agents' , 'insurances.agent_id' , '=' , 'agents.agent_id')
            ->leftJoin('clearances_m' , 'clearances_m.clearance_id' , '=' , self::$main_table.'.clearance_id')
            ->leftJoin('customers' , 'customers.customer_id' , '=' , 'clearances_m.customer_id');
        if( isset($params['lang']) ){
            $query->selectRaw('
                insurance_id as id ,
                insurances.agent_id ,
                agents.agent_name_'.$params['lang'].' as agent_name ,
                insurances.action_date ,
                insurances.clearance_id,
                customers.customer_id ,
                customers.name_'.$params['lang'].' as customer_name,
                amount,
                status ,
                return_amount ,
                return_file ,
                return_date ,
                total_containers ,
                return_containers ,
                clearances_m.shipping_policy_no,
                (total_containers - return_containers ) as rem_containers
            ');
        }
        if(isset($params['active']))
            $query->where('insurance_active' , $params['active'] );

        if(isset($params['agent_id']))
            $query->where('insurances.agent_id' , $params['agent_id'] );

        if(isset($params['clearance_id']))
            $query->where('insurances.clearance_id' , $params['clearance_id'] );

        if(isset($params['from_date']))
            $query->where('insurances.action_date' , '>=' , $params['from_date'] );

        if(isset($params['to_date']))
            $query->where('insurances.action_date' , '<=' , $params['to_date'] );

        if(isset($params['status']))
            $query->where('insurances.status' , $params['status'] );


        if(isset($params['complete']) && $params['complete']){
            $query->whereRaw('total_containers = return_containers')
                ->where('total_containers' , '>' , 0 );
        }

        if(isset($params['complete']) && !$params['complete']){
            $query->whereRaw('total_containers > return_containers')
                ->where('total_containers' , '>' , 0 );
        }


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('agents.agent_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('agents.agent_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('insurances.clearance_id' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();

        }



        return $data;
    }

    public static function get_insurance($id , $lang = null)
    {
        $query = DB::connection('other_mysql')->table(self::$main_table)
            ->leftJoin('agents' , 'insurances.agent_id' , '=' , 'agents.agent_id')
            ->leftJoin('clearances_m' , self::$main_table.'.clearance_id' , '=' , 'clearances_m.clearance_id')
            ->leftJoin('customers' , 'clearances_m.customer_id' , '=' , 'customers.customer_id')
            ->where('insurance_id' , $id);

        if($lang != null){
            $query->selectRaw('
                insurance_id as id ,
                insurances.agent_id ,
                agents.agent_name_'.$lang.' as agent_name ,
                customers.name_'.$lang.' as customer_name ,'
                .self::$main_table.'.action_date ,'
                .self::$main_table.'.clearance_id,
                amount,
                status ,
                return_amount ,
                return_file ,
                return_file as file_upload ,
                return_date ,
                total_containers ,
                return_containers ,
                (total_containers - return_containers ) as rem_containers
            ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_insurance($seed)
    {
        return DB::connection('other_mysql')->table(self::$main_table)->insertGetId($seed);
    }

    public static function update_insurance($seed ,  $id)
    {
        return DB::connection('other_mysql')->table(self::$main_table)->where('insurance_id' , $id )->update($seed);
    }

    public static function delete_insurance($id)
    {
        return DB::connection('other_mysql')->table(self::$main_table)->where('insurance_id' , $id )->update( array('insurance_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
