<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class Item extends Model
{
    protected $primaryKey     = 'item_id';
    public static $main_table = 'items';
    public $table             = 'items';

    public static $rules = array(
        'name_en'          => 'required|string',
        'name_ar'          => 'required|string',
    );

    public static function all_items($params = null)
    {
        $query = DB::table(self::$main_table)
            ->orderBy(self::$main_table.'.item_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                item_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.item_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('item_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_item($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('item_id', $id);

        if($lang != null)
            $query->selectRaw('
                item_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.item_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_item($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_item($seed, $id)
    {
        return DB::table(self::$main_table)->where('item_id', $id)->update($seed);
    }

    public static function delete_item($id)
    {
        return DB::table(self::$main_table)->where('item_id', $id)->update(array('item_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }
}
