<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class Quotation extends Model
{

    protected $primaryKey = 'id';

    public static $main_table = 'quotations';
    public $table             = 'quotations';

    public static $rules = array(
        'customer_name'      => 'required|string|unique:quotations' ,
    );



    public static function get_all($params = null)
    {
        $query =  DB::table(self::$main_table);

        if( isset($params['lang']) ){
            $query->selectRaw('
                id ,
                customer_name ,
                responsible_person ,
                customer_code ,
                active ,
                add_user ,
                add_date

            ');
        }

        if(isset($params['active']))
            $query->where('active' , $params['active'] );

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('quotations.customer_name' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('quotations.responsible_person' , 'LIKE' , "%{$params['word']}%");
                });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();

        }

        return $data;
    }

    public static function get_one($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('id' , $id);

        if($lang != null){
            $query->selectRaw('
                id ,
                customer_name ,
                responsible_person ,
                customer_code ,
                active ,
                add_user ,
                add_date
            ');
        }

        $data = $query->first();

        return $data;
    }
    public static function get_by_customer($customer_name , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('customer_name' , $customer_name);

        if($lang != null){
            $query->selectRaw('
                id ,
                customer_name ,
                responsible_person ,
                customer_code ,
                active ,
                add_user ,
                add_date
            ');
        }

        $data = $query->first();

        return $data;
    }

    public static function get_details($id , $lang = null)
    {
        $query = DB::table('quotation_details')
            ->where('m_id' , $id);

        if($lang != null){
            $query->selectRaw('
                d_id ,
                m_id ,
                statement ,
                amount

            ');
        }

        $data = $query->get();

        return $data;
    }

    public static function add_master($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }
    public static function add_detail($seed)
    {
        return DB::table('quotation_details')->insertGetId($seed);
    }
    public static function update_one($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('id' , $id )->update($seed);
    }

    public static function delete_one($id)
    {
        return DB::table(self::$main_table)->where('id' , $id )->update( array('active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
