<?php

namespace App\Models\Sales;

use Illuminate\Database\Eloquent\Model;
use DB;

class Unit extends Model
{
    protected $primaryKey     = 'unit_id';
    public static $main_table = 'units';
    public $table             = 'units';

    public static $rules = array(
        'name_en'          => 'required|string',
        'name_ar'          => 'required|string',
    );

    public static function all_units($params = null)
    {
        $query = DB::table(self::$main_table)
            ->orderBy(self::$main_table.'.unit_id', 'desc');

        if(isset($params['lang']))
            $query->selectRaw('
                unit_id as id ,'.
                self::$main_table.'.name_'.$params['lang'].' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.unit_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        if(isset($params['active']))
            $query->where('unit_active', $params['active']);

        if(isset($params['word']) && strlen($params['word']) > 0){
            $query->where(function ($stmt) use($params) {
                $stmt->where(self::$main_table.'.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere(self::$main_table.'.unit_id' , 'LIKE' , "%{$params['word']}%");
            });
        }

        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_unit($id, $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('unit_id', $id);

        if($lang != null)
            $query->selectRaw('
                unit_id as id ,'.
                self::$main_table.'.name_'.$lang.' as name ,'.
                self::$main_table.'.name_ar ,'.
                self::$main_table.'.name_en ,'.
                self::$main_table.'.unit_active,'.
                self::$main_table.'.add_date,'.
                self::$main_table.'.add_user
            ');

        $data = $query->first();
        return $data;
    }

    public static function add_unit($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_unit($seed, $id)
    {
        return DB::table(self::$main_table)->where('unit_id', $id)->update($seed);
    }

    public static function delete_unit($id)
    {
        return DB::table(self::$main_table)->where('unit_id', $id)->update(array('unit_active' => 0, 'disable_date' => date("Y-m-d H:i:s"), 'disable_user' => auth()->user()->id));
    }

    public function units(array $row)
    {
        return new Unit([
            'name_ar' => $row['name'],
            'name_en'=>$row['email'],
        ]);
    }
}
