<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Area extends Model
{

    protected $primaryKey = 'area_id';

    public static $main_table = 'areas';

    public static $rules = array(
        'area_name_en'               => 'required|string|unique:areas' ,
        'area_name_ar'               => 'required|string|unique:areas'
    );



    public static function all_areas($params = null)
    {
        $query =  DB::table(self::$main_table);

        if( isset($params['lang']) )
            $query->selectRaw('area_id as id , area_name_'.$params['lang'].' as name ');

        if(isset($params['active']))
            $query->where('area_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('area_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('area_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('area_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_area($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('area_id' , $id);

        if($lang != null){
            $query->selectRaw('area_id as id , area_name_'.$lang.' as name ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_area($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_area($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('area_id' , $id )->update($seed);
    }

    public static function delete_area($id)
    {
        return DB::table(self::$main_table)->where('area_id' , $id )->update( array('area_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
