<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Car extends Model
{

    protected $primaryKey = 'car_id';

    public static $main_table = 'cars';

    public static $rules = array(
        'car_name_en'               => 'required|string' ,
        'car_name_ar'               => 'required|string'
    );



    public static function all_cars($params = null)
    {
        $query =  DB::table(self::$main_table)
        ->leftjoin('drivers' ,'cars.default_driver','drivers.driver_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                cars.car_id as id ,
                cars.car_name_'.$params['lang'].' as name  ,
                drivers.driver_name_'.$params['lang'].' as driver_name ,
                cars.default_driver ,
                cars.weight,
                cars.car_code ,
                cars.surface_number ,
                cars.axes_number ,
                cars.car_type ,
                cars.form_expiry_date ,
                cars.form_expiry_image ,
                cars.card_expiry_date ,
                cars.card_expiry_image
                  ');

        if(isset($params['active']))
            $query->where('car_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('car_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('car_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_car($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('car_id' , $id);

        if($lang != null){
            $query->selectRaw('car_id as id , car_name_'.$lang.' as name , default_driver , weight , car_code ,surface_number , axes_number , car_type ,form_expiry_date , form_expiry_image , card_expiry_date , card_expiry_image');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_car($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_car($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('car_id' , $id )->update($seed);
    }

    public static function delete_car($id)
    {
        return DB::table(self::$main_table)->where('car_id' , $id )->update( array('car_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
