<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Driver extends Model
{

    protected $primaryKey = 'driver_id';

    public static $main_table = 'drivers';

    public static $rules = array(
        'driver_name_en'        => 'required|string' ,
        'driver_name_ar'        => 'required|string'
    );

    public static function all_drivers($params = null)
    {
        $query =  DB::table(self::$main_table)
        ->leftJoin('cars',self::$main_table.'.driver_id','=','cars.default_driver')
        ->leftJoin('branchs',self::$main_table.'.branch_id','=','branchs.branch_id')
        ->leftJoin('ports',self::$main_table.'.port_id','=','ports.port_id')

        ->groupBy(self::$main_table.'.driver_id');

        if( isset($params['lang']) )
            $query->selectRaw('
                car_name_'.$params['lang'].' as car_name ,
                branchs.name_'.$params['lang'].' as branch_name ,
                ports.port_name_'.$params['lang'].' as port_name ,
                drivers.driver_id as id ,
                drivers.driver_name_'.$params['lang'].' as name  ,
                drivers.mobile ,
                drivers.port_id ,
                drivers.bank_number ,
                drivers.iban ,

                drivers.licence_expiry_date ,
                drivers.residency_number ,
                drivers.permit_expiry_date  ,
                drivers.card_expiry_date
          ');

        if(isset($params['active']))
            $query->where('driver_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('driver_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('driver_id' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('car_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('car_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('residency_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('ports.port_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('branchs.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('branchs.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('drivers.mobile' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('driver_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_driver($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('driver_id' , $id);

        if($lang != null){
            $query->selectRaw('driver_id as id , driver_name_'.$lang.' as name  , mobile , port_id ,drivers.bank_number ,
                drivers.iban , licence_expiry_date, residency_number , permit_expiry_date , card_expiry_image , card_expiry_date');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_driver($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_driver($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('driver_id' , $id )->update($seed);
    }

    public static function delete_driver($id)
    {
        return DB::table(self::$main_table)->where('driver_id' , $id )->update( array('driver_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
