<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class DriverCard extends Model
{

    protected $primaryKey = 'driver_card_id';

    public static $main_table = 'driver_cards';

    public static $rules = array(
        'amount'        => 'required|numeric|min:1',
        'driver_id'     => 'required|numeric'
    );



    public static function all_driver_cards($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->join('drivers' , 'driver_cards.driver_id' , '=' , 'drivers.driver_id')
            ->orderBy('driver_cards.driver_card_id' , 'desc');

        if( isset($params['lang']) )
            $query->selectRaw('
                driver_card_id as id ,
                drivers.driver_id ,
                driver_name_'.$params['lang'].' as name  ,
                drivers.bank_number ,
                drivers.iban ,
                action_date ,
                driver_card_active as active ,
                amount
             ');

        if(isset($params['active']))
            $query->where('driver_card_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('driver_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('driver_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_driver_card($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('drivers' , 'driver_cards.driver_id' , '=' , 'drivers.driver_id')
            ->leftjoin('cars' , 'driver_cards.driver_id' , '=' , 'cars.default_driver')
            ->where('driver_card_id' , $id);

        if($lang != null){
            $query->selectRaw('
            driver_card_id as id ,
            driver_name_'.$lang.' as name ,
            drivers.bank_number ,
            drivers.iban ,
            from_date ,
            to_date ,
            action_date ,
            driver_card_items,
            cars.car_name_'.$lang.' as car_name
                 ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_driver_card($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_driver_card($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('driver_card_id' , $id )->update($seed);
    }

    public static function delete_driver_card($id)
    {
        return DB::table(self::$main_table)->where('driver_card_id' , $id )->delete();
        // return DB::table(self::$main_table)->where('driver_card_id' , $id )->update( array('driver_card_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
