<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Invoice extends Model
{

    protected $primaryKey = 'invoice_id';

    public static $main_table = 'invoices';

    public static $rules = array(
        'invoice_name_en'               => 'required|string' ,
        'invoice_name_ar'               => 'required|string'
    );



    public static function all_invoices($params = null)
    {
        $query =  DB::table(self::$main_table)
            ->leftjoin('daily_journal_m' , 'invoices.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id' )
            ->orderBy('invoices.invoice_id' , 'desc');

        if( isset($params['lang']) ){
            $query->selectRaw('

                invoice_id as m_id ,
                invoice_id ,
                invoices.type ,
                invoices.daily_journal_m_id as daily_journal_id ,
                daily_journal_m.year_ser as journal_code ,
                invoice_date ,
                invoices.branch_id,
                invoices.finance_year_id,
                trips_total ,
                transactions_total ,
                total ,
                discount ,
                total_without_vat ,
                vat_per ,
                total_vat ,
                total_with_vat ,
                invoices.add_date,
                if(invoices.customer_id = 0 , "شركه امواج المد للخدمات اللوجستية" , customers.name_'.$params['lang'].') as customer_name
                ');
        }


        if(isset($params['active']))
            $query->where('invoice_active' , $params['active'] );

        if(isset($params['type']))
            $query->where('invoices.type' , $params['type'] );

        if(isset($params['customer']))
            $query->where( self::$main_table.'.customer_id' , $params['customer'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_invoice($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->leftjoin('daily_journal_m' , 'invoices.daily_journal_m_id' , '=' , 'daily_journal_m.daily_journal_m_id')
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id' )
            ->leftjoin('branchs' , self::$main_table.'.branch_id' , '=' , 'branchs.branch_id' )
            ->where('invoice_id' , $id);

        if( isset($lang) ){
            $query->selectRaw('

                invoice_id as m_id ,
                invoice_id ,
                invoices.type,
                customers.customer_id ,
                customers.account_tree_id as customer_account_tree_id,
                invoices.daily_journal_m_id as daily_journal_id ,
                daily_journal_m.year_ser as journal_code ,
                invoice_date ,
                invoices.branch_id,
                branchs.name_'.$lang.' as branch_name ,
                invoices.finance_year_id,
                trips_total ,
                transactions_total ,
                total ,
                discount ,
                total_without_vat ,
                vat_per ,
                total_vat ,
                total_with_vat ,
                invoices.add_date,
                invoices.description ,
                invoices.from_date ,
                invoices.to_date ,
                invoices.invoice_active ,
                
                if(invoices.customer_id = 0 , "شركه امواج المد للخدمات اللوجستية" , customers.name_'.$lang.') as customer_name ,
                if(invoices.customer_id = 0 , "300151673100003" , customers.vat_register_number) as vat_register_number ,
                if(invoices.customer_id = 0 , " " , customers.address) as address
                ');
        }
        $data = $query->first();

        return $data;
    }

    public static function add_invoice($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_invoice($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('invoice_id' , $id )->update($seed);
    }

    public static function delete_invoice($id)
    {
        return DB::table(self::$main_table)->where('invoice_id' , $id )->update( array('invoice_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
