<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use DB;

class Wearhouse extends Model
{

    protected $primaryKey = 'wearhouse_id';

    public static $main_table = 'wearhouses';

    public static $rules = array(
        'wearhouse_name_en'      => 'required|string' ,
        'wearhouse_name_ar'      => 'required|string' ,
        'area_id'           => 'required|numeric'
    );

    

    public static function all_wearhouses($params = null)
    {
        $query =  DB::table(self::$main_table);

        if( isset($params['lang']) )
            $query->selectRaw('wearhouse_id as id , wearhouse_name_'.$params['lang'].' as name  , area_id , price , empty_20 , filled_20 , empty_40 , filled_40 , package_price ,loading_price , switching_price , allowed_days'); 

        if(isset($params['active']))
            $query->where('wearhouse_active' , $params['active'] );


        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('wearhouse_name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('wearhouse_name_ar' , 'LIKE' , "%{$params['word']}%");
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_wearhouse($id , $lang = null)
    {
        $query = DB::table(self::$main_table)
            ->where('wearhouse_id' , $id);

        if($lang != null){
            $query->selectRaw('wearhouse_id as id , wearhouse_name_'.$lang.' as name  , area_id , price ,empty_20 , filled_20 , empty_40 , filled_40 , package_price ,loading_price , switching_price , allowed_days'); 
        }

        $data = $query->first();

        return $data;
    }

    public static function add_wearhouse($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_wearhouse($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('wearhouse_id' , $id )->update($seed);
    }

    public static function delete_wearhouse($id)
    {
        return DB::table(self::$main_table)->where('wearhouse_id' , $id )->update( array('wearhouse_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }
}
