<?php

namespace App\Models\Trans;

use Illuminate\Database\Eloquent\Model;
use Illuminate\Support\Facades\DB;

class WearhouseTransaction extends Model
{

    protected $primaryKey = 'wearhouse_transaction_id';

    public static $main_table = 'wearhouse_transactions';

    public static $rules = array(
        'wearhouse_id'       => 'required|numeric',
        'customer_id'       => 'required|numeric'
    );



    public static function all_wearhouse_transactions($params = null)
    {
        $other_db = session()->get('otherDB');
        $query =  DB::table(self::$main_table)
            ->leftjoin('trips as in_trips' , self::$main_table.'.in_trip_id' , '=' , 'in_trips.trip_id' )
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id' )
            ->leftjoin('branchs' , self::$main_table.'.branch_id' , '=' , 'branchs.branch_id' )
            ->leftjoin($other_db.'.clearances_d' , self::$main_table.'.clearance_d_id' , '=' , 'clearances_d.clearances_d_id' )
            ->leftjoin($other_db.'.clearances_m' , function ($join) {
                $join->on(self::$main_table.'.clearance_m_id' , '=', 'clearances_m.clearance_id')
                    ->where('clearances_m.clearance_active', 1);
            })
            ->leftjoin($other_db.'.customers as amwag_cust' , self::$main_table.'.customer_id' , '=' , 'amwag_cust.customer_id')
            ->leftjoin('areas as final_dest','final_dest.area_id','clearances_d.dest_id')
            // ->where('clearances_m.clearance_active', 1)
            ->orderBy('wearhouse_transaction_id' , 'desc');

        if( isset($params['lang']) ){
            $query->selectRaw('
                wearhouse_transaction_id as id , '.
                self::$main_table.'.wearhouse_id as wearhouse_id ,'.
                self::$main_table.'.customer_id ,'.
                self::$main_table.'.clearance_m_id ,'.
                self::$main_table.'.clearance_d_id ,
                wearhouse_transaction_id as code_number,
                clearances_m.note ,
                if( wearhouse_transactions.transaction_type = 1 , amwag_cust.name_'.$params['lang'].' , customers.name_'.$params['lang'].' ) as customer_name ,
                if( wearhouse_transactions.transaction_type = 1 , amwag_cust.vat_register_number , customers.vat_register_number ) as customer_vat_number ,
                if( wearhouse_transactions.transaction_type = 1 , amwag_cust.mobile , customers.mobile ) as customer_mobile ,

                if(wearhouse_transactions.transaction_type = 1 , clearances_d.container_number , '.self::$main_table.'.container_number ) as container_number ,
                if(wearhouse_transactions.transaction_type = 1  , clearances_d.container_size , '.self::$main_table.'.container_size ) as container_size ,
                clearances_d.dest_id,
                final_dest.area_name_'.$params['lang'].' as final_dest_name ,
                transaction_type ,
                is_box ,
                wearhouse_transactions.is_empty ,
                requested_by ,

                wearhouse_transactions.in_trip_id ,
                wearhouse_transactions.in_driver_residency_number ,
                wearhouse_transactions.in_driver_name ,
                wearhouse_transactions.in_driver_mobile ,
                wearhouse_transactions.in_car_number,

                wearhouse_transactions.out_trip_id ,
                ifnull(wearhouse_transactions.out_driver_residency_number , wearhouse_transactions.in_driver_residency_number ) as in_driver_residency_number ,
                ifnull(wearhouse_transactions.out_driver_name , wearhouse_transactions.in_driver_name ) as out_driver_name ,
                ifnull(wearhouse_transactions.out_driver_mobile , wearhouse_transactions.in_driver_mobile ) as out_driver_mobile ,
                ifnull(wearhouse_transactions.out_car_number , wearhouse_transactions.in_car_number ) as out_car_number ,

                store_place ,
                cost_per_day ,'.
                self::$main_table.'.in_date ,'.
                self::$main_table.'.out_date ,'.
                self::$main_table.'.out_user ,'.
                self::$main_table.'.out_notes ,
                in_user ,
                wearhouse_transactions.status ,
                in_notes ,
                total_days ,
                total_cost , '.
                self::$main_table.'.invoice_id,
                branchs.name_'.$params['lang'].' as branch_name,
                clearances_d.recommended_return_date ,
                wearhouse_transactions.company_bears_expenses
            ');
        }



        if(isset($params['ids']))
            $query->whereIn('wearhouse_transactions.wearhouse_transaction_id' , $params['ids'] );

        if(isset($params['active']))
            $query->where('wearhouse_transaction_active' , $params['active'] );

        if(isset($params['status']))
            $query->where('wearhouse_transactions.status' , $params['status'] );

        if(isset($params['invoice_id']))
            $query->where( self::$main_table.'.invoice_id' , $params['invoice_id'] );

        if(isset($params['customer']) && $params['customer'] == 0 ){
            $query->where(self::$main_table.'.transaction_type' , 1 );
        }elseif(isset($params['customer'])){
            $query->where(self::$main_table.'.customer_id' , $params['customer'] );
        }


        if(isset($params['wearhouse']))
            $query->where(self::$main_table.'.wearhouse_id' , $params['wearhouse'] );

        if(isset($params['from']))
            $query->where(self::$main_table.'.in_date' , '>=' , $params['from']." 00:00:00" );

        if(isset($params['to']))
            $query->where(self::$main_table.'.in_date' , '<=' , $params['to']." 23:59:59" );

        if(isset($params['branch_id']))
            $query->where(self::$main_table.'.branch_id' , $params['branch_id']);


        if(isset($params['clearance_id']))
            $query->where(self::$main_table.'.clearance_m_id' , $params['clearance_id']);

        if(isset($params['port_id'])){

            $query->where(function($where) use($params){
                $where->where('clearances_m.port_id' , $params['port_id']);
                $where->orWhereNull('clearances_m.port_id' );
            });
        }

        if(isset($params['word']) && strlen($params['word']) >  0){

            $query->where(function ($stmt) use($params) {
                $stmt->where('customers.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('customers.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('amwag_cust.name_en' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('amwag_cust.name_ar' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_d.container_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_d.container_size' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('wearhouse_transactions.container_number' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('wearhouse_transactions.container_size' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('wearhouse_transactions.in_driver_name' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('wearhouse_transactions.out_driver_name' , 'LIKE' , "%{$params['word']}%")
                    ->orWhere('clearances_m.clearance_id' , $params['word'] )
                    ->orWhere('wearhouse_transactions.wearhouse_transaction_id' , $params['word'] );
            });

        }


        if(isset($params['rows'])){
            $data = $query->paginate($params['rows'], ['*'], 'page', $params['page'] ?? 1);
        }else{
            $data = $query->get();
        }

        return $data;
    }

    public static function get_wearhouse_transaction($id , $lang = null)
    {
        $other_db = session()->get('otherDB');
        $query = DB::table(self::$main_table)

            // ->leftjoin('trips' , self::$main_table.'.trip_id' , '=' , 'trips.trip_id' )
            ->leftjoin('customers' , self::$main_table.'.customer_id' , '=' , 'customers.customer_id' )
            ->leftjoin($other_db.'.clearances_d' , self::$main_table.'.clearance_d_id' , '=' , 'clearances_d.clearances_d_id' )
            ->leftjoin($other_db.'.clearances_m' , self::$main_table.'.clearance_m_id' , '=' , 'clearances_m.clearance_id' )
            ->leftjoin($other_db.'.customers as amwag_cust' , self::$main_table.'.customer_id' , '=' , 'amwag_cust.customer_id')


            ->where('wearhouse_transaction_id' , $id);

        if( isset($lang) ){
            $query->selectRaw('
                wearhouse_transaction_id as id , '.
                self::$main_table.'.wearhouse_id as wearhouse_id ,'.
                self::$main_table.'.customer_id ,'.
                self::$main_table.'.clearance_m_id ,'.
                self::$main_table.'.clearance_d_id ,
                code_number ,
                clearances_d.is_hold,
                if( transaction_type = 1 , amwag_cust.name_'.$lang.' , customers.name_'.$lang.' ) as customer_name ,
                if( transaction_type = 1 , amwag_cust.vat_register_number , customers.vat_register_number ) as customer_vat_number ,
                if( transaction_type = 1 , amwag_cust.mobile , customers.mobile ) as customer_mobile ,

                if(transaction_type = 1 , clearances_d.container_number , '.self::$main_table.'.container_number ) as container_number ,
                if(transaction_type = 1 , clearances_d.container_size , '.self::$main_table.'.container_size ) as container_size ,
                if(transaction_type = 1 , clearances_d.seal_number , null ) as seal_number ,

                transaction_type ,
                requested_by ,

                is_box ,
                wearhouse_transactions.is_empty ,

                wearhouse_transactions.in_trip_id ,
                wearhouse_transactions.in_driver_residency_number ,
                wearhouse_transactions.in_driver_name ,
                wearhouse_transactions.in_driver_mobile ,
                wearhouse_transactions.in_car_number,
                wearhouse_transactions.notes ,
                wearhouse_transactions.out_trip_id ,
                ifnull(wearhouse_transactions.out_driver_residency_number , wearhouse_transactions.in_driver_residency_number ) as out_driver_residency_number ,
                ifnull(wearhouse_transactions.out_driver_name , wearhouse_transactions.in_driver_name ) as out_driver_name ,
                ifnull(wearhouse_transactions.out_driver_mobile , wearhouse_transactions.in_driver_mobile ) as out_driver_mobile ,
                ifnull(wearhouse_transactions.out_car_number , wearhouse_transactions.in_car_number ) as out_car_number ,


                store_place ,
                cost_per_day ,

                in_date ,
                in_user ,
                out_date ,
                out_user ,
                wearhouse_transactions.status ,
                in_notes ,
                out_notes ,
                total_days ,
                total_cost , '.
                self::$main_table.'.invoice_id,
                wearhouse_transactions.company_bears_expenses,
                clearances_m.shipping_policy_no
            ');
        }

        $data = $query->first();

        return $data;
    }

    public static function add_wearhouse_transaction($seed)
    {
        return DB::table(self::$main_table)->insertGetId($seed);
    }

    public static function update_wearhouse_transaction($seed ,  $id)
    {
        return DB::table(self::$main_table)->where('wearhouse_transaction_id' , $id )->update($seed);
    }

    public static function delete_wearhouse_transaction($id)
    {
        return DB::table(self::$main_table)->where('wearhouse_transaction_id' , $id )->delete();
        // return DB::table(self::$main_table)->where('wearhouse_transaction_id' , $id )->update( array('wearhouse_transaction_active' => '0' , 'disable_date' => date("Y-m-d H:i:s") , 'disable_user' => auth()->user()->id));
    }



}
