/**
 * First we will load all of this project's JavaScript dependencies which
 * includes Vue and other libraries. It is a great starting point when
 * building robust, powerful web applications using Vue and Laravel.
 */




import Vue from 'vue';
import VueProgressBar from 'vue-progressbar'
import App from "./App.vue";


import router from './router';
import store from './store';
import i18n from "./services/langs";
import { BootstrapVue, IconsPlugin } from 'bootstrap-vue'
// Import Bootstrap an BootstrapVue CSS files (order is important)
import 'bootstrap/dist/css/bootstrap.css'
import 'bootstrap-vue/dist/bootstrap-vue.css'
import 'sweetalert2/dist/sweetalert2.min.css';
import GeneralPlugins from "./services/general-plugins.js";
import VueFullscreen from 'vue-fullscreen'
import HighchartsVue from "highcharts-vue";
import Print from 'vue-print-nb'
import VueQr from 'vue-qr'

import('./assets/css/main-style.scss')
const EngLang = () => import('./assets/css/style.scss');
const AraLang = () => import('./assets/css/style-rtl.scss');
localStorage.getItem("lang") === "ar" ? AraLang() : EngLang();

const options = {
  color: 'green',
  failedColor: '#874b4b',
  thickness: '5px',
  transition: {
    speed: '0.2s',
    opacity: '0.6s',
    termination: 300
  },
  autoRevert: true,
  // location: 'left',
  inverse: false
}


Vue.use(BootstrapVue);
Vue.use(IconsPlugin);
Vue.use(VueProgressBar, options);
Vue.use(GeneralPlugins , store );
Vue.use(VueFullscreen);
Vue.use(HighchartsVue);
Vue.use(Print);
Vue.use(VueQr);

router.beforeEach((to, from, next) => {

  Promise.all([store.dispatch('updateAuthData')]).then(next);
  if (to.meta.auth && !store.getters.isAuthenticated) {
    next({ name: 'Login' })
  } else if (!to.meta.auth && store.getters.isAuthenticated) {
    next({ name: 'Home' });
  } else {
    next()
  }
});


new Vue({
  i18n,
  router,
  store,
  render: h => h(App)
}).$mount("#app");






