import Vue from "vue";
import VueRouter from "vue-router";

Vue.use(VueRouter);

const routes = [
  {
    path: "/login",
    name: "Login",
    meta:{auth:false},
    component: () => import( "../views/auth/Login.vue"), 
  },
  {
    path: "/printInvoiceByQty/:id",
    name: "printInvoiceByQty",
    props: true,
    meta:{auth:true,loading:true},
    component: () => import( "../views/sales/byQTY/print.vue"), 
  },
  {
    path: "/printInvoiceByPiece/:id",
    name: "printInvoiceByPiece",
    props: true,
    meta:{auth:true,loading:true},
    component: () => import( "../views/sales/byPiece/print.vue"), 
  },
  {
    path: "/printUsedInvoice/:id",
    name: "printUsedInvoice",
    props: true,
    meta:{auth:true,loading:true},
    component: () => import( "../views/used/print.vue"), 
  },
  {
    path: "/printReturnInvoiceByQty/:id",
    name: "printReturnInvoiceByQty",
    props: true,
    meta:{auth:true,loading:true},
    component: () => import( "../views/sales/returnQty/print.vue"), 
  },
  {
    path: "/",
    name: "Home",
    meta:{auth:true,loading:true},
    component: () => import( "../views/Home.vue"), 
    children: [
      {
        path: "/dashboard",
        name: "dashboard",
        meta:{auth:true,loading:true},
        component: () => import( "../views/dashboard.vue"), 
      },{
        path: "/invoiceByQty",
        name: "invoiceByQty",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/byQTY/index.vue"), 
      },{
        path: "/createSalesInvoiceByQty",
        name: "createSalesInvoiceByQty",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/byQTY/create.vue"), 
      },{
        path: "/invoiceByPiece",
        name: "invoiceByPiece",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/byPiece/index.vue"), 
      },{
        path: "/createSalesInvoiceByPiece",
        name: "createSalesInvoiceByPiece",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/byPiece/create.vue"), 
      },{
        path: "/usedInvoice",
        name: "usedInvoice",
        meta:{auth:true,loading:true},
        component: () => import( "../views/used/index.vue"), 
      },{
        path: "/createUsedInvoice",
        name: "createUsedInvoice",
        meta:{auth:true,loading:true},
        component: () => import( "../views/used/create.vue"), 
      },{
        path: "/salesReturnQty",
        name: "salesReturnQty",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/returnQty/index.vue"), 
      },{
        path: "/createSalesReturnQty",
        name: "createSalesReturnQty",
        meta:{auth:true,loading:true},
        component: () => import( "../views/sales/returnQty/create.vue"), 
      }
    ]
  },
 
];

const router = new VueRouter({
  mode: "history",
  base: process.env.BASE_URL,
  routes,
});

export default router;